@@ -18,6 +18,7 @@ source $LIBDIR/fip_port_forwarding
 source $LIBDIR/uplink_status_propagation
 source $LIBDIR/tag_ports_during_bulk_creation
 source $LIBDIR/octavia
+source $LIBDIR/loki
 
 Q_BUILD_OVS_FROM_GIT=$(trueorfalse False Q_BUILD_OVS_FROM_GIT)
 
@@ -40,11 +41,6 @@ if [[ "$1" == "stack" ]]; then
                 load_conntrack_gre_module
                 start_new_ovs
             fi
-            if is_ovn_enabled; then
-                install_ovn
-                configure_ovn
-                init_ovn
-            fi
             ;;
         post-config)
             if is_service_enabled neutron-tag-ports-during-bulk-creation; then
@@ -110,24 +106,26 @@ if [[ "$1" == "stack" ]]; then
                 configure_ml2_extension_drivers
             fi
             if is_ovn_enabled; then
-                configure_ovn_plugin
-                start_ovn
+                if is_service_enabled q-port-forwarding neutron-port-forwarding; then
+                    configure_port_forwarding
+                fi
+            fi
+            if is_service_enabled neutron-loki; then
+                configure_loki
             fi
             ;;
         extra)
             if is_service_enabled q-sriov-agt neutron-sriov-agent; then
                 start_l2_agent_sriov
             fi
+            if is_service_enabled br-ex-tcpdump ; then
+                # tcpdump monitor on br-ex for ARP, reverse ARP and ICMP v4 / v6 packets
+                sudo ip link set dev $PUBLIC_BRIDGE up
+                run_process br-ex-tcpdump "/usr/sbin/tcpdump -i $PUBLIC_BRIDGE arp or rarp or icmp or icmp6 -enlX" "$STACK_GROUP" root
+            fi
 
-            if is_ovn_enabled; then
-                if [[ "$OVN_L3_CREATE_PUBLIC_NETWORK" == "True" ]]; then
-                    if [[ "$NEUTRON_CREATE_INITIAL_NETWORKS" != "True" ]]; then
-                        echo "OVN_L3_CREATE_PUBLIC_NETWORK=True is being ignored because"
-                        echo "NEUTRON_CREATE_INITIAL_NETWORKS is set to False"
-                    else
-                        create_public_bridge
-                    fi
-                fi
+            if is_service_enabled br-int-flows ; then
+                run_process br-int-flows "/bin/sh -c \"set +e; while true; do echo ovs-ofctl dump-flows br-int; ovs-ofctl dump-flows br-int ; sleep 30; done; \"" "$STACK_GROUP" root
             fi
             ;;
     esac
@@ -139,9 +137,4 @@ elif [[ "$1" == "unstack" ]]; then
        [[ "$Q_BUILD_OVS_FROM_GIT" == "True" ]]; then
         stop_new_ovs
     fi
-
-    if is_ovn_enabled; then
-        stop_ovn
-        cleanup_ovn
-    fi
 fi
