@@ -16,8 +16,6 @@
 import abc
 import contextlib
 
-import six
-
 from neutron_lib.api.definitions import port_security as psec
 from neutron_lib import constants as n_const
 from neutron_lib.utils import runtime
@@ -36,11 +34,8 @@ ICMPV6_ALLOWED_INGRESS_TYPES = (n_const.ICMPV6_TYPE_MLD_QUERY,
 # List of ICMPv6 types that should be permitted (egress) by default.
 ICMPV6_ALLOWED_EGRESS_TYPES = (n_const.ICMPV6_TYPE_MLD_QUERY,
                                n_const.ICMPV6_TYPE_RS,
-                               n_const.ICMPV6_TYPE_NS)
-
-# List of ICMPv6 types that should be permitted depending on payload content
-# to avoid spoofing (egress) by default.
-ICMPV6_RESTRICTED_EGRESS_TYPES = (n_const.ICMPV6_TYPE_NA, )
+                               n_const.ICMPV6_TYPE_NS,
+                               n_const.ICMPV6_TYPE_NA)
 
 
 def port_sec_enabled(port):
@@ -52,8 +47,7 @@ def load_firewall_driver_class(driver):
         'neutron.agent.firewall_drivers', driver)
 
 
-@six.add_metaclass(abc.ABCMeta)
-class FirewallDriver(object):
+class FirewallDriver(object, metaclass=abc.ABCMeta):
     """Firewall Driver base class.
 
     Defines methods that any driver providing security groups
