@@ -15,12 +15,13 @@
 
 import sys
 
+from neutron_lib.agent import l2_extension
 from neutron_lib import constants
 from neutron_lib.utils import helpers
 from oslo_config import cfg
 from oslo_log import log as logging
+from pyroute2.netlink import exceptions as netlink_exceptions
 
-from neutron.agent.l2 import l2_agent_extension
 from neutron.agent.linux import bridge_lib
 from neutron.conf.agent import l2_ext_fdb_population
 from neutron.plugins.ml2.drivers.linuxbridge.agent.common import (
@@ -34,7 +35,7 @@ LOG = logging.getLogger(__name__)
 
 
 class FdbPopulationAgentExtension(
-        l2_agent_extension.L2AgentExtension):
+        l2_extension.L2AgentExtension):
     """The FDB population is an agent extension to OVS or linux bridge
     who's objective is to update the FDB table for existing instance
     using normal port, thus enabling communication between SR-IOV instances
@@ -70,13 +71,14 @@ class FdbPopulationAgentExtension(
             # update macs already in the physical interface's FDB table
             for device in devices:
                 try:
-                    _stdout = bridge_lib.FdbInterface.show(device)
-                except RuntimeError as e:
+                    rules = bridge_lib.FdbInterface.show(dev=device)
+                except (OSError, netlink_exceptions.NetlinkError) as e:
                     LOG.warning(
                         'Unable to find FDB Interface %(device)s. '
                         'Exception: %(e)s', {'device': device, 'e': e})
                     continue
-                self.device_to_macs[device] = _stdout.split()[::3]
+                self.device_to_macs[device] = [rule['mac'] for rule in
+                                               rules[device]]
 
         def update_port(self, device, port_id, mac):
             # check if device is updated
@@ -90,14 +92,9 @@ class FdbPopulationAgentExtension(
             # check if rule for mac already exists
             if mac in self.device_to_macs[device]:
                 return
-            try:
-                bridge_lib.FdbInterface.add(mac, device)
-            except RuntimeError as e:
-                LOG.warning(
-                    'Unable to add mac %(mac)s '
-                    'to FDB Interface %(device)s. '
-                    'Exception: %(e)s',
-                    {'mac': mac, 'device': device, 'e': e})
+            if not bridge_lib.FdbInterface.add(mac, device):
+                LOG.warning('Unable to add mac %(mac)s to FDB Interface '
+                            '%(device)s.', {'mac': mac, 'device': device})
                 return
             self.device_to_macs[device].append(mac)
 
@@ -110,14 +107,10 @@ class FdbPopulationAgentExtension(
                 return
             for device in devices:
                 if mac in self.device_to_macs[device]:
-                    try:
-                        bridge_lib.FdbInterface.delete(mac, device)
-                    except RuntimeError as e:
-                        LOG.warning(
-                            'Unable to delete mac %(mac)s '
-                            'from FDB Interface %(device)s. '
-                            'Exception: %(e)s',
-                            {'mac': mac, 'device': device, 'e': e})
+                    if not bridge_lib.FdbInterface.delete(mac, device):
+                        LOG.warning('Unable to delete mac %(mac)s from FDB '
+                                    'Interface %(device)s.',
+                                    {'mac': mac, 'device': device})
                         return
                     self.device_to_macs[device].remove(mac)
                     del self.portid_to_mac[port_id]
@@ -160,8 +153,8 @@ class FdbPopulationAgentExtension(
         self.fdb_tracker.delete_port(devices, port_id)
 
     def _get_devices(self):
-        def _flatten_list(l):
-            return [item for sublist in l for item in sublist]
+        def _flatten_list(li):
+            return [item for sublist in li for item in sublist]
 
         return _flatten_list(self.device_mappings.values())
 
