@@ -18,7 +18,6 @@ import sys
 import threading
 
 import httplib2
-import netaddr
 from oslo_config import cfg
 from oslo_log import log as logging
 
@@ -90,15 +89,6 @@ class MonitorDaemon(daemon.Daemon):
                     new_state = 'backup'
                 self.write_state_change(new_state)
                 self.notify_agent(new_state)
-            elif event['name'] != self.interface and event['event'] == 'added':
-                # Send GARPs for all new router interfaces.
-                # REVISIT(jlibosva): keepalived versions 1.2.19 and below
-                # contain bug where gratuitous ARPs are not sent on receiving
-                # SIGHUP signal. This is a workaround to this bug. keepalived
-                # has this issue fixed since 1.2.20 but the version is not
-                # packaged in some distributions (RHEL/CentOS/Ubuntu Xenial).
-                # Remove this code once new keepalived versions are available.
-                self.send_garp(event)
 
     def handle_initial_state(self):
         try:
@@ -107,12 +97,12 @@ class MonitorDaemon(daemon.Daemon):
             for address in ip.addr.list():
                 if address.get('cidr') == self.cidr:
                     state = 'primary'
-                    self.write_state_change(state)
-                    self.notify_agent(state)
                     break
 
             LOG.debug('Initial status of router %s is %s',
                       self.router_id, state)
+            self.write_state_change(state)
+            self.notify_agent(state)
         except Exception:
             LOG.exception('Failed to get initial status of router %s',
                           self.router_id)
@@ -137,19 +127,6 @@ class MonitorDaemon(daemon.Daemon):
 
         LOG.debug('Notified agent router %s, state %s', self.router_id, state)
 
-    def send_garp(self, event):
-        """Send gratuitous ARP for given event."""
-        ip_address = str(netaddr.IPNetwork(event['cidr']).ip)
-        ip_lib.send_ip_addr_adv_notif(
-            self.namespace,
-            event['name'],
-            ip_address,
-            log_exception=False,
-            use_eventlet=False
-        )
-        LOG.debug('Sent GARP to %(ip_address)s from %(device_name)s',
-                  {'ip_address': ip_address, 'device_name': event['name']})
-
     def handle_sigterm(self, signum, frame):
         self.event_stop.set()
         self._thread_read_queue.join(timeout=5)
