@@ -589,10 +589,7 @@ class RouterInfo(BaseRouterInfo):
                       self.router_id)
             self.radvd.disable()
 
-    def internal_network_updated(self, port):
-        interface_name = self.get_internal_device_name(port['id'])
-        ip_cidrs = common_utils.fixed_ip_cidrs(port['fixed_ips'])
-        mtu = port['mtu']
+    def internal_network_updated(self, interface_name, ip_cidrs, mtu):
         self.driver.set_mtu(interface_name, mtu, namespace=self.ns_name,
                             prefix=INTERNAL_DEV_PREFIX)
         self.driver.init_router_port(
@@ -651,8 +648,12 @@ class RouterInfo(BaseRouterInfo):
         updated_cidrs = []
         for p in updated_ports:
             self._update_internal_ports_cache(p)
-            updated_cidrs += common_utils.fixed_ip_cidrs(p['fixed_ips'])
-            self.internal_network_updated(p)
+            interface_name = self.get_internal_device_name(p['id'])
+            ip_cidrs = common_utils.fixed_ip_cidrs(p['fixed_ips'])
+            LOG.debug("updating internal network for port %s", p)
+            updated_cidrs += ip_cidrs
+            self.internal_network_updated(
+                interface_name, ip_cidrs, p['mtu'])
             enable_ra = enable_ra or self._port_has_ipv6_subnet(p)
 
         # Check if there is any pd prefix update
@@ -1104,11 +1105,15 @@ class RouterInfo(BaseRouterInfo):
             self.iptables_manager.ipv6['mangle'].add_rule(
                 'PREROUTING', mark_metadata_v6_for_internal_interfaces)
 
-    def _get_port_devicename_scopemark(self, ports, name_generator):
+    def _get_port_devicename_scopemark(
+            self, ports, name_generator, interface_name=None):
         devicename_scopemark = {lib_constants.IP_VERSION_4: dict(),
                                 lib_constants.IP_VERSION_6: dict()}
         for p in ports:
-            device_name = name_generator(p['id'])
+            if interface_name is None:
+                device_name = name_generator(p['id'])
+            else:
+                device_name = interface_name
             ip_cidrs = common_utils.fixed_ip_cidrs(p['fixed_ips'])
             port_as_marks = self.get_port_address_scope_mark(p)
             for ip_version in {common_utils.get_ip_version(cidr)
