@@ -35,7 +35,6 @@ from oslo_utils import fileutils
 from oslo_utils import netutils
 from oslo_utils import uuidutils
 
-from neutron._i18n import _
 from neutron.agent.common import utils as agent_common_utils
 from neutron.agent.linux import external_process
 from neutron.agent.linux import ip_lib
@@ -195,7 +194,8 @@ class DhcpBase(object, metaclass=abc.ABCMeta):
         self.disable(retain_port=True, block=True)
         self.enable()
 
-    @abc.abstractproperty
+    @property
+    @abc.abstractmethod
     def active(self):
         """Boolean representing the running state of the DHCP server."""
 
@@ -302,7 +302,7 @@ class DhcpLocalProcess(DhcpBase, metaclass=abc.ABCMeta):
             common_utils.wait_until_true(lambda: not self.active)
         if not retain_port:
             self._destroy_namespace_and_port()
-            self._remove_config_files()
+        self._remove_config_files()
 
     def _destroy_namespace_and_port(self):
         try:
@@ -320,17 +320,13 @@ class DhcpLocalProcess(DhcpBase, metaclass=abc.ABCMeta):
     def _get_value_from_conf_file(self, kind, converter=None):
         """A helper function to read a value from one of the state files."""
         file_name = self.get_conf_file_name(kind)
-        msg = _('Error while reading %s')
-
         try:
             with open(file_name, 'r') as f:
-                try:
-                    return converter(f.read()) if converter else f.read()
-                except ValueError:
-                    msg = _('Unable to convert value in %s')
+                return converter(f.read()) if converter else f.read()
+        except ValueError:
+            msg = "Unable to convert value in %s"
         except IOError:
-            msg = _('Unable to access %s')
-
+            msg = "Unable to access %s"
         LOG.debug(msg, file_name)
         return None
 
@@ -802,50 +798,17 @@ class Dnsmasq(DhcpLocalProcess):
         dhcpv4_enabled_subnet_ids = [
             s.id for s in self._get_all_subnets(self.network)
             if s.enable_dhcp and s.ip_version == constants.IP_VERSION_4]
-        dhcpv6_enabled_subnet_ids = [
-            s.id for s in self._get_all_subnets(self.network)
-            if s.enable_dhcp and s.ip_version == constants.IP_VERSION_6]
-
-        existing_ipv6_leases = {}
-        if os.path.isfile(filename):
-            # The IPv6 leases can't be generated as their IAID is unknown. To
-            # not loose active leases, read the existing leases and add them to
-            # the generated file.
-            LOG.debug('Reading IPv6 leases from existing lease file.')
-            with open(filename) as leasefile:
-                for line in leasefile:
-                    if line.startswith('duid '):
-                        # Keep the DUID
-                        buf.write(line)
-                        continue
-                    try:
-                        ts, mac, ip, host, iaid = line.split(' ')
-                    except ValueError:
-                        # not the correct format for a lease, skip this line
-                        continue
-
-                    if netaddr.valid_ipv6(ip):
-                        existing_ipv6_leases[netaddr.IPAddress(ip)] = line
-
         for host_tuple in self._iter_hosts():
             port, alloc, hostname, name, no_dhcp, no_opts, tag = host_tuple
-
-            if no_dhcp:
+            # don't write ip address which belongs to a dhcp disabled subnet
+            # or an IPv6 subnet.
+            if no_dhcp or alloc.subnet_id not in dhcpv4_enabled_subnet_ids:
                 continue
 
-            if alloc.subnet_id in dhcpv4_enabled_subnet_ids:
-                # all that matters is the mac address and IP. the hostname and
-                # client ID will be overwritten on the next renewal.
-                buf.write('%s %s %s * *\n' %
-                          (timestamp, port.mac_address, alloc.ip_address))
-            elif (alloc.subnet_id in dhcpv6_enabled_subnet_ids and
-                  netaddr.IPAddress(alloc.ip_address) in existing_ipv6_leases):
-                # Keep the existing IPv6 lease if the port still exists and is
-                # still configured for DHCPv6
-                buf.write(
-                    existing_ipv6_leases[netaddr.IPAddress(alloc.ip_address)]
-                )
-
+            # all that matters is the mac address and IP. the hostname and
+            # client ID will be overwritten on the next renewal.
+            buf.write('%s %s %s * *\n' %
+                      (timestamp, port.mac_address, alloc.ip_address))
         contents = buf.getvalue()
         file_utils.replace_file(filename, contents)
         LOG.debug('Done building initial lease file %s with contents:\n%s',
@@ -1149,8 +1112,10 @@ class Dnsmasq(DhcpLocalProcess):
                  addr_mode == constants.IPV6_SLAAC)):
                 continue
             if subnet.dns_nameservers:
-                if common_utils.is_dns_servers_any_address(
-                        subnet.dns_nameservers, subnet.ip_version):
+                if ((subnet.ip_version == 4 and
+                     subnet.dns_nameservers == ['0.0.0.0']) or
+                    (subnet.ip_version == 6 and
+                     subnet.dns_nameservers == ['::'])):
                     # Special case: Do not announce DNS servers
                     options.append(
                         self._format_option(
@@ -1309,7 +1274,6 @@ class Dnsmasq(DhcpLocalProcess):
     def _format_option(self, ip_version, tag, option, *args):
         """Format DHCP option by option name or code."""
         option = str(option)
-        option = option.split("\n", 1)[0]
         pattern = "(tag:(.*),)?(.*)$"
         matches = re.match(pattern, option)
         extra_tag = matches.groups()[0]
@@ -1325,11 +1289,10 @@ class Dnsmasq(DhcpLocalProcess):
         elif not option.isdigit():
             option = 'option:%s' % option
         if extra_tag:
-            tags = ['tag:' + tag, extra_tag[:-1], '%s' % option]
+            tags = ('tag:' + tag, extra_tag[:-1], '%s' % option)
         else:
-            tags = ['tag:' + tag, '%s' % option]
-
-        return ','.join(tags + [v.split("\n", 1)[0] for v in args])
+            tags = ('tag:' + tag, '%s' % option)
+        return ','.join(tags + args)
 
     @staticmethod
     def _convert_to_literal_addrs(ip_version, ips):
