@@ -37,7 +37,8 @@ agent_cfg.register_process_monitor_opts(cfg.CONF)
 
 
 class MonitoredProcess(object, metaclass=abc.ABCMeta):
-    @abc.abstractproperty
+    @property
+    @abc.abstractmethod
     def active(self):
         """Boolean representing the running state of the process."""
 
@@ -105,15 +106,18 @@ class ProcessManager(MonitoredProcess):
                 cmd = get_stop_command(self.get_pid_file_name())
                 ip_wrapper = ip_lib.IPWrapper(namespace=self.namespace)
                 ip_wrapper.netns.execute(cmd, addl_env=self.cmd_addl_env,
-                                         run_as_root=self.run_as_root)
+                                         run_as_root=self.run_as_root,
+                                         privsep_exec=True)
             else:
                 cmd = self.get_kill_cmd(sig, pid)
-                utils.execute(cmd, run_as_root=self.run_as_root)
+                utils.execute(cmd, run_as_root=self.run_as_root,
+                              privsep_exec=True)
                 # In the case of shutting down, remove the pid file
                 if sig == '9':
-                    fileutils.delete_if_exists(self.get_pid_file_name())
+                    utils.delete_if_exists(self.get_pid_file_name(),
+                                           run_as_root=self.run_as_root)
         elif pid:
-            LOG.debug('%{service}s process for %(uuid)s pid %(pid)d is stale, '
+            LOG.debug('%(service)s process for %(uuid)s pid %(pid)d is stale, '
                       'ignoring signal %(signal)s',
                       {'service': self.service, 'uuid': self.uuid,
                        'pid': pid, 'signal': sig})
