@@ -14,7 +14,6 @@
 #    under the License.
 
 import errno
-from os import path
 import re
 import threading
 import time
@@ -36,7 +35,6 @@ from neutron._i18n import _
 from neutron.agent.common import utils
 from neutron.common import utils as common_utils
 from neutron.privileged.agent.linux import ip_lib as privileged
-from neutron.privileged.agent.linux import utils as priv_utils
 
 LOG = logging.getLogger(__name__)
 
@@ -137,7 +135,7 @@ class SubProcessBase(object):
         opt_list = ['-%s' % o for o in options]
         ip_cmd = add_namespace_to_cmd(['ip'], namespace)
         cmd = ip_cmd + opt_list + [command] + list(args)
-        return utils.execute(cmd, run_as_root=run_as_root,
+        return utils.execute(cmd, run_as_root=run_as_root, privsep_exec=True,
                              log_fail_as_error=self.log_fail_as_error)
 
     def set_log_fail_as_error(self, fail_with_error):
@@ -276,20 +274,11 @@ class IPWrapper(SubProcessBase):
                                     vlan_id=vlan_id)
         return IPDevice(name, namespace=self.namespace)
 
-    def add_vxlan(self, name, vni, dev, group=None, ttl=None, tos=None,
+    def add_vxlan(self, name, vni, group=None, dev=None, ttl=None, tos=None,
                   local=None, srcport=None, dstport=None, proxy=False):
-        kwargs = {'vxlan_id': vni,
-                  'physical_interface': dev}
+        kwargs = {'vxlan_id': vni}
         if group:
-            try:
-                ip_version = common_utils.get_ip_version(group)
-                if ip_version == constants.IP_VERSION_6:
-                    kwargs['vxlan_group6'] = group
-                else:
-                    kwargs['vxlan_group'] = group
-            except netaddr.core.AddrFormatError:
-                err_msg = _("Invalid group address: %s") % group
-                raise exceptions.InvalidInput(error_message=err_msg)
+            kwargs['vxlan_group'] = group
         if dev:
             kwargs['physical_interface'] = dev
         if ttl:
@@ -297,15 +286,7 @@ class IPWrapper(SubProcessBase):
         if tos:
             kwargs['vxlan_tos'] = tos
         if local:
-            try:
-                ip_version = common_utils.get_ip_version(local)
-                if ip_version == constants.IP_VERSION_6:
-                    kwargs['vxlan_local6'] = local
-                else:
-                    kwargs['vxlan_local'] = local
-            except netaddr.core.AddrFormatError:
-                err_msg = _("Invalid local address: %s") % local
-                raise exceptions.InvalidInput(error_message=err_msg)
+            kwargs['vxlan_local'] = local
         if proxy:
             kwargs['vxlan_proxy'] = proxy
         # tuple: min,max
@@ -373,7 +354,7 @@ class IPDevice(SubProcessBase):
         try:
             ip_wrapper.netns.execute(["conntrack", "-D", "-d", ip_str],
                                      check_exit_code=True,
-                                     extra_ok_codes=[1])
+                                     extra_ok_codes=[1], privsep_exec=True)
 
         except RuntimeError:
             LOG.exception("Failed deleting ingress connection state of"
@@ -383,7 +364,7 @@ class IPDevice(SubProcessBase):
         try:
             ip_wrapper.netns.execute(["conntrack", "-D", "-q", ip_str],
                                      check_exit_code=True,
-                                     extra_ok_codes=[1])
+                                     extra_ok_codes=[1], privsep_exec=True)
         except RuntimeError:
             LOG.exception("Failed deleting egress connection state of"
                           " floatingip %s", ip_str)
@@ -395,7 +376,7 @@ class IPDevice(SubProcessBase):
                '--dport', dport]
         try:
             ip_wrapper.netns.execute(cmd, check_exit_code=True,
-                                     extra_ok_codes=[1])
+                                     extra_ok_codes=[1], privsep_exec=True)
 
         except RuntimeError:
             LOG.exception("Failed deleting ingress connection state of "
@@ -420,23 +401,11 @@ class IPDevice(SubProcessBase):
         self._name = name
 
 
-class IpCommandBase(object):
-    COMMAND = ''
+class IpDeviceCommandBase(object):
 
     def __init__(self, parent):
         self._parent = parent
 
-    def _run(self, options, args):
-        return self._parent._run(options, self.COMMAND, args)
-
-    def _as_root(self, options, args, use_root_namespace=False):
-        return self._parent._as_root(options,
-                                     self.COMMAND,
-                                     args,
-                                     use_root_namespace=use_root_namespace)
-
-
-class IpDeviceCommandBase(IpCommandBase):
     @property
     def name(self):
         return self._parent.name
@@ -447,7 +416,6 @@ class IpDeviceCommandBase(IpCommandBase):
 
 
 class IpLinkCommand(IpDeviceCommandBase):
-    COMMAND = 'link'
 
     def set_address(self, mac_address):
         privileged.set_link_attribute(
@@ -473,8 +441,6 @@ class IpLinkCommand(IpDeviceCommandBase):
         privileged.set_link_attribute(
             self.name, self._parent.namespace, net_ns_fd=namespace)
         self._parent.namespace = namespace
-        common_utils.wait_until_true(lambda: self.exists, timeout=5,
-                                     sleep=0.5)
 
     def set_name(self, name):
         privileged.set_link_attribute(
@@ -542,7 +508,6 @@ class IpLinkCommand(IpDeviceCommandBase):
 
 
 class IpAddrCommand(IpDeviceCommandBase):
-    COMMAND = 'addr'
 
     def add(self, cidr, scope='global', add_broadcast=True):
         add_ip_address(cidr, self.name, self._parent.namespace, scope,
@@ -615,7 +580,6 @@ class IpAddrCommand(IpDeviceCommandBase):
 
 
 class IpRouteCommand(IpDeviceCommandBase):
-    COMMAND = 'route'
 
     def __init__(self, parent, table=None):
         super(IpRouteCommand, self).__init__(parent)
@@ -678,13 +642,13 @@ class IPRoute(SubProcessBase):
 
 
 class IpNeighCommand(IpDeviceCommandBase):
-    COMMAND = 'neigh'
 
-    def add(self, ip_address, mac_address, **kwargs):
+    def add(self, ip_address, mac_address, nud_state=None, **kwargs):
         add_neigh_entry(ip_address,
                         mac_address,
                         self.name,
-                        self._parent.namespace,
+                        namespace=self._parent.namespace,
+                        nud_state=nud_state,
                         **kwargs)
 
     def delete(self, ip_address, mac_address, **kwargs):
@@ -706,21 +670,33 @@ class IpNeighCommand(IpDeviceCommandBase):
         Given address entry is removed from neighbour cache (ARP or NDP). To
         flush all entries pass string 'all' as an address.
 
+        From https://man.archlinux.org/man/core/iproute2/ip-neighbour.8.en:
+          "the default neighbour states to be flushed do not include permanent
+           and noarp".
+
         :param ip_version: Either 4 or 6 for IPv4 or IPv6 respectively
-        :param ip_address: The prefix selecting the neighbours to flush
+        :param ip_address: The prefix selecting the neighbours to flush or
+                           "all"
         """
-        # NOTE(haleyb): There is no equivalent to 'flush' in pyroute2
-        self._as_root([ip_version], ('flush', 'to', ip_address))
+        cidr = netaddr.IPNetwork(ip_address) if ip_address != 'all' else None
+        for entry in self.dump(ip_version):
+            if entry['state'] in ('permanent', 'noarp'):
+                continue
+            if ip_address == 'all' or entry['dst'] in cidr:
+                self.delete(entry['dst'], entry['lladdr'])
 
 
-class IpNetnsCommand(IpCommandBase):
-    COMMAND = 'netns'
+class IpNetnsCommand(object):
+
+    def __init__(self, parent):
+        self._parent = parent
 
     def add(self, name):
         create_network_namespace(name)
         wrapper = IPWrapper(namespace=name)
         wrapper.netns.execute(['sysctl', '-w',
-                               'net.ipv4.conf.all.promote_secondaries=1'])
+                               'net.ipv4.conf.all.promote_secondaries=1'],
+                              privsep_exec=True)
         return wrapper
 
     def delete(self, name):
@@ -728,7 +704,7 @@ class IpNetnsCommand(IpCommandBase):
 
     def execute(self, cmds, addl_env=None, check_exit_code=True,
                 log_fail_as_error=True, extra_ok_codes=None,
-                run_as_root=False):
+                run_as_root=False, privsep_exec=False):
         ns_params = []
         if self._parent.namespace:
             run_as_root = True
@@ -742,7 +718,8 @@ class IpNetnsCommand(IpCommandBase):
         return utils.execute(cmd, check_exit_code=check_exit_code,
                              extra_ok_codes=extra_ok_codes,
                              log_fail_as_error=log_fail_as_error,
-                             run_as_root=run_as_root)
+                             run_as_root=run_as_root,
+                             privsep_exec=privsep_exec)
 
     def exists(self, name):
         return network_namespace_exists(name)
@@ -862,20 +839,25 @@ def get_routing_table(ip_version, namespace=None):
 
 # NOTE(haleyb): These neighbour functions live outside the IpNeighCommand
 # class since not all callers require it.
-def add_neigh_entry(ip_address, mac_address, device, namespace=None, **kwargs):
+def add_neigh_entry(ip_address, mac_address, device, namespace=None,
+                    nud_state=None, **kwargs):
     """Add a neighbour entry.
 
     :param ip_address: IP address of entry to add
     :param mac_address: MAC address of entry to add
     :param device: Device name to use in adding entry
     :param namespace: The name of the namespace in which to add the entry
+    :param nud_state: The NUD (Neighbour Unreachability Detection) state of
+                      the entry; defaults to "permanent"
     """
     ip_version = common_utils.get_ip_version(ip_address)
+    nud_state = nud_state or 'permanent'
     privileged.add_neigh_entry(ip_version,
                                ip_address,
                                mac_address,
                                device,
                                namespace,
+                               nud_state,
                                **kwargs)
 
 
@@ -952,16 +934,9 @@ def network_namespace_exists(namespace, try_is_ready=False, **kwargs):
                          is ready to be operated.
     :param kwargs: Callers add any filters they use as kwargs
     """
-    if not namespace:
-        return False
-
     if not try_is_ready:
-        nspath = kwargs.get('nspath') or netns.NETNS_RUN_DIR
-        nspath += '/' + namespace
-        if cfg.CONF.AGENT.use_helper_for_ns_read:
-            return priv_utils.path_exists(nspath)
-        else:
-            return path.exists(nspath)
+        output = list_network_namespaces(**kwargs)
+        return namespace in output
 
     try:
         privileged.open_namespace(namespace)
@@ -1042,7 +1017,8 @@ def _arping(ns_name, iface_name, address, count, log_exception):
                           '-w', 2, address]
             try:
                 ip_wrapper.netns.execute(arping_cmd,
-                                         extra_ok_codes=extra_ok_codes)
+                                         extra_ok_codes=extra_ok_codes,
+                                         privsep_exec=True)
             except Exception as exc:
                 # Since this is spawned in a thread and executed 2 seconds
                 # apart, something may have been deleted while we were
@@ -1052,8 +1028,8 @@ def _arping(ns_name, iface_name, address, count, log_exception):
                                                         [address],
                                                         mac=None,
                                                         namespace=ns_name)
-                msg = _("Failed sending gratuitous ARP to %(addr)s on "
-                        "%(iface)s in namespace %(ns)s: %(err)s")
+                msg = ("Failed sending gratuitous ARP to %(addr)s on "
+                       "%(iface)s in namespace %(ns)s: %(err)s")
                 logger_method = LOG.exception
                 if not (log_exception and (first or exists)):
                     logger_method = LOG.info
@@ -1129,7 +1105,8 @@ def sysctl(cmd, namespace=None, log_fail_as_error=True):
     ip_wrapper = IPWrapper(namespace=namespace)
     try:
         ip_wrapper.netns.execute(cmd, run_as_root=True,
-                                 log_fail_as_error=log_fail_as_error)
+                                 log_fail_as_error=log_fail_as_error,
+                                 privsep_exec=True)
     except RuntimeError as rte:
         LOG.warning(
             "Setting %(cmd)s in namespace %(ns)s failed: %(err)s.",
@@ -1155,7 +1132,8 @@ def get_ip_nonlocal_bind(namespace=None):
     """Get kernel option value of ip_nonlocal_bind in given namespace."""
     cmd = ['sysctl', '-bn', IP_NONLOCAL_BIND]
     ip_wrapper = IPWrapper(namespace)
-    return int(ip_wrapper.netns.execute(cmd, run_as_root=True))
+    return int(ip_wrapper.netns.execute(cmd, run_as_root=True,
+                                        privsep_exec=True))
 
 
 def set_ip_nonlocal_bind(value, namespace=None, log_fail_as_error=True):
@@ -1190,7 +1168,8 @@ def get_ipv6_forwarding(device, namespace=None):
     """Get kernel value of IPv6 forwarding for device in given namespace."""
     cmd = ['sysctl', '-b', "net.ipv6.conf.%s.forwarding" % device]
     ip_wrapper = IPWrapper(namespace)
-    return int(ip_wrapper.netns.execute(cmd, run_as_root=True))
+    return int(ip_wrapper.netns.execute(cmd, run_as_root=True,
+                                        privsep_exec=True))
 
 
 def _parse_ip_rule(rule, ip_version):
@@ -1521,14 +1500,14 @@ def ip_monitor(namespace, queue, event_stop, event_started):
 
 
 def add_ip_route(namespace, cidr, device=None, via=None, table=None,
-                 metric=None, scope=None, **kwargs):
+                 metric=None, scope=None, proto='static', **kwargs):
     """Add an IP route"""
     if table:
         table = IP_RULE_TABLES.get(table, table)
     ip_version = common_utils.get_ip_version(cidr or via)
     privileged.add_ip_route(namespace, cidr, ip_version,
                             device=device, via=via, table=table,
-                            metric=metric, scope=scope, **kwargs)
+                            metric=metric, scope=scope, proto=proto, **kwargs)
 
 
 def list_ip_routes(namespace, ip_version, scope=None, via=None, table=None,
@@ -1538,6 +1517,12 @@ def list_ip_routes(namespace, ip_version, scope=None, via=None, table=None,
         for device in (d for d in devices if d['index'] == index):
             return get_attr(device, 'IFLA_IFNAME')
 
+    def get_proto(proto_number):
+        if proto_number in rtnl.rt_proto:
+            return rtnl.rt_proto[proto_number]
+        elif str(proto_number) in constants.IP_PROTOCOL_NUM_TO_NAME_MAP:
+            return constants.IP_PROTOCOL_NUM_TO_NAME_MAP[str(proto_number)]
+
     table = table if table else 'main'
     table = IP_RULE_TABLES.get(table, table)
     routes = privileged.list_ip_routes(namespace, ip_version, device=device,
@@ -1553,6 +1538,7 @@ def list_ip_routes(namespace, ip_version, scope=None, via=None, table=None,
         table = int(get_attr(route, 'RTA_TABLE'))
         metric = (get_attr(route, 'RTA_PRIORITY') or
                   IP_ROUTE_METRIC_DEFAULT[ip_version])
+        proto = get_proto(route['proto'])
         value = {
             'table': IP_RULE_TABLES_NAMES.get(table, table),
             'source_prefix': get_attr(route, 'RTA_PREFSRC'),
@@ -1561,6 +1547,7 @@ def list_ip_routes(namespace, ip_version, scope=None, via=None, table=None,
             'device': get_device(int(get_attr(route, 'RTA_OIF')), devices),
             'via': get_attr(route, 'RTA_GATEWAY'),
             'metric': metric,
+            'proto': proto,
         }
 
         ret.append(value)
