@@ -16,8 +16,6 @@
 import collections
 import contextlib
 import copy
-import itertools
-import re
 
 import eventlet
 import netaddr
@@ -43,7 +41,6 @@ from neutron.plugins.ml2.drivers.openvswitch.agent.common import constants \
         as ovs_consts
 
 LOG = logging.getLogger(__name__)
-CONJ_ID_REGEX = re.compile(r"conj_id=(\d+),")
 
 
 def _replace_register(flow_params, register_number, register_value):
@@ -152,7 +149,7 @@ class SecurityGroup(object):
                 else:
                     rule['protocol'] = lib_const.IP_PROTOCOL_MAP.get(
                         protocol, protocol)
-            if 'remote_group_id' in rule:
+            if 'remote_group_id' in rule or 'remote_address_group_id' in rule:
                 self.remote_rules.append(rule)
             else:
                 self.raw_rules.append(rule)
@@ -265,75 +262,33 @@ class SGPortMap(object):
 class ConjIdMap(object):
     """Handle conjunction ID allocations and deallocations."""
 
-    CONJ_ID_BLOCK_SIZE = 8
-    MAX_CONJ_ID = 2 ** 32 - 8
-
-    def __new__(cls, int_br):
+    def __new__(cls):
         if not hasattr(cls, '_instance'):
             cls._instance = super(ConjIdMap, cls).__new__(cls)
         return cls._instance
 
-    def __init__(self, int_br):
+    def __init__(self):
         self.id_map = collections.defaultdict(self._conj_id_factory)
         # Stores the set of conjuntion IDs used for each unique tuple
-        # (sg_id, remote_sg_id, direction, ethertype). Each tuple can have up
-        # to 8 conjuntion IDs (see ConjIPFlowManager.add()).
+        # (sg_id, remote_id, direction, ethertype). Each tuple
+        # can have up to 8 conjuntion IDs (see ConjIPFlowManager.add()).
         self.id_map_group = collections.defaultdict(set)
         self.id_free = collections.deque()
-        self._max_id = self._init_max_id(int_br)
-
-    def _init_max_id(self, int_br):
-        """Read the maximum conjunction ID number in the integration bridge
-
-        This method will dump all integration bridge flows, parse them and
-        return the maximum conjunction ID number. By default, "int_br" is a
-        ``OVSAgentBridge`` instance, using "os-ken" library to access to the OF
-        rules.
-        If not, "int_br" will default to a ``OVSBridge`` instance. The CLI
-        command "ovs-ofctl" will be used instead.
-
-        :param int_br: ``OVSAgentBridge`` or ``OVSBridge`` instance.
-        :returns: The maximum conjunction ID number in the integration bridge
-        """
-        conj_id_max = 0
-        try:
-            for flow in itertools.chain(
-                    *[int_br.dump_flows(table)
-                      for table in ovs_consts.OVS_FIREWALL_TABLES]):
-                conj_id_max = max(conj_id_max, flow.match.get('conj_id', 0))
-        except AttributeError:  # br_int is a ``OVSBridge`` instance.
-            flows_iter = itertools.chain(
-                *[int_br.dump_flows_for_table(table)
-                  for table in ovs_consts.OVS_FIREWALL_TABLES])
-            conj_ids = CONJ_ID_REGEX.findall(" | ".join(flows_iter))
-            try:
-                conj_id_max = max([int(conj_id) for conj_id in conj_ids])
-            except ValueError:
-                conj_id_max = 0
-
-        max_id = conj_id_max - conj_id_max % self.CONJ_ID_BLOCK_SIZE
-        return self._next_max_id(max_id)
-
-    def _next_max_id(self, max_id):
-        max_id += self.CONJ_ID_BLOCK_SIZE
-        if max_id >= self.MAX_CONJ_ID:
-            max_id = 0
-        return max_id
+        self.max_id = 0
 
     def _conj_id_factory(self):
         # If there is any freed ID, use one.
         if self.id_free:
             return self.id_free.popleft()
         # Allocate new one. It must be divisible by 8. (See the next function.)
-        self._max_id = self._next_max_id(self._max_id)
-        return self._max_id
+        self.max_id += 8
+        return self.max_id
 
-    def get_conj_id(self, sg_id, remote_sg_id, direction, ethertype):
+    def get_conj_id(self, sg_id, remote_id, direction, ethertype):
         """Return a conjunction ID specified by the arguments.
-        Allocate one if necessary.  The returned ID is divisible by 8
-        (CONJ_ID_BLOCK_SIZE), as there are 4 priority levels
-        (see rules.flow_priority_offset) and 2 conjunction IDs are needed per
-        priority.
+        Allocate one if necessary.  The returned ID is divisible by 8,
+        as there are 4 priority levels (see rules.flow_priority_offset)
+        and 2 conjunction IDs are needed per priority.
         """
         if direction not in [lib_const.EGRESS_DIRECTION,
                              lib_const.INGRESS_DIRECTION]:
@@ -341,7 +296,7 @@ class ConjIdMap(object):
         if ethertype not in [lib_const.IPv4, lib_const.IPv6]:
             raise ValueError(_("Invalid ethertype '%s'") % ethertype)
 
-        return self.id_map[(sg_id, remote_sg_id, direction, ethertype)]
+        return self.id_map[(sg_id, remote_id, direction, ethertype)]
 
     def delete_sg(self, sg_id):
         """Free all conj_ids associated with the sg_id and
@@ -382,7 +337,7 @@ class ConjIPFlowManager(object):
     """
 
     def __init__(self, driver):
-        self.conj_id_map = ConjIdMap(driver.int_br.br)
+        self.conj_id_map = ConjIdMap()
         self.driver = driver
         # The following two are dict of dicts and are indexed like:
         #     self.x[vlan_tag][(direction, ethertype)]
@@ -390,18 +345,18 @@ class ConjIPFlowManager(object):
         self.flow_state = collections.defaultdict(
             lambda: collections.defaultdict(dict))
 
-    def _build_addr_conj_id_map(self, ethertype, sg_conj_id_map):
+    def _build_addr_conj_id_map(self, ethertype, sg_ag_conj_id_map):
         """Build a map of addr -> list of conj_ids."""
         addr_to_conj = collections.defaultdict(list)
-        for remote_id, conj_id_set in sg_conj_id_map.items():
+        for remote_id, conj_id_set in sg_ag_conj_id_map.items():
             remote_group = self.driver.sg_port_map.get_sg(remote_id)
             if not remote_group or not remote_group.members:
-                LOG.debug('No member for SG %s', remote_id)
+                LOG.debug('No member for security group or'
+                          'address group %s', remote_id)
                 continue
             for addr in remote_group.get_ethertype_filtered_addresses(
                     ethertype):
                 addr_to_conj[addr].extend(conj_id_set)
-
         return addr_to_conj
 
     def _update_flows_for_vlan_subr(self, direction, ethertype, vlan_tag,
@@ -414,16 +369,39 @@ class ConjIPFlowManager(object):
         self.driver.delete_flows_for_flow_state(
             flow_state, addr_to_conj, direction, ethertype, vlan_tag)
         for conj_id_set in conj_id_to_remove:
-            # Remove any remaining flow with remote SG ID conj_id_to_remove
+            # Remove any remaining flow with remote SG/AG ID conj_id_to_remove
             for current_ip, conj_ids in flow_state.items():
                 conj_ids_to_remove = conj_id_set & set(conj_ids)
                 self.driver.delete_flow_for_ip(
                     current_ip, direction, ethertype, vlan_tag,
                     conj_ids_to_remove)
 
+        # NOTE(hangyang): Handle add/delete overlapped IPs among
+        # remote security groups and remote address groups
+        removed_ips = set([str(netaddr.IPNetwork(addr[0]).cidr) for addr in (
+                set(flow_state.keys()) - set(addr_to_conj.keys()))])
+        ip_to_conj = collections.defaultdict(set)
         for addr, conj_ids in addr_to_conj.items():
+            # Addresses from remote security groups have mac addresses,
+            # others from remote address groups have not.
+            ip_to_conj[str(netaddr.IPNetwork(addr[0]).cidr)].update(conj_ids)
+
+        for addr in addr_to_conj.keys():
+            ip_cidr = str(netaddr.IPNetwork(addr[0]).cidr)
+            # When the overlapped IP in remote security group and remote
+            # address group have different conjunction ids but with the
+            # same priority offset, we need to combine the conj_ids together
+            # before create flows otherwise flows will be overridden in the
+            # creation sequence.
+            conj_ids = list(ip_to_conj[ip_cidr])
             conj_ids.sort()
-            if flow_state.get(addr) == conj_ids:
+            if flow_state.get(addr) == conj_ids and ip_cidr not in removed_ips:
+                # When there are IP overlaps among remote security groups
+                # and remote address groups, removal of the overlapped ips
+                # from one remote group will also delete the flows for the
+                # other groups because the non-strict delete method cannot
+                # match flow priority or actions for different conjunction
+                # ids, therefore we need to recreate the affected flows.
                 continue
             for flow in rules.create_flows_for_ip_address(
                     addr, direction, ethertype, vlan_tag, conj_ids):
@@ -431,40 +409,41 @@ class ConjIPFlowManager(object):
 
     def update_flows_for_vlan(self, vlan_tag, conj_id_to_remove=None):
         """Install action=conjunction(conj_id, 1/2) flows,
-        which depend on IP addresses of remote_group_id.
+        which depend on IP addresses of remote_group_id or
+        remote_address_group_id.
         """
-        for (direction, ethertype), sg_conj_id_map in (
+        for (direction, ethertype), sg_ag_conj_id_map in (
                 self.conj_ids[vlan_tag].items()):
             # TODO(toshii): optimize when remote_groups have
             # no address overlaps.
             addr_to_conj = self._build_addr_conj_id_map(
-                ethertype, sg_conj_id_map)
+                ethertype, sg_ag_conj_id_map)
             self._update_flows_for_vlan_subr(
                 direction, ethertype, vlan_tag,
                 self.flow_state[vlan_tag][(direction, ethertype)],
                 addr_to_conj, conj_id_to_remove)
             self.flow_state[vlan_tag][(direction, ethertype)] = addr_to_conj
 
-    def add(self, vlan_tag, sg_id, remote_sg_id, direction, ethertype,
+    def add(self, vlan_tag, sg_id, remote_id, direction, ethertype,
             priority_offset):
         """Get conj_id specified by the arguments
         and notify the manager that
-        (remote_sg_id, direction, ethertype, conj_id) flows need to be
-        populated on the vlan_tag network.
+        (remote_id, direction, ethertype, conj_id) flows need
+        to be populated on the vlan_tag network.
 
         A caller must call update_flows_for_vlan to have the change in effect.
 
         """
         conj_id = self.conj_id_map.get_conj_id(
-            sg_id, remote_sg_id, direction, ethertype) + priority_offset * 2
+            sg_id, remote_id, direction, ethertype) + priority_offset * 2
 
         if (direction, ethertype) not in self.conj_ids[vlan_tag]:
             self.conj_ids[vlan_tag][(direction, ethertype)] = (
                 collections.defaultdict(set))
-        self.conj_ids[vlan_tag][(direction, ethertype)][remote_sg_id].add(
+        self.conj_ids[vlan_tag][(direction, ethertype)][remote_id].add(
             conj_id)
 
-        conj_id_tuple = (sg_id, remote_sg_id, direction, ethertype)
+        conj_id_tuple = (sg_id, remote_id, direction, ethertype)
         self.conj_id_map.id_map_group[conj_id_tuple].add(conj_id)
         return conj_id
 
@@ -591,6 +570,14 @@ class OVSFirewallDriver(firewall.FirewallDriver):
         else:
             self.int_br.br.delete_flows(**kwargs)
 
+    def _strict_delete_flow(self, **kwargs):
+        """Delete given flow right away even if bridge is deferred.
+
+        Delete command will use strict delete.
+        """
+        create_reg_numbers(kwargs)
+        self.int_br.br.delete_flows(strict=True, **kwargs)
+
     @staticmethod
     def initialize_bridge(int_br):
         int_br.add_protocols(*OVSFirewallDriver.REQUIRED_PROTOCOLS)
@@ -772,6 +759,13 @@ class OVSFirewallDriver(firewall.FirewallDriver):
     def _update_flows_for_port(self, of_port, old_of_port):
         with self.update_cookie_context():
             self._set_port_filters(of_port)
+        # Flush the flows caused by changes made to deferred bridge. The reason
+        # is that following delete_all_port_flows() call uses --strict
+        # parameter that cannot be combined with other non-strict rules, hence
+        # all parameters with --strict are applied right away. In order to
+        # avoid applying delete rules with --strict *before*
+        # _set_port_filters() we dump currently cached flows here.
+        self.int_br.apply_flows()
         self.delete_all_port_flows(old_of_port)
         # Rewrite update cookie with default cookie
         self._set_port_filters(of_port)
@@ -884,13 +878,15 @@ class OVSFirewallDriver(firewall.FirewallDriver):
 
     def delete_physical_direct_flow(self, mac, segment_id):
         if segment_id:
-            self._delete_flows(table=ovs_consts.TRANSIENT_TABLE,
-                               dl_dst=mac,
-                               dl_vlan=segment_id)
+            self._strict_delete_flow(priority=90,
+                                     table=ovs_consts.TRANSIENT_TABLE,
+                                     dl_dst=mac,
+                                     dl_vlan=segment_id)
         else:
-            self._delete_flows(table=ovs_consts.TRANSIENT_TABLE,
-                               dl_dst=mac,
-                               vlan_tci=ovs_consts.FLAT_VLAN_TCI)
+            self._strict_delete_flow(priority=90,
+                                     table=ovs_consts.TRANSIENT_TABLE,
+                                     dl_dst=mac,
+                                     vlan_tci=ovs_consts.FLAT_VLAN_TCI)
 
     def initialize_port_flows(self, port):
         """Set base flows for port
@@ -938,7 +934,8 @@ class OVSFirewallDriver(firewall.FirewallDriver):
         self._initialize_ingress(port)
 
     def _initialize_egress_ipv6_icmp(self, port, allowed_pairs):
-        allowed_pairs = allowed_pairs.union({(port.mac, port.lla_address)})
+        # NOTE(slaweq): should we include also fe80::/64 (link-local) subnet
+        # in the allowed pairs here?
         for mac_addr, ip_addr in allowed_pairs:
             for icmp_type in firewall.ICMPV6_ALLOWED_EGRESS_TYPES:
                 self._add_flow(
@@ -954,19 +951,6 @@ class OVSFirewallDriver(firewall.FirewallDriver):
                     actions='resubmit(,%d)' % (
                         ovs_consts.ACCEPTED_EGRESS_TRAFFIC_NORMAL_TABLE)
                 )
-            for icmp_type in firewall.ICMPV6_RESTRICTED_EGRESS_TYPES:
-                self._add_flow(
-                    table=ovs_consts.BASE_EGRESS_TABLE,
-                    priority=95,
-                    in_port=port.ofport,
-                    reg_port=port.ofport,
-                    dl_type=lib_const.ETHERTYPE_IPV6,
-                    nw_proto=lib_const.PROTO_NUM_IPV6_ICMP,
-                    icmp_type=icmp_type,
-                    nd_target=ip_addr,
-                    actions='resubmit(,%d)' % (
-                        ovs_consts.ACCEPTED_EGRESS_TRAFFIC_NORMAL_TABLE)
-                )
 
     def _initialize_egress_no_port_security(self, port_id, ovs_ports=None):
         try:
@@ -1041,9 +1025,9 @@ class OVSFirewallDriver(firewall.FirewallDriver):
         """Identify egress traffic and send it to egress base"""
 
         # Apply mac/ip pairs for IPv4
-        allowed_mac_ipv4_pairs = port.allowed_pairs_v4.union(
+        allowed_pairs = port.allowed_pairs_v4.union(
             {(port.mac, ip_addr) for ip_addr in port.ipv4_addresses})
-        for mac_addr, ip_addr in allowed_mac_ipv4_pairs:
+        for mac_addr, ip_addr in allowed_pairs:
             self._add_flow(
                 table=ovs_consts.BASE_EGRESS_TABLE,
                 priority=95,
@@ -1069,10 +1053,10 @@ class OVSFirewallDriver(firewall.FirewallDriver):
             )
 
         # Apply mac/ip pairs for IPv6
-        allowed_mac_ipv6_pairs = port.allowed_pairs_v6.union(
+        allowed_pairs = port.allowed_pairs_v6.union(
             {(port.mac, ip_addr) for ip_addr in port.ipv6_addresses})
-        self._initialize_egress_ipv6_icmp(port, allowed_mac_ipv6_pairs)
-        for mac_addr, ip_addr in allowed_mac_ipv6_pairs:
+        self._initialize_egress_ipv6_icmp(port, allowed_pairs)
+        for mac_addr, ip_addr in allowed_pairs:
             self._add_flow(
                 table=ovs_consts.BASE_EGRESS_TABLE,
                 priority=65,
@@ -1087,30 +1071,21 @@ class OVSFirewallDriver(firewall.FirewallDriver):
             )
 
         # DHCP discovery
-        additional_ipv4_filters = [
-            {"dl_src": mac, "nw_src": ip}
-            for mac, ip in (*allowed_mac_ipv4_pairs,
-                            (port.mac, '0.0.0.0'),)]
-        additional_ipv6_filters = [
-            {"dl_src": mac, "ipv6_src": ip}
-            for mac, ip in allowed_mac_ipv6_pairs]
-        for dl_type, src_port, dst_port, filters_list in (
-                (lib_const.ETHERTYPE_IP, 68, 67, additional_ipv4_filters),
-                (lib_const.ETHERTYPE_IPV6, 546, 547, additional_ipv6_filters)):
-            for additional_filters in filters_list:
-                self._add_flow(
-                    table=ovs_consts.BASE_EGRESS_TABLE,
-                    priority=80,
-                    reg_port=port.ofport,
-                    in_port=port.ofport,
-                    dl_type=dl_type,
-                    **additional_filters,
-                    nw_proto=lib_const.PROTO_NUM_UDP,
-                    tp_src=src_port,
-                    tp_dst=dst_port,
-                    actions='resubmit(,{:d})'.format(
-                        ovs_consts.ACCEPT_OR_INGRESS_TABLE)
-                )
+        for dl_type, src_port, dst_port in (
+                (lib_const.ETHERTYPE_IP, 68, 67),
+                (lib_const.ETHERTYPE_IPV6, 546, 547)):
+            self._add_flow(
+                table=ovs_consts.BASE_EGRESS_TABLE,
+                priority=80,
+                reg_port=port.ofport,
+                in_port=port.ofport,
+                dl_type=dl_type,
+                nw_proto=lib_const.PROTO_NUM_UDP,
+                tp_src=src_port,
+                tp_dst=dst_port,
+                actions='resubmit(,{:d})'.format(
+                    ovs_consts.ACCEPT_OR_INGRESS_TABLE)
+            )
         # Ban dhcp service running on an instance
         for dl_type, src_port, dst_port in (
                 (lib_const.ETHERTYPE_IP, 67, 68),
@@ -1455,20 +1430,28 @@ class OVSFirewallDriver(firewall.FirewallDriver):
             protocol = rule.get('protocol')
             priority_offset = rules.flow_priority_offset(rule)
 
+            if rule.get('remote_group_id'):
+                remote_type = 'security group'
+                remote_id = rule.get('remote_group_id')
+            else:
+                remote_type = 'address group'
+                remote_id = rule.get('remote_address_group_id')
             conj_id = self.conj_ip_manager.add(port.vlan_tag, sec_group_id,
-                                               rule['remote_group_id'],
+                                               remote_id,
                                                direction, ethertype,
                                                priority_offset)
             LOG.debug("Created conjunction %(conj_id)s for SG %(sg_id)s "
-                      "referencing remote SG ID %(remote_sg_id)s on port "
-                      "%(port_id)s.",
+                      "referencing remote %(remote_type)s ID %(remote_id)s "
+                      "on port %(port_id)s.",
                       {'conj_id': conj_id,
                        'sg_id': sec_group_id,
-                       'remote_sg_id': rule['remote_group_id'],
+                       'remote_type': remote_type,
+                       'remote_id': remote_id,
                        'port_id': port.id})
 
             rule1 = rule.copy()
-            del rule1['remote_group_id']
+            rule1.pop('remote_group_id', None)
+            rule1.pop('remote_address_group_id', None)
             port_rules_key = (direction, ethertype, protocol)
             port_rules[port_rules_key].append((rule1, conj_id))
 
@@ -1535,17 +1518,19 @@ class OVSFirewallDriver(firewall.FirewallDriver):
     def delete_all_port_flows(self, port):
         """Delete all flows for given port"""
         for mac_addr in port.all_allowed_macs:
-            self._delete_flows(table=ovs_consts.TRANSIENT_TABLE,
-                               dl_dst=mac_addr,
-                               dl_vlan=port.vlan_tag)
+            self._strict_delete_flow(priority=90,
+                                     table=ovs_consts.TRANSIENT_TABLE,
+                                     dl_dst=mac_addr,
+                                     dl_vlan=port.vlan_tag)
             self.delete_physical_direct_flow(mac_addr, port.segment_id)
             self._delete_flows(table=ovs_consts.ACCEPT_OR_INGRESS_TABLE,
                                dl_dst=mac_addr, reg_net=port.vlan_tag)
 
         self.delete_accepted_egress_direct_flow(
             port.mac, port.vlan_tag)
-        self._delete_flows(table=ovs_consts.TRANSIENT_TABLE,
-                           in_port=port.ofport)
+        self._strict_delete_flow(priority=100,
+                                 table=ovs_consts.TRANSIENT_TABLE,
+                                 in_port=port.ofport)
         self._delete_flows(reg_port=port.ofport)
 
     def delete_flows_for_flow_state(
