@@ -25,13 +25,14 @@ from neutron_lib import constants
 from neutron_lib import exceptions
 from oslo_config import cfg
 from oslo_log import log as logging
+from oslo_utils import netutils
 
 from neutron._i18n import _
 from neutron.agent.l3 import ha_router
 from neutron.agent.l3 import namespaces
 from neutron.agent.linux import external_process
+from neutron.agent.linux import ip_lib
 from neutron.agent.linux import utils as linux_utils
-from neutron.common import coordination
 from neutron.common import utils as common_utils
 
 
@@ -69,6 +70,7 @@ defaults
 
 listen listener
     bind %(host)s:%(port)s
+    %(bind_v6_line)s
     server metadata %(unix_socket_path)s
     http-request del-header X-Neutron-%(res_type_del)s-ID
     http-request set-header X-Neutron-%(res_type)s-ID %(res_id)s
@@ -81,13 +83,16 @@ class InvalidUserOrGroupException(Exception):
 
 class HaproxyConfigurator(object):
     def __init__(self, network_id, router_id, unix_socket_path, host, port,
-                 user, group, state_path, pid_file):
+                 user, group, state_path, pid_file, host_v6=None,
+                 bind_interface=None):
         self.network_id = network_id
         self.router_id = router_id
         if network_id is None and router_id is None:
             raise exceptions.NetworkIdOrRouterIdRequiredError()
 
         self.host = host
+        self.host_v6 = host_v6
+        self.bind_interface = bind_interface
         self.port = port
         self.user = user
         self.group = group
@@ -132,8 +137,14 @@ class HaproxyConfigurator(object):
             'group': groupname,
             'pidfile': self.pidfile,
             'log_level': self.log_level,
-            'log_tag': self.log_tag
+            'log_tag': self.log_tag,
+            'bind_v6_line': '',
         }
+        if self.host_v6 and self.bind_interface:
+            cfg_info['bind_v6_line'] = (
+                'bind %s:%s interface %s' % (
+                    self.host_v6, self.port, self.bind_interface)
+            )
         # If using the network ID, delete any spurious router ID that might
         # have been in the request, same for network ID when using router ID.
         if self.network_id:
@@ -193,12 +204,14 @@ class MetadataDriver(object):
                  '-j DROP' % port)]
 
     @classmethod
-    def metadata_nat_rules(cls, port):
-        return [('PREROUTING', '-d 169.254.169.254/32 '
+    def metadata_nat_rules(
+            cls, port, metadata_address=constants.METADATA_V4_CIDR):
+        return [('PREROUTING', '-d %(metadata_address)s '
                  '-i %(interface_name)s '
                  '-p tcp -m tcp --dport 80 -j REDIRECT '
                  '--to-ports %(port)s' %
-                 {'interface_name': namespaces.INTERNAL_DEV_PREFIX + '+',
+                 {'metadata_address': metadata_address,
+                  'interface_name': namespaces.INTERNAL_DEV_PREFIX + '+',
                   'port': port})]
 
     @classmethod
@@ -210,7 +223,9 @@ class MetadataDriver(object):
 
     @classmethod
     def _get_metadata_proxy_callback(cls, bind_address, port, conf,
-                                     network_id=None, router_id=None):
+                                     network_id=None, router_id=None,
+                                     bind_address_v6=None,
+                                     bind_interface=None):
         def callback(pid_file):
             metadata_proxy_socket = conf.metadata_proxy_socket
             user, group = (
@@ -223,7 +238,9 @@ class MetadataDriver(object):
                                           user,
                                           group,
                                           conf.state_path,
-                                          pid_file)
+                                          pid_file,
+                                          bind_address_v6,
+                                          bind_interface)
             haproxy.create_config_file()
             proxy_cmd = [HAPROXY_SERVICE,
                          '-f', haproxy.cfg_path]
@@ -234,14 +251,23 @@ class MetadataDriver(object):
     @classmethod
     def spawn_monitored_metadata_proxy(cls, monitor, ns_name, port, conf,
                                        bind_address="0.0.0.0", network_id=None,
-                                       router_id=None):
+                                       router_id=None, bind_address_v6=None,
+                                       bind_interface=None):
         uuid = network_id or router_id
         callback = cls._get_metadata_proxy_callback(
-            bind_address, port, conf, network_id=network_id,
-            router_id=router_id)
+            bind_address, port, conf,
+            network_id=network_id, router_id=router_id,
+            bind_address_v6=bind_address_v6, bind_interface=bind_interface)
         pm = cls._get_metadata_proxy_process_manager(uuid, conf,
                                                      ns_name=ns_name,
                                                      callback=callback)
+        if bind_interface is not None and bind_address_v6 is not None:
+            # HAProxy cannot bind() until IPv6 Duplicate Address Detection
+            # completes. We must wait until the address leaves its 'tentative'
+            # state.
+            ip_lib.IpAddrCommand(
+                parent=ip_lib.IPDevice(name=bind_interface, namespace=ns_name)
+            ).wait_until_address_ready(address=bind_address_v6)
         pm.enable()
         monitor.register(uuid, METADATA_SERVICE_NAME, pm)
         cls.monitors[router_id] = pm
@@ -281,14 +307,34 @@ class MetadataDriver(object):
 def after_router_added(resource, event, l3_agent, **kwargs):
     router = kwargs['router']
     proxy = l3_agent.metadata_driver
-    apply_metadata_nat_rules(router, proxy)
+    ipv6_enabled = netutils.is_ipv6_enabled()
+    for c, r in proxy.metadata_filter_rules(proxy.metadata_port,
+                                            proxy.metadata_access_mark):
+        router.iptables_manager.ipv4['filter'].add_rule(c, r)
+    if ipv6_enabled:
+        for c, r in proxy.metadata_filter_rules(proxy.metadata_port,
+                                                proxy.metadata_access_mark):
+            router.iptables_manager.ipv6['filter'].add_rule(c, r)
+    for c, r in proxy.metadata_nat_rules(proxy.metadata_port):
+        router.iptables_manager.ipv4['nat'].add_rule(c, r)
+    if ipv6_enabled:
+        for c, r in proxy.metadata_nat_rules(
+                proxy.metadata_port,
+                metadata_address=(constants.METADATA_V6_IP + '/128')):
+            router.iptables_manager.ipv6['nat'].add_rule(c, r)
+    router.iptables_manager.apply()
+
     if not isinstance(router, ha_router.HaRouter):
+        spawn_kwargs = {}
+        if ipv6_enabled:
+            spawn_kwargs['bind_address'] = '::'
         proxy.spawn_monitored_metadata_proxy(
             l3_agent.process_monitor,
             router.ns_name,
             proxy.metadata_port,
             l3_agent.conf,
-            router_id=router.router_id)
+            router_id=router.router_id,
+            **spawn_kwargs)
 
 
 def after_router_updated(resource, event, l3_agent, **kwargs):
@@ -296,12 +342,16 @@ def after_router_updated(resource, event, l3_agent, **kwargs):
     proxy = l3_agent.metadata_driver
     if (not proxy.monitors.get(router.router_id) and
             not isinstance(router, ha_router.HaRouter)):
+        spawn_kwargs = {}
+        if netutils.is_ipv6_enabled():
+            spawn_kwargs['bind_address'] = '::'
         proxy.spawn_monitored_metadata_proxy(
             l3_agent.process_monitor,
             router.ns_name,
             proxy.metadata_port,
             l3_agent.conf,
-            router_id=router.router_id)
+            router_id=router.router_id,
+            **spawn_kwargs)
 
 
 def before_router_removed(resource, event, l3_agent, payload=None):
@@ -312,13 +362,3 @@ def before_router_removed(resource, event, l3_agent, payload=None):
                                            router.router['id'],
                                            l3_agent.conf,
                                            router.ns_name)
-
-
-@coordination.synchronized('router-lock-ns-{router.ns_name}')
-def apply_metadata_nat_rules(router, proxy):
-    for c, r in proxy.metadata_filter_rules(proxy.metadata_port,
-                                            proxy.metadata_access_mark):
-        router.iptables_manager.ipv4['filter'].add_rule(c, r)
-    for c, r in proxy.metadata_nat_rules(proxy.metadata_port):
-        router.iptables_manager.ipv4['nat'].add_rule(c, r)
-    router.iptables_manager.apply()
