@@ -15,7 +15,6 @@
 import collections
 import functools
 import re
-import threading
 import uuid
 
 from neutron_lib import constants as n_const
@@ -49,8 +48,6 @@ OVN_VIF_PORT_TYPES = ("", "external", )
 MetadataPortInfo = collections.namedtuple('MetadataPortInfo', ['mac',
                                                                'ip_addresses'])
 
-OVN_METADATA_UUID_NAMESPACE = uuid.UUID('d34bf9f6-da32-4871-9af8-15a4626b41ab')
-
 
 def _sync_lock(f):
     """Decorator to block all operations for a global sync call."""
@@ -203,31 +200,12 @@ class MetadataAgent(object):
         self._process_monitor = external_process.ProcessMonitor(
             config=self.conf,
             resource_type='metadata')
-        self._sb_idl = None
-        self._post_fork_event = threading.Event()
-
-    @property
-    def sb_idl(self):
-        if not self._sb_idl:
-            self._post_fork_event.wait()
-        return self._sb_idl
-
-    @sb_idl.setter
-    def sb_idl(self, val):
-        self._sb_idl = val
 
     def _load_config(self):
         self.chassis = self._get_own_chassis_name()
-        try:
-            self.chassis_id = uuid.UUID(self.chassis)
-        except ValueError:
-            # OVS system-id could be a non UUID formatted string.
-            self.chassis_id = uuid.uuid5(OVN_METADATA_UUID_NAMESPACE,
-                                         self.chassis)
-
         self.ovn_bridge = self._get_ovn_bridge()
-        LOG.info("Loaded chassis name %s (UUID: %s) and ovn bridge %s.",
-                 self.chassis, self.chassis_id, self.ovn_bridge)
+        LOG.debug("Loaded chassis %s and ovn bridge %s.",
+                  self.chassis, self.ovn_bridge)
 
     @_sync_lock
     def resync(self):
@@ -244,8 +222,7 @@ class MetadataAgent(object):
         self._load_config()
 
         # Launch the server that will act as a proxy between the VM's and Nova.
-        proxy = metadata_server.UnixDomainMetadataProxy(self.conf,
-                self.chassis)
+        proxy = metadata_server.UnixDomainMetadataProxy(self.conf)
         proxy.run()
 
         tables = ('Encap', 'Port_Binding', 'Datapath_Binding', 'SB_Global',
@@ -259,7 +236,6 @@ class MetadataAgent(object):
         # Chassis table.
         # Open the connection to OVN SB database.
         self.has_chassis_private = False
-        self._post_fork_event.clear()
         try:
             self.sb_idl = ovsdb.MetadataAgentOvnSbIdl(
                 chassis=self.chassis, tables=tables + ('Chassis_Private', ),
@@ -270,9 +246,6 @@ class MetadataAgent(object):
                 chassis=self.chassis, tables=tables,
                 events=events + (ChassisCreateEvent(self), )).start()
 
-        # Now IDL connections can be safely used.
-        self._post_fork_event.set()
-
         # Do the initial sync.
         self.sync()
 
@@ -289,8 +262,9 @@ class MetadataAgent(object):
         # NOTE(lucasagomes): db_add() will not overwrite the UUID if
         # it's already set.
         table = ('Chassis_Private' if self.has_chassis_private else 'Chassis')
+        chassis_id = uuid.UUID(self._get_own_chassis_name())
         # Generate unique, but consistent metadata id for chassis name
-        agent_id = uuid.uuid5(self.chassis_id, 'metadata_agent')
+        agent_id = uuid.uuid5(chassis_id, 'metadata_agent')
         ext_ids = {ovn_const.OVN_AGENT_METADATA_ID_KEY: str(agent_id)}
         self.sb_idl.db_add(table, self.chassis, 'external_ids',
                            ext_ids).execute(check_error=True)
