@@ -12,7 +12,6 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from oslo_log import log
 from ovs.db import idl
 from ovsdbapp.backend.ovs_idl import connection
 from ovsdbapp.backend.ovs_idl import idlutils
@@ -24,9 +23,6 @@ from neutron.plugins.ml2.drivers.ovn.mech_driver.ovsdb import impl_idl_ovn
 from neutron.plugins.ml2.drivers.ovn.mech_driver.ovsdb import ovsdb_monitor
 
 
-LOG = log.getLogger(__name__)
-
-
 class MetadataAgentOvnSbIdl(ovsdb_monitor.OvnIdl):
 
     SCHEMA = 'OVN_Southbound'
@@ -40,17 +36,12 @@ class MetadataAgentOvnSbIdl(ovsdb_monitor.OvnIdl):
                       'SB_Global')
         for table in tables:
             helper.register_table(table)
-        try:
-            super(MetadataAgentOvnSbIdl, self).__init__(
-                None, connection_string, helper, leader_only=False)
-        except TypeError:
-            # TODO(twilson) We can remove this when we require ovs>=2.12.0
-            super(MetadataAgentOvnSbIdl, self).__init__(
-                None, connection_string, helper)
+        super(MetadataAgentOvnSbIdl, self).__init__(
+            None, connection_string, helper)
         if chassis:
-            for table in set(tables).intersection({'Chassis',
-                                                   'Chassis_Private'}):
-                self.set_table_condition(table, [['name', '==', chassis]])
+            table = ('Chassis_Private' if 'Chassis_Private' in tables
+                     else 'Chassis')
+            self.tables[table].condition = [['name', '==', chassis]]
         if events:
             self.notify_handler.watch_events(events)
 
@@ -60,12 +51,7 @@ class MetadataAgentOvnSbIdl(ovsdb_monitor.OvnIdl):
     def _get_ovsdb_helper(self, connection_string):
         return idlutils.get_schema_helper(connection_string, self.SCHEMA)
 
-    @tenacity.retry(
-        wait=tenacity.wait_exponential(
-            max=config.get_ovn_ovsdb_retry_max_interval()),
-        reraise=True)
     def start(self):
-        LOG.info('Getting OvsdbSbOvnIdl for MetadataAgent with retry')
         conn = connection.Connection(
             self, timeout=config.get_ovn_ovsdb_timeout())
         return impl_idl_ovn.OvsdbSbOvnIdl(conn)
