@@ -43,9 +43,8 @@ MODE_MAP = {
 
 class MetadataProxyHandler(object):
 
-    def __init__(self, conf, chassis):
+    def __init__(self, conf):
         self.conf = conf
-        self.chassis = chassis
         self.subscribe()
 
     def subscribe(self):
@@ -57,8 +56,7 @@ class MetadataProxyHandler(object):
         # We need to open a connection to OVN SouthBound database for
         # each worker so that we can process the metadata requests.
         self.sb_idl = ovsdb.MetadataAgentOvnSbIdl(
-            tables=('Port_Binding', 'Datapath_Binding', 'Chassis'),
-            chassis=self.chassis).start()
+            tables=('Port_Binding', 'Datapath_Binding')).start()
 
     @webob.dec.wsgify(RequestClass=webob.Request)
     def __call__(self, req):
@@ -175,9 +173,8 @@ class MetadataProxyHandler(object):
 
 class UnixDomainMetadataProxy(object):
 
-    def __init__(self, conf, chassis):
+    def __init__(self, conf):
         self.conf = conf
-        self.chassis = chassis
         agent_utils.ensure_directory_exists_without_file(
             cfg.CONF.metadata_proxy_socket)
 
@@ -206,7 +203,7 @@ class UnixDomainMetadataProxy(object):
         md_workers = self.conf.metadata_workers
         if md_workers is None:
             md_workers = 2
-        self.server.start(MetadataProxyHandler(self.conf, self.chassis),
+        self.server.start(MetadataProxyHandler(self.conf),
                           self.conf.metadata_proxy_socket,
                           workers=md_workers,
                           backlog=self.conf.metadata_backlog,
