@@ -15,16 +15,17 @@
 
 import random
 
+from neutron_lib.agent import topics
+from neutron_lib.api import extensions
 from neutron_lib import constants
 from neutron_lib.plugins import constants as plugin_constants
 from neutron_lib.plugins import directory
+from neutron_lib import rpc as n_rpc
 from oslo_log import log as logging
 import oslo_messaging
 
 from neutron.api.rpc.agentnotifiers import utils as ag_utils
-from neutron.common import rpc as n_rpc
-from neutron.common import topics
-from neutron.common import utils
+from neutron.common import utils as common_utils
 
 
 LOG = logging.getLogger(__name__)
@@ -55,7 +56,9 @@ class L3AgentNotifyAPI(object):
     def _agent_notification(self, context, method, router_ids, operation,
                             shuffle_agents):
         """Notify changed routers to hosting l3 agents."""
-        adminContext = context if context.is_admin else context.elevated()
+        adminContext = (
+            context if context.is_admin else
+            common_utils.get_elevated_context(context))
         plugin = directory.get_plugin(plugin_constants.L3)
         for router_id in router_ids:
             hosts = plugin.get_hosts_to_notify(adminContext, router_id)
@@ -90,10 +93,11 @@ class L3AgentNotifyAPI(object):
             LOG.error('No plugin for L3 routing registered. Cannot notify '
                       'agents with the message %s', method)
             return
-        if utils.is_extension_supported(
+        if extensions.is_extension_supported(
                 plugin, constants.L3_AGENT_SCHEDULER_EXT_ALIAS):
-            adminContext = (context.is_admin and
-                            context or context.elevated())
+            adminContext = (
+                context.is_admin and
+                context or common_utils.get_elevated_context(context))
             if schedule_routers:
                 plugin.schedule_routers(adminContext, router_ids)
             self._agent_notification(
