@@ -11,8 +11,11 @@
 #    under the License.
 
 from neutron._i18n import _
+from neutron.objects import address_group
+from neutron.objects import conntrack_helper
 from neutron.objects.logapi import logging_resource as log_object
 from neutron.objects import network
+from neutron.objects import port_forwarding
 from neutron.objects import ports
 from neutron.objects.qos import policy
 from neutron.objects import securitygroup
@@ -30,6 +33,9 @@ NETWORK = network.Network.obj_name()
 SUBNET = subnet.Subnet.obj_name()
 SECURITYGROUP = securitygroup.SecurityGroup.obj_name()
 SECURITYGROUPRULE = securitygroup.SecurityGroupRule.obj_name()
+PORTFORWARDING = port_forwarding.PortForwarding.obj_name()
+CONNTRACKHELPER = conntrack_helper.ConntrackHelper.obj_name()
+ADDRESSGROUP = address_group.AddressGroup.obj_name()
 
 
 _VALID_CLS = (
@@ -42,6 +48,9 @@ _VALID_CLS = (
     securitygroup.SecurityGroup,
     securitygroup.SecurityGroupRule,
     log_object.Log,
+    port_forwarding.PortForwarding,
+    conntrack_helper.ConntrackHelper,
+    address_group.AddressGroup,
 )
 
 _TYPE_TO_CLS_MAP = {cls.obj_name(): cls for cls in _VALID_CLS}
