@@ -258,6 +258,12 @@ class SecurityGroupServerAPIShim(sg_rpc_base.SecurityGroupInfoAPIMixin):
                            'Port', events.AFTER_DELETE)
         registry.subscribe(self._handle_sg_member_update,
                            'Port', events.AFTER_UPDATE)
+        self._register_legacy_ag_notification_callbacks(sg_agent)
+
+    def _register_legacy_ag_notification_callbacks(self, sg_agent):
+        for event in (events.AFTER_UPDATE, events.AFTER_DELETE):
+            registry.subscribe(self._handle_address_group_event,
+                               resources.ADDRESSGROUP, event)
 
     def security_group_info_for_devices(self, context, devices):
         ports = self._get_devices_info(context, devices)
@@ -270,6 +276,11 @@ class SecurityGroupServerAPIShim(sg_rpc_base.SecurityGroupInfoAPIMixin):
         # error.
         raise NotImplementedError()
 
+    def get_secgroup_ids_for_address_group(self, address_group_id):
+        filters = {'remote_address_group_id': (address_group_id, )}
+        return set([rule.security_group_id for rule in
+                    self.rcache.get_resources('SecurityGroupRule', filters)])
+
     def _add_child_sg_rules(self, rtype, event, trigger, context, updated,
                             **kwargs):
         # whenever we receive a full security group, add all child rules
@@ -318,10 +329,17 @@ class SecurityGroupServerAPIShim(sg_rpc_base.SecurityGroupInfoAPIMixin):
                                             'allowed_address_pairs'}):
             # none of the relevant fields to SG calculations changed
             return
-        sgs.update({sg_id for sg_id in updated.security_group_ids})
+        sgs.update(set(updated.security_group_ids))
         if sgs:
             self._sg_agent.security_groups_member_updated(sgs)
 
+    def _handle_address_group_event(self, rtype, event, trigger, context,
+                                    resource_id, **kwargs):
+        if event == events.AFTER_UPDATE:
+            self._sg_agent.address_group_updated(resource_id)
+        else:
+            self._sg_agent.address_group_deleted(resource_id)
+
     def _get_devices_info(self, context, devices):
         # NOTE(kevinbenton): this format is required by the sg code, it is
         # defined in get_port_from_device and mimics
@@ -339,6 +357,7 @@ class SecurityGroupServerAPIShim(sg_rpc_base.SecurityGroupInfoAPIMixin):
             port['security_groups'] = list(ovo.security_group_ids)
             port['security_group_rules'] = []
             port['security_group_source_groups'] = []
+            port['security_group_remote_address_groups'] = []
             port['fixed_ips'] = [str(f['ip_address'])
                                  for f in port['fixed_ips']]
             # NOTE(kevinbenton): this id==device is only safe for OVS. a lookup
@@ -366,6 +385,19 @@ class SecurityGroupServerAPIShim(sg_rpc_base.SecurityGroupInfoAPIMixin):
                     ips_by_group[sg_id].update(set(port_ips))
         return ips_by_group
 
+    def _select_ips_for_remote_address_group(self, context,
+                                             remote_address_group_ids):
+        if not remote_address_group_ids:
+            return {}
+        ips_by_group = {ag: set() for ag in remote_address_group_ids}
+        for ag_id in remote_address_group_ids:
+            ag = self.rcache.get_resource_by_id('AddressGroup', ag_id)
+            # NOTE(hangyang) Expected data structure is (ip, mac) tuple
+            ips = [(str(addr_assoc.address), None)
+                   for addr_assoc in ag.addresses]
+            ips_by_group[ag_id].update(set(ips))
+        return ips_by_group
+
     def _select_rules_for_ports(self, context, ports):
         if not ports:
             return []
