@@ -65,7 +65,7 @@ class cache_method_results(object):
         self._first_call = True
         self._not_cached = cache.NO_VALUE
 
-    def _get_from_cache(self, target_self, *args, **kwargs):
+    def _get_from_cache(self, target_self, *args, skip_cache=False, **kwargs):
         target_self_cls_name = reflection.get_class_name(target_self,
                                                          fully_qualified=False)
         func_name = "%(module)s.%(class)s.%(func_name)s" % {
@@ -73,13 +73,12 @@ class cache_method_results(object):
             'class': target_self_cls_name,
             'func_name': self.func.__name__,
         }
-        skip_cache = kwargs.pop('skip_cache', False)
-
         key = (func_name,) + args
         if kwargs:
             key += helpers.dict2tuple(kwargs)
         # oslo.cache expects a string or a buffer
         key = str(key)
+
         if not skip_cache:
             try:
                 item = target_self._cache.get(key)
