@@ -15,8 +15,6 @@ import re
 from neutron_lib.api.definitions import portbindings
 from neutron_lib import constants as const
 
-# TODO(lucasagomes): Remove OVN_SG_NAME_EXT_ID_KEY in the Rocky release
-OVN_SG_NAME_EXT_ID_KEY = 'neutron:security_group_name'
 OVN_SG_EXT_ID_KEY = 'neutron:security_group_id'
 OVN_SG_RULE_EXT_ID_KEY = 'neutron:security_group_rule_id'
 OVN_ML2_MECH_DRIVER_NAME = 'ovn'
@@ -39,6 +37,7 @@ OVN_CIDRS_EXT_ID_KEY = 'neutron:cidrs'
 OVN_FIP_EXT_ID_KEY = 'neutron:fip_id'
 OVN_FIP_PORT_EXT_ID_KEY = 'neutron:fip_port_id'
 OVN_FIP_EXT_MAC_KEY = 'neutron:fip_external_mac'
+OVN_FIP_NET_ID = 'neutron:fip_network_id'
 OVN_REV_NUM_EXT_ID_KEY = 'neutron:revision_number'
 OVN_QOS_POLICY_EXT_ID_KEY = 'neutron:qos_policy_id'
 OVN_SG_IDS_EXT_ID_KEY = 'neutron:security_group_ids'
@@ -51,12 +50,10 @@ OVN_PORT_BINDING_PROFILE_PARAMS = [{'parent_name': str,
                                    {'vtep-physical-switch': str,
                                     'vtep-logical-switch': str}]
 MIGRATING_ATTR = 'migrating_to'
-OVN_ROUTER_PORT_OPTION_KEYS = ['router-port', 'nat-addresses',
-                               'exclude-lb-vips-from-garp']
+OVN_ROUTER_PORT_OPTION_KEYS = ['router-port', 'nat-addresses']
 OVN_GATEWAY_CHASSIS_KEY = 'redirect-chassis'
 OVN_CHASSIS_REDIRECT = 'chassisredirect'
 OVN_GATEWAY_NAT_ADDRESSES_KEY = 'nat-addresses'
-OVN_ROUTER_PORT_EXCLUDE_LB_VIPS_GARP = 'exclude-lb-vips-from-garp'
 OVN_DROP_PORT_GROUP_NAME = 'neutron_pg_drop'
 OVN_ROUTER_PORT_GW_MTU_OPTION = 'gateway_mtu'
 
@@ -70,9 +67,6 @@ OVN_AGENT_METADATA_ID_KEY = 'neutron:ovn-metadata-id'
 OVN_CONTROLLER_AGENT = 'OVN Controller agent'
 OVN_CONTROLLER_GW_AGENT = 'OVN Controller Gateway agent'
 OVN_METADATA_AGENT = 'OVN Metadata agent'
-OVN_CONTROLLER_TYPES = (OVN_CONTROLLER_AGENT,
-                        OVN_CONTROLLER_GW_AGENT,
-                        )
 
 # OVN ACLs have priorities.  The highest priority ACL that matches is the one
 # that takes effect.  Our choice of priority numbers is arbitrary, but it
@@ -83,6 +77,7 @@ ACL_PRIORITY_ALLOW = 1002
 ACL_PRIORITY_DROP = 1001
 
 ACL_ACTION_DROP = 'drop'
+ACL_ACTION_REJECT = 'reject'
 ACL_ACTION_ALLOW_RELATED = 'allow-related'
 ACL_ACTION_ALLOW = 'allow'
 
@@ -164,7 +159,7 @@ SUPPORTED_DHCP_OPTS_MAPPING = {
     6: {'server-id': 'server_id',
         'dns-server': 'dns_server',
         'domain-search': 'domain_search',
-        'ia-addr': 'ia_addr',
+        'ia-addr': 'ip_addr',
         '2': 'server_id',
         '5': 'ia_addr',
         '24': 'domain_search',
@@ -224,8 +219,6 @@ _TYPES_PRIORITY_ORDER = (
     TYPE_FLOATINGIPS,
     TYPE_SECURITY_GROUP_RULES)
 
-DB_CONSISTENCY_CHECK_INTERVAL = 300  # 5 minutes
-
 # The order in which the resources should be created or updated by the
 # maintenance task: Root ones first and leafs at the end.
 MAINTENANCE_CREATE_UPDATE_TYPE_ORDER = {
@@ -284,12 +277,6 @@ LSP_OPTIONS_VIRTUAL_IP_KEY = 'virtual-ip'
 LSP_OPTIONS_MCAST_FLOOD_REPORTS = 'mcast_flood_reports'
 LSP_OPTIONS_MCAST_FLOOD = 'mcast_flood'
 
-LRP_OPTIONS_RESIDE_REDIR_CH = 'reside-on-redirect-chassis'
-
-# Port Binding types
-PB_TYPE_PATCH = 'patch'
-PB_TYPE_VIRTUAL = 'virtual'
-
 HA_CHASSIS_GROUP_DEFAULT_NAME = 'default_ha_chassis_group'
 HA_CHASSIS_GROUP_HIGHEST_PRIORITY = 32767
 
@@ -305,3 +292,6 @@ NEUTRON_AVAILABILITY_ZONES = 'neutron-availability-zones'
 OVN_CMS_OPTIONS = 'ovn-cms-options'
 CMS_OPT_CHASSIS_AS_GW = 'enable-chassis-as-gw'
 CMS_OPT_AVAILABILITY_ZONES = 'availability-zones'
+
+# OVN vlan transparency option
+VLAN_PASSTHRU = 'vlan-passthru'
