@@ -40,8 +40,6 @@ from ovsdbapp import constants as ovsdbapp_const
 from neutron._i18n import _
 from neutron.common.ovn import constants
 from neutron.common.ovn import exceptions as ovn_exc
-from neutron.common import utils as common_utils
-from neutron.conf.plugins.ml2.drivers.ovn import ovn_conf
 
 LOG = log.getLogger(__name__)
 
@@ -76,6 +74,12 @@ def ovn_lrouter_port_name(id):
     return constants.LRP_PREFIX + '%s' % id
 
 
+def ovn_cr_lrouter_port_name(_id):
+    # The name of the OVN chassisredirect lrouter port entry will be
+    # cr-lrp-<UUID>
+    return 'cr-lrp-%s' % _id
+
+
 def ovn_provnet_port_name(network_id):
     # The name of OVN lswitch provider network port entry will be
     # provnet-<Network-UUID>. The port is created for network having
@@ -363,19 +367,15 @@ def sort_ips_by_version(addresses):
 
 
 def is_lsp_router_port(port):
-    return port.get('device_owner') in const.ROUTER_PORT_OWNERS
+    return port.get('device_owner') in [const.DEVICE_OWNER_ROUTER_INTF,
+                                        const.DEVICE_OWNER_ROUTER_GW]
 
 
 def get_lrouter_ext_gw_static_route(ovn_router):
-    # TODO(lucasagomes): Remove the try...except block after OVS 2.8.2
-    # is tagged.
-    try:
-        return [route for route in getattr(ovn_router, 'static_routes', []) if
-                strutils.bool_from_string(getattr(
-                    route, 'external_ids', {}).get(
-                        constants.OVN_ROUTER_IS_EXT_GW, 'false'))]
-    except KeyError:
-        pass
+    return [route for route in getattr(ovn_router, 'static_routes', []) if
+            strutils.bool_from_string(getattr(
+                route, 'external_ids', {}).get(
+                    constants.OVN_ROUTER_IS_EXT_GW, 'false'))]
 
 
 def get_lrouter_snats(ovn_router):
@@ -384,19 +384,14 @@ def get_lrouter_snats(ovn_router):
 
 def get_lrouter_non_gw_routes(ovn_router):
     routes = []
-    # TODO(lucasagomes): Remove the try...except block after OVS 2.8.2
-    # is tagged.
-    try:
-        for route in getattr(ovn_router, 'static_routes', []):
-            external_ids = getattr(route, 'external_ids', {})
-            if strutils.bool_from_string(
-                    external_ids.get(constants.OVN_ROUTER_IS_EXT_GW, 'false')):
-                continue
+    for route in getattr(ovn_router, 'static_routes', []):
+        external_ids = getattr(route, 'external_ids', {})
+        if strutils.bool_from_string(
+                external_ids.get(constants.OVN_ROUTER_IS_EXT_GW, 'false')):
+            continue
 
-            routes.append({'destination': route.ip_prefix,
-                           'nexthop': route.nexthop})
-    except KeyError:
-        pass
+        routes.append({'destination': route.ip_prefix,
+                       'nexthop': route.nexthop})
     return routes
 
 
@@ -421,26 +416,6 @@ def get_system_dns_resolvers(resolver_file=DNS_RESOLVER_FILE):
     return resolvers
 
 
-def get_dhcp_dns_servers(subnet, ip_version=const.IP_VERSION_4):
-    """Retrieve the DHCP option DNS servers
-
-    The DHCP should not announce any DNS resolver at all on the subnet if any
-    configured DNS server is "0.0.0.0" (IPv4) or "::" (IPv6).
-    https://docs.openstack.org/neutron/latest/admin/config-dns-res.html
-    """
-    if ip_version == const.IP_VERSION_4:
-        dns_servers = (subnet.get('dns_nameservers') or
-                       ovn_conf.get_dns_servers() or
-                       get_system_dns_resolvers())
-    else:
-        dns_servers = subnet['dns_nameservers']
-
-    if common_utils.is_dns_servers_any_address(dns_servers, ip_version):
-        return []
-
-    return dns_servers
-
-
 def get_port_subnet_ids(port):
     fixed_ips = list(port['fixed_ips'])
     return [f['subnet_id'] for f in fixed_ips]
@@ -590,23 +565,3 @@ def parse_ovn_lb_port_forwarding(ovn_rtr_lb_pfs):
 
 def get_network_name_from_datapath(datapath):
     return datapath.external_ids['name'].replace('neutron-', '')
-
-
-def connection_config_to_target_string(connection_config):
-    """Converts the Neutron NB/SB connection parameter to the OVN target string
-
-    :param connection_config: Neutron OVN config parameter for the OVN NB or SB
-                              database. See "ovn_sb_connection" or
-                              "ovn_nb_connection" params.
-    :returns: (String) OVN NB/SB ``connection.target`` column value.
-    """
-    regex = re.compile(r'^(?P<proto>\w+)\:((?P<ip>.+)\:(?P<port>\d+)|'
-                       r'(?P<file>[\w\/\.]+))')
-    m = regex.match(connection_config)
-    if m:
-        _dict = m.groupdict()
-        if _dict['ip'] and _dict['port']:
-            return ('p' + _dict['proto'] + ':' + _dict['port'] + ':' +
-                    _dict['ip'])
-        elif _dict['file']:
-            return 'p' + _dict['proto'] + ':' + _dict['file']
