@@ -18,6 +18,7 @@
 
 """Utilities and helper functions."""
 
+import datetime
 import functools
 import importlib
 import os
@@ -36,6 +37,7 @@ from eventlet.green import subprocess
 import netaddr
 from neutron_lib import constants as n_const
 from neutron_lib.db import api as db_api
+from neutron_lib import exceptions as n_exc
 from neutron_lib.services.trunk import constants as trunk_constants
 from neutron_lib.utils import helpers
 from oslo_config import cfg
@@ -46,6 +48,10 @@ from oslo_utils import timeutils
 from oslo_utils import uuidutils
 from osprofiler import profiler
 import pkg_resources
+from sqlalchemy.dialects.mysql import dialect as mysql_dialect
+from sqlalchemy.dialects.postgresql import dialect as postgresql_dialect
+from sqlalchemy.dialects.sqlite import dialect as sqlite_dialect
+from sqlalchemy.sql.expression import func as sql_func
 
 import neutron
 from neutron._i18n import _
@@ -63,6 +69,10 @@ class WaitTimeout(Exception):
     """Default exception coming from wait_until_true() function."""
 
 
+class TimerTimeout(n_exc.NeutronException):
+    message = _('Timer timeout expired after %(timeout)s second(s).')
+
+
 class LockWithTimer(object):
     def __init__(self, threshold):
         self._threshold = threshold
@@ -859,6 +869,72 @@ def validate_rp_bandwidth(rp_bandwidths, device_names):
                 "device mappings") % {'dev_name': dev_name})
 
 
+class Timer(object):
+    """Timer context manager class
+
+    This class creates a context that:
+    - Triggers a timeout exception if the timeout is set.
+    - Returns the time elapsed since the context was initialized.
+    - Returns the time spent in the context once it's closed.
+
+    The timeout exception can be suppressed; when the time expires, the context
+    finishes without rising TimerTimeout.
+
+    NOTE(ralonsoh): this class, when a timeout is defined, cannot be used in
+    other than the main thread. When a timeout is defined, an alarm signal is
+    set. Only the main thread is allowed to set a signal handler and the signal
+    handlers are always executed in this main thread [1].
+    [1] https://docs.python.org/3/library/signal.html#signals-and-threads
+    """
+    def __init__(self, timeout=None, raise_exception=True):
+        self.start = self.delta = None
+        self._timeout = int(timeout) if timeout else None
+        self._timeout_flag = False
+        self._raise_exception = raise_exception
+
+    def _timeout_handler(self, *_):
+        self._timeout_flag = True
+        if self._raise_exception:
+            raise TimerTimeout(timeout=self._timeout)
+        self.__exit__()
+
+    def __enter__(self):
+        self.start = datetime.datetime.now()
+        if self._timeout:
+            signal.signal(signal.SIGALRM, self._timeout_handler)
+            signal.alarm(self._timeout)
+        return self
+
+    def __exit__(self, *_):
+        if self._timeout:
+            signal.alarm(0)
+        self.delta = datetime.datetime.now() - self.start
+
+    def __getattr__(self, item):
+        return getattr(self.delta, item)
+
+    def __iter__(self):
+        self._raise_exception = False
+        return self.__enter__()
+
+    def next(self):  # pragma: no cover
+        # NOTE(ralonsoh): Python 2 support.
+        if not self._timeout_flag:
+            return datetime.datetime.now()
+        raise StopIteration()
+
+    def __next__(self):  # pragma: no cover
+        # NOTE(ralonsoh): Python 3 support.
+        return self.next()
+
+    def __del__(self):
+        signal.alarm(0)
+
+    @property
+    def delta_time_sec(self):
+        return (datetime.datetime.now() - self.start).total_seconds()
+
+
 def collect_profiler_info():
     p = profiler.get()
     if p:
@@ -965,3 +1041,26 @@ def with_metaclass(meta, *bases):
             return meta(name, bases, d)
 
     return metaclass('temporary_class', None, {})
+
+
+def get_elevated_context(context):
+    admin_context = context.elevated()
+    # NOTE(slaweq): system_scope='all' is needed if new policies are
+    # enforced. This should be set in context.elevated() method in the
+    # neutron-lib but as a temporary workaround it is done here
+    if cfg.CONF.oslo_policy.enforce_new_defaults:
+        admin_context.system_scope = 'all'
+    return admin_context
+
+
+def get_sql_random_method(sql_dialect_name):
+    """Return the SQL random method supported depending on the dialect."""
+    # NOTE(ralonsoh): this method is a good candidate to be implemented in
+    # oslo.db.
+    # https://www.postgresql.org/docs/8.2/functions-math.html
+    # https://www.sqlite.org/c3ref/randomness.html
+    if sql_dialect_name in (postgresql_dialect.name, sqlite_dialect.name):
+        return sql_func.random
+    # https://dev.mysql.com/doc/refman/8.0/en/mathematical-functions.html
+    elif sql_dialect_name == mysql_dialect.name:
+        return sql_func.rand
