@@ -84,13 +84,8 @@ ovs_opts = [
                        "hypervisor name is used to locate the parent of the "
                        "resource provider tree. Only needs to be set in the "
                        "rare case when the hypervisor name is different from "
-                       "the resource_provider_default_hypervisor config "
-                       "option value as known by the nova-compute managing "
-                       "that hypervisor.")),
-    cfg.StrOpt('resource_provider_default_hypervisor',
-               help=_("The default hypervisor name used to locate the parent "
-                      "of the resource provider. If this option is not set, "
-                      "canonical name is used")),
+                       "the DEFAULT.host config option value as known by the "
+                       "nova-compute managing that hypervisor.")),
     cfg.DictOpt('resource_provider_inventory_defaults',
                 default={'allocation_ratio': 1.0,
                          'min_unit': 1,
@@ -105,11 +100,6 @@ ovs_opts = [
                        "See also: "
                        "https://docs.openstack.org/api-ref/placement/"
                        "#update-resource-provider-inventories")),
-    cfg.BoolOpt('use_veth_interconnection', default=False,
-                help=_("Use veths instead of patch ports to interconnect the "
-                       "integration bridge to physical networks. "
-                       "Support kernel without Open vSwitch patch port "
-                       "support so long as it is set to True.")),
     cfg.StrOpt('datapath_type', default=constants.OVS_DATAPATH_SYSTEM,
                choices=[constants.OVS_DATAPATH_SYSTEM,
                         constants.OVS_DATAPATH_NETDEV],
