@@ -20,6 +20,7 @@ from eventlet import greenthread
 from neutron_lib.agent import constants as agent_consts
 from neutron_lib.api import converters
 from neutron_lib.api.definitions import agent as agent_apidef
+from neutron_lib.api.definitions import availability_zone_filter as azfil_ext
 from neutron_lib.api import extensions
 from neutron_lib.callbacks import events
 from neutron_lib.callbacks import registry
@@ -43,7 +44,6 @@ from neutron.agent.common import utils
 from neutron.api.rpc.callbacks import version_manager
 from neutron.conf.agent.database import agents_db
 from neutron.db.models import agent as agent_model
-from neutron.extensions import _availability_zone_filter_lib as azfil_ext
 from neutron.extensions import agent as ext_agent
 from neutron.extensions import availability_zone as az_ext
 from neutron.objects import agent as agent_obj
@@ -269,9 +269,6 @@ class AgentDbMixin(ext_agent.AgentPluginBase, AgentAvailabilityZoneMixin):
                          payload=events.DBEventPayload(
                              context, states=(agent,), resource_id=id))
         agent.delete()
-        registry.publish(resources.AGENT, events.AFTER_DELETE, self,
-                         payload=events.DBEventPayload(
-                             context, states=(agent,), resource_id=id))
 
     @db_api.retry_if_session_inactive()
     def update_agent(self, context, id, agent):
@@ -397,7 +394,6 @@ class AgentDbMixin(ext_agent.AgentPluginBase, AgentAvailabilityZoneMixin):
                 agent = self._get_agent_by_type_and_host(
                     context, agent_state['agent_type'], agent_state['host'])
                 agent_state_orig = copy.deepcopy(agent_state)
-                agent_state_previous = copy.deepcopy(agent)
                 if not agent.is_active:
                     status = agent_consts.AGENT_REVIVED
                     if 'resource_versions' not in agent_state:
@@ -418,7 +414,6 @@ class AgentDbMixin(ext_agent.AgentPluginBase, AgentAvailabilityZoneMixin):
                 event_type = events.AFTER_UPDATE
             except agent_exc.AgentNotFoundByTypeHost:
                 agent_state_orig = None
-                agent_state_previous = None
                 greenthread.sleep(0)
                 res['created_at'] = current_time
                 res['started_at'] = current_time
@@ -443,7 +438,7 @@ class AgentDbMixin(ext_agent.AgentPluginBase, AgentAvailabilityZoneMixin):
                                  'plugin': self,
                                  'status': status
                              },
-                             states=(agent_state_orig, agent_state_previous),
+                             states=(agent_state_orig, ),
                              desired_state=agent_state,
                              resource_id=agent.id
                          ))
