@@ -16,8 +16,8 @@
 import functools
 
 import netaddr
-from neutron_lib.api.definitions import external_net as extnet_def
 from neutron_lib.api.definitions import ip_allocation as ipalloc_apidef
+from neutron_lib.api.definitions import port as port_def
 from neutron_lib.api.definitions import portbindings as portbindings_def
 from neutron_lib.api.definitions import subnetpool as subnetpool_def
 from neutron_lib.api import validators
@@ -419,12 +419,10 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
         return network
 
     @db_api.retry_if_session_inactive()
-    def update_network(self, context, id, network, db_network=None):
+    def update_network(self, context, id, network):
         n = network['network']
-        # we dont't use DB objects not belonging to the current active session
-        db_network = db_network if context.session.is_active else None
         with db_api.CONTEXT_WRITER.using(context):
-            network = db_network or self._get_network(context, id)
+            network = self._get_network(context, id)
             # validate 'shared' parameter
             if 'shared' in n:
                 entry = None
@@ -487,7 +485,7 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
                     _constants.AUTO_DELETE_PORT_OWNERS))]
         for port_id in auto_delete_port_ids:
             try:
-                self.delete_port(context.elevated(), port_id)
+                self.delete_port(utils.get_elevated_context(context), port_id)
             except exc.PortNotFound:
                 # Don't raise if something else concurrently deleted the port
                 LOG.debug("Ignoring PortNotFound when deleting port '%s'. "
@@ -507,8 +505,7 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
                 network_db = self._get_network(context, id)
                 network = self._make_network_dict(network_db, context=context)
                 registry.notify(resources.NETWORK, events.PRECOMMIT_DELETE,
-                                self, context=context, network_id=id,
-                                network=network)
+                                self, context=context, network_id=id)
                 # We expire network_db here because precommit deletion
                 # might have left the relationship stale, for example,
                 # if we deleted a segment.
@@ -628,20 +625,17 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
             # NOTE(salv-orlando): There is slight chance of a race, when
             # a subnet-update and a router-interface-add operation are
             # executed concurrently
-            s_gateway_ip = netaddr.IPAddress(s['gateway_ip'])
-            if (cur_subnet and
-                    s_gateway_ip != cur_subnet['gateway_ip'] and
-                    not ipv6_utils.is_ipv6_pd_enabled(s)):
+            if cur_subnet and not ipv6_utils.is_ipv6_pd_enabled(s):
                 gateway_ip = str(cur_subnet['gateway_ip'])
                 with db_api.CONTEXT_READER.using(context):
-                    alloc = port_obj.IPAllocation.get_alloc_routerports(
+                    allocated = port_obj.IPAllocation.get_alloc_routerports(
                         context, cur_subnet['id'], gateway_ip=gateway_ip,
                         first=True)
 
-                if alloc and alloc.port_id:
+                if allocated and allocated.port_id:
                     raise exc.GatewayIpInUse(
                         ip_address=gateway_ip,
-                        port_id=alloc.port_id)
+                        port_id=allocated.port_id)
 
         if validators.is_attr_set(s.get('dns_nameservers')):
             if len(s['dns_nameservers']) > cfg.CONF.max_dns_nameservers:
@@ -716,7 +710,7 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
 
     def _update_router_gw_port(self, context, router_id, network, subnet):
         l3plugin = directory.get_plugin(plugin_constants.L3)
-        ctx_admin = context.elevated()
+        ctx_admin = utils.get_elevated_context(context)
         ext_subnets_dict = {s['id']: s for s in network['subnets']}
         router = l3plugin.get_router(ctx_admin, router_id)
         external_gateway_info = router['external_gateway_info']
@@ -740,12 +734,11 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
     @db_api.retry_if_session_inactive()
     def _create_subnet_postcommit(self, context, result,
                                   network=None, ipam_subnet=None):
-        # need to get db net obj with full subnet info either if no network
-        # is passed or if passed network is an external net dict from
-        # ml2 plugin (with only IDs in 'subnets')
-        if not network or network.get(extnet_def.EXTERNAL):
+        if not network:
             network = self._get_network(context,
                                         result['network_id'])
+        if not ipam_subnet:
+            ipam_subnet = self.ipam.get_subnet(context, result['id'])
 
         if hasattr(network, 'external') and network.external:
             self._update_router_gw_ports(context,
@@ -754,8 +747,6 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
         # If this subnet supports auto-addressing, then update any
         # internal ports on the network with addresses for this subnet.
         if ipv6_utils.is_auto_address_subnet(result):
-            if not ipam_subnet:
-                ipam_subnet = self.ipam.get_subnet(context, result['id'])
             updated_ports = self.ipam.add_auto_addrs_on_network_ports(
                 context, result, ipam_subnet)
             for port_id in updated_ports:
@@ -948,8 +939,8 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
                         self, **kwargs)
 
         with db_api.CONTEXT_WRITER.using(context):
-            subnet, changes = self.ipam.update_db_subnet(
-                context, id, s, db_pools, subnet_obj=subnet_obj)
+            subnet, changes = self.ipam.update_db_subnet(context, id, s,
+                                                         db_pools)
         return self._make_subnet_dict(subnet, context=context), orig
 
     @property
@@ -1060,8 +1051,7 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
         with db_api.exc_to_retry(sql_exc.IntegrityError), \
                 db_api.CONTEXT_WRITER.using(context):
             registry.notify(resources.SUBNET, events.PRECOMMIT_DELETE,
-                            self, context=context, subnet_id=subnet.id,
-                            subnet_obj=subnet)
+                            self, context=context, subnet_id=subnet.id)
             subnet.delete()
             # Delete related ipam subnet manually,
             # since there is no FK relationship
@@ -1468,13 +1458,11 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
                                         new_mac, current_owner)
 
     @db_api.retry_if_session_inactive()
-    def update_port(self, context, id, port, db_port=None):
-        # we dont't use DB objects not belonging to the current active session
-        db_port = db_port if context.session.is_active else None
+    def update_port(self, context, id, port):
         new_port = port['port']
 
         with db_api.CONTEXT_WRITER.using(context):
-            db_port = db_port or self._get_port(context, id)
+            db_port = self._get_port(context, id)
             new_mac = new_port.get('mac_address')
             self._validate_port_for_update(context, db_port, new_port, new_mac)
             # Note: _make_port_dict is called here to load extension data
@@ -1495,6 +1483,13 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
                     old_port_db=db_port,
                     old_port=self._make_port_dict(db_port),
                     new_port=new_port)
+            except ipam_exc.IpAddressAllocationNotFound as e:
+                # If a port update and a subnet delete interleave, there is a
+                # chance that the IPAM update operation raises this exception.
+                # Rather than throwing that up to the user under some sort of
+                # conflict, bubble up a retry instead that should bring things
+                # back to sanity.
+                raise os_db_exc.RetryRequest(e)
             except ipam_exc.IPAddressChangeNotAllowed as e:
                 raise exc.BadRequest(resource='ports', msg=e)
         return self._make_port_dict(db_port)
@@ -1568,8 +1563,7 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
                                       marker_obj=marker_obj,
                                       page_reverse=page_reverse)
         items = [self._make_port_dict(c, fields, bulk=True) for c in query]
-        # TODO(obondarev): use neutron_lib constant
-        resource_extend.apply_funcs('ports_bulk', items, None)
+        resource_extend.apply_funcs(port_def.COLLECTION_NAME_BULK, items, None)
         if limit and page_reverse:
             items.reverse()
         return items
@@ -1593,7 +1587,7 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
             if device_id:
                 if hasattr(self, 'get_router'):
                     try:
-                        ctx_admin = context.elevated()
+                        ctx_admin = utils.get_elevated_context(context)
                         router = self.get_router(ctx_admin, device_id)
                     except l3_exc.RouterNotFound:
                         return
@@ -1601,7 +1595,7 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
                     l3plugin = directory.get_plugin(plugin_constants.L3)
                     if l3plugin:
                         try:
-                            ctx_admin = context.elevated()
+                            ctx_admin = utils.get_elevated_context(context)
                             router = l3plugin.get_router(ctx_admin,
                                                          device_id)
                         except l3_exc.RouterNotFound:
