@@ -20,14 +20,16 @@ from neutron_lib.api.definitions import l3 as l3_apidef
 from neutron_lib.callbacks import events
 from neutron_lib.callbacks import registry
 from neutron_lib.callbacks import resources
+from neutron_lib.db import api as db_api
 from neutron_lib.db import resource_extend
 from neutron_lib.exceptions import extraroute as xroute_exc
 from neutron_lib.utils import helpers
+from neutron_lib.utils import net as net_utils
 from oslo_config import cfg
 from oslo_log import log as logging
 
 from neutron._i18n import _
-from neutron.common import utils
+from neutron.common import utils as common_utils
 from neutron.conf.db import extraroute_db
 from neutron.db import l3_db
 from neutron.objects import router as l3_obj
@@ -53,7 +55,7 @@ class ExtraRoute_dbonly_mixin(l3_db.L3_NAT_dbonly_mixin):
     def update_router(self, context, id, router):
         r = router['router']
         if 'routes' in r:
-            with context.session.begin(subtransactions=True):
+            with db_api.CONTEXT_WRITER.using(context):
                 # check if route exists and have permission to access
                 router_db = self._get_router(context, id)
                 old_router = self._make_router_dict(router_db)
@@ -67,9 +69,6 @@ class ExtraRoute_dbonly_mixin(l3_db.L3_NAT_dbonly_mixin):
                                      context, request_body=router_data,
                                      states=(old_router,), resource_id=id,
                                      desired_state=router_db))
-            # NOTE(yamamoto): expire to ensure the following update_router
-            # see the effects of the above _update_extra_routes.
-            context.session.expire(router_db, attribute_names=['route_list'])
         return super(ExtraRoute_dbonly_mixin, self).update_router(
             context, id, router)
 
@@ -94,7 +93,7 @@ class ExtraRoute_dbonly_mixin(l3_db.L3_NAT_dbonly_mixin):
                 router_id=router_id,
                 quota=cfg.CONF.max_routes)
 
-        context = context.elevated()
+        context = common_utils.get_elevated_context(context)
         filters = {'device_id': [router_id]}
         ports = self._core_plugin.get_ports(context, filters)
         cidrs = []
@@ -117,7 +116,7 @@ class ExtraRoute_dbonly_mixin(l3_db.L3_NAT_dbonly_mixin):
             l3_obj.RouterRoute(
                 context,
                 router_id=router['id'],
-                destination=utils.AuthenticIPNetwork(route['destination']),
+                destination=net_utils.AuthenticIPNetwork(route['destination']),
                 nexthop=netaddr.IPAddress(route['nexthop'])).create()
 
         LOG.debug('Removed routes are %s', removed)
@@ -141,17 +140,98 @@ class ExtraRoute_dbonly_mixin(l3_db.L3_NAT_dbonly_mixin):
         return self._make_extra_route_list(router_objs)
 
     def _confirm_router_interface_not_in_use(self, context, router_id,
-                                             subnet_id):
+                                             subnet):
         super(ExtraRoute_dbonly_mixin,
               self)._confirm_router_interface_not_in_use(
-            context, router_id, subnet_id)
-        subnet = self._core_plugin.get_subnet(context, subnet_id)
+            context, router_id, subnet)
         subnet_cidr = netaddr.IPNetwork(subnet['cidr'])
         extra_routes = self._get_extra_routes_by_router_id(context, router_id)
         for route in extra_routes:
             if netaddr.all_matching_cidrs(route['nexthop'], [subnet_cidr]):
                 raise xroute_exc.RouterInterfaceInUseByRoute(
-                    router_id=router_id, subnet_id=subnet_id)
+                    router_id=router_id, subnet_id=subnet['id'])
+
+    @staticmethod
+    def _add_extra_routes(old, add):
+        """Add two lists of extra routes.
+
+        Exact duplicates (both destination and nexthop) in old and add are
+        merged into one item.
+        Same destinations with different nexthops are accepted and all of
+        them are returned.
+        Overlapping destinations are accepted and all of them are returned.
+        """
+        routes_dict = {}  # its values are sets of nexthops
+        for r in old + add:
+            dst = r['destination']
+            nexthop = r['nexthop']
+            if dst not in routes_dict:
+                routes_dict[dst] = set()
+            routes_dict[dst].add(nexthop)
+        routes_list = []
+        for dst, nexthops in routes_dict.items():
+            for nexthop in nexthops:
+                routes_list.append({'destination': dst, 'nexthop': nexthop})
+        return routes_list
+
+    @staticmethod
+    def _remove_extra_routes(old, remove):
+        """Remove the 2nd list of extra routes from the first.
+
+        Since we care about the end state if an extra route to be removed
+        is already missing from old, that's not an error, but accepted.
+        """
+        routes_dict = {}  # its values are sets of nexthops
+        for r in old:
+            dst = r['destination']
+            nexthop = r['nexthop']
+            if dst not in routes_dict:
+                routes_dict[dst] = set()
+            routes_dict[dst].add(nexthop)
+        for r in remove:
+            dst = r['destination']
+            nexthop = r['nexthop']
+            if dst in routes_dict:
+                routes_dict[dst].discard(nexthop)
+        routes_list = []
+        for dst, nexthops in routes_dict.items():
+            for nexthop in nexthops:
+                routes_list.append({'destination': dst, 'nexthop': nexthop})
+        return routes_list
+
+    @db_api.retry_if_session_inactive()
+    def add_extraroutes(self, context, router_id, body=None):
+        # NOTE(bence romsics): The input validation is delayed until
+        # update_router() validates the whole set of routes. Until then
+        # do not trust 'routes'.
+        routes = body['router']['routes']
+        with db_api.CONTEXT_WRITER.using(context):
+            old_routes = self._get_extra_routes_by_router_id(
+                context, router_id)
+            router = self.update_router(
+                context,
+                router_id,
+                {'router':
+                    {'routes':
+                        self._add_extra_routes(old_routes, routes)}})
+            return {'router': router}
+
+    @db_api.retry_if_session_inactive()
+    def remove_extraroutes(self, context, router_id, body=None):
+        # NOTE(bence romsics): The input validation is delayed until
+        # update_router() validates the whole set of routes. Until then
+        # do not trust 'routes'.
+        routes = body['router']['routes']
+        with db_api.CONTEXT_WRITER.using(context):
+            old_routes = self._get_extra_routes_by_router_id(
+                context, router_id)
+            router = self.update_router(
+                context,
+                router_id,
+                {'router':
+                    {'routes':
+                        self._remove_extra_routes(old_routes, routes)}})
+            return {'router': router}
 
 
 class ExtraRoute_db_mixin(ExtraRoute_dbonly_mixin, l3_db.L3_NAT_db_mixin):
