@@ -179,24 +179,6 @@ class IpamPluggableBackend(ipam_backend_mixin.IpamBackendMixin):
         ipam_driver = driver.Pool.get_instance(None, context)
         return ipam_driver.get_subnet(subnet_id)
 
-    @db_api.retry_if_session_inactive()
-    @db_api.CONTEXT_WRITER
-    def deallocate_ips_from_port(self, context, port, ips):
-        """Deallocate set of ips from port.
-
-        Deallocate IP addresses previosly allocated for given port.
-        Format of the ips:
-            [{
-                "ip_address": IP.ADDRESS,
-                "subnet_id": subnet_id
-            }]
-        """
-        if not ips:
-            return
-        ipam_driver = driver.Pool.get_instance(None, context)
-        self._ipam_deallocate_ips(
-            context, ipam_driver, port['port'], ips)
-
     def allocate_ips_for_port_and_store(self, context, port, port_id):
         # Make a copy of port dict to prevent changing
         # incoming dict by adding 'id' to it.
@@ -216,34 +198,28 @@ class IpamPluggableBackend(ipam_backend_mixin.IpamBackendMixin):
 
         port_copy['port']['id'] = port_id
         network_id = port_copy['port']['network_id']
-        ips = self.allocate_ips_for_port(context, port_copy)
-        self.store_ip_allocation_for_port(context, ips, network_id, port_copy)
-        return ips
-
-    @db_api.retry_if_session_inactive()
-    @db_api.CONTEXT_WRITER
-    def allocate_ips_for_port(self, context, port):
-        return self._allocate_ips_for_port(context, port)
-
-    def store_ip_allocation_for_port(self, context, ips, network_id, port):
+        ips = []
         try:
+            ips = self._allocate_ips_for_port(context, port_copy)
             for ip in ips:
                 ip_address = ip['ip_address']
                 subnet_id = ip['subnet_id']
                 IpamPluggableBackend._store_ip_allocation(
                     context, ip_address, network_id,
-                    subnet_id, port['port']['id'])
+                    subnet_id, port_id)
+            return ips
         except Exception:
             with excutils.save_and_reraise_exception():
-                ipam_driver = driver.Pool.get_instance(None, context)
-                if not ipam_driver.needs_rollback():
-                    return
+                if ips:
+                    ipam_driver = driver.Pool.get_instance(None, context)
+                    if not ipam_driver.needs_rollback():
+                        return
 
-                LOG.debug("An exception occurred during port creation. "
-                          "Reverting IP allocation")
-                self._safe_rollback(self._ipam_deallocate_ips, context,
-                                    ipam_driver, port['port'], ips,
-                                    revert_on_fail=False)
+                    LOG.debug("An exception occurred during port creation. "
+                              "Reverting IP allocation")
+                    self._safe_rollback(self._ipam_deallocate_ips, context,
+                                        ipam_driver, port_copy['port'], ips,
+                                        revert_on_fail=False)
 
     def _allocate_ips_for_port(self, context, port):
         """Allocate IP addresses for the port. IPAM version.
@@ -519,8 +495,8 @@ class IpamPluggableBackend(ipam_backend_mixin.IpamBackendMixin):
         self._ipam_deallocate_ips(context, ipam_driver, port,
                                   port['fixed_ips'])
 
-    def update_db_subnet(self, context, id, s, old_pools, subnet_obj=None):
-        subnet = subnet_obj or obj_subnet.Subnet.get_object(context, id=id)
+    def update_db_subnet(self, context, id, s, old_pools):
+        subnet = obj_subnet.Subnet.get_object(context, id=id)
         old_segment_id = subnet.segment_id if subnet else None
         if 'segment_id' in s:
             self._validate_segment(
@@ -531,7 +507,7 @@ class IpamPluggableBackend(ipam_backend_mixin.IpamBackendMixin):
         # so create unchanged copy for ipam driver
         subnet_copy = copy.deepcopy(s)
         subnet, changes = super(IpamPluggableBackend, self).update_db_subnet(
-            context, id, s, old_pools, subnet_obj=subnet_obj)
+            context, id, s, old_pools)
         ipam_driver = driver.Pool.get_instance(None, context)
 
         # Set old allocation pools if no new pools are provided by user.
@@ -546,41 +522,40 @@ class IpamPluggableBackend(ipam_backend_mixin.IpamBackendMixin):
 
     def add_auto_addrs_on_network_ports(self, context, subnet, ipam_subnet):
         """For an auto-address subnet, add addrs for ports on the net."""
-        # TODO(ataraday): switched for writer when flush_on_subtransaction
-        # will be available for neutron
-        with context.session.begin(subtransactions=True):
-            network_id = subnet['network_id']
+        network_id = subnet['network_id']
+        with db_api.CONTEXT_READER.using(context):
             ports = port_obj.Port.get_objects(
                 context, network_id=network_id,
                 device_owner=obj_utils.NotIn(
                     constants.ROUTER_INTERFACE_OWNERS_SNAT))
-            updated_ports = []
-            ipam_driver = driver.Pool.get_instance(None, context)
-            factory = ipam_driver.get_address_request_factory()
-            for port in ports:
-                # Find candidate subnets based on host_id and existing
-                # fixed_ips. This will filter subnets on other segments. Only
-                # allocate if this subnet is a valid candidate.
-                p = self._make_port_dict(port)
-                fixed_configured = (p['fixed_ips'] is not
-                                    constants.ATTR_NOT_SPECIFIED)
+        updated_ports = []
+        ipam_driver = driver.Pool.get_instance(None, context)
+        factory = ipam_driver.get_address_request_factory()
+        for port in ports:
+            # Find candidate subnets based on host_id and existing
+            # fixed_ips. This will filter subnets on other segments. Only
+            # allocate if this subnet is a valid candidate.
+            p = self._make_port_dict(port)
+            fixed_configured = (p['fixed_ips'] is not
+                                constants.ATTR_NOT_SPECIFIED)
+            with db_api.CONTEXT_READER.using(context):
                 subnet_candidates = obj_subnet.Subnet.find_candidate_subnets(
                     context,
                     network_id,
                     p.get(portbindings.HOST_ID),
                     p.get('device_owner'),
                     fixed_configured,
-                    p.get('fixed_ips'),
-                    distributed_service=self._is_distributed_service(p))
+                    p.get('fixed_ips'))
                 if subnet['id'] not in [s['id'] for s in subnet_candidates]:
                     continue
 
-                ip = {'subnet_id': subnet['id'],
-                      'subnet_cidr': subnet['cidr'],
-                      'eui64_address': True,
-                      'mac': port.mac_address}
-                ip_request = factory.get_request(context, port, ip)
-                try:
+            ip = {'subnet_id': subnet['id'],
+                  'subnet_cidr': subnet['cidr'],
+                  'eui64_address': True,
+                  'mac': port.mac_address}
+            ip_request = factory.get_request(context, port, ip)
+            try:
+                with db_api.CONTEXT_WRITER.using(context):
                     ip_address = ipam_subnet.allocate(ip_request)
                     allocated = port_obj.IPAllocation(
                         context, network_id=network_id, port_id=port.id,
@@ -593,21 +568,22 @@ class IpamPluggableBackend(ipam_backend_mixin.IpamBackendMixin):
                     # it explicitly here.
                     allocated.create()
                     updated_ports.append(port.id)
-                except db_exc.DBReferenceError:
-                    LOG.debug("Port %s was deleted while updating it with an "
-                              "IPv6 auto-address. Ignoring.", port.id)
-                    LOG.debug("Reverting IP allocation for %s", ip_address)
-                    # Do not fail if reverting allocation was unsuccessful
+            except db_exc.DBReferenceError:
+                LOG.debug("Port %s was deleted while updating it with an "
+                          "IPv6 auto-address. Ignoring.", port.id)
+                LOG.debug("Reverting IP allocation for %s", ip_address)
+                # Do not fail if reverting allocation was unsuccessful
+                with db_api.CONTEXT_WRITER.using(context):
                     try:
                         ipam_subnet.deallocate(ip_address)
                     except Exception:
                         LOG.debug("Reverting IP allocation failed for %s",
                                   ip_address)
-                except ipam_exc.IpAddressAlreadyAllocated:
-                    LOG.debug("Port %s got IPv6 auto-address in a concurrent "
-                              "create or update port request. Ignoring.",
-                              port.id)
-            return updated_ports
+            except ipam_exc.IpAddressAlreadyAllocated:
+                LOG.debug("Port %s got IPv6 auto-address in a concurrent "
+                          "create or update port request. Ignoring.",
+                          port.id)
+        return updated_ports
 
     def allocate_subnet(self, context, network, subnet, subnetpool_id):
         subnetpool = None
