@@ -55,6 +55,7 @@ from neutron.objects import base as base_obj
 from neutron.objects import port_forwarding
 from neutron.objects import ports as port_obj
 from neutron.objects import router as l3_obj
+from neutron.objects import subnet as subnet_obj
 from neutron import worker as neutron_worker
 
 LOG = logging.getLogger(__name__)
@@ -192,6 +193,7 @@ class L3_NAT_dbonly_mixin(l3.RouterPluginBase,
                    'device_owner': [DEVICE_OWNER_FLOATINGIP]}
         return {p['id'] for p in self._core_plugin.get_ports(context, filters)}
 
+    @db_api.CONTEXT_READER
     def _get_router(self, context, router_id):
         try:
             router = model_query.get_by_id(
@@ -227,7 +229,7 @@ class L3_NAT_dbonly_mixin(l3.RouterPluginBase,
         router['tenant_id'] = tenant_id
         registry.notify(resources.ROUTER, events.BEFORE_CREATE,
                         self, context=context, router=router)
-        with context.session.begin(subtransactions=True):
+        with db_api.CONTEXT_WRITER.using(context):
             # pre-generate id so it will be available when
             # configuring external gw port
             router_db = l3_models.Router(
@@ -245,10 +247,15 @@ class L3_NAT_dbonly_mixin(l3.RouterPluginBase,
 
     def _update_gw_for_create_router(self, context, gw_info, router_id):
         if gw_info:
-            router_db = self._get_router(context, router_id)
+            with db_utils.context_if_transaction(
+                    context, not context.session.is_active, writer=False):
+                router_db = self._get_router(context, router_id)
             self._update_router_gw_info(context, router_id,
                                         gw_info, router=router_db)
 
+            return self._get_router(context, router_id), None
+        return None, None
+
     @db_api.retry_if_session_inactive()
     def create_router(self, context, router):
         r = router['router']
@@ -288,9 +295,6 @@ class L3_NAT_dbonly_mixin(l3.RouterPluginBase,
         gw_info = r.pop(EXTERNAL_GW_INFO, constants.ATTR_NOT_SPECIFIED)
         original = self.get_router(context, id)
         if gw_info != constants.ATTR_NOT_SPECIFIED:
-            # Update the gateway outside of the DB update since it involves L2
-            # calls that don't make sense to rollback and may cause deadlocks
-            # in a transaction.
             self._update_router_gw_info(context, id, gw_info)
         router_db = self._update_router_db(context, id, r)
         updated = self._make_router_dict(router_db)
@@ -308,24 +312,33 @@ class L3_NAT_dbonly_mixin(l3.RouterPluginBase,
                      'device_owner': DEVICE_OWNER_ROUTER_GW,
                      'admin_state_up': True,
                      'name': ''}
+
+        if context.session.is_active:
+            # TODO(ralonsoh): ML2 plugin "create_port" should be called outside
+            # a DB transaction. In this case an exception is made but in order
+            # to prevent future errors, this call should be moved outside
+            # the current transaction.
+            context.GUARD_TRANSACTION = False
         gw_port = plugin_utils.create_port(
-            self._core_plugin, context.elevated(), {'port': port_data})
+            self._core_plugin, utils.get_elevated_context(context),
+            {'port': port_data})
 
         if not gw_port['fixed_ips']:
             LOG.debug('No IPs available for external network %s',
                       network_id)
         with plugin_utils.delete_port_on_error(
-                self._core_plugin, context.elevated(), gw_port['id']):
-            with context.session.begin(subtransactions=True):
+                self._core_plugin, utils.get_elevated_context(context),
+                gw_port['id']):
+            with db_api.CONTEXT_WRITER.using(context):
+                router = self._get_router(context, router['id'])
                 router.gw_port = self._core_plugin._get_port(
-                    context.elevated(), gw_port['id'])
+                    utils.get_elevated_context(context), gw_port['id'])
                 router_port = l3_obj.RouterPort(
                     context,
                     router_id=router.id,
                     port_id=gw_port['id'],
                     port_type=DEVICE_OWNER_ROUTER_GW
                 )
-                context.session.add(router)
                 router_port.create()
 
     def _validate_gw_info(self, context, gw_port, info, ext_ips):
@@ -362,7 +375,7 @@ class L3_NAT_dbonly_mixin(l3.RouterPluginBase,
             router.gw_port and router.gw_port['network_id'] != new_network_id)
         if not port_requires_deletion:
             return
-        admin_ctx = context.elevated()
+        admin_ctx = utils.get_elevated_context(context)
         old_network_id = router.gw_port['network_id']
 
         if self.router_gw_port_has_floating_ips(admin_ctx, router_id):
@@ -371,10 +384,14 @@ class L3_NAT_dbonly_mixin(l3.RouterPluginBase,
         gw_ips = [x['ip_address'] for x in router.gw_port['fixed_ips']]
         gw_port_id = router.gw_port['id']
         self._delete_router_gw_port_db(context, router)
+        if admin_ctx.session.is_active:
+            # TODO(ralonsoh): ML2 plugin "delete_port" should be called outside
+            # a DB transaction. In this case an exception is made but in order
+            # to prevent future errors, this call should be moved outside
+            # the current transaction.
+            admin_ctx.GUARD_TRANSACTION = False
         self._core_plugin.delete_port(
             admin_ctx, gw_port_id, l3_port_check=False)
-        with context.session.begin(subtransactions=True):
-            context.session.refresh(router)
         # TODO(boden): normalize metadata
         metadata = {'network_id': old_network_id,
                     'new_network_id': new_network_id,
@@ -387,7 +404,7 @@ class L3_NAT_dbonly_mixin(l3.RouterPluginBase,
                              resource_id=router_id))
 
     def _delete_router_gw_port_db(self, context, router):
-        with context.session.begin(subtransactions=True):
+        with db_api.CONTEXT_WRITER.using(context):
             router.gw_port = None
             if router not in context.session:
                 context.session.add(router)
@@ -405,10 +422,12 @@ class L3_NAT_dbonly_mixin(l3.RouterPluginBase,
 
     def _create_gw_port(self, context, router_id, router, new_network_id,
                         ext_ips):
-        new_valid_gw_port_attachment = (
-            new_network_id and
-            (not router.gw_port or
-             router.gw_port['network_id'] != new_network_id))
+        with db_api.CONTEXT_READER.using(context):
+            router = self._get_router(context, router_id)
+            new_valid_gw_port_attachment = (
+                new_network_id and
+                (not router.gw_port or
+                router.gw_port['network_id'] != new_network_id))
         if new_valid_gw_port_attachment:
             subnets = self._core_plugin.get_subnets_by_network(context,
                                                                new_network_id)
@@ -427,12 +446,13 @@ class L3_NAT_dbonly_mixin(l3.RouterPluginBase,
 
             self._check_for_dup_router_subnets(context, router,
                                                new_network_id,
-                                               subnets,
-                                               include_gateway=True)
+                                               subnets)
             self._create_router_gw_port(context, router,
                                         new_network_id, ext_ips)
 
-            gw_ips = [x['ip_address'] for x in router.gw_port['fixed_ips']]
+            with db_api.CONTEXT_READER.using(context):
+                router = self._get_router(context, router_id)
+                gw_ips = [x['ip_address'] for x in router.gw_port['fixed_ips']]
 
             registry.publish(resources.ROUTER_GATEWAY,
                              events.AFTER_CREATE,
@@ -444,14 +464,11 @@ class L3_NAT_dbonly_mixin(l3.RouterPluginBase,
                                  resource_id=router_id))
 
     def _update_current_gw_port(self, context, router_id, router, ext_ips):
-        self._core_plugin.update_port(context.elevated(), router.gw_port['id'],
-                                      {'port': {'fixed_ips': ext_ips}})
-        context.session.expire(router.gw_port)
+        self._core_plugin.update_port(
+            utils.get_elevated_context(context), router.gw_port['id'],
+            {'port': {'fixed_ips': ext_ips}})
 
     def _update_router_gw_info(self, context, router_id, info, router=None):
-        # TODO(salvatore-orlando): guarantee atomic behavior also across
-        # operations that span beyond the model classes handled by this
-        # class (e.g.: delete_port)
         router = router or self._get_router(context, router_id)
         gw_port = router.gw_port
         ext_ips = info.get('external_fixed_ips') if info else []
@@ -503,28 +520,35 @@ class L3_NAT_dbonly_mixin(l3.RouterPluginBase,
         registry.publish(resources.ROUTER, events.BEFORE_DELETE, self,
                          payload=events.DBEventPayload(
                              context, resource_id=id))
-        # TODO(nati) Refactor here when we have router insertion model
-        router = self._ensure_router_not_in_use(context, id)
-        original = self._make_router_dict(router)
-        self._delete_current_gw_port(context, id, router, None)
-        with context.session.begin(subtransactions=True):
-            context.session.refresh(router)
 
-        router_ports = router.attached_ports
-        for rp in router_ports:
-            self._core_plugin.delete_port(context.elevated(),
-                                          rp.port.id,
-                                          l3_port_check=False)
-        with context.session.begin(subtransactions=True):
-            context.session.refresh(router)
+        # TODO(nati) Refactor here when we have router insertion model
+        with db_api.CONTEXT_READER.using(context):
+            router = self._ensure_router_not_in_use(context, id)
+            original = self._make_router_dict(router)
+        self._delete_current_gw_port(context, id, router, None)
+
+        with db_api.CONTEXT_WRITER.using(context):
+            # TODO(ralonsoh): move this section (port deletion) out of the DB
+            # transaction.
+            router_ports_ids = (rp.port_id for rp in
+                l3_obj.RouterPort.get_objects(context, router_id=id))
+            if context.session.is_active:
+                context.GUARD_TRANSACTION = False
+            for rp_id in router_ports_ids:
+                self._core_plugin.delete_port(
+                    utils.get_elevated_context(context), rp_id,
+                    l3_port_check=False)
+
+            router = self._get_router(context, id)
             registry.notify(resources.ROUTER, events.PRECOMMIT_DELETE,
                             self, context=context, router_db=router,
                             router_id=id)
             # we bump the revision even though we are about to delete to throw
-            # staledataerror if something snuck in with a new interface
+            # staledataerror if something stuck in with a new interface
             router.bump_revision()
             context.session.flush()
             context.session.delete(router)
+
         registry.notify(resources.ROUTER, events.AFTER_DELETE, self,
                         context=context, router_id=id, original=original)
 
@@ -553,8 +577,7 @@ class L3_NAT_dbonly_mixin(l3.RouterPluginBase,
                                                 filters=filters)
 
     def _check_for_dup_router_subnets(self, context, router,
-                                      network_id, new_subnets,
-                                      include_gateway=False):
+                                      network_id, new_subnets):
         # It's possible these ports are on the same network, but
         # different subnets.
         new_subnet_ids = {s['id'] for s in new_subnets}
@@ -565,16 +588,21 @@ class L3_NAT_dbonly_mixin(l3.RouterPluginBase,
                     msg = (_("Router already has a port on subnet %s")
                            % ip['subnet_id'])
                     raise n_exc.BadRequest(resource='router', msg=msg)
-                gw_owner = (p.get('device_owner') == DEVICE_OWNER_ROUTER_GW)
-                if include_gateway == gw_owner:
+                if p.get('device_owner') == DEVICE_OWNER_ROUTER_GW:
+                    ext_subts = self._core_plugin.get_subnets(
+                        utils.get_elevated_context(context),
+                        filters={'network_id': [p['network_id']]})
+                    for sub in ext_subts:
+                        router_subnets.append(sub['id'])
+                else:
                     router_subnets.append(ip['subnet_id'])
 
         # Ignore temporary Prefix Delegation CIDRs
         new_subnets = [s for s in new_subnets
                        if s['cidr'] != constants.PROVISIONAL_IPV6_PD_PREFIX]
         id_filter = {'id': router_subnets}
-        subnets = self._core_plugin.get_subnets(context.elevated(),
-                                                filters=id_filter)
+        subnets = self._core_plugin.get_subnets(
+            utils.get_elevated_context(context), filters=id_filter)
         for sub in subnets:
             cidr = sub['cidr']
             ipnet = netaddr.IPNetwork(cidr)
@@ -667,7 +695,7 @@ class L3_NAT_dbonly_mixin(l3.RouterPluginBase,
                 raise n_exc.BadRequest(resource='router', msg=msg)
 
     def _validate_router_port_info(self, context, router, port_id):
-        with db_api.autonested_transaction(context.session):
+        with db_api.CONTEXT_READER.using(context):
             # check again within transaction to mitigate race
             port = self._check_router_port(context, port_id, router.id)
 
@@ -873,8 +901,6 @@ class L3_NAT_dbonly_mixin(l3.RouterPluginBase,
                         new_interface=new_router_intf,
                         interface_info=interface_info)
 
-        with context.session.begin(subtransactions=True):
-            context.session.refresh(router)
         return self._make_router_interface_info(
             router.id, port['tenant_id'], port['id'], port['network_id'],
             subnets[-1]['id'], [subnet['id'] for subnet in subnets])
@@ -894,13 +920,16 @@ class L3_NAT_dbonly_mixin(l3.RouterPluginBase,
             context, port_id, {'port': {'device_id': router_id,
                                         'device_owner': device_owner}})
 
-    def _check_router_interface_not_in_use(self, router_id, subnet_id):
+    def _check_router_interface_not_in_use(self, router_id, subnet):
         context = n_ctx.get_admin_context()
-        subnet = self._core_plugin.get_subnet(context, subnet_id)
         subnet_cidr = netaddr.IPNetwork(subnet['cidr'])
 
         fip_objs = l3_obj.FloatingIP.get_objects(context, router_id=router_id)
         pf_plugin = directory.get_plugin(plugin_constants.PORTFORWARDING)
+        subnet_port_ids = [
+            port.id for port in
+            port_obj.Port.get_ports_allocated_by_subnet_id(
+                context, subnet_id=subnet['id'])]
         if pf_plugin:
             fip_ids = [fip_obj.id for fip_obj in fip_objs]
             pf_objs = port_forwarding.PortForwarding.get_objects(
@@ -908,23 +937,25 @@ class L3_NAT_dbonly_mixin(l3.RouterPluginBase,
             for pf_obj in pf_objs:
                 if (pf_obj.internal_ip_address and
                         pf_obj.internal_ip_address in subnet_cidr):
-                    raise l3_exc.RouterInterfaceInUseByFloatingIP(
-                        router_id=router_id, subnet_id=subnet_id)
+                    if pf_obj.internal_port_id in subnet_port_ids:
+                        raise l3_exc.RouterInterfaceInUseByFloatingIP(
+                            router_id=router_id, subnet_id=subnet['id'])
 
         for fip_obj in fip_objs:
             if (fip_obj.fixed_ip_address and
                     fip_obj.fixed_ip_address in subnet_cidr):
-                raise l3_exc.RouterInterfaceInUseByFloatingIP(
-                    router_id=router_id, subnet_id=subnet_id)
+                if fip_obj.fixed_port_id in subnet_port_ids:
+                    raise l3_exc.RouterInterfaceInUseByFloatingIP(
+                        router_id=router_id, subnet_id=subnet['id'])
 
     def _confirm_router_interface_not_in_use(self, context, router_id,
-                                             subnet_id):
+                                             subnet):
         try:
             registry.publish(
                 resources.ROUTER_INTERFACE,
                 events.BEFORE_DELETE, self,
                 payload=events.DBEventPayload(
-                    context, metadata={'subnet_id': subnet_id},
+                    context, metadata={'subnet_id': subnet['id']},
                     resource_id=router_id))
         except exceptions.CallbackFailure as e:
             # NOTE(armax): preserve old check's behavior
@@ -932,7 +963,7 @@ class L3_NAT_dbonly_mixin(l3.RouterPluginBase,
                 raise e.errors[0].error
             raise l3_exc.RouterInUse(router_id=router_id, reason=e)
 
-        self._check_router_interface_not_in_use(router_id, subnet_id)
+        self._check_router_interface_not_in_use(router_id, subnet)
 
     def _remove_interface_by_port(self, context, router_id,
                                   port_id, subnet_id, owner):
@@ -948,22 +979,22 @@ class L3_NAT_dbonly_mixin(l3.RouterPluginBase,
         if subnet_id and subnet_id not in port_subnet_ids:
             raise n_exc.SubnetMismatchForPort(
                 port_id=port_id, subnet_id=subnet_id)
-        subnets = [self._core_plugin.get_subnet(context, port_subnet_id)
-                   for port_subnet_id in port_subnet_ids]
-        for port_subnet_id in port_subnet_ids:
+        subnets = subnet_obj.Subnet.get_objects(context, id=port_subnet_ids)
+        for subnet in subnets:
             self._confirm_router_interface_not_in_use(
-                    context, router_id, port_subnet_id)
+                    context, router_id, subnet)
         self._core_plugin.delete_port(context, port['id'],
                                       l3_port_check=False)
         return port, subnets
 
     def _remove_interface_by_subnet(self, context,
                                     router_id, subnet_id, owner):
-        self._confirm_router_interface_not_in_use(
-            context, router_id, subnet_id)
         subnet = self._core_plugin.get_subnet(context, subnet_id)
         ports = port_obj.Port.get_ports_by_router_and_network(
             context, router_id, owner, subnet['network_id'])
+        if ports:
+            self._confirm_router_interface_not_in_use(
+                context, router_id, subnet)
 
         for p in ports:
             try:
@@ -1018,8 +1049,6 @@ class L3_NAT_dbonly_mixin(l3.RouterPluginBase,
                         port=port,
                         router_id=router_id,
                         interface_info=interface_info)
-        with context.session.begin(subtransactions=True):
-            context.session.refresh(router)
         return self._make_router_interface_info(router_id, port['tenant_id'],
                                                 port['id'], port['network_id'],
                                                 subnets[0]['id'],
@@ -1045,7 +1074,9 @@ class L3_NAT_dbonly_mixin(l3.RouterPluginBase,
                'router_id': floatingip.router_id,
                'port_id': floatingip.fixed_port_id,
                'fixed_ip_address': fixed_ip_address,
-               'status': floatingip.status}
+               'status': floatingip.status,
+               'standard_attr_id': floatingip.db_obj.standard_attr.id,
+               }
         # NOTE(mlavalle): The following assumes this mixin is used in a
         # class inheriting from CommonDbMixin, which is true for all existing
         # plugins.
@@ -1313,13 +1344,13 @@ class L3_NAT_dbonly_mixin(l3.RouterPluginBase,
         # 'status' in port dict could not be updated by default, use
         # check_allow_post to stop the verification of system
         external_port = plugin_utils.create_port(
-            self._core_plugin, context.elevated(),
+            self._core_plugin, utils.get_elevated_context(context),
             {'port': port}, check_allow_post=False)
 
         with plugin_utils.delete_port_on_error(
-                self._core_plugin, context.elevated(),
+                self._core_plugin, utils.get_elevated_context(context),
                 external_port['id']),\
-                context.session.begin(subtransactions=True):
+                db_api.CONTEXT_WRITER.using(context):
             # Ensure IPv4 addresses are allocated on external port
             external_ipv4_ips = self._port_ipv4_fixed_ips(external_port)
             if not external_ipv4_ips:
@@ -1347,17 +1378,14 @@ class L3_NAT_dbonly_mixin(l3.RouterPluginBase,
                     context, floatingip_dict, fip)
             if self._is_fip_qos_supported:
                 self._process_extra_fip_qos_create(context, fip_id, fip)
-            floatingip_obj = l3_obj.FloatingIP.get_object(
-                context, id=floatingip_obj.id)
-            floatingip_db = floatingip_obj.db_obj
 
             registry.notify(resources.FLOATING_IP, events.PRECOMMIT_CREATE,
                             self, context=context, floatingip=fip,
                             floatingip_id=fip_id,
-                            floatingip_db=floatingip_db)
+                            floatingip_db=floatingip_obj.db_obj)
 
         self._core_plugin.update_port(
-            context.elevated(), external_port['id'],
+            utils.get_elevated_context(context), external_port['id'],
             {'port': {'device_id': fip_id,
                       'project_id': fip['tenant_id']}})
         registry.notify(resources.FLOATING_IP,
@@ -1378,7 +1406,7 @@ class L3_NAT_dbonly_mixin(l3.RouterPluginBase,
         # TODO(lujinluo): Change floatingip_db to floatingip_obj once all
         # codes are migrated to use Floating IP OVO object.
         resource_extend.apply_funcs(l3_apidef.FLOATINGIPS, floatingip_dict,
-                                    floatingip_db)
+                                    floatingip_obj.db_obj)
         return floatingip_dict
 
     @db_api.retry_if_session_inactive()
@@ -1397,7 +1425,7 @@ class L3_NAT_dbonly_mixin(l3.RouterPluginBase,
             raise e.errors[0].error
 
         fip = floatingip['floatingip']
-        with context.session.begin(subtransactions=True):
+        with db_api.CONTEXT_WRITER.using(context):
             floatingip_obj = self._get_floatingip(context, id)
             old_floatingip = self._make_floatingip_dict(floatingip_obj)
             old_fixed_port_id = floatingip_obj.fixed_port_id
@@ -1481,11 +1509,17 @@ class L3_NAT_dbonly_mixin(l3.RouterPluginBase,
         # floating IP record once the port is deleted. We can't start
         # a transaction first to remove it ourselves because the delete_port
         # method will yield in its post-commit activities.
-        self._core_plugin.delete_port(context.elevated(),
+        self._core_plugin.delete_port(utils.get_elevated_context(context),
                                       floatingip.floating_port_id,
                                       l3_port_check=False)
         registry.notify(resources.FLOATING_IP, events.AFTER_DELETE,
                         self, context=context, **floatingip_dict)
+        if floatingip.fixed_port_id:
+            LOG.info(FIP_ASSOC_MSG,
+                     {'fip_id': floatingip.id,
+                      'ext_ip': str(floatingip.floating_ip_address),
+                      'port_id': floatingip.fixed_port_id,
+                      'assoc': 'disassociated (deleted)'})
         return floatingip_dict
 
     @db_api.retry_if_session_inactive()
@@ -1565,8 +1599,8 @@ class L3_NAT_dbonly_mixin(l3.RouterPluginBase,
                           "%(port_id)s no longer exists, allowing deletion.",
                           {'f_id': port['device_id'], 'port_id': port['id']})
                 return
-        elif not l3_obj.Router.objects_exist(context.elevated(),
-                                             id=port['device_id']):
+        elif not l3_obj.Router.objects_exist(
+                utils.get_elevated_context(context), id=port['device_id']):
             LOG.debug("Router %(router_id)s corresponding to port "
                       "%(port_id)s no longer exists, allowing deletion.",
                       {'router_id': port['device_id'],
@@ -1586,7 +1620,7 @@ class L3_NAT_dbonly_mixin(l3.RouterPluginBase,
                           This parameter is ignored.
         @return: set of router-ids that require notification updates
         """
-        with context.session.begin(subtransactions=True):
+        with db_api.CONTEXT_WRITER.using(context):
             floating_ip_objs = l3_obj.FloatingIP.get_objects(
                 context, fixed_port_id=port_id)
             router_ids = {fip.router_id for fip in floating_ip_objs}
@@ -1826,7 +1860,7 @@ class L3_NAT_dbonly_mixin(l3.RouterPluginBase,
     def _get_router_info_list(self, context, router_ids=None, active=None,
                               device_owners=None):
         """Query routers and their related floating_ips, interfaces."""
-        with context.session.begin(subtransactions=True):
+        with db_api.CONTEXT_WRITER.using(context):
             routers = self._get_sync_routers(context,
                                              router_ids=router_ids,
                                              active=active)
