@@ -13,9 +13,12 @@
 #    under the License.
 import collections
 
+import netaddr
+from neutron_lib.api.definitions import external_net as extnet_apidef
 from neutron_lib.api.definitions import l3 as l3_apidef
 from neutron_lib.api.definitions import portbindings
 from neutron_lib.api.definitions import portbindings_extended
+from neutron_lib.api.definitions import router_admin_state_down_before_update
 from neutron_lib.api import validators
 from neutron_lib.callbacks import events
 from neutron_lib.callbacks import exceptions
@@ -45,9 +48,9 @@ from neutron.db import l3_attrs_db
 from neutron.db import l3_db
 from neutron.db.models import allowed_address_pair as aap_models
 from neutron.db import models_v2
-from neutron.extensions import _admin_state_down_before_update_lib
 from neutron.ipam import utils as ipam_utils
 from neutron.objects import agent as ag_obj
+from neutron.objects import base as base_obj
 from neutron.objects import l3agent as rb_obj
 from neutron.objects import router as l3_obj
 
@@ -61,7 +64,7 @@ def is_admin_state_down_necessary():
     global _IS_ADMIN_STATE_DOWN_NECESSARY
     if _IS_ADMIN_STATE_DOWN_NECESSARY is None:
         _IS_ADMIN_STATE_DOWN_NECESSARY = \
-            _admin_state_down_before_update_lib.ALIAS in (extensions.
+            router_admin_state_down_before_update.ALIAS in (extensions.
                     PluginAwareExtensionManager.get_instance().extensions)
     return _IS_ADMIN_STATE_DOWN_NECESSARY
 
@@ -208,7 +211,7 @@ class DVRResourceOperationHandler(object):
         if (old_router and old_router['distributed'] and not
                 router['distributed']):
             self._core_plugin.delete_distributed_port_bindings_by_router_id(
-                context.elevated(), router_db['id'])
+                n_utils.get_elevated_context(context), router_db['id'])
 
     @registry.receives(resources.ROUTER, [events.AFTER_UPDATE],
                        priority_group.PRIORITY_ROUTER_EXTENDED_ATTRIBUTE)
@@ -221,7 +224,7 @@ class DVRResourceOperationHandler(object):
             old_router = kwargs['old_router']
             if old_router and old_router['distributed']:
                 self.delete_csnat_router_interface_ports(
-                    context.elevated(), router_db)
+                    n_utils.get_elevated_context(context), router_db)
 
     @registry.receives(resources.ROUTER,
                        [events.AFTER_CREATE, events.AFTER_UPDATE],
@@ -243,8 +246,8 @@ class DVRResourceOperationHandler(object):
                          not old_router.get(l3_apidef.EXTERNAL_GW_INFO))
             if not do_create:
                 return
-        if not self._create_snat_intf_ports_if_not_exists(context.elevated(),
-                                                          router_db):
+        if not self._create_snat_intf_ports_if_not_exists(
+                n_utils.get_elevated_context(context), router_db):
             LOG.debug("SNAT interface ports not created: %s",
                       router_db['id'])
         return router_db
@@ -277,9 +280,10 @@ class DVRResourceOperationHandler(object):
             msg = _("Unable to create the SNAT Interface Port")
             raise n_exc.BadRequest(resource='router', msg=msg)
 
-        with plugin_utils.delete_port_on_error(self.l3plugin._core_plugin,
-                                               context.elevated(),
-                                               snat_port['id']):
+        with plugin_utils.delete_port_on_error(
+                self.l3plugin._core_plugin,
+                n_utils.get_elevated_context(context),
+                snat_port['id']):
             l3_obj.RouterPort(
                 context,
                 port_id=snat_port['id'],
@@ -354,7 +358,8 @@ class DVRResourceOperationHandler(object):
             return
         if not payload.metadata.get('new_network_id'):
             self.delete_csnat_router_interface_ports(
-                payload.context.elevated(), payload.latest_state)
+                n_utils.get_elevated_context(payload.context),
+                payload.latest_state)
 
         network_id = payload.metadata.get('network_id')
         # NOTE(Swami): Delete the Floatingip agent gateway port
@@ -363,10 +368,11 @@ class DVRResourceOperationHandler(object):
         filters = {'network_id': [network_id],
                    'device_owner': [const.DEVICE_OWNER_ROUTER_GW]}
         ext_net_gw_ports = self._core_plugin.get_ports(
-            payload.context.elevated(), filters)
+            n_utils.get_elevated_context(payload.context), filters)
         if not ext_net_gw_ports:
             self.delete_floatingip_agent_gateway_port(
-                payload.context.elevated(), None, network_id)
+                n_utils.get_elevated_context(payload.context),
+                None, network_id)
             # Send the information to all the L3 Agent hosts
             # to clean up the fip namespace as it is no longer required.
             self.l3plugin.l3_rpc_notifier.delete_fipnamespace_for_ext_net(
@@ -409,6 +415,15 @@ class DVRResourceOperationHandler(object):
                 if host_id:
                     return
 
+    @registry.receives(resources.NETWORK, [events.AFTER_DELETE])
+    def delete_fip_namespaces_for_ext_net(self, rtype, event, trigger,
+                                          context, network, **kwargs):
+        if network.get(extnet_apidef.EXTERNAL):
+            # Send the information to all the L3 Agent hosts
+            # to clean up the fip namespace as it is no longer required.
+            self.l3plugin.l3_rpc_notifier.delete_fipnamespace_for_ext_net(
+                context, network['id'])
+
     def _get_ports_for_allowed_address_pair_ip(self, context, network_id,
                                                fixed_ip):
         """Return all active ports associated with the allowed_addr_pair ip."""
@@ -431,7 +446,7 @@ class DVRResourceOperationHandler(object):
         floatingIP association happens.
         """
         if association_event and router_id:
-            admin_ctx = context.elevated()
+            admin_ctx = n_utils.get_elevated_context(context)
             router_dict = self.get_router(admin_ctx, router_id)
             # Check if distributed router and then create the
             # FloatingIP agent gateway port
@@ -469,6 +484,17 @@ class DVRResourceOperationHandler(object):
                                 fixed_ip_address))
                         if not addr_pair_active_service_port_list:
                             return
+                        self._inherit_service_port_and_arp_update(
+                            context, addr_pair_active_service_port_list[0])
+
+    def _inherit_service_port_and_arp_update(self, context, service_port):
+        """Function inherits port host bindings for allowed_address_pair."""
+        service_port_dict = self.l3plugin._core_plugin._make_port_dict(
+            service_port)
+        address_pair_list = service_port_dict.get('allowed_address_pairs')
+        for address_pair in address_pair_list:
+            self.update_arp_entry_for_dvr_service_port(context,
+                                                       service_port_dict)
 
     @registry.receives(resources.ROUTER_INTERFACE, [events.BEFORE_CREATE])
     @db_api.retry_if_session_inactive()
@@ -477,7 +503,7 @@ class DVRResourceOperationHandler(object):
         """Event handler to for csnat port creation on interface creation."""
         if not router_db.extra_attributes.distributed or not router_db.gw_port:
             return
-        admin_context = context.elevated()
+        admin_context = n_utils.get_elevated_context(context)
         self._add_csnat_router_interface_port(
             admin_context, router_db, port['network_id'],
             [{'subnet_id': port['fixed_ips'][-1]['subnet_id']}])
@@ -501,7 +527,7 @@ class DVRResourceOperationHandler(object):
         # IPv6 subnet
         # Add new prefix to an existing ipv6 csnat port with the
         # same network id if one exists
-        admin_ctx = context.elevated()
+        admin_ctx = n_utils.get_elevated_context(context)
         router = self.l3plugin._get_router(admin_ctx, router_id)
         cs_port = self._find_v6_router_port_by_network_and_device_owner(
             router, subnet['network_id'], const.DEVICE_OWNER_ROUTER_SNAT)
@@ -580,7 +606,7 @@ class DVRResourceOperationHandler(object):
                 if fixed_ips:
                     # multiple prefix port - delete prefix from port
                     self.l3plugin._core_plugin.update_port(
-                        context.elevated(),
+                        n_utils.get_elevated_context(context),
                         cs_port['id'], {'port': {'fixed_ips': fixed_ips}})
                     return True
         return False
@@ -670,7 +696,8 @@ class DVRResourceOperationHandler(object):
         if not is_multiple_prefix_csport:
             # Single prefix port - go ahead and delete the port
             self.delete_csnat_router_interface_ports(
-                context.elevated(), router, subnet_id=sub_id)
+                n_utils.get_elevated_context(context),
+                router, subnet_id=sub_id)
 
     def _cleanup_related_hosts_after_interface_removal(
             self, context, router_id, subnet_id):
@@ -872,7 +899,8 @@ class _DVRAgentInterfaceMixin(object):
             return []
         filters = {'device_id': [fip_agent_id],
                    'device_owner': [const.DEVICE_OWNER_AGENT_GW]}
-        ports = self._core_plugin.get_ports(context.elevated(), filters)
+        ports = self._core_plugin.get_ports(
+            n_utils.get_elevated_context(context), filters)
         LOG.debug("Return the FIP ports: %s ", ports)
         return ports
 
@@ -1030,7 +1058,7 @@ class _DVRAgentInterfaceMixin(object):
         fip = fips[0]
         network_id = fip.get('floating_network_id')
         self.create_fip_agent_gw_port_if_not_exists(
-            context.elevated(), network_id, host)
+            n_utils.get_elevated_context(context), network_id, host)
 
     def create_fip_agent_gw_port_if_not_exists(self, context, network_id,
                                                host):
@@ -1104,6 +1132,21 @@ class _DVRAgentInterfaceMixin(object):
         self._populate_mtu_and_subnets_for_ports(context, [agent_port])
         return agent_port
 
+    def _generate_arp_table_and_notify_agent(self, context, fixed_ip,
+                                             mac_address, notifier):
+        """Generates the arp table entry and notifies the l3 agent."""
+        ip_address = fixed_ip['ip_address']
+        subnet = fixed_ip['subnet_id']
+        arp_table = {'ip_address': ip_address,
+                     'mac_address': mac_address,
+                     'subnet_id': subnet}
+        filters = {'fixed_ips': {'subnet_id': [subnet]},
+                   'device_owner': [const.DEVICE_OWNER_DVR_INTERFACE]}
+        ports = self._core_plugin.get_ports(context, filters=filters)
+        routers = [port['device_id'] for port in ports]
+        for router_id in routers:
+            notifier(context, router_id, arp_table)
+
     def _get_subnet_id_for_given_fixed_ip(self, context, fixed_ip, port_dict):
         """Returns the subnet_id that matches the fixedip on a network."""
         filters = {'network_id': [port_dict['network_id']]}
@@ -1112,6 +1155,78 @@ class _DVRAgentInterfaceMixin(object):
             if ipam_utils.check_subnet_ip(subnet['cidr'], fixed_ip):
                 return subnet['id']
 
+    def _get_allowed_address_pair_fixed_ips(self, context, port_dict):
+        """Returns all fixed_ips associated with the allowed_address_pair."""
+        aa_pair_fixed_ips = []
+        if port_dict.get('allowed_address_pairs'):
+            for address_pair in port_dict['allowed_address_pairs']:
+                aap_ip_cidr = address_pair['ip_address'].split("/")
+                if len(aap_ip_cidr) == 1 or int(aap_ip_cidr[1]) == 32:
+                    subnet_id = self._get_subnet_id_for_given_fixed_ip(
+                        context, aap_ip_cidr[0], port_dict)
+                    if subnet_id is not None:
+                        fixed_ip = {'subnet_id': subnet_id,
+                                    'ip_address': aap_ip_cidr[0]}
+                        aa_pair_fixed_ips.append(fixed_ip)
+                    else:
+                        LOG.debug("Subnet does not match for the given "
+                                  "fixed_ip %s for arp update", aap_ip_cidr[0])
+        return aa_pair_fixed_ips
+
+    def update_arp_entry_for_dvr_service_port(self, context, port_dict):
+        """Notify L3 agents of ARP table entry for dvr service port.
+
+        When a dvr service port goes up, look for the DVR router on
+        the port's subnet, and send the ARP details to all
+        L3 agents hosting the router to add it.
+        If there are any allowed_address_pairs associated with the port
+        those fixed_ips should also be updated in the ARP table.
+        """
+        fixed_ips = port_dict['fixed_ips']
+        if not fixed_ips:
+            return
+        allowed_address_pair_fixed_ips = (
+            self._get_allowed_address_pair_fixed_ips(context, port_dict))
+        changed_fixed_ips = fixed_ips + allowed_address_pair_fixed_ips
+        for fixed_ip in changed_fixed_ips:
+            self._generate_arp_table_and_notify_agent(
+                context, fixed_ip, port_dict['mac_address'],
+                self.l3_rpc_notifier.add_arp_entry)
+
+    def delete_arp_entry_for_dvr_service_port(self, context, port_dict,
+                                              fixed_ips_to_delete=None):
+        """Notify L3 agents of ARP table entry for dvr service port.
+
+        When a dvr service port goes down, look for the DVR
+        router on the port's subnet, and send the ARP details to all
+        L3 agents hosting the router to delete it.
+        If there are any allowed_address_pairs associated with the
+        port, those fixed_ips should be removed from the ARP table.
+        """
+        fixed_ips = port_dict['fixed_ips']
+        if not fixed_ips:
+            return
+        if not fixed_ips_to_delete:
+            allowed_address_pair_fixed_ips = (
+                self._get_allowed_address_pair_fixed_ips(context, port_dict))
+            fixed_ips_to_delete = fixed_ips + allowed_address_pair_fixed_ips
+        for fixed_ip in fixed_ips_to_delete:
+            self._generate_arp_table_and_notify_agent(
+                context, fixed_ip, port_dict['mac_address'],
+                self.l3_rpc_notifier.del_arp_entry)
+
+    def _get_address_pair_active_port_with_fip(
+            self, context, port_dict, port_addr_pair_ip):
+        port_valid_state = (port_dict['admin_state_up'] or
+                            port_dict['status'] == const.PORT_STATUS_ACTIVE)
+        if not port_valid_state:
+            return
+        fips = l3_obj.FloatingIP.get_objects(
+            context, _pager=base_obj.Pager(limit=1),
+            fixed_ip_address=netaddr.IPAddress(port_addr_pair_ip))
+        return self._core_plugin.get_port(
+            context, fips[0].fixed_port_id) if fips else None
+
 
 class L3_NAT_with_dvr_db_mixin(_DVRAgentInterfaceMixin,
                                DVRResourceOperationHandler,
@@ -1155,7 +1270,8 @@ class L3_NAT_with_dvr_db_mixin(_DVRAgentInterfaceMixin,
 
         try:
             # using admin context as router may belong to admin tenant
-            router = self._get_router(context.elevated(), router_id)
+            router = self._get_router(n_utils.get_elevated_context(context),
+                                      router_id)
         except l3_exc.RouterNotFound:
             LOG.warning("Router %s was not found. "
                         "Skipping agent notification.",
@@ -1189,7 +1305,7 @@ class L3_NAT_with_dvr_db_mixin(_DVRAgentInterfaceMixin,
                         context, [router_id], dest_host)
             else:
                 centralized_agent_list = self.list_l3_agents_hosting_router(
-                    context.elevated(), router_id)['agents']
+                    n_utils.get_elevated_context(context), router_id)['agents']
                 for agent in centralized_agent_list:
                     self.l3_rpc_notifier.routers_updated_on_host(
                         context, [router_id], agent['host'])
@@ -1215,7 +1331,8 @@ class L3_NAT_with_dvr_db_mixin(_DVRAgentInterfaceMixin,
     def is_router_distributed(self, context, router_id):
         if router_id:
             return is_distributed_router(
-                self.get_router(context.elevated(), router_id))
+                self.get_router(n_utils.get_elevated_context(context),
+                                router_id))
         return False
 
     def get_ports_under_dvr_connected_subnet(self, context, subnet_id):
