@@ -40,13 +40,11 @@ from oslo_db import exception as db_exc
 from oslo_log import helpers as log_helpers
 from oslo_log import log as logging
 from oslo_utils import excutils
-import six
 import sqlalchemy as sa
 from sqlalchemy import exc as sql_exc
 from sqlalchemy import orm
 
 from neutron._i18n import _
-from neutron.common import constants as n_const
 from neutron.common import utils as n_utils
 from neutron.conf.db import l3_hamode_db
 from neutron.db import _utils as db_utils
@@ -122,12 +120,12 @@ class L3_HA_NAT_db_mixin(l3_dvr_db.L3_NAT_with_dvr_db_mixin,
         network_id = ha_network.network_id
 
         # TODO(kevinbenton): let decorator handle duplicate retry
-        # like in review.openstack.org/#/c/367179/1/neutron/db/l3_hamode_db.py
+        # like in review.opendev.org/#/c/367179/1/neutron/db/l3_hamode_db.py
         for count in range(MAX_ALLOCATION_TRIES):
             try:
                 # NOTE(kevinbenton): we disallow subtransactions because the
                 # retry logic will bust any parent transactions
-                with context.session.begin():
+                with db_api.CONTEXT_WRITER.using(context):
                     if router_db.extra_attributes.ha_vr_id:
                         LOG.debug(
                             "Router %(router_id)s has already been "
@@ -145,7 +143,10 @@ class L3_HA_NAT_db_mixin(l3_dvr_db.L3_NAT_with_dvr_db_mixin,
                         context, network_id=network_id, vr_id=vr_id)
                     allocation.create()
 
-                    router_db.extra_attributes.ha_vr_id = allocation.vr_id
+                    router_db.extra_attributes.update(
+                        {'ha_vr_id': allocation.vr_id})
+                    context.session.add(router_db.extra_attributes)
+
                     LOG.debug(
                         "Router %(router_id)s has been allocated a ha_vr_id "
                         "%(ha_vr_id)d.",
@@ -180,7 +181,7 @@ class L3_HA_NAT_db_mixin(l3_dvr_db.L3_NAT_with_dvr_db_mixin,
     def _create_ha_subnet(self, context, network_id, tenant_id):
         args = {'network_id': network_id,
                 'tenant_id': '',
-                'name': n_const.HA_SUBNET_NAME % tenant_id,
+                'name': constants.HA_SUBNET_NAME % tenant_id,
                 'ip_version': 4,
                 'cidr': cfg.CONF.l3_ha_net_cidr,
                 'enable_dhcp': False,
@@ -202,7 +203,7 @@ class L3_HA_NAT_db_mixin(l3_dvr_db.L3_NAT_with_dvr_db_mixin,
             # and the process is started over where the existing
             # network will be selected.
             raise db_exc.DBDuplicateEntry(columns=['tenant_id'])
-        return ha_network
+        return None, ha_network
 
     def _add_ha_network_settings(self, network):
         if cfg.CONF.l3_ha_network_type:
@@ -213,10 +214,10 @@ class L3_HA_NAT_db_mixin(l3_dvr_db.L3_NAT_with_dvr_db_mixin,
                 cfg.CONF.l3_ha_network_physical_name)
 
     def _create_ha_network(self, context, tenant_id):
-        admin_ctx = context.elevated()
+        admin_ctx = n_utils.get_elevated_context(context)
 
         args = {'network':
-                {'name': n_const.HA_NETWORK_NAME % tenant_id,
+                {'name': constants.HA_NETWORK_NAME % tenant_id,
                  'tenant_id': '',
                  'shared': False,
                  'admin_state_up': True}}
@@ -260,17 +261,16 @@ class L3_HA_NAT_db_mixin(l3_dvr_db.L3_NAT_with_dvr_db_mixin,
     @db_api.retry_if_session_inactive()
     def _create_ha_port_binding(self, context, router_id, port_id):
         try:
-            with context.session.begin():
-                l3_obj.RouterPort(
-                    context,
-                    port_id=port_id,
-                    router_id=router_id,
-                    port_type=constants.DEVICE_OWNER_ROUTER_HA_INTF).create()
-                portbinding = l3_hamode.L3HARouterAgentPortBinding(
-                    context, port_id=port_id, router_id=router_id)
-                portbinding.create()
+            l3_obj.RouterPort(
+                context,
+                port_id=port_id,
+                router_id=router_id,
+                port_type=constants.DEVICE_OWNER_ROUTER_HA_INTF).create()
+            portbinding = l3_hamode.L3HARouterAgentPortBinding(
+                context, port_id=port_id, router_id=router_id)
+            portbinding.create()
 
-            return portbinding
+            return None, portbinding
         except db_exc.DBReferenceError as e:
             with excutils.save_and_reraise_exception() as ctxt:
                 if isinstance(e.inner_exception, sql_exc.IntegrityError):
@@ -296,7 +296,7 @@ class L3_HA_NAT_db_mixin(l3_dvr_db.L3_NAT_with_dvr_db_mixin,
                 'admin_state_up': True,
                 'device_id': router_id,
                 'device_owner': constants.DEVICE_OWNER_ROUTER_HA_INTF,
-                'name': n_const.HA_PORT_NAME % tenant_id}
+                'name': constants.HA_PORT_NAME % tenant_id}
         creation = functools.partial(p_utils.create_port, self._core_plugin,
                                      context, {'port': args})
         content = functools.partial(self._create_ha_port_binding, context,
@@ -311,7 +311,7 @@ class L3_HA_NAT_db_mixin(l3_dvr_db.L3_NAT_with_dvr_db_mixin,
         return binding.db_obj
 
     def _delete_ha_interfaces(self, context, router_id):
-        admin_ctx = context.elevated()
+        admin_ctx = n_utils.get_elevated_context(context)
         device_filter = {'device_id': [router_id],
                          'device_owner':
                          [constants.DEVICE_OWNER_ROUTER_HA_INTF]}
@@ -322,7 +322,7 @@ class L3_HA_NAT_db_mixin(l3_dvr_db.L3_NAT_with_dvr_db_mixin,
                                           l3_port_check=False)
 
     def delete_ha_interfaces_on_host(self, context, router_id, host):
-        admin_ctx = context.elevated()
+        admin_ctx = n_utils.get_elevated_context(context)
         port_ids = (binding.port_id for binding
                     in self.get_ha_router_port_bindings(admin_ctx,
                                                         [router_id], host))
@@ -343,7 +343,7 @@ class L3_HA_NAT_db_mixin(l3_dvr_db.L3_NAT_with_dvr_db_mixin,
 
     def _get_device_owner(self, context, router=None):
         """Get device_owner for the specified router."""
-        router_is_uuid = isinstance(router, six.string_types)
+        router_is_uuid = isinstance(router, str)
         if router_is_uuid:
             router = self._get_router(context, router)
         if (is_ha_router(router) and not
@@ -497,7 +497,7 @@ class L3_HA_NAT_db_mixin(l3_dvr_db.L3_NAT_with_dvr_db_mixin,
         self._notify_router_updated(context, router_db.id)
 
     def _delete_ha_network(self, context, net):
-        admin_ctx = context.elevated()
+        admin_ctx = n_utils.get_elevated_context(context)
         self._core_plugin.delete_network(admin_ctx, net.network_id)
 
     def safe_delete_ha_network(self, context, ha_network, tenant_id):
@@ -572,7 +572,7 @@ class L3_HA_NAT_db_mixin(l3_dvr_db.L3_NAT_with_dvr_db_mixin,
            states on this agent.
 
         """
-        with context.session.begin(subtransactions=True):
+        with db_api.CONTEXT_WRITER.using(context):
             bindings = self.get_ha_router_port_bindings(context, [router_id])
             router_active_agents_dead = []
             router_standby_agents_dead = []
@@ -581,9 +581,9 @@ class L3_HA_NAT_db_mixin(l3_dvr_db.L3_NAT_with_dvr_db_mixin,
             for binding in bindings:
                 if not (binding.agent.is_active and
                         binding.agent.admin_state_up):
-                    if binding.state == n_const.HA_ROUTER_STATE_ACTIVE:
+                    if binding.state == constants.HA_ROUTER_STATE_ACTIVE:
                         router_active_agents_dead.append(binding.agent)
-                    elif binding.state == n_const.HA_ROUTER_STATE_STANDBY:
+                    elif binding.state == constants.HA_ROUTER_STATE_STANDBY:
                         router_standby_agents_dead.append(binding.agent)
             if router_active_agents_dead:
                 # Just check if all l3_agents are down
@@ -594,12 +594,12 @@ class L3_HA_NAT_db_mixin(l3_dvr_db.L3_NAT_with_dvr_db_mixin,
                     # agent communication may be issue but router
                     # may still be active. We do not know the
                     # exact status of router.
-                    state = n_const.HA_ROUTER_STATE_UNKNOWN
+                    state = constants.HA_ROUTER_STATE_UNKNOWN
                 else:
                     # Make router status as standby on all dead agents
                     # as some other agents are alive , router can become
                     # active on them after some time
-                    state = n_const.HA_ROUTER_STATE_STANDBY
+                    state = constants.HA_ROUTER_STATE_STANDBY
                 for dead_agent in router_active_agents_dead:
                     self.update_routers_states(context, {router_id: state},
                                                dead_agent.host)
@@ -626,7 +626,7 @@ class L3_HA_NAT_db_mixin(l3_dvr_db.L3_NAT_with_dvr_db_mixin,
         # and l2pop would not work correctly.
         return next(
             (agent.host for agent, state in bindings
-             if state == n_const.HA_ROUTER_STATE_ACTIVE),
+             if state == constants.HA_ROUTER_STATE_ACTIVE),
             None)
 
     @log_helpers.log_method_call
@@ -649,7 +649,7 @@ class L3_HA_NAT_db_mixin(l3_dvr_db.L3_NAT_with_dvr_db_mixin,
 
             router = routers_dict.get(binding.router_id)
             router[constants.HA_INTERFACE_KEY] = port_dict
-            router[n_const.HA_ROUTER_STATE_KEY] = binding.state
+            router[constants.HA_ROUTER_STATE_KEY] = binding.state
 
         interfaces = []
         for router in routers_dict.values():
@@ -689,7 +689,8 @@ class L3_HA_NAT_db_mixin(l3_dvr_db.L3_NAT_with_dvr_db_mixin,
     def _set_router_states(cls, context, bindings, states):
         for binding in bindings:
             try:
-                with context.session.begin(subtransactions=True):
+                with db_api.CONTEXT_WRITER.using(context):
+                    context.session.add(binding)
                     binding.state = states[binding.router_id]
             except (orm.exc.StaleDataError, orm.exc.ObjectDeletedError):
                 # Take concurrently deleted routers in to account
@@ -699,13 +700,14 @@ class L3_HA_NAT_db_mixin(l3_dvr_db.L3_NAT_with_dvr_db_mixin,
     def update_routers_states(self, context, states, host):
         """Receive dict of router ID to state and update them all."""
 
-        bindings = self.get_ha_router_port_bindings(
-            context, router_ids=states.keys(), host=host)
+        with db_api.CONTEXT_READER.using(context):
+            bindings = self.get_ha_router_port_bindings(
+                context, router_ids=states.keys(), host=host)
         self._set_router_states(context, bindings, states)
         self._update_router_port_bindings(context, states, host)
 
     def _update_router_port_bindings(self, context, states, host):
-        admin_ctx = context.elevated()
+        admin_ctx = n_utils.get_elevated_context(context)
         device_filter = {'device_id': list(states.keys()),
                          'device_owner':
                          [constants.DEVICE_OWNER_HA_REPLICATED_INT,
@@ -714,7 +716,7 @@ class L3_HA_NAT_db_mixin(l3_dvr_db.L3_NAT_with_dvr_db_mixin,
         ports = self._core_plugin.get_ports(admin_ctx, filters=device_filter)
         active_ports = (
             port for port in ports
-            if states[port['device_id']] == n_const.HA_ROUTER_STATE_ACTIVE)
+            if states[port['device_id']] == constants.HA_ROUTER_STATE_ACTIVE)
 
         for port in active_ports:
             try:
@@ -738,7 +740,7 @@ class L3_HA_NAT_db_mixin(l3_dvr_db.L3_NAT_with_dvr_db_mixin,
         gateway_port_status = gateway_port['status']
         gateway_port_binding_host = gateway_port[portbindings.HOST_ID]
 
-        admin_ctx = context.elevated()
+        admin_ctx = n_utils.get_elevated_context(context)
         router_id = router['id']
         ha_bindings = self.get_l3_bindings_hosting_router_with_ha_states(
             admin_ctx, router_id)
@@ -749,13 +751,13 @@ class L3_HA_NAT_db_mixin(l3_dvr_db.L3_NAT_with_dvr_db_mixin,
                    "host": gateway_port_binding_host,
                    "status": gateway_port_status})
         for ha_binding_agent, ha_binding_state in ha_bindings:
-            if ha_binding_state != n_const.HA_ROUTER_STATE_ACTIVE:
+            if ha_binding_state != constants.HA_ROUTER_STATE_ACTIVE:
                 continue
             # For create router gateway, the gateway port may not be ACTIVE
-            # yet, so we return 'master' host directly.
+            # yet, so we return 'primary' host directly.
             if gateway_port_status != constants.PORT_STATUS_ACTIVE:
                 return ha_binding_agent.host
-            # Do not let the original 'master' (current is backup) host,
+            # Do not let the original 'primary' (current is backup) host,
             # override the gateway port binding host.
             if (gateway_port_status == constants.PORT_STATUS_ACTIVE and
                     ha_binding_agent.host == gateway_port_binding_host):
