@@ -22,7 +22,6 @@ from neutron_lib.api import extensions as api_extensions
 from neutron_lib.plugins import constants
 from neutron_lib.plugins import directory
 from neutron_lib.services import base as service_base
-import six
 
 from neutron.api import extensions
 from neutron.api.v2 import base
@@ -80,8 +79,7 @@ class Qos(api_extensions.APIExtensionDescriptor):
         return resources
 
 
-@six.add_metaclass(abc.ABCMeta)
-class QoSPluginBase(service_base.ServicePluginBase):
+class QoSPluginBase(service_base.ServicePluginBase, metaclass=abc.ABCMeta):
 
     path_prefix = apidef.API_PREFIX
 
