@@ -19,6 +19,7 @@ import itertools
 from neutron_lib.db import api as db_api
 from neutron_lib import exceptions as n_exc
 from neutron_lib.objects import exceptions as o_exc
+from neutron_lib.objects.extensions import standardattributes
 from oslo_db import exception as obj_exc
 from oslo_db.sqlalchemy import enginefacade
 from oslo_db.sqlalchemy import utils as db_utils
@@ -28,13 +29,11 @@ from oslo_utils import versionutils
 from oslo_versionedobjects import base as obj_base
 from oslo_versionedobjects import exception as obj_exception
 from oslo_versionedobjects import fields as obj_fields
-import six
 from sqlalchemy import orm
 
 from neutron._i18n import _
 from neutron.db import standard_attr
 from neutron.objects.db import api as obj_db_api
-from neutron.objects.extensions import standardattributes
 
 
 LOG = logging.getLogger(__name__)
@@ -55,7 +54,7 @@ def register_filter_hook_on_model(model, filter_name):
     obj_class.add_extra_filter_name(filter_name)
 
 
-class LazyQueryIterator(six.Iterator):
+class LazyQueryIterator(object):
     def __init__(self, obj_class, lazy_query):
         self.obj_class = obj_class
         self.context = None
@@ -142,10 +141,10 @@ class NeutronObjectRegistry(obj_base.VersionedObjectRegistry):
         return self
 
 
-@six.add_metaclass(abc.ABCMeta)
 class NeutronObject(obj_base.VersionedObject,
                     obj_base.VersionedObjectDictCompat,
-                    obj_base.ComparableVersionedObject):
+                    obj_base.ComparableVersionedObject,
+                    metaclass=abc.ABCMeta):
 
     synthetic_fields = []
     extra_filter_names = set()
@@ -398,8 +397,7 @@ class DeclarativeObject(abc.ABCMeta):
             raise o_exc.NeutronObjectValidatorException(fields=invalid_fields)
 
 
-@six.add_metaclass(DeclarativeObject)
-class NeutronDbObject(NeutronObject):
+class NeutronDbObject(NeutronObject, metaclass=DeclarativeObject):
 
     # should be overridden for all persistent objects
     db_model = None
@@ -407,9 +405,6 @@ class NeutronDbObject(NeutronObject):
     # should be overridden for all rbac aware objects
     rbac_db_cls = None
 
-    # whether to use new engine facade for the object
-    new_facade = False
-
     primary_keys = ['id']
 
     # 'unique_keys' is a list of unique keys that can be used with get_object
@@ -573,16 +568,12 @@ class NeutronDbObject(NeutronObject):
     @classmethod
     def db_context_writer(cls, context):
         """Return read-write session activation decorator."""
-        if cls.new_facade or cls._use_db_facade(context):
-            return db_api.CONTEXT_WRITER.using(context)
-        return db_api.autonested_transaction(context.session)
+        return db_api.CONTEXT_WRITER.using(context)
 
     @classmethod
     def db_context_reader(cls, context):
         """Return read-only session activation decorator."""
-        if cls.new_facade or cls._use_db_facade(context):
-            return db_api.CONTEXT_READER.using(context)
-        return db_api.autonested_transaction(context.session)
+        return db_api.CONTEXT_READER.using(context)
 
     @classmethod
     def get_object(cls, context, fields=None, **kwargs):
