@@ -288,7 +288,9 @@ class Port(base.NeutronDbObject):
     # Version 1.3: distributed_binding -> distributed_bindings
     # Version 1.4: Attribute binding becomes ListOfObjectsField
     # Version 1.5: Added qos_network_policy_id field
-    VERSION = '1.5'
+    # Version 1.6: Added numa_affinity_policy field
+    # Version 1.7: Added port_device field
+    VERSION = '1.7'
 
     db_model = models_v2.Port
 
@@ -338,6 +340,8 @@ class Port(base.NeutronDbObject):
         'binding_levels': obj_fields.ListOfObjectsField(
             'PortBindingLevel', nullable=True
         ),
+        'numa_affinity_policy': obj_fields.StringField(nullable=True),
+        'device_profile': obj_fields.StringField(nullable=True),
 
         # TODO(ihrachys): consider adding a 'dns_assignment' fully synthetic
         # field in later object iterations
@@ -352,10 +356,12 @@ class Port(base.NeutronDbObject):
         'bindings',
         'binding_levels',
         'data_plane_status',
+        'device_profile',
         'dhcp_options',
         'distributed_bindings',
         'dns',
         'fixed_ips',
+        'numa_affinity_policy',
         'qos_policy_id',
         'qos_network_policy_id',
         'security',
@@ -516,6 +522,15 @@ class Port(base.NeutronDbObject):
                 db_obj.qos_network_policy_binding.policy_id)
             fields_to_change.append('qos_network_policy_binding')
 
+        if db_obj.get('numa_affinity_policy'):
+            self.numa_affinity_policy = (
+                db_obj.numa_affinity_policy.numa_affinity_policy)
+            fields_to_change.append('numa_affinity_policy')
+
+        if db_obj.get('device_profile'):
+            self.device_profile = db_obj.device_profile.device_profile
+            fields_to_change.append('device_profile')
+
         self.obj_reset_changes(fields_to_change)
 
     def obj_make_compatible(self, primitive, target_version):
@@ -544,6 +559,10 @@ class Port(base.NeutronDbObject):
                         break
         if _target_version < (1, 5):
             primitive.pop('qos_network_policy_id', None)
+        if _target_version < (1, 6):
+            primitive.pop('numa_affinity_policy', None)
+        if _target_version < (1, 7):
+            primitive.pop('device_profile', None)
 
     @classmethod
     def get_ports_by_router_and_network(cls, context, router_id, owner,
@@ -619,14 +638,6 @@ class Port(base.NeutronDbObject):
                 ~models_v2.Port.device_owner.in_(excluded_device_owners))
         return [port_binding['port_id'] for port_binding in query.all()]
 
-    @classmethod
-    def get_ports_by_host(cls, context, host):
-        query = context.session.query(models_v2.Port.id).join(
-            ml2_models.PortBinding)
-        query = query.filter(
-            ml2_models.PortBinding.host == host)
-        return [port_id[0] for port_id in query.all()]
-
     @classmethod
     def get_ports_by_binding_type_and_host(cls, context,
                                            binding_type, host):
