@@ -24,11 +24,11 @@ from neutron_lib.plugins import directory
 from oslo_db import exception as db_exc
 from oslo_log import log
 from oslo_utils import uuidutils
-import six
 from sqlalchemy import or_
 from sqlalchemy.orm import exc
 
 from neutron._i18n import _
+from neutron.common import utils as common_utils
 from neutron.db.models import securitygroup as sg_models
 from neutron.db import models_v2
 from neutron.objects import base as objects_base
@@ -168,7 +168,7 @@ def get_ports_and_sgs(context, port_ids):
         return []
     ports_to_sg_ids = get_sg_ids_grouped_by_port(context, port_ids)
     return [make_port_dict_with_security_groups(port, sec_groups)
-            for port, sec_groups in six.iteritems(ports_to_sg_ids)]
+            for port, sec_groups in ports_to_sg_ids.items()]
 
 
 def get_sg_ids_grouped_by_port(context, port_ids):
@@ -207,6 +207,7 @@ def make_port_dict_with_security_groups(port, sec_groups):
     port_dict['security_groups'] = sec_groups
     port_dict['security_group_rules'] = []
     port_dict['security_group_source_groups'] = []
+    port_dict['security_group_remote_address_groups'] = []
     port_dict['fixed_ips'] = [ip['ip_address']
                               for ip in port['fixed_ips']]
     return port_dict
@@ -339,7 +340,8 @@ def _prevent_segment_delete_with_port_bound(resource, event, trigger,
         plugin = directory.get_plugin()
     for port_id in auto_delete_port_ids:
         try:
-            plugin.delete_port(payload.context.elevated(), port_id)
+            plugin.delete_port(
+                common_utils.get_elevated_context(payload.context), port_id)
         except nlib_exc.PortNotFound:
             # Don't raise if something else concurrently deleted the port
             LOG.debug("Ignoring PortNotFound when deleting port '%s'. "
