@@ -33,6 +33,7 @@ import oslo_messaging
 from oslo_service import service
 from oslo_utils import excutils
 
+from neutron._i18n import _
 from neutron.agent.linux import bridge_lib
 from neutron.agent.linux import ip_lib
 from neutron.api.rpc.handlers import securitygroups_rpc as sg_rpc
@@ -327,7 +328,8 @@ class LinuxBridgeManager(amb.CommonAgentManagerBase):
                       "VNI %(segmentation_id)s",
                       {'interface': interface,
                        'segmentation_id': segmentation_id})
-            args = {'srcport': (cfg.CONF.VXLAN.udp_srcport_min,
+            args = {'dev': self.local_int,
+                    'srcport': (cfg.CONF.VXLAN.udp_srcport_min,
                                 cfg.CONF.VXLAN.udp_srcport_max),
                     'dstport': cfg.CONF.VXLAN.udp_dstport,
                     'ttl': cfg.CONF.VXLAN.ttl}
@@ -344,16 +346,12 @@ class LinuxBridgeManager(amb.CommonAgentManagerBase):
 
             if self.vxlan_mode == lconst.VXLAN_MCAST:
                 args['group'] = self.get_vxlan_group(segmentation_id)
-
             if cfg.CONF.VXLAN.l2_population:
                 args['proxy'] = cfg.CONF.VXLAN.arp_responder
-                # L2population should set the local ip address to handle
-                # a source dev with multiple ip addresses configured.
-                args['local'] = self.local_ip
 
             try:
                 int_vxlan = self.ip.add_vxlan(interface, segmentation_id,
-                                              self.local_int, **args)
+                                              **args)
             except RuntimeError:
                 with excutils.save_and_reraise_exception() as ctxt:
                     # perform this check after an attempt rather than before
@@ -439,9 +437,9 @@ class LinuxBridgeManager(amb.CommonAgentManagerBase):
                       "%(interface)s",
                       {'bridge_name': bridge_name, 'interface': interface})
             bridge_device = bridge_lib.BridgeDevice.addbr(bridge_name)
-            if bridge_device.setfd(0):
+            if not bridge_device.setfd(0):
                 return
-            if bridge_device.disable_stp():
+            if not bridge_device.disable_stp():
                 return
             if bridge_device.link.set_up():
                 return
@@ -460,19 +458,14 @@ class LinuxBridgeManager(amb.CommonAgentManagerBase):
 
         # Check if the interface is part of the bridge
         if not bridge_device.owns_interface(interface):
-            try:
-                # Check if the interface is attached to another bridge
-                bridge = bridge_lib.BridgeDevice.get_interface_bridge(
-                    interface)
-                if bridge:
-                    bridge.delif(interface)
+            # Check if the interface is attached to another bridge
+            bridge = bridge_lib.BridgeDevice.get_interface_bridge(interface)
+            if bridge:
+                bridge.delif(interface)
 
-                bridge_device.addif(interface)
-            except Exception as e:
-                LOG.error("Unable to add %(interface)s to %(bridge_name)s"
-                          "! Exception: %(e)s",
-                          {'interface': interface, 'bridge_name': bridge_name,
-                           'e': e})
+            if not bridge_device.addif(interface):
+                LOG.error("Unable to add %(interface)s to %(bridge_name)s",
+                          {'interface': interface, 'bridge_name': bridge_name})
                 return
         return bridge_name
 
@@ -565,7 +558,8 @@ class LinuxBridgeManager(amb.CommonAgentManagerBase):
                         'bridge_name': bridge_name}
                 LOG.debug("Adding device %(tap_device_name)s to bridge "
                           "%(bridge_name)s", data)
-                if bridge_lib.BridgeDevice(bridge_name).addif(tap_device_name):
+                if not bridge_lib.BridgeDevice(bridge_name).addif(
+                        tap_device_name):
                     return False
         else:
             data = {'tap_device_name': tap_device_name,
@@ -639,24 +633,24 @@ class LinuxBridgeManager(amb.CommonAgentManagerBase):
                       "%(bridge_name)s",
                       {'interface_name': interface_name,
                        'bridge_name': bridge_name})
-            try:
-                bridge_device.delif(interface_name)
+
+            if bridge_device.delif(interface_name):
                 LOG.debug("Done removing device %(interface_name)s from "
                           "bridge %(bridge_name)s",
                           {'interface_name': interface_name,
                            'bridge_name': bridge_name})
                 return True
-            except RuntimeError:
-                with excutils.save_and_reraise_exception() as ctxt:
-                    if not bridge_device.owns_interface(interface_name):
-                        # the exception was likely a side effect of the tap
-                        # being deleted by some other agent during handling
-                        ctxt.reraise = False
-                        LOG.debug("Cannot remove %(interface_name)s from "
-                                  "%(bridge_name)s. It is not on the bridge.",
-                                  {'interface_name': interface_name,
-                                   'bridge_name': bridge_name})
-                        return False
+            else:
+                if not bridge_device.owns_interface(interface_name):
+                    LOG.debug("Cannot remove %(interface_name)s from "
+                              "%(bridge_name)s. It is not on the bridge.",
+                              {'interface_name': interface_name,
+                               'bridge_name': bridge_name})
+                    return False
+                msg = _("Error deleting %(interface_name)s from bridge "
+                        "%(bridge_name)s") % {'interface_name': interface_name,
+                                              'bridge_name': bridge_name}
+                raise RuntimeError(msg)
         else:
             LOG.debug("Cannot remove device %(interface_name)s bridge "
                       "%(bridge_name)s does not exist",
@@ -692,16 +686,7 @@ class LinuxBridgeManager(amb.CommonAgentManagerBase):
     def vxlan_ucast_supported(self):
         if not cfg.CONF.VXLAN.l2_population:
             return False
-        if not ip_lib.iproute_arg_supported(
-                ['bridge', 'fdb'], 'append'):
-            LOG.warning('Option "%(option)s" must be supported by command '
-                        '"%(command)s" to enable %(mode)s mode',
-                        {'option': 'append',
-                         'command': 'bridge fdb',
-                         'mode': 'VXLAN UCAST'})
-            return False
 
-        test_iface = None
         for seg_id in range(1, constants.MAX_VXLAN_VNI + 1):
             if (ip_lib.device_exists(self.get_vxlan_device_name(seg_id)) or
                     ip_lib.vxlan_in_use(seg_id)):
@@ -712,15 +697,10 @@ class LinuxBridgeManager(amb.CommonAgentManagerBase):
             LOG.error('No valid Segmentation ID to perform UCAST test.')
             return False
 
-        try:
-            bridge_lib.FdbInterface.append(constants.FLOODING_ENTRY[0],
-                                           test_iface, '1.1.1.1',
-                                           log_fail_as_error=False)
-            return True
-        except RuntimeError:
-            return False
-        finally:
-            self.delete_interface(test_iface)
+        ret = bridge_lib.FdbInterface.append(constants.FLOODING_ENTRY[0],
+                                             test_iface, '1.1.1.1')
+        self.delete_interface(test_iface)
+        return ret
 
     @staticmethod
     def vxlan_mcast_supported():
@@ -760,11 +740,13 @@ class LinuxBridgeManager(amb.CommonAgentManagerBase):
 
     @staticmethod
     def fdb_bridge_entry_exists(mac, interface, agent_ip=None):
-        entries = bridge_lib.FdbInterface.show(interface)
+        entries = bridge_lib.FdbInterface.show(dev=interface)
+        macs = [entry['mac'] for entry in entries[interface]]
+        ips = [entry['dst_ip'] for entry in entries[interface]]
         if not agent_ip:
-            return mac in entries
+            return mac in macs
 
-        return (agent_ip in entries and mac in entries)
+        return agent_ip in ips and mac in macs
 
     @staticmethod
     def add_fdb_ip_entry(mac, ip, interface):
@@ -780,25 +762,23 @@ class LinuxBridgeManager(amb.CommonAgentManagerBase):
         for mac, ip in ports:
             if mac != constants.FLOODING_ENTRY[0]:
                 self.add_fdb_ip_entry(mac, ip, interface)
-                bridge_lib.FdbInterface.replace(mac, interface, agent_ip,
-                                                check_exit_code=False)
+                bridge_lib.FdbInterface.replace(mac, interface,
+                                                dst_ip=agent_ip)
             elif self.vxlan_mode == lconst.VXLAN_UCAST:
                 if self.fdb_bridge_entry_exists(mac, interface):
-                    bridge_lib.FdbInterface.append(mac, interface, agent_ip,
-                                                   check_exit_code=False)
+                    bridge_lib.FdbInterface.append(mac, interface,
+                                                   dst_ip=agent_ip)
                 else:
-                    bridge_lib.FdbInterface.add(mac, interface, agent_ip,
-                                                check_exit_code=False)
+                    bridge_lib.FdbInterface.add(mac, interface,
+                                                dst_ip=agent_ip)
 
     def remove_fdb_entries(self, agent_ip, ports, interface):
         for mac, ip in ports:
             if mac != constants.FLOODING_ENTRY[0]:
                 self.remove_fdb_ip_entry(mac, ip, interface)
-                bridge_lib.FdbInterface.delete(mac, interface, agent_ip,
-                                               check_exit_code=False)
+                bridge_lib.FdbInterface.delete(mac, interface, dst_ip=agent_ip)
             elif self.vxlan_mode == lconst.VXLAN_UCAST:
-                bridge_lib.FdbInterface.delete(mac, interface, agent_ip,
-                                               check_exit_code=False)
+                bridge_lib.FdbInterface.delete(mac, interface, dst_ip=agent_ip)
 
     def get_agent_id(self):
         if self.bridge_mappings:
