@@ -58,9 +58,12 @@ class SriovNicSwitchMechanismDriver(mech_agent.SimpleAgentMechanismDriverBase):
                  vif_details={portbindings.CAP_PORT_FILTER: False,
                               portbindings.VIF_DETAILS_CONNECTIVITY:
                                   portbindings.CONNECTIVITY_L2},
-                 supported_vnic_types=[portbindings.VNIC_DIRECT,
-                                       portbindings.VNIC_MACVTAP,
-                                       portbindings.VNIC_DIRECT_PHYSICAL]):
+                 supported_vnic_types=[
+                     portbindings.VNIC_DIRECT,
+                     portbindings.VNIC_MACVTAP,
+                     portbindings.VNIC_DIRECT_PHYSICAL,
+                     portbindings.VNIC_ACCELERATOR_DIRECT,
+                 ]):
         """Initialize base class for SriovNicSwitch L2 agent type.
 
         :param agent_type: Constant identifying agent type in agents_db
@@ -178,9 +181,6 @@ class SriovNicSwitchMechanismDriver(mech_agent.SimpleAgentMechanismDriverBase):
         :param agent: agents_db entry describing agent to bind or None
         :returns: True if segment can be bound for agent
         """
-        if agent and agent['agent_type'] != self.agent_type:
-            return False
-
         network_type = segment[api.NETWORK_TYPE]
         if network_type in self.get_allowed_network_types(agent):
             if agent:
