@@ -59,7 +59,8 @@ MAC_SPOOF_TABLE = 25
 
 # Table to decide whether further filtering is needed
 TRANSIENT_TABLE = 60
-TRANSIENT_EGRESS_TABLE = 61
+LOCAL_MAC_DIRECT = 61
+TRANSIENT_EGRESS_TABLE = 62
 
 # Tables used for ovs firewall
 BASE_EGRESS_TABLE = 71
@@ -89,6 +90,7 @@ INT_BR_ALL_TABLES = (
     CANARY_TABLE,
     ARP_SPOOF_TABLE,
     MAC_SPOOF_TABLE,
+    LOCAL_MAC_DIRECT,
     TRANSIENT_TABLE,
     TRANSIENT_EGRESS_TABLE,
     BASE_EGRESS_TABLE,
@@ -114,6 +116,9 @@ LEARN_FROM_TUN = 10
 UCAST_TO_TUN = 20
 ARP_RESPONDER = 21
 FLOOD_TO_TUN = 22
+# NOTE(vsaienko): transit table used by networking-bagpipe driver to
+# mirror traffic to EVPN and standard tunnels to gateway nodes
+BAGPIPE_FLOOD_TO_TUN_BROADCAST = 222
 
 TUN_BR_ALL_TABLES = (
     LOCAL_SWITCHING,
