@@ -14,9 +14,51 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from ryu.lib.packet import ether_types
-from ryu.lib.packet import icmpv6
-from ryu.lib.packet import in_proto
+from os_ken.lib.packet import arp
+from os_ken.lib.packet import ether_types
+from os_ken.lib.packet import icmpv6
+from os_ken.lib.packet import in_proto
+
+from neutron.plugins.ml2.drivers.openvswitch.agent.common import constants
+
+
+class OVSDVRInterfaceMixin(object):
+
+    def delete_arp_destination_change(self, target_mac_address,
+                                      orig_mac_address):
+        (_dp, _ofp, ofpp) = self._get_dp()
+        match = ofpp.OFPMatch(eth_dst=orig_mac_address,
+                              eth_type=ether_types.ETH_TYPE_ARP,
+                              arp_tha=target_mac_address,
+                              arp_op=arp.ARP_REPLY)
+        self.uninstall_flows(table_id=constants.LOCAL_SWITCHING,
+                             match=match)
+
+    def change_arp_destination_mac(self, target_mac_address,
+                                   orig_mac_address):
+        """Change destination MAC from dvr_host_mac to internal gateway MAC.
+        """
+        (_dp, ofp, ofpp) = self._get_dp()
+        # TODO(liuyulong): except ARP, reconsider if we can change all IPv4
+        # packet's destination MAC to internal gateway MAC based
+        # on the match rule nw_dst=gateway_ip.
+        match = ofpp.OFPMatch(eth_dst=orig_mac_address,
+                              eth_type=ether_types.ETH_TYPE_ARP,
+                              arp_tha=target_mac_address,
+                              arp_op=arp.ARP_REPLY)
+
+        # dst_mac: dvr_host_mac -> qr_dev_mac
+        actions = [
+            ofpp.OFPActionSetField(eth_dst=target_mac_address),
+        ]
+        instructions = [
+            ofpp.OFPInstructionActions(ofp.OFPIT_APPLY_ACTIONS, actions),
+            ofpp.OFPInstructionGotoTable(table_id=constants.TRANSIENT_TABLE)]
+
+        self.install_instructions(table_id=constants.LOCAL_SWITCHING,
+                                  priority=99,
+                                  match=match,
+                                  instructions=instructions)
 
 
 class OVSDVRProcessMixin(object):
@@ -35,17 +77,17 @@ class OVSDVRProcessMixin(object):
     def install_dvr_process_ipv4(self, vlan_tag, gateway_ip):
         # block ARP
         (_dp, ofp, ofpp) = self._get_dp()
-        match = self._dvr_process_ipv4_match(ofp, ofpp,
-            vlan_tag=vlan_tag, gateway_ip=gateway_ip)
-        self.install_drop(table_id=self.dvr_process_table_id,
+        match = self._dvr_process_ipv4_match(ofp, ofpp, vlan_tag=vlan_tag,
+                                             gateway_ip=gateway_ip)
+        self.install_drop(table_id=constants.FLOOD_TO_TUN,
                           priority=3,
                           match=match)
 
     def delete_dvr_process_ipv4(self, vlan_tag, gateway_ip):
         (_dp, ofp, ofpp) = self._get_dp()
-        match = self._dvr_process_ipv4_match(ofp, ofpp,
-            vlan_tag=vlan_tag, gateway_ip=gateway_ip)
-        self.uninstall_flows(table_id=self.dvr_process_table_id,
+        match = self._dvr_process_ipv4_match(ofp, ofpp, vlan_tag=vlan_tag,
+                                             gateway_ip=gateway_ip)
+        self.uninstall_flows(table_id=constants.FLOOD_TO_TUN,
                              match=match)
 
     @staticmethod
@@ -59,16 +101,16 @@ class OVSDVRProcessMixin(object):
     def install_dvr_process_ipv6(self, vlan_tag, gateway_mac):
         # block RA
         (_dp, ofp, ofpp) = self._get_dp()
-        match = self._dvr_process_ipv6_match(ofp, ofpp,
-            vlan_tag=vlan_tag, gateway_mac=gateway_mac)
-        self.install_drop(table_id=self.dvr_process_table_id, priority=3,
+        match = self._dvr_process_ipv6_match(ofp, ofpp, vlan_tag=vlan_tag,
+                                             gateway_mac=gateway_mac)
+        self.install_drop(table_id=constants.FLOOD_TO_TUN, priority=3,
                           match=match)
 
     def delete_dvr_process_ipv6(self, vlan_tag, gateway_mac):
         (_dp, ofp, ofpp) = self._get_dp()
-        match = self._dvr_process_ipv6_match(ofp, ofpp,
-            vlan_tag=vlan_tag, gateway_mac=gateway_mac)
-        self.uninstall_flows(table_id=self.dvr_process_table_id,
+        match = self._dvr_process_ipv6_match(ofp, ofpp, vlan_tag=vlan_tag,
+                                             gateway_mac=gateway_mac)
+        self.uninstall_flows(table_id=constants.FLOOD_TO_TUN,
                              match=match)
 
     @staticmethod
