@@ -27,6 +27,8 @@ from os_ken.lib.packet import in_proto
 from oslo_log import log as logging
 
 from neutron.plugins.ml2.drivers.openvswitch.agent.common import constants
+from neutron.plugins.ml2.drivers.openvswitch.agent.openflow.native \
+    import br_dvr_process
 from neutron.plugins.ml2.drivers.openvswitch.agent.openflow.native \
     import ovs_bridge
 
@@ -34,22 +36,20 @@ from neutron.plugins.ml2.drivers.openvswitch.agent.openflow.native \
 LOG = logging.getLogger(__name__)
 
 
-class OVSIntegrationBridge(ovs_bridge.OVSAgentBridge):
+class OVSIntegrationBridge(ovs_bridge.OVSAgentBridge,
+                           br_dvr_process.OVSDVRInterfaceMixin):
     """openvswitch agent br-int specific logic."""
 
     of_tables = constants.INT_BR_ALL_TABLES
 
     def setup_default_table(self):
-        (_dp, ofp, ofpp) = self._get_dp()
         self.setup_canary_table()
         self.install_goto(dest_table_id=constants.TRANSIENT_TABLE)
         self.install_normal(table_id=constants.TRANSIENT_TABLE, priority=3)
         self.install_drop(table_id=constants.ARP_SPOOF_TABLE)
-        self.install_drop(
-            table_id=constants.LOCAL_SWITCHING,
-            priority=constants.OPENFLOW_MAX_PRIORITY,
-            vlan_vid=ofp.OFPVID_PRESENT | constants.DEAD_VLAN_TAG,
-        )
+        self.install_drop(table_id=constants.LOCAL_SWITCHING,
+                          priority=constants.OPENFLOW_MAX_PRIORITY,
+                          vlan_vid=constants.DEAD_VLAN_TAG)
         # When openflow firewall is not enabled, we use this table to
         # deal with all egress flow.
         self.install_normal(table_id=constants.TRANSIENT_EGRESS_TABLE,
