@@ -515,6 +515,14 @@ class OVSDVRNeutronAgent(object):
         self.local_ports[port.vif_id] = ovsport
         ldm.add_dvr_ofport(port.vif_id, port.ofport)
 
+        if (ip_version == n_const.IP_VERSION_4 and
+                subnet_info.get('gateway_mac')):
+            # Change ARP reply destination MAC address from
+            # dvr_host_mac to gateway_mac.
+            self.int_br.change_arp_destination_mac(
+                target_mac_address=subnet_info['gateway_mac'],
+                orig_mac_address=self.dvr_mac_address)
+
     def _bind_port_on_dvr_subnet(self, port, lvm, fixed_ips,
                                  device_owner):
         # Handle new compute port added use-case
@@ -638,10 +646,9 @@ class OVSDVRNeutronAgent(object):
                      "%(ofport)s, rebinding.",
                      {'vif': port.vif_id, 'ofport': port.ofport})
             self.unbind_port_from_dvr(port, local_vlan_map)
-
-        if port.ofport in (ovs_lib.INVALID_OFPORT,
-                           ovs_lib.UNASSIGNED_OFPORT):
-            return
+            if port.ofport in (ovs_lib.INVALID_OFPORT,
+                               ovs_lib.UNASSIGNED_OFPORT):
+                return
 
         if device_owner == n_const.DEVICE_OWNER_DVR_INTERFACE:
             self._bind_distributed_router_interface_port(port,
@@ -735,6 +742,13 @@ class OVSDVRNeutronAgent(object):
                 self.firewall.delete_accepted_egress_direct_flow(
                     subnet_info['gateway_mac'], lvm.vlan)
 
+            if (ip_version == n_const.IP_VERSION_4 and
+                    subnet_info.get('gateway_mac')):
+                # remove ARP reply destination MAC address change flow
+                self.int_br.delete_arp_destination_change(
+                    target_mac_address=subnet_info['gateway_mac'],
+                    orig_mac_address=self.dvr_mac_address)
+
         if lvm.network_type in constants.DVR_PHYSICAL_NETWORK_TYPES:
             br = self.phys_brs[physical_network]
         if lvm.network_type in constants.TUNNEL_NETWORK_TYPES:
