@@ -20,9 +20,10 @@ import operator
 import signal
 import threading
 import types
+import uuid
 
-import netaddr
 from neutron_lib.api.definitions import portbindings
+from neutron_lib.api.definitions import provider_net
 from neutron_lib.api.definitions import segment as segment_def
 from neutron_lib.callbacks import events
 from neutron_lib.callbacks import registry
@@ -32,25 +33,21 @@ from neutron_lib import context as n_context
 from neutron_lib import exceptions as n_exc
 from neutron_lib.plugins import directory
 from neutron_lib.plugins.ml2 import api
-from neutron_lib.utils import helpers
-from oslo_concurrency import processutils
 from oslo_config import cfg
 from oslo_db import exception as os_db_exc
 from oslo_log import log
 from oslo_utils import timeutils
-from ovsdbapp.backend.ovs_idl import idlutils
 
 from neutron._i18n import _
 from neutron.common.ovn import acl as ovn_acl
 from neutron.common.ovn import constants as ovn_const
-from neutron.common.ovn import exceptions as ovn_exceptions
+from neutron.common.ovn import extensions as ovn_extensions
 from neutron.common.ovn import utils as ovn_utils
 from neutron.common import utils as n_utils
 from neutron.conf.plugins.ml2.drivers.ovn import ovn_conf
 from neutron.db import ovn_hash_ring_db
 from neutron.db import ovn_revision_numbers_db
 from neutron.db import provisioning_blocks
-from neutron.extensions import securitygroup as ext_sg
 from neutron.plugins.ml2 import db as ml2_db
 from neutron.plugins.ml2.drivers.ovn.agent import neutron_agent as n_agent
 from neutron.plugins.ml2.drivers.ovn.mech_driver.ovsdb import impl_idl_ovn
@@ -59,7 +56,7 @@ from neutron.plugins.ml2.drivers.ovn.mech_driver.ovsdb import ovn_client
 from neutron.plugins.ml2.drivers.ovn.mech_driver.ovsdb import ovn_db_sync
 from neutron.plugins.ml2.drivers.ovn.mech_driver.ovsdb import ovsdb_monitor
 from neutron.plugins.ml2.drivers.ovn.mech_driver.ovsdb import worker
-from neutron import service
+from neutron.services.logapi.drivers.ovn import driver as log_driver
 from neutron.services.qos.drivers.ovn import driver as qos_driver
 from neutron.services.segments import db as segment_service_db
 from neutron.services.trunk.drivers.ovn import trunk_driver
@@ -68,7 +65,6 @@ import neutron.wsgi
 
 LOG = log.getLogger(__name__)
 METADATA_READY_WAIT_TIMEOUT = 15
-AGENTS = {}
 
 
 class MetadataServiceReadyWaitTimeoutException(Exception):
@@ -99,6 +95,9 @@ class OVNMechanismDriver(api.MechanismDriver):
     update network/port case, all data validation must be done within
     methods that are part of the database transaction.
     """
+    resource_provider_uuid5_namespace = uuid.UUID(
+        '5533233b-800c-11eb-b1f4-000056b2f5b8')
+
     def initialize(self):
         """Perform driver initialization.
 
@@ -123,12 +122,22 @@ class OVNMechanismDriver(api.MechanismDriver):
         if cfg.CONF.SECURITYGROUP.firewall_driver:
             LOG.warning('Firewall driver configuration is ignored')
         self._setup_vif_port_bindings()
+        if impl_idl_ovn.OvsdbSbOvnIdl.schema_has_table('Chassis_Private'):
+            self.agent_chassis_table = 'Chassis_Private'
+        else:
+            self.agent_chassis_table = 'Chassis'
         self.subscribe()
         self.qos_driver = qos_driver.OVNQosDriver.create(self)
         self.trunk_driver = trunk_driver.OVNTrunkDriver.create(self)
-        # The agent_chassis_table will be changed to Chassis_Private if it
-        # exists, we need to have a connection in order to check that.
-        self.agent_chassis_table = 'Chassis'
+        self.log_driver = log_driver.register(self)
+
+    @property
+    def nb_schema_helper(self):
+        return impl_idl_ovn.OvsdbNbOvnIdl.schema_helper
+
+    @property
+    def sb_schema_helper(self):
+        return impl_idl_ovn.OvsdbSbOvnIdl.schema_helper
 
     @property
     def _plugin(self):
@@ -157,34 +166,41 @@ class OVNMechanismDriver(api.MechanismDriver):
     def sb_ovn(self):
         return self._sb_ovn
 
+    def check_vlan_transparency(self, context):
+        """OVN driver vlan transparency support."""
+        vlan_transparency_network_types = [
+            const.TYPE_LOCAL,
+            const.TYPE_GENEVE,
+            const.TYPE_VXLAN,
+            const.TYPE_VLAN
+        ]
+        return (context.current.get(provider_net.NETWORK_TYPE)
+                in vlan_transparency_network_types)
+
     def _setup_vif_port_bindings(self):
         self.supported_vnic_types = [portbindings.VNIC_NORMAL,
                                      portbindings.VNIC_DIRECT,
                                      portbindings.VNIC_DIRECT_PHYSICAL,
-                                     portbindings.VNIC_MACVTAP]
+                                     portbindings.VNIC_MACVTAP,
+                                     portbindings.VNIC_VHOST_VDPA,
+                                     ]
         self.vif_details = {
             portbindings.VIF_TYPE_OVS: {
-                portbindings.CAP_PORT_FILTER: self.sg_enabled,
-                portbindings.VIF_DETAILS_CONNECTIVITY:
-                    portbindings.CONNECTIVITY_L2,
+                portbindings.CAP_PORT_FILTER: self.sg_enabled
             },
             portbindings.VIF_TYPE_VHOST_USER: {
                 portbindings.CAP_PORT_FILTER: False,
                 portbindings.VHOST_USER_MODE:
                 portbindings.VHOST_USER_MODE_SERVER,
-                portbindings.VHOST_USER_OVS_PLUG: True,
-                portbindings.VIF_DETAILS_CONNECTIVITY:
-                    portbindings.CONNECTIVITY_L2,
+                portbindings.VHOST_USER_OVS_PLUG: True
             },
-            # NOTE(ralonsoh): for stable releases, this parameter is left here
-            # to allow "_check_drivers_connectivity" to check the OVN mech
-            # driver connectivity correctly. This addresses LP#1959125, that
-            # in master branch ("Y") was solved by adding a "connectivity"
-            # property to the "MechanismDriver" class.
             portbindings.VIF_DETAILS_CONNECTIVITY:
                 portbindings.CONNECTIVITY_L2,
         }
 
+    def supported_extensions(self, extensions):
+        return set(ovn_extensions.ML2_SUPPORTED_API_EXTENSIONS) & extensions
+
     def subscribe(self):
         registry.subscribe(self.pre_fork_initialize,
                            resources.PROCESS,
@@ -236,7 +252,6 @@ class OVNMechanismDriver(api.MechanismDriver):
         atexit.register(self._clean_hash_ring)
         signal.signal(signal.SIGTERM, self._clean_hash_ring)
         self._create_neutron_pg_drop()
-        self._set_inactivity_probe()
 
     def _create_neutron_pg_drop(self):
         """Create neutron_pg_drop Port Group.
@@ -248,20 +263,11 @@ class OVNMechanismDriver(api.MechanismDriver):
         otherwise the error is something else and it's raised to the caller.
         """
         idl = ovsdb_monitor.OvnInitPGNbIdl.from_server(
-            ovn_conf.get_ovn_nb_connection(), 'OVN_Northbound', self)
+            ovn_conf.get_ovn_nb_connection(), self.nb_schema_helper, self)
         with ovsdb_monitor.short_living_ovsdb_api(
                 impl_idl_ovn.OvsdbNbOvnIdl, idl) as pre_ovn_nb_api:
             try:
                 create_default_drop_port_group(pre_ovn_nb_api)
-            except KeyError:
-                # Due to a bug in python-ovs, we can send transactions before
-                # the initial OVSDB is populated in memory. This can break
-                # the AddCommand post_commit method which tries to return a
-                # row looked up by the newly commited row's uuid. Since we
-                # don't care about the return value from the PgAddCommand, we
-                # can just catch the KeyError and continue. This can be
-                # removed when the python-ovs bug is resolved.
-                pass
             except RuntimeError as re:
                 if pre_ovn_nb_api.get_port_group(
                         ovn_const.OVN_DROP_PORT_GROUP_NAME):
@@ -273,33 +279,10 @@ class OVNMechanismDriver(api.MechanismDriver):
                 else:
                     raise
 
-    def _set_inactivity_probe(self):
-        """Set 'connection.inactivity_probe' in NB and SB databases"""
-        inactivity_probe = ovn_conf.get_ovn_ovsdb_probe_interval()
-        dbs = [(ovn_conf.get_ovn_nb_connection(), 'OVN_Northbound',
-                impl_idl_ovn.OvsdbNbOvnIdl),
-               (ovn_conf.get_ovn_sb_connection(), 'OVN_Southbound',
-                impl_idl_ovn.OvsdbSbOvnIdl)]
-        for connection, schema, klass in dbs:
-            target = ovn_utils.connection_config_to_target_string(connection)
-            if not target:
-                continue
-
-            idl = ovsdb_monitor.BaseOvnIdl.from_server(connection, schema)
-            with ovsdb_monitor.short_living_ovsdb_api(klass, idl) as idl_api:
-                conn = idlutils.row_by_value(idl_api, 'Connection', 'target',
-                                             target, None)
-                if conn:
-                    idl_api.db_set(
-                        'Connection', target,
-                        ('inactivity_probe', int(inactivity_probe))).execute(
-                        check_error=True)
-
     @staticmethod
     def should_post_fork_initialize(worker_class):
         return worker_class in (neutron.wsgi.WorkerService,
-                                worker.MaintenanceWorker,
-                                service.RpcWorker)
+                                worker.MaintenanceWorker)
 
     def post_fork_initialize(self, resource, event, trigger, payload=None):
         # Initialize API/Maintenance workers with OVN IDL connections
@@ -316,15 +299,9 @@ class OVNMechanismDriver(api.MechanismDriver):
             self.node_uuid = ovn_hash_ring_db.add_node(admin_context,
                                                        self.hash_ring_group)
 
+        n_agent.AgentCache(self)  # Initialize singleton agent cache
         self._nb_ovn, self._sb_ovn = impl_idl_ovn.get_ovn_idls(self, trigger)
 
-        if self._sb_ovn.is_table_present('Chassis_Private'):
-            self.agent_chassis_table = 'Chassis_Private'
-
-        # AGENTS must be populated after fork so if ovn-controller is stopped
-        # before a worker handles a get_agents request, we still show agents
-        populate_agents(self)
-
         # Override agents API methods
         self.patch_plugin_merge("get_agents", get_agents)
         self.patch_plugin_choose("get_agent", get_agent)
@@ -377,7 +354,7 @@ class OVNMechanismDriver(api.MechanismDriver):
                is available to the IDL.
         """
         idl = ovsdb_monitor.OvnInitPGNbIdl.from_server(
-            ovn_conf.get_ovn_nb_connection(), 'OVN_Northbound', self,
+            ovn_conf.get_ovn_nb_connection(), self.nb_schema_helper, self,
             pg_only=True)
         with ovsdb_monitor.short_living_ovsdb_api(
                 impl_idl_ovn.OvsdbNbOvnIdl, idl) as ovn_nb_api:
@@ -387,7 +364,8 @@ class OVNMechanismDriver(api.MechanismDriver):
                                          **kwargs):
         ovn_revision_numbers_db.create_initial_revision(
             kwargs['context'], kwargs['security_group']['id'],
-            ovn_const.TYPE_SECURITY_GROUPS)
+            ovn_const.TYPE_SECURITY_GROUPS,
+            std_attr_id=kwargs['security_group']['standard_attr_id'])
 
     def _create_security_group(self, resource, event, trigger,
                                security_group, **kwargs):
@@ -411,7 +389,8 @@ class OVNMechanismDriver(api.MechanismDriver):
         sg_rule = kwargs.get('security_group_rule')
         context = kwargs.get('context')
         ovn_revision_numbers_db.create_initial_revision(
-            context, sg_rule['id'], ovn_const.TYPE_SECURITY_GROUP_RULES)
+            context, sg_rule['id'], ovn_const.TYPE_SECURITY_GROUP_RULES,
+            std_attr_id=sg_rule['standard_attr_id'])
 
     def _process_sg_rule_notification(
             self, resource, event, trigger, **kwargs):
@@ -419,12 +398,8 @@ class OVNMechanismDriver(api.MechanismDriver):
             self._ovn_client.create_security_group_rule(
                 kwargs['context'], kwargs.get('security_group_rule'))
         elif event == events.BEFORE_DELETE:
-            try:
-                sg_rule = self._plugin.get_security_group_rule(
-                    kwargs['context'], kwargs.get('security_group_rule_id'))
-            except ext_sg.SecurityGroupRuleNotFound:
-                return
-
+            sg_rule = self._plugin.get_security_group_rule(
+                kwargs['context'], kwargs.get('security_group_rule_id'))
             if sg_rule.get('remote_ip_prefix') is not None:
                 if self._sg_has_rules_with_same_normalized_cidr(sg_rule):
                     return
@@ -439,9 +414,6 @@ class OVNMechanismDriver(api.MechanismDriver):
         sg_rules = self._plugin.get_security_group_rules(
             n_context.get_admin_context(),
             {'security_group_id': [sg_rule['security_group_id']]})
-        cidr_sg_rule = netaddr.IPNetwork(sg_rule['remote_ip_prefix'])
-        normalized_sg_rule_prefix = "%s/%s" % (cidr_sg_rule.network,
-                                               cidr_sg_rule.prefixlen)
 
         def _rules_equal(rule1, rule2):
             return not any(
@@ -450,10 +422,7 @@ class OVNMechanismDriver(api.MechanismDriver):
         for rule in sg_rules:
             if not rule.get('remote_ip_prefix') or rule['id'] == sg_rule['id']:
                 continue
-            cidr_rule = netaddr.IPNetwork(rule['remote_ip_prefix'])
-            normalized_rule_prefix = "%s/%s" % (cidr_rule.network,
-                                                cidr_rule.prefixlen)
-            if normalized_sg_rule_prefix != normalized_rule_prefix:
+            if sg_rule.get('normalized_cidr') != rule.get('normalized_cidr'):
                 continue
             if _rules_equal(sg_rule, rule):
                 return True
@@ -463,9 +432,18 @@ class OVNMechanismDriver(api.MechanismDriver):
         return (network_type in [const.TYPE_LOCAL,
                                  const.TYPE_FLAT,
                                  const.TYPE_GENEVE,
+                                 const.TYPE_VXLAN,
                                  const.TYPE_VLAN])
 
+    def _get_max_tunid(self):
+        try:
+            return int(self._nb_ovn.nb_global.options.get('max_tunid'))
+        except (ValueError, TypeError):
+            # max_tunid may be absent in older OVN versions, return None
+            pass
+
     def _validate_network_segments(self, network_segments):
+        max_tunid = self._get_max_tunid()
         for network_segment in network_segments:
             network_type = network_segment['network_type']
             segmentation_id = network_segment['segmentation_id']
@@ -480,6 +458,12 @@ class OVNMechanismDriver(api.MechanismDriver):
             if not self._is_network_type_supported(network_type):
                 msg = _('Network type %s is not supported') % network_type
                 raise n_exc.InvalidInput(error_message=msg)
+            if segmentation_id and max_tunid and segmentation_id > max_tunid:
+                m = (
+                    _('Segmentation ID should be lower or equal to %d') %
+                    max_tunid
+                )
+                raise n_exc.InvalidInput(error_message=m)
 
     def create_segment_provnet_port(self, resource, event, trigger,
                                     context, segment, payload=None):
@@ -509,7 +493,8 @@ class OVNMechanismDriver(api.MechanismDriver):
         self._validate_network_segments(context.network_segments)
         ovn_revision_numbers_db.create_initial_revision(
             context._plugin_context, context.current['id'],
-            ovn_const.TYPE_NETWORKS)
+            ovn_const.TYPE_NETWORKS,
+            std_attr_id=context.current['standard_attr_id'])
 
     def create_network_postcommit(self, context):
         """Create a network.
@@ -586,7 +571,8 @@ class OVNMechanismDriver(api.MechanismDriver):
     def create_subnet_precommit(self, context):
         ovn_revision_numbers_db.create_initial_revision(
             context._plugin_context, context.current['id'],
-            ovn_const.TYPE_SUBNETS)
+            ovn_const.TYPE_SUBNETS,
+            std_attr_id=context.current['standard_attr_id'])
 
     def create_subnet_postcommit(self, context):
         self._ovn_client.create_subnet(context._plugin_context,
@@ -632,14 +618,16 @@ class OVNMechanismDriver(api.MechanismDriver):
                                                  port['id'])
 
         ovn_revision_numbers_db.create_initial_revision(
-            context._plugin_context, port['id'], ovn_const.TYPE_PORTS)
+            context._plugin_context, port['id'], ovn_const.TYPE_PORTS,
+            std_attr_id=context.current['standard_attr_id'])
 
         # in the case of router ports we also need to
         # track the creation and update of the LRP OVN objects
         if ovn_utils.is_lsp_router_port(port):
             ovn_revision_numbers_db.create_initial_revision(
                 context._plugin_context, port['id'],
-                ovn_const.TYPE_ROUTER_PORTS)
+                ovn_const.TYPE_ROUTER_PORTS,
+                std_attr_id=context.current['standard_attr_id'])
 
     def _is_port_provisioning_required(self, port, host, original_host=None):
         vnic_type = port.get(portbindings.VNIC_TYPE, portbindings.VNIC_NORMAL)
@@ -707,32 +695,6 @@ class OVNMechanismDriver(api.MechanismDriver):
                     'device_owner': original_port['device_owner']})
             raise OVNPortUpdateError(resource='port', msg=msg)
 
-    def _ovn_update_port(self, plugin_context, port, original_port,
-                         retry_on_revision_mismatch):
-        try:
-            self._ovn_client.update_port(plugin_context, port,
-                                         port_object=original_port)
-        except ovn_exceptions.RevisionConflict:
-            if retry_on_revision_mismatch:
-                # NOTE(slaweq): I know this is terrible hack but there is no
-                # other way to workaround possible race between port update
-                # event from the OVN (port down on the src node) and API
-                # request from nova-compute to activate binding of the port on
-                # the dest node.
-                original_port_migrating_to = original_port.get(
-                    portbindings.PROFILE, {}).get('migrating_to')
-                port_host_id = port.get(portbindings.HOST_ID)
-                if (original_port_migrating_to is not None and
-                        original_port_migrating_to == port_host_id):
-                    LOG.debug("Revision number of the port %s has changed "
-                              "probably during live migration. Retrying "
-                              "update port in OVN.", port)
-                    db_port = self._plugin.get_port(plugin_context,
-                                                    port['id'])
-                    port['revision_number'] = db_port['revision_number']
-                    self._ovn_update_port(plugin_context, port, original_port,
-                                          retry_on_revision_mismatch=False)
-
     def create_port_postcommit(self, context):
         """Create a port.
 
@@ -779,7 +741,8 @@ class OVNMechanismDriver(api.MechanismDriver):
             if not ovn_utils.is_lsp_router_port(original_port):
                 ovn_revision_numbers_db.create_initial_revision(
                     context._plugin_context, port['id'],
-                    ovn_const.TYPE_ROUTER_PORTS, may_exist=True)
+                    ovn_const.TYPE_ROUTER_PORTS, may_exist=True,
+                    std_attr_id=context.current['standard_attr_id'])
 
     def update_port_postcommit(self, context):
         """Update a port.
@@ -823,8 +786,8 @@ class OVNMechanismDriver(api.MechanismDriver):
             # will fail that OVN has port with bigger revision.
             return
 
-        self._ovn_update_port(context._plugin_context, port, original_port,
-                              retry_on_revision_mismatch=True)
+        self._ovn_client.update_port(context._plugin_context, port,
+                                     port_object=original_port)
         self._notify_dhcp_updated(port['id'])
 
     def delete_port_postcommit(self, context):
@@ -1092,19 +1055,10 @@ class OVNMechanismDriver(api.MechanismDriver):
 
     def delete_mac_binding_entries(self, external_ip):
         """Delete all MAC_Binding entries associated to this IP address"""
-        cmd = ['ovsdb-client', 'transact', ovn_conf.get_ovn_sb_connection(),
-               '--timeout', str(ovn_conf.get_ovn_ovsdb_timeout())]
-
-        if ovn_conf.get_ovn_sb_private_key():
-            cmd += ['-p', ovn_conf.get_ovn_sb_private_key(), '-c',
-                    ovn_conf.get_ovn_sb_certificate(), '-C',
-                    ovn_conf.get_ovn_sb_ca_cert()]
-
-        cmd += ['["OVN_Southbound", {"op": "delete", "table": "MAC_Binding", '
-                '"where": [["ip", "==", "%s"]]}]' % external_ip]
-
-        return processutils.execute(*cmd,
-                                    log_errors=processutils.LOG_FINAL_ERROR)
+        mac_binds = self._sb_ovn.db_find_rows(
+            'MAC_Binding', ('ip', '=', external_ip)).execute() or []
+        for entry in mac_binds:
+            self._sb_ovn.db_destroy('MAC_Binding', entry.uuid).execute()
 
     def update_segment_host_mapping(self, host, phy_nets):
         """Update SegmentHostMapping in DB"""
@@ -1188,52 +1142,6 @@ class OVNMechanismDriver(api.MechanismDriver):
                             " neutron-ovn-metadata-agent status/logs.",
                             port_id)
 
-    def agent_alive(self, agent, update_db):
-        # Allow a maximum of 1 difference between expected and read values
-        # to avoid false positives.
-        if self._nb_ovn.nb_global.nb_cfg - agent.nb_cfg <= 1:
-            if update_db:
-                self.mark_agent_alive(agent)
-            return True
-
-        now = timeutils.utcnow(with_timezone=True)
-        if (now - agent.updated_at).total_seconds() < cfg.CONF.agent_down_time:
-            # down, but not yet timed out
-            return True
-        return False
-
-    def mark_agent_alive(self, agent):
-        # Update the time of our successful check
-        value = timeutils.utcnow(with_timezone=True).isoformat()
-        self._sb_ovn.db_set(
-            self.agent_chassis_table, agent.chassis_private.uuid,
-            ('external_ids', {agent.key: value})).execute(check_error=True)
-
-    def agents_from_chassis(self, chassis_private, update_db=True):
-        agent_dict = {}
-        # For each Chassis there will possibly be a Metadata agent and either
-        # a Controller or Controller Gateway agent.
-        for agent in n_agent.NeutronAgent.agents_from_chassis(chassis_private):
-            if not agent.agent_id:
-                continue
-            alive = self.agent_alive(agent, update_db)
-            agent_dict[agent.agent_id] = agent.as_dict(alive)
-        return agent_dict
-
-    def check_segment_for_agent(self, segment, agent):
-        """Check if the OVN controller agent br mappings has segment physnet
-
-        Only segments on physical networks (flat or vlan) can be associated
-        to a host.
-        """
-        if agent['agent_type'] not in ovn_const.OVN_CONTROLLER_TYPES:
-            return False
-
-        br_map = agent.get('configurations', {}).get('bridge-mappings', '')
-        mapping_dict = helpers.parse_mappings(br_map.split(','),
-                                              unique_values=False)
-        return segment['physical_network'] in mapping_dict
-
     def patch_plugin_merge(self, method_name, new_fn, op=operator.add):
         old_method = getattr(self._plugin, method_name)
 
@@ -1300,42 +1208,22 @@ class OVNMechanismDriver(api.MechanismDriver):
         return azs
 
 
-def populate_agents(driver):
-    for ch in driver._sb_ovn.tables[driver.agent_chassis_table].rows.values():
-        # update the cache, rows are hashed on uuid but it is the name that
-        # stays consistent across ovn-controller restarts
-        AGENTS.update({ch.name: ch})
-
-
 def get_agents(self, context, filters=None, fields=None, _driver=None):
-    update_db = _driver.ping_all_chassis()
+    _driver.ping_all_chassis()
     filters = filters or {}
     agent_list = []
-    populate_agents(_driver)
-    for ch in AGENTS.values():
-        for agent in _driver.agents_from_chassis(ch, update_db).values():
-            if all(agent[k] in v for k, v in filters.items()):
-                agent_list.append(agent)
+    for agent in n_agent.AgentCache():
+        agent_dict = agent.as_dict()
+        if all(agent_dict[k] in v for k, v in filters.items()):
+            agent_list.append(agent_dict)
     return agent_list
 
 
 def get_agent(self, context, id, fields=None, _driver=None):
-    chassis = None
     try:
-        # look up Chassis by *name*, which the id attribute is
-        chassis = _driver._sb_ovn.lookup(_driver.agent_chassis_table, id)
-    except idlutils.RowNotFound:
-        # If the UUID is not found, check for the metadata agent ID
-        for ch in _driver._sb_ovn.tables[
-                _driver.agent_chassis_table].rows.values():
-            metadata_agent_id = ch.external_ids.get(
-                ovn_const.OVN_AGENT_METADATA_ID_KEY)
-            if id == metadata_agent_id:
-                chassis = ch
-                break
-        else:
-            raise n_exc.agent.AgentNotFound(id=id)
-    return _driver.agents_from_chassis(chassis)[id]
+        return n_agent.AgentCache()[id].as_dict()
+    except KeyError:
+        raise n_exc.agent.AgentNotFound(id=id)
 
 
 def update_agent(self, context, id, agent, _driver=None):
@@ -1361,9 +1249,28 @@ def update_agent(self, context, id, agent, _driver=None):
 
 
 def delete_agent(self, context, id, _driver=None):
-    get_agent(self, None, id, _driver=_driver)
-    raise n_exc.BadRequest(resource='agent',
-                           msg='OVN agents cannot be deleted')
+    # raise AgentNotFound if this isn't an ml2/ovn-related agent
+    agent = get_agent(self, None, id, _driver=_driver)
+
+    # NOTE(twilson) According to the API docs, an agent must be disabled
+    # before deletion. Otherwise, behavior seems to be undefined. We could
+    # check that alive=False before allowing deletion, but depending on the
+    # agent_down_time setting, that could take quite a while.
+    # If ovn-controller is up, the Chassis will be recreated and so the agent
+    # will still show as up. The recreated Chassis will cause all kinds of
+    # events to fire. But again, undefined behavior.
+    chassis_name = agent['configurations']['chassis_name']
+    _driver._sb_ovn.chassis_del(chassis_name, if_exists=True).execute(
+        check_error=True)
+    # Send a specific event that all API workers can get to delete the agent
+    # from their caches. Ideally we could send a single transaction that both
+    # created and deleted the key, but alas python-ovs is too "smart"
+    _driver._sb_ovn.db_set(
+        'SB_Global', '.', ('external_ids', {'delete_agent': str(id)})).execute(
+            check_error=True)
+    _driver._sb_ovn.db_remove(
+        'SB_Global', '.', 'external_ids', delete_agent=str(id),
+        if_exists=True).execute(check_error=True)
 
 
 def create_default_drop_port_group(nb_idl):
