@@ -19,11 +19,14 @@ from neutron.objects.qos import policy as qos_policy
 from neutron.objects.qos import rule as qos_rule
 from neutron_lib import constants
 from neutron_lib import context as n_context
+from neutron_lib.plugins import constants as plugins_const
 from neutron_lib.plugins import directory
 from neutron_lib.services.qos import constants as qos_consts
 from oslo_log import log as logging
 
+from neutron.common.ovn import constants as ovn_const
 from neutron.common.ovn import utils
+from neutron.conf.plugins.ml2.drivers.ovn import ovn_conf
 
 
 LOG = logging.getLogger(__name__)
@@ -38,6 +41,7 @@ class OVNClientQosExtension(object):
         super(OVNClientQosExtension, self).__init__()
         self._driver = driver
         self._plugin_property = None
+        self._plugin_l3_property = None
 
     @property
     def _plugin(self):
@@ -45,6 +49,12 @@ class OVNClientQosExtension(object):
             self._plugin_property = directory.get_plugin()
         return self._plugin_property
 
+    @property
+    def _plugin_l3(self):
+        if self._plugin_l3_property is None:
+            self._plugin_l3_property = directory.get_plugin(plugins_const.L3)
+        return self._plugin_l3_property
+
     @staticmethod
     def _qos_rules(context, policy_id):
         """QoS Neutron rules classified per direction and type
@@ -81,7 +91,24 @@ class OVNClientQosExtension(object):
                              'policy_id': policy_id})
         return qos_rules
 
+    @staticmethod
+    def _ovn_qos_rule_match(direction, port_id, ip_address, resident_port):
+        if direction == constants.EGRESS_DIRECTION:
+            in_or_out = 'inport'
+            src_or_dst = 'src'
+        else:
+            in_or_out = 'outport'
+            src_or_dst = 'dst'
+
+        match = '%s == "%s"' % (in_or_out, port_id)
+        if ip_address and resident_port:
+            match += (' && ip4.%s == %s && is_chassis_resident("%s")' %
+                      (src_or_dst, ip_address, resident_port))
+
+        return match
+
     def _ovn_qos_rule(self, rules_direction, rules, port_id, network_id,
+                      fip_id=None, ip_address=None, resident_port=None,
                       delete=False):
         """Generate an OVN QoS register based on several Neutron QoS rules
 
@@ -96,8 +123,16 @@ class OVNClientQosExtension(object):
         :param rules_direction: (string) rules direction (egress, ingress).
         :param rules: (dict) {bw_limit: {max_kbps, max_burst_kbps},
                               dscp: {dscp_mark}}
-        :param port_id: (string) port ID.
+        :param port_id: (string) port ID; for L3 floating IP bandwidth
+                        limit this is the router gateway port ID.
         :param network_id: (string) network ID.
+        :param fip_id: (string) floating IP ID, for L3 floating IP bandwidth
+                       limit.
+        :param ip_address: (string) IP address, for L3 floating IP bandwidth
+                           limit.
+        :param resident_port: (string) for L3 floating IP bandwidth, this is
+                              a logical switch port located in the chassis
+                              where the floating IP traffic is NATed.
         :param delete: (bool) defines if this rule if going to be a partial
                        one (without any bandwidth or DSCP information) to be
                        used only as deletion rule.
@@ -108,17 +143,18 @@ class OVNClientQosExtension(object):
             return
 
         lswitch_name = utils.ovn_name(network_id)
-
-        if rules_direction == constants.EGRESS_DIRECTION:
-            direction = 'from-lport'
-            match = 'inport == "{}"'.format(port_id)
-        else:
-            direction = 'to-lport'
-            match = 'outport == "{}"'.format(port_id)
+        direction = (
+            'from-lport' if rules_direction == constants.EGRESS_DIRECTION else
+            'to-lport')
+        match = self._ovn_qos_rule_match(rules_direction, port_id, ip_address,
+                                         resident_port)
 
         ovn_qos_rule = {'switch': lswitch_name, 'direction': direction,
                         'priority': OVN_QOS_DEFAULT_RULE_PRIORITY,
                         'match': match}
+        if fip_id:
+            ovn_qos_rule['external_ids'] = {
+                ovn_const.OVN_FIP_EXT_ID_KEY: fip_id}
 
         if delete:
             # Any specific rule parameter is left undefined.
@@ -180,14 +216,19 @@ class OVNClientQosExtension(object):
             if ovn_rule:
                 txn.add(self._driver._nb_idl.qos_add(**ovn_rule))
 
-    def create_port(self, txn, port):
-        self.update_port(txn, port, None, reset=True)
+    def create_port(self, txn, port, port_type=None):
+        self.update_port(txn, port, None, reset=True, port_type=port_type)
 
     def delete_port(self, txn, port):
         self.update_port(txn, port, None, delete=True)
 
     def update_port(self, txn, port, original_port, reset=False, delete=False,
-                    qos_rules=None):
+                    qos_rules=None, port_type=None):
+        if port_type == ovn_const.LSP_TYPE_EXTERNAL:
+            # External ports (SR-IOV) QoS is handled the SR-IOV agent QoS
+            # extension.
+            return
+
         if (not reset and not original_port) and not delete:
             # If there is no information about the previous QoS policy, do not
             # make any change, unless the port is new or the QoS information
@@ -234,6 +275,50 @@ class OVNClientQosExtension(object):
 
         return updated_port_ids
 
+    def create_floatingip(self, txn, floatingip):
+        self.update_floatingip(txn, floatingip)
+
+    def update_floatingip(self, txn, floatingip):
+        router_id = floatingip.get('router_id')
+        qos_policy_id = floatingip.get('qos_policy_id')
+        if floatingip['floating_network_id']:
+            lswitch_name = utils.ovn_name(floatingip['floating_network_id'])
+            txn.add(self._driver._nb_idl.qos_del_ext_ids(
+                lswitch_name,
+                {ovn_const.OVN_FIP_EXT_ID_KEY: floatingip['id']}))
+
+        if not (router_id and qos_policy_id):
+            return
+
+        admin_context = n_context.get_admin_context()
+        router_db = self._plugin_l3._get_router(admin_context, router_id)
+        gw_port_id = router_db.get('gw_port_id')
+        if not gw_port_id:
+            return
+
+        if ovn_conf.is_ovn_distributed_floating_ip():
+            # DVR, floating IP GW is in the same compute node as private port.
+            resident_port = floatingip['port_id']
+        else:
+            # Non-DVR, floating IP GW is located where chassisredirect lrp is.
+            resident_port = utils.ovn_cr_lrouter_port_name(gw_port_id)
+
+        qos_rules = self._qos_rules(admin_context, qos_policy_id)
+        for direction, rules in qos_rules.items():
+            ovn_rule = self._ovn_qos_rule(
+                direction, rules, gw_port_id,
+                floatingip['floating_network_id'], fip_id=floatingip['id'],
+                ip_address=floatingip['floating_ip_address'],
+                resident_port=resident_port)
+            if ovn_rule:
+                txn.add(self._driver._nb_idl.qos_add(**ovn_rule))
+
+    def delete_floatingip(self, txn, floatingip):
+        self.update_floatingip(txn, floatingip)
+
+    def disassociate_floatingip(self, txn, floatingip):
+        self.delete_floatingip(txn, floatingip)
+
     def update_policy(self, context, policy):
         updated_port_ids = set([])
         bound_networks = policy.get_bound_networks()
@@ -256,3 +341,8 @@ class OVNClientQosExtension(object):
                                                filters={'id': port_ids}):
                 self.update_port(txn, port, {}, reset=True,
                                  qos_rules=qos_rules)
+
+            for fip_binding in policy.get_bound_floatingips():
+                fip = self._plugin_l3.get_floatingip(context,
+                                                     fip_binding.fip_id)
+                self.update_floatingip(txn, fip)
