@@ -11,16 +11,12 @@
 #    under the License.
 
 import contextlib
-import functools
-import socket
 import uuid
 
 from neutron_lib import exceptions as n_exc
 from neutron_lib.utils import helpers
 from oslo_log import log
 from oslo_utils import uuidutils
-from ovs import socket_util
-from ovs import stream
 from ovsdbapp.backend import ovs_idl
 from ovsdbapp.backend.ovs_idl import connection
 from ovsdbapp.backend.ovs_idl import idlutils
@@ -45,19 +41,6 @@ from neutron.services.portforwarding import constants as pf_const
 LOG = log.getLogger(__name__)
 
 
-# Override wait_for_change to not use a timeout so we always try to reconnect
-def wait_for_change(idl_, timeout, seqno=None):
-    if seqno is None:
-        seqno = idl_.change_seqno
-        while idl_.change_seqno == seqno and not idl_.run():
-            poller = idlutils.poller.Poller()
-            idl_.wait(poller)
-            poller.block()
-
-
-idlutils.wait_for_change = wait_for_change
-
-
 class OvnNbTransaction(idl_trans.Transaction):
 
     def __init__(self, *args, **kwargs):
@@ -72,45 +55,6 @@ class OvnNbTransaction(idl_trans.Transaction):
         self.api.nb_global.increment('nb_cfg')
 
 
-def add_keepalives(fn):
-    @functools.wraps(fn)
-    def _open(*args, **kwargs):
-        error, sock = fn(*args, **kwargs)
-        try:
-            sock.setsockopt(socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1)
-        except socket.error as e:
-            sock.close()
-            return socket_util.get_exception_errno(e), None
-        return error, sock
-    return _open
-
-
-class NoProbesMixin:
-    @staticmethod
-    def needs_probes():
-        # If we are using keepalives, we can force probe_interval=0
-        return False
-
-
-class TCPStream(stream.TCPStream, NoProbesMixin):
-    @classmethod
-    @add_keepalives
-    def _open(cls, suffix, dscp):
-        return super()._open(suffix, dscp)
-
-
-class SSLStream(stream.SSLStream, NoProbesMixin):
-    @classmethod
-    @add_keepalives
-    def _open(cls, suffix, dscp):
-        return super()._open(suffix, dscp)
-
-
-# Overwriting globals in a library is clearly a good idea
-stream.Stream.register_method("tcp", TCPStream)
-stream.Stream.register_method("ssl", SSLStream)
-
-
 # This version of Backend doesn't use a class variable for ovsdb_connection
 # and therefor allows networking-ovn to manage connection scope on its own
 class Backend(ovs_idl.Backend):
@@ -144,6 +88,25 @@ class Backend(ovs_idl.Backend):
     def connection_string(cls):
         raise NotImplementedError()
 
+    @n_utils.classproperty
+    def schema_helper(cls):
+        # SchemaHelper.get_idl_schema() sets schema_json to None which is
+        # called in Idl.__init__(), so if we've done that return new helper
+        try:
+            if cls._schema_helper.schema_json:
+                return cls._schema_helper
+        except AttributeError:
+            pass
+
+        ovsdb_monitor._check_and_set_ssl_files(cls.schema)
+        cls._schema_helper = idlutils.get_schema_helper(cls.connection_string,
+                                                        cls.schema)
+        return cls._schema_helper
+
+    @classmethod
+    def schema_has_table(cls, table_name):
+        return table_name in cls.schema_helper.schema_json['tables']
+
     def is_table_present(self, table_name):
         return table_name in self._tables
 
@@ -210,7 +173,7 @@ class OvsdbNbOvnIdl(nb_impl_idl.OvnNbApiIdlImpl, Backend):
 
     @classmethod
     def from_worker(cls, worker_class, driver=None):
-        args = (cfg.get_ovn_nb_connection(), 'OVN_Northbound')
+        args = (cls.connection_string, cls.schema_helper)
         if worker_class == worker.MaintenanceWorker:
             idl_ = ovsdb_monitor.BaseOvnIdl.from_server(*args)
         else:
@@ -236,14 +199,11 @@ class OvsdbNbOvnIdl(nb_impl_idl.OvnNbApiIdlImpl, Backend):
         This method is just a wrapper around the ovsdbapp transaction
         to handle revision conflicts correctly.
         """
-        revision_mismatch_raise = kwargs.pop('revision_mismatch_raise', False)
         try:
             with super(OvsdbNbOvnIdl, self).transaction(*args, **kwargs) as t:
                 yield t
         except ovn_exc.RevisionConflict as e:
             LOG.info('Transaction aborted. Reason: %s', e)
-            if revision_mismatch_raise:
-                raise e
 
     def create_lswitch_port(self, lport_name, lswitch_name, may_exist=True,
                             **columns):
@@ -531,7 +491,7 @@ class OvsdbNbOvnIdl(nb_impl_idl.OvnNbApiIdlImpl, Backend):
                 'external_ids': ext_ids, 'uuid': row.uuid}
 
     def get_subnet_dhcp_options(self, subnet_id, with_ports=False):
-        subnet = {}
+        subnet = None
         ports = []
         for row in self._tables['DHCP_Options'].rows.values():
             external_ids = getattr(row, 'external_ids', {})
@@ -578,10 +538,7 @@ class OvsdbNbOvnIdl(nb_impl_idl.OvnNbApiIdlImpl, Backend):
     def get_address_sets(self):
         address_sets = {}
         for row in self._tables['Address_Set'].rows.values():
-            # TODO(lucasagomes): Remove OVN_SG_NAME_EXT_ID_KEY in the
-            # Rocky release
-            if not (ovn_const.OVN_SG_EXT_ID_KEY in row.external_ids or
-               ovn_const.OVN_SG_NAME_EXT_ID_KEY in row.external_ids):
+            if not (ovn_const.OVN_SG_EXT_ID_KEY in row.external_ids):
                 continue
             name = getattr(row, 'name')
             data = {}
@@ -621,12 +578,7 @@ class OvsdbNbOvnIdl(nb_impl_idl.OvnNbApiIdlImpl, Backend):
 
         nat_rules = []
         for nat_rule in getattr(lrouter, 'nat', []):
-            ext_ids = {}
-            # TODO(dalvarez): remove this check once the minimum OVS required
-            # version contains the column (when OVS 2.8.2 is released).
-            if self.is_col_present('NAT', 'external_ids'):
-                ext_ids = dict(getattr(nat_rule, 'external_ids', {}))
-
+            ext_ids = dict(getattr(nat_rule, 'external_ids', {}))
             nat_rules.append({'external_ip': nat_rule.external_ip,
                               'logical_ip': nat_rule.logical_ip,
                               'type': nat_rule.type,
@@ -697,11 +649,6 @@ class OvsdbNbOvnIdl(nb_impl_idl.OvnNbApiIdlImpl, Backend):
         return result[0] if result else None
 
     def get_floatingip(self, fip_id):
-        # TODO(dalvarez): remove this check once the minimum OVS required
-        # version contains the column (when OVS 2.8.2 is released).
-        if not self.is_col_present('NAT', 'external_ids'):
-            return
-
         fip = self.db_find('NAT', ('external_ids', '=',
                                    {ovn_const.OVN_FIP_EXT_ID_KEY: fip_id}))
         result = fip.execute(check_error=True)
@@ -725,12 +672,11 @@ class OvsdbNbOvnIdl(nb_impl_idl.OvnNbApiIdlImpl, Backend):
     def get_lrouter(self, lrouter_name):
         if uuidutils.is_uuid_like(lrouter_name):
             lrouter_name = utils.ovn_name(lrouter_name)
-
-        # TODO(lucasagomes): Use lr_get() once we start refactoring this
-        # API to use methods from ovsdbapp.
-        lr = self.db_find_rows('Logical_Router', ('name', '=', lrouter_name))
-        result = lr.execute(check_error=True)
-        return result[0] if result else None
+        try:
+            return self.lr_get(lrouter_name).execute(log_errors=False,
+                                                     check_error=True)
+        except idlutils.RowNotFound:
+            return None
 
     def get_lrouter_port(self, lrp_name):
         # TODO(mangelajo): Implement lrp_get() ovsdbapp and use from here
@@ -810,7 +756,7 @@ class OvsdbSbOvnIdl(sb_impl_idl.OvnSbApiIdlImpl, Backend):
 
     @classmethod
     def from_worker(cls, worker_class, driver=None):
-        args = (cfg.get_ovn_sb_connection(), 'OVN_Southbound')
+        args = (cls.connection_string, cls.schema_helper)
         if worker_class == worker.MaintenanceWorker:
             idl_ = ovsdb_monitor.BaseOvnSbIdl.from_server(*args)
         else:
@@ -820,7 +766,8 @@ class OvsdbSbOvnIdl(sb_impl_idl.OvnSbApiIdlImpl, Backend):
 
     def _get_chassis_physnets(self, chassis):
         bridge_mappings = chassis.external_ids.get('ovn-bridge-mappings', '')
-        mapping_dict = helpers.parse_mappings(bridge_mappings.split(','))
+        mapping_dict = helpers.parse_mappings(bridge_mappings.split(','),
+                                              unique_values=False)
         return list(mapping_dict.keys())
 
     def chassis_exists(self, hostname):
@@ -904,24 +851,15 @@ class OvsdbSbOvnIdl(sb_impl_idl.OvnSbApiIdlImpl, Backend):
         rows = self.db_list_rows('Port_Binding').execute(check_error=True)
         # TODO(twilson) It would be useful to have a db_find that takes a
         # comparison function
-
-        def check_net_and_ip(port):
-            # If the port is not bound to any chassis it is not relevant
-            if not port.chassis:
-                return False
-
-            # TODO(dalvarez): Remove the comparison to port.datapath.uuid in Y
-            # cycle when we are sure that all namespaces will be created with
-            # the Neutron network UUID and not anymore with the OVN datapath
-            # UUID.
-            is_in_network = lambda port: (
-                str(port.datapath.uuid) == network or
-                utils.get_network_name_from_datapath(port.datapath) == network)
-
-            return port.mac and is_in_network(port) and (
-                    ip_address in port.mac[0].split(' '))
-
-        return [r for r in rows if check_net_and_ip(r)]
+        # TODO(dalvarez): Remove the comparison to r.datapath.uuid in Y cycle
+        # when we are sure that all namespaces will be created with the
+        # Neutron network UUID and not anymore with the OVN datapath UUID.
+        return [r for r in rows
+                if (r.mac and (
+                    str(r.datapath.uuid) == network or
+                    utils.get_network_name_from_datapath(
+                        r.datapath) == network)) and
+                ip_address in r.mac[0].split(' ')]
 
     def set_port_cidrs(self, name, cidrs):
         # TODO(twilson) add if_exists to db commands
@@ -937,7 +875,6 @@ class OvsdbSbOvnIdl(sb_impl_idl.OvnSbApiIdlImpl, Backend):
     def get_logical_port_chassis_and_datapath(self, name):
         for port in self._tables['Port_Binding'].rows.values():
             if port.logical_port == name:
-                network_id = utils.get_network_name_from_datapath(
-                    port.datapath)
+                datapath = str(port.datapath.uuid)
                 chassis = port.chassis[0].name if port.chassis else None
-                return chassis, network_id
+                return chassis, datapath
