@@ -19,6 +19,7 @@ from neutron_lib.api.definitions import l3
 from neutron_lib.api.definitions import segment as segment_def
 from neutron_lib import constants
 from neutron_lib import context
+from neutron_lib.db import api as db_api
 from neutron_lib import exceptions as n_exc
 from neutron_lib.plugins import constants as plugin_constants
 from neutron_lib.plugins import directory
@@ -164,7 +165,7 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
 
         neutron_sgs = {}
         neutron_pgs = set()
-        with ctx.session.begin(subtransactions=True):
+        with db_api.CONTEXT_READER.using(ctx):
             for sg in self.core_plugin.get_security_groups(ctx):
                 pg_name = utils.ovn_port_group_name(sg['id'])
                 neutron_pgs.add(pg_name)
@@ -223,14 +224,16 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
 
     def _get_acls_from_port_groups(self):
         ovn_acls = []
-        acl_columns = (self.ovn_api._tables['ACL'].columns.keys() &
-                       set(ovn_const.ACL_EXPECTED_COLUMNS_NBDB))
-        acl_columns.discard('external_ids')
-        for pg in self.ovn_api.db_list_rows('Port_Group').execute():
+        port_groups = self.ovn_api.db_list_rows('Port_Group').execute()
+        for pg in port_groups:
             acls = getattr(pg, 'acls', [])
             for acl in acls:
-                acl_string = {k: getattr(acl, k) for k in acl_columns}
+                acl_string = {}
                 acl_string['port_group'] = pg.name
+                for acl_key in getattr(acl, "_data", {}):
+                    acl_string[acl_key] = getattr(acl, acl_key)
+                acl_string.pop('meter')
+                acl_string.pop('external_ids')
                 ovn_acls.append(acl_string)
         return ovn_acls
 
@@ -282,7 +285,7 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
                 for acla in neutron_acls:
                     LOG.warning('ACL found in Neutron but not in '
                                 'OVN DB for port group %s', acla['port_group'])
-                    txn.add(self.ovn_api.pg_acl_add(**acla, may_exist=True))
+                    txn.add(self.ovn_api.pg_acl_add(**acla))
 
             with self.ovn_api.transaction(check_error=True) as txn:
                 for aclr in ovn_acls:
@@ -857,16 +860,16 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
         # TODO(mjozefcz): Remove constants.DEVICE_OWNER_DHCP
         # from get_ports in W-release.
         for net in self.core_plugin.get_networks(ctx):
-            # Get only DHCP ports that don't belong to agent, it should return
-            # only OVN metadata ports
-            dhcp_ports = [
-                p for p in self.core_plugin.get_ports(
-                    ctx, filters=dict(
-                        network_id=[net['id']],
-                        device_owner=[
-                            constants.DEVICE_OWNER_DISTRIBUTED,
-                            constants.DEVICE_OWNER_DHCP]))
-                if not utils.is_neutron_dhcp_agent_port(p)]
+            dhcp_ports = self.core_plugin.get_ports(ctx, filters=dict(
+                network_id=[net['id']],
+                device_owner=[
+                    constants.DEVICE_OWNER_DISTRIBUTED,
+                    constants.DEVICE_OWNER_DHCP]))
+
+            for port in dhcp_ports:
+                # Do not touch the Neutron DHCP agents ports
+                if utils.is_neutron_dhcp_agent_port(port):
+                    dhcp_ports.remove(port)
 
             if not dhcp_ports:
                 LOG.warning('Missing metadata port found in Neutron for '
