@@ -23,7 +23,6 @@ from neutron_lib.utils import helpers
 from oslo_config import cfg
 from oslo_log import log
 from oslo_utils import timeutils
-from ovs.db import idl as ovs_idl_mod
 from ovs.stream import Stream
 from ovsdbapp.backend.ovs_idl import connection
 from ovsdbapp.backend.ovs_idl import event as row_event
@@ -35,7 +34,7 @@ from neutron.common.ovn import hash_ring_manager
 from neutron.common.ovn import utils
 from neutron.conf.plugins.ml2.drivers.ovn import ovn_conf
 from neutron.db import ovn_hash_ring_db
-from neutron.plugins.ml2.drivers.ovn.mech_driver.ovsdb import backports
+from neutron.plugins.ml2.drivers.ovn.agent import neutron_agent as n_agent
 
 
 CONF = cfg.CONF
@@ -138,7 +137,8 @@ class ChassisEvent(row_event.RowEvent):
         phy_nets = []
         if event != self.ROW_DELETE:
             bridge_mappings = row.external_ids.get('ovn-bridge-mappings', '')
-            mapping_dict = helpers.parse_mappings(bridge_mappings.split(','))
+            mapping_dict = helpers.parse_mappings(bridge_mappings.split(','),
+                                                  unique_values=False)
             phy_nets = list(mapping_dict)
 
         self.driver.update_segment_host_mapping(host, phy_nets)
@@ -173,6 +173,143 @@ class ChassisEvent(row_event.RowEvent):
         self.handle_ha_chassis_group_changes(event, row, old)
 
 
+class PortBindingChassisUpdateEvent(row_event.RowEvent):
+    """Event for matching a port moving chassis
+
+    If the LSP is up and the Port_Binding chassis has just changed,
+    there is a good chance the host died without cleaning up the chassis
+    column on the Port_Binding. The port never goes down, so we won't
+    see update the driver with the LogicalSwitchPortUpdateUpEvent which
+    only monitors for transitions from DOWN to UP.
+    """
+
+    def __init__(self, driver):
+        self.driver = driver
+        table = 'Port_Binding'
+        events = (self.ROW_UPDATE,)
+        super(PortBindingChassisUpdateEvent, self).__init__(
+            events, table, None)
+        self.event_name = self.__class__.__name__
+
+    def match_fn(self, event, row, old=None):
+        # NOTE(twilson) ROW_UPDATE events always pass old, but chassis will
+        # only be set if chassis has changed
+        old_chassis = getattr(old, 'chassis', None)
+        if not (row.chassis and old_chassis) or row.chassis == old_chassis:
+            return False
+        if row.type == ovn_const.OVN_CHASSIS_REDIRECT:
+            return False
+        try:
+            lsp = self.driver._nb_ovn.lookup('Logical_Switch_Port',
+                                             row.logical_port)
+        except idlutils.RowNotFound:
+            LOG.warning("Logical Switch Port %(port)s not found for "
+                        "Port_Binding %(binding)s",
+                        {'port': row.logical_port, 'binding': row.uuid})
+            return False
+
+        return bool(lsp.up)
+
+    def run(self, event, row, old=None):
+        self.driver.set_port_status_up(row.logical_port)
+
+
+class ChassisAgentEvent(BaseEvent):
+    GLOBAL = True
+
+    # NOTE (twilson) Do not run new transactions out of a GLOBAL Event since
+    # it will be running on every single process, and you almost certainly
+    # don't want to insert/update/delete something a bajillion times.
+    def __init__(self, driver):
+        self.driver = driver
+        super().__init__()
+
+    @property
+    def table(self):
+        # It probably doesn't matter, but since agent_chassis_table changes
+        # in post_fork_initialize(), resolve this at runtime
+        return self.driver.agent_chassis_table
+
+    @table.setter
+    def table(self, value):
+        pass
+
+
+class ChassisAgentDownEvent(ChassisAgentEvent):
+    events = (BaseEvent.ROW_DELETE,)
+
+    def run(self, event, row, old):
+        for agent in n_agent.AgentCache().agents_by_chassis_private(row):
+            agent.set_down = True
+
+    def match_fn(self, event, row, old=None):
+        return True
+
+
+class ChassisAgentDeleteEvent(ChassisAgentEvent):
+    events = (BaseEvent.ROW_UPDATE,)
+    table = 'SB_Global'
+
+    def match_fn(self, event, row, old=None):
+        try:
+            return (old.external_ids.get('delete_agent') !=
+                    row.external_ids['delete_agent'])
+        except (AttributeError, KeyError):
+            return False
+
+    def run(self, event, row, old):
+        del n_agent.AgentCache()[row.external_ids['delete_agent']]
+
+
+class ChassisAgentWriteEvent(ChassisAgentEvent):
+    events = (BaseEvent.ROW_CREATE, BaseEvent.ROW_UPDATE)
+
+    def match_fn(self, event, row, old=None):
+        # On updates to Chassis_Private because the Chassis has been deleted,
+        # don't update the AgentCache. We use chassis_private.chassis to return
+        # data about the agent.
+        return event == self.ROW_CREATE or (
+            getattr(old, 'nb_cfg', False) and not
+            (self.table == 'Chassis_Private' and not row.chassis))
+
+    def run(self, event, row, old):
+        n_agent.AgentCache().update(ovn_const.OVN_CONTROLLER_AGENT, row,
+                                    clear_down=event == self.ROW_CREATE)
+
+
+class ChassisMetadataAgentWriteEvent(ChassisAgentEvent):
+    events = (BaseEvent.ROW_CREATE, BaseEvent.ROW_UPDATE)
+
+    @staticmethod
+    def _metadata_nb_cfg(row):
+        return int(
+            row.external_ids.get(ovn_const.OVN_AGENT_METADATA_SB_CFG_KEY, -1))
+
+    @staticmethod
+    def agent_id(row):
+        return row.external_ids.get(ovn_const.OVN_AGENT_METADATA_ID_KEY)
+
+    def match_fn(self, event, row, old=None):
+        if not self.agent_id(row):
+            # Don't create a cached object with an agent_id of 'None'
+            return False
+        if event == self.ROW_CREATE:
+            return True
+        try:
+            # On updates to Chassis_Private because the Chassis has been
+            # deleted, don't update the AgentCache. We use
+            # chassis_private.chassis to return data about the agent.
+            if self.table == 'Chassis_Private' and not row.chassis:
+                return False
+            return self._metadata_nb_cfg(row) != self._metadata_nb_cfg(old)
+        except (AttributeError, KeyError):
+            return False
+
+    def run(self, event, row, old):
+        n_agent.AgentCache().update(ovn_const.OVN_METADATA_AGENT, row,
+                                    clear_down=True)
+
+
 class PortBindingChassisEvent(row_event.RowEvent):
     """Port_Binding update event - set chassis for chassisredirect port.
 
@@ -206,8 +343,8 @@ class PortBindingChassisEvent(row_event.RowEvent):
             router, host)
 
 
-class PortBindingCreateUpEvent(row_event.RowEvent):
-    """Row create event - Port_Binding 'up' = True.
+class LogicalSwitchPortCreateUpEvent(row_event.RowEvent):
+    """Row create event - Logical_Switch_Port 'up' = True.
 
     On connection, we get a dump of all ports, so if there is a neutron
     port that is down that has since been activated, we'll catch it here.
@@ -216,200 +353,73 @@ class PortBindingCreateUpEvent(row_event.RowEvent):
 
     def __init__(self, driver):
         self.driver = driver
-        table = 'Port_Binding'
+        table = 'Logical_Switch_Port'
         events = (self.ROW_CREATE,)
-        super().__init__(events, table, None)
-        self.event_name = 'PortBindingCreateUpEvent'
-
-    def match_fn(self, event, row, old):
-        if row.type in (ovn_const.PB_TYPE_VIRTUAL,
-                        ovn_const.OVN_CHASSIS_REDIRECT):
-            # NOTE(ltomasbo): Skipping virtual ports as they are not being
-            # set to ACTIVE
-            # NOTE(ltomasbo): No need to handle cr ports
-            return False
-        if row.type == ovn_const.PB_TYPE_PATCH:
-            # NOTE(ltomasbo): Only handle the logical_switch_port side,
-            # not the router side.
-            if (row.logical_port.startswith('lrp-') or
-                    row.logical_port.startswith('cr-lrp')):
-                return False
-            return True
-        # TODO(ltomasbo): Remove the checkings for 'up' column once minimal
-        # ovn version has it (v21.03.0). The match_fn can be then replaced
-        # by different init method above:
-        # super().__init__(
-        #     events, table, (('up', '=', True), ('type', '=', ''),))
-        if hasattr(row, 'up'):
-            # NOTE(ltomasbo): Due to bug in core ovn not setting the up field
-            # to DOWN in some cases (for example subports detachment from
-            # trunks), we need to also check the chassis is set to claim the
-            # port as ACTIVE
-            return row.chassis and bool(row.up[0])
-        elif row.chassis:
-            return True
-        return False
+        super(LogicalSwitchPortCreateUpEvent, self).__init__(
+            events, table, (('up', '=', True),))
+        self.event_name = 'LogicalSwitchPortCreateUpEvent'
 
     def run(self, event, row, old):
-        self.driver.set_port_status_up(row.logical_port)
+        self.driver.set_port_status_up(row.name)
 
 
-class PortBindingCreateDownEvent(row_event.RowEvent):
-    """Row create event - Port_Binding 'up' = False
+class LogicalSwitchPortCreateDownEvent(row_event.RowEvent):
+    """Row create event - Logical_Switch_Port 'up' = False
 
     On connection, we get a dump of all ports, so if there is a neutron
     port that is up that has since been deactivated, we'll catch it here.
     This event will not be generated for new ports getting created.
     """
-
     def __init__(self, driver):
         self.driver = driver
-        table = 'Port_Binding'
+        table = 'Logical_Switch_Port'
         events = (self.ROW_CREATE,)
-        super().__init__(events, table, None)
-        self.event_name = 'PortBindingCreateDownEvent'
-
-    def match_fn(self, event, row, old):
-        if row.type in [ovn_const.PB_TYPE_VIRTUAL, ovn_const.PB_TYPE_PATCH,
-                        ovn_const.OVN_CHASSIS_REDIRECT]:
-            # NOTE(ltomasbo): Skipping as virtual ports are not being set to
-            # ACTIVE
-            # Patch ports are set to UP on creation, no need to update
-            # No need to handle cr ports
-            return False
-
-        # TODO(ltomasbo): Remove the checkings for 'up' column once minimal
-        # ovn version has it (v21.03.0). The match_fn can be then replaced
-        # by different init method above:
-        # super().__init__(
-        #     events, table, (('up', '=', False), ('type', '=', ''),))
-        if hasattr(row, 'up'):
-            # NOTE(ltomasbo): Due to bug in core ovn not setting the up field
-            # to DOWN in some cases (for example subports detachment from
-            # trunks), we need to also check if the chassis is unset to set
-            # the port as DOWN
-            return not row.chassis or not bool(row.up[0])
-        elif not row.chassis:
-            return True
-        return False
+        super(LogicalSwitchPortCreateDownEvent, self).__init__(
+            events, table, (('up', '=', False),))
+        self.event_name = 'LogicalSwitchPortCreateDownEvent'
 
     def run(self, event, row, old):
-        self.driver.set_port_status_down(row.logical_port)
+        self.driver.set_port_status_down(row.name)
 
 
-class PortBindingUpdateUpEvent(row_event.RowEvent):
-    """Row update event - Port_Binding 'up' going from False to True
+class LogicalSwitchPortUpdateUpEvent(row_event.RowEvent):
+    """Row update event - Logical_Switch_Port 'up' going from False to True
 
     This happens when the VM goes up.
-    New value of Port_Binding 'up' will be True and the old value will
-    be False. Or if that column does not exists, the chassis will be set
-    and the old chassis value will be empty.
+    New value of Logical_Switch_Port 'up' will be True and the old value will
+    be False.
     """
-
     def __init__(self, driver):
         self.driver = driver
-        table = 'Port_Binding'
+        table = 'Logical_Switch_Port'
         events = (self.ROW_UPDATE,)
-        super().__init__(events, table, None)
-        self.event_name = 'PortBindingUpdateUpEvent'
-
-    def match_fn(self, event, row, old):
-        if row.type in (ovn_const.PB_TYPE_VIRTUAL,
-                        ovn_const.OVN_CHASSIS_REDIRECT):
-            # NOTE(ltomasbo): Skipping virtual ports as they are not being
-            # set to ACTIVE
-            # NOTE(ltomasbo): No need to handle cr ports
-            return False
-        if row.type == ovn_const.PB_TYPE_PATCH:
-            # NOTE(ltomasbo): Only handle the logical_switch_port side,
-            # not the router side.
-            if (row.logical_port.startswith('lrp-') or
-                    row.logical_port.startswith('cr-lrp')):
-                return False
-            try:
-                if old.mac:
-                    # NOTE(ltomasbo): only execute it once (the first update
-                    # event for this port), as you don't need to set it to
-                    # active several time
-                    return True
-            except AttributeError:
-                return False
-            return False
-        # TODO(ltomasbo): Remove the checkings for 'up' column once minimal
-        # ovn version has it (v21.03.0). The match_fn can be then replaced
-        # by different init method above:
-        # super().__init__(
-        #     events, table, (('up', '=', True), ('type', '=', '')),
-        #     old_conditions=(('up', '=', False),))
-        try:
-            if hasattr(row, 'up'):
-                # NOTE(ltomasbo): Due to bug in core ovn not setting the up
-                # field to DOWN in some cases (for example subports detachment
-                # from trunks), we need to also check the chassis is set to
-                # claim the port as ACTIVE
-                return (bool(row.up[0]) and not bool(old.up[0]) and
-                        row.chassis)
-            elif row.chassis and not old.chassis:
-                return True
-        except AttributeError:
-            # NOTE(ltomasbo): do not process if there is no old up/chassis
-            # information
-            return False
-        return False
+        super(LogicalSwitchPortUpdateUpEvent, self).__init__(
+            events, table, (('up', '=', True),),
+            old_conditions=(('up', '=', False),))
+        self.event_name = 'LogicalSwitchPortUpdateUpEvent'
 
     def run(self, event, row, old):
-        self.driver.set_port_status_up(row.logical_port)
+        self.driver.set_port_status_up(row.name)
 
 
-class PortBindingUpdateDownEvent(row_event.RowEvent):
-    """Row update event - Port_Binding 'up' going from True to False
+class LogicalSwitchPortUpdateDownEvent(row_event.RowEvent):
+    """Row update event - Logical_Switch_Port 'up' going from True to False
 
     This happens when the VM goes down.
-    New value of Port_Binding 'up' will be False and the old value will
-    be True. Or if that column does not exists, the chassis will be unset
-    and the old chassis will be set.
+    New value of Logical_Switch_Port 'up' will be False and the old value will
+    be True.
     """
-
     def __init__(self, driver):
         self.driver = driver
-        table = 'Port_Binding'
+        table = 'Logical_Switch_Port'
         events = (self.ROW_UPDATE,)
-        super().__init__(events, table, None)
-        self.event_name = 'PortBindingUpdateDownEvent'
-
-    def match_fn(self, event, row, old):
-        if row.type in [ovn_const.PB_TYPE_VIRTUAL, ovn_const.PB_TYPE_PATCH,
-                        ovn_const.OVN_CHASSIS_REDIRECT]:
-            # NOTE(ltomasbo): Skipping as virtual ports are not being set to
-            # ACTIVE
-            # Patch ports are meant to be always UP, after creation, no need
-            # to update
-            # No need to handle cr ports
-            return False
-        # TODO(ltomasbo): Remove the checkings for 'up' column once minimal
-        # ovn version has it (v21.03.0). The match_fn can be then replaced
-        # by different init method above:
-        # super().__init__(
-        #     events, table, (('up', '=', False), ('type', '=', '')),
-        #     old_conditions=(('up', '=', True),))
-        try:
-            if hasattr(row, 'up'):
-                # NOTE(ltomasbo): Due to bug in core ovn not setting the up
-                # field to DOWN in some cases (for example subports detachment
-                # from trunks), we need to also check if the chassis is being
-                # unset to set the port as DOWN
-                return ((not bool(row.up[0]) and bool(old.up[0])) or
-                        (not row.chassis and old.chassis))
-            elif not row.chassis and old.chassis:
-                return True
-        except AttributeError:
-            # NOTE(ltomasbo): do not process if there is no old up/chassis
-            # information
-            return False
-        return False
+        super(LogicalSwitchPortUpdateDownEvent, self).__init__(
+            events, table, (('up', '=', False),),
+            old_conditions=(('up', '=', True),))
+        self.event_name = 'LogicalSwitchPortUpdateDownEvent'
 
     def run(self, event, row, old):
-        self.driver.set_port_status_down(row.logical_port)
+        self.driver.set_port_status_down(row.name)
 
 
 class FIPAddDeleteEvent(row_event.RowEvent):
@@ -444,10 +454,27 @@ class NeutronPgDropPortGroupCreated(row_event.WaitEvent):
         self.event_name = 'PortGroupCreated'
 
 
-class OvnDbNotifyHandler(backports.RowEventHandler):
+class OvnDbNotifyHandler(row_event.RowEventHandler):
     def __init__(self, driver):
-        super(OvnDbNotifyHandler, self).__init__()
         self.driver = driver
+        super(OvnDbNotifyHandler, self).__init__()
+        try:
+            self._lock = self._RowEventHandler__lock
+            self._watched_events = self._RowEventHandler__watched_events
+        except AttributeError:
+            pass
+
+    def notify(self, event, row, updates=None, global_=False):
+        row = idlutils.frozen_row(row)
+        matching = self.matching_events(event, row, updates, global_)
+        for match in matching:
+            self.notifications.put((match, event, row, updates))
+
+    def matching_events(self, event, row, updates, global_=False):
+        with self._lock:
+            return tuple(t for t in self._watched_events
+                         if getattr(t, 'GLOBAL', False) == global_ and
+                         self.match(t, event, row, updates))
 
 
 class Ml2OvnIdlBase(connection.OvsdbIdl):
@@ -457,25 +484,14 @@ class Ml2OvnIdlBase(connection.OvsdbIdl):
         super(Ml2OvnIdlBase, self).__init__(
             remote, schema, probe_interval=probe_interval, **kwargs)
 
-    def set_table_condition(self, table_name, condition):
-        # Prior to ovs commit 46d44cf3be0, self.cond_change() doesn't work here
-        # but after that commit, setting table.condtion doesn't work.
-        if hasattr(ovs_idl_mod, 'ConditionState'):
-            self.cond_change(table_name, condition)
-        else:
-            # Can be removed after the minimum ovs version >= 2.17.0
-            self.tables[table_name].condition = condition
-
 
 class BaseOvnIdl(Ml2OvnIdlBase):
-    def __init__(self, remote, schema, **kwargs):
-        self.notify_handler = backports.RowEventHandler()
-        super(BaseOvnIdl, self).__init__(remote, schema, **kwargs)
+    def __init__(self, remote, schema):
+        self.notify_handler = row_event.RowEventHandler()
+        super(BaseOvnIdl, self).__init__(remote, schema)
 
     @classmethod
-    def from_server(cls, connection_string, schema_name):
-        _check_and_set_ssl_files(schema_name)
-        helper = idlutils.get_schema_helper(connection_string, schema_name)
+    def from_server(cls, connection_string, helper):
         helper.register_all()
         return cls(connection_string, helper)
 
@@ -485,25 +501,18 @@ class BaseOvnIdl(Ml2OvnIdlBase):
 
 class BaseOvnSbIdl(Ml2OvnIdlBase):
     @classmethod
-    def from_server(cls, connection_string, schema_name):
-        _check_and_set_ssl_files(schema_name)
-        helper = idlutils.get_schema_helper(connection_string, schema_name)
+    def from_server(cls, connection_string, helper):
         helper.register_table('Chassis')
         helper.register_table('Encap')
         helper.register_table('Port_Binding')
         helper.register_table('Datapath_Binding')
-        # Used by MaintenanceWorker which can use ovsdb locking
-        try:
-            return cls(connection_string, helper, leader_only=True)
-        except TypeError:
-            # TODO(twilson) We can remove this when we require ovs>=2.12.0
-            return cls(connection_string, helper)
+        return cls(connection_string, helper)
 
 
 class OvnIdl(BaseOvnIdl):
 
-    def __init__(self, driver, remote, schema, **kwargs):
-        super(OvnIdl, self).__init__(remote, schema, **kwargs)
+    def __init__(self, driver, remote, schema):
+        super(OvnIdl, self).__init__(remote, schema)
         self.driver = driver
         self.notify_handler = OvnDbNotifyHandler(driver)
         # ovsdb lock name to acquire.
@@ -539,22 +548,44 @@ class OvnIdl(BaseOvnIdl):
 
 class OvnIdlDistributedLock(BaseOvnIdl):
 
-    def __init__(self, driver, remote, schema, **kwargs):
-        super(OvnIdlDistributedLock, self).__init__(remote, schema, **kwargs)
+    def __init__(self, driver, remote, schema):
+        super(OvnIdlDistributedLock, self).__init__(remote, schema)
         self.driver = driver
         self.notify_handler = OvnDbNotifyHandler(driver)
         self._node_uuid = self.driver.node_uuid
         self._hash_ring = hash_ring_manager.HashRingManager(
             self.driver.hash_ring_group)
         self._last_touch = None
+        # This is a map of tables that may be new after OVN database is updated
+        self._tables_to_register = {
+            'OVN_Southbound': ['Chassis_Private'],
+        }
+
+    def handle_db_schema_changes(self, event, row):
+        if (event == row_event.RowEvent.ROW_CREATE and
+                row._table.name == 'Database'):
+            try:
+                tables = self._tables_to_register[row.name]
+            except KeyError:
+                return
+
+            self.update_tables(tables, row.schema[0])
+
+            if 'Chassis_Private' == self.driver.agent_chassis_table:
+                if 'Chassis_Private' not in self.tables:
+                    self.driver.agent_chassis_table = 'Chassis'
+            else:
+                if 'Chassis_Private' in self.tables:
+                    self.driver.agent_chassis_table = 'Chassis_Private'
 
     def notify(self, event, row, updates=None):
+        self.handle_db_schema_changes(event, row)
+        self.notify_handler.notify(event, row, updates, global_=True)
         try:
             target_node = self._hash_ring.get_node(str(row.uuid))
         except exceptions.HashRingIsEmpty as e:
             LOG.error('HashRing is empty, error: %s', e)
             return
-
         if target_node != self._node_uuid:
             return
 
@@ -591,51 +622,63 @@ class OvnNbIdl(OvnIdlDistributedLock):
 
     def __init__(self, driver, remote, schema):
         super(OvnNbIdl, self).__init__(driver, remote, schema)
+        self._lsp_update_up_event = LogicalSwitchPortUpdateUpEvent(driver)
+        self._lsp_update_down_event = LogicalSwitchPortUpdateDownEvent(driver)
+        self._lsp_create_up_event = LogicalSwitchPortCreateUpEvent(driver)
+        self._lsp_create_down_event = LogicalSwitchPortCreateDownEvent(driver)
         self._fip_create_delete_event = FIPAddDeleteEvent(driver)
 
-        self.notify_handler.watch_events([self._fip_create_delete_event])
+        self.notify_handler.watch_events([self._lsp_create_up_event,
+                                          self._lsp_create_down_event,
+                                          self._lsp_update_up_event,
+                                          self._lsp_update_down_event,
+                                          self._fip_create_delete_event])
 
     @classmethod
-    def from_server(cls, connection_string, schema_name, driver):
+    def from_server(cls, connection_string, helper, driver):
 
-        _check_and_set_ssl_files(schema_name)
-        helper = idlutils.get_schema_helper(connection_string, schema_name)
         helper.register_all()
         return cls(driver, connection_string, helper)
 
+    def unwatch_logical_switch_port_create_events(self):
+        """Unwatch the logical switch port create events.
+
+        When the ovs idl client connects to the ovsdb-server, it gets
+        a dump of all logical switch ports as events and we need to process
+        them at start up.
+        After the startup, there is no need to watch these events.
+        So unwatch these events.
+        """
+        self.notify_handler.unwatch_events([self._lsp_create_up_event,
+                                            self._lsp_create_down_event])
+        self._lsp_create_up_event = None
+        self._lsp_create_down_event = None
+
+    def post_connect(self):
+        self.unwatch_logical_switch_port_create_events()
+
 
 class OvnSbIdl(OvnIdlDistributedLock):
 
-    def __init__(self, driver, remote, schema, **kwargs):
-        super(OvnSbIdl, self).__init__(driver, remote, schema, **kwargs)
-
-        self._pb_create_up_event = PortBindingCreateUpEvent(driver)
-        self._pb_create_down_event = PortBindingCreateDownEvent(driver)
-
+    def __init__(self, driver, remote, schema):
+        super(OvnSbIdl, self).__init__(driver, remote, schema)
         self.notify_handler.watch_events([
-            self._pb_create_up_event,
-            self._pb_create_down_event,
-            PortBindingUpdateUpEvent(driver),
-            PortBindingUpdateDownEvent(driver)])
+            ChassisAgentDeleteEvent(self.driver),
+            ChassisAgentDownEvent(self.driver),
+            ChassisAgentWriteEvent(self.driver),
+            ChassisMetadataAgentWriteEvent(self.driver)])
 
     @classmethod
-    def from_server(cls, connection_string, schema_name, driver):
-        _check_and_set_ssl_files(schema_name)
-        helper = idlutils.get_schema_helper(connection_string, schema_name)
+    def from_server(cls, connection_string, helper, driver):
         if 'Chassis_Private' in helper.schema_json['tables']:
             helper.register_table('Chassis_Private')
-        if 'FDB' in helper.schema_json['tables']:
-            helper.register_table('FDB')
         helper.register_table('Chassis')
         helper.register_table('Encap')
         helper.register_table('Port_Binding')
         helper.register_table('Datapath_Binding')
-        helper.register_table('Connection')
-        try:
-            return cls(driver, connection_string, helper, leader_only=False)
-        except TypeError:
-            # TODO(twilson) We can remove this when we require ovs>=2.12.0
-            return cls(driver, connection_string, helper)
+        helper.register_table('MAC_Binding')
+        helper.register_columns('SB_Global', ['external_ids'])
+        return cls(driver, connection_string, helper)
 
     def post_connect(self):
         """Watch Chassis events.
@@ -648,23 +691,8 @@ class OvnSbIdl(OvnIdlDistributedLock):
         self._chassis_event = ChassisEvent(self.driver)
         self._portbinding_event = PortBindingChassisEvent(self.driver)
         self.notify_handler.watch_events(
-            [self._chassis_event, self._portbinding_event])
-
-        self.unwatch_port_binding_create_events()
-
-    def unwatch_port_binding_create_events(self):
-        """Unwatch the port binding create events.
-
-        When the ovs idl client connects to the ovsdb-server, it gets
-        a dump of all port binding events and we need to process
-        them at start up.
-        After the startup, there is no need to watch these events.
-        So unwatch these events.
-        """
-        self.notify_handler.unwatch_events([self._pb_create_up_event,
-                                            self._pb_create_down_event])
-        self._pb_create_up_event = None
-        self._pb_create_down_event = None
+            [self._chassis_event, self._portbinding_event,
+             PortBindingChassisUpdateEvent(self.driver)])
 
 
 class OvnInitPGNbIdl(OvnIdl):
@@ -678,16 +706,14 @@ class OvnInitPGNbIdl(OvnIdl):
 
     def __init__(self, driver, remote, schema):
         super(OvnInitPGNbIdl, self).__init__(driver, remote, schema)
-        self.set_table_condition(
-            'Port_Group', [['name', '==', ovn_const.OVN_DROP_PORT_GROUP_NAME]])
+        self.cond_change(
+            'Port_Group',
+            [['name', '==', ovn_const.OVN_DROP_PORT_GROUP_NAME]])
         self.neutron_pg_drop_event = NeutronPgDropPortGroupCreated()
         self.notify_handler.watch_event(self.neutron_pg_drop_event)
 
     @classmethod
-    def from_server(
-            cls, connection_string, schema_name, driver, pg_only=False):
-        _check_and_set_ssl_files(schema_name)
-        helper = idlutils.get_schema_helper(connection_string, schema_name)
+    def from_server(cls, connection_string, helper, driver, pg_only=False):
         if pg_only:
             helper.register_table('Port_Group')
         else:
