@@ -23,7 +23,6 @@ from keystoneauth1 import token_endpoint
 from neutron_lib import constants
 from neutron_lib.exceptions import dns as dns_exc
 from oslo_config import cfg
-from oslo_log import log
 
 from neutron.conf.services import extdns_designate_driver
 from neutron.services.externaldns import driver
@@ -38,8 +37,6 @@ _SESSION = None
 CONF = cfg.CONF
 extdns_designate_driver.register_designate_opts()
 
-LOG = log.getLogger(__name__)
-
 
 def get_clients(context):
     global _SESSION
@@ -60,7 +57,8 @@ def get_clients(context):
             password=CONF.designate.admin_password,
             tenant_name=CONF.designate.admin_tenant_name,
             tenant_id=CONF.designate.admin_tenant_id)
-    admin_client = d_client.Client(session=_SESSION, auth=admin_auth)
+    admin_client = d_client.Client(session=_SESSION, auth=admin_auth,
+                                   endpoint_override=CONF.designate.url)
     return client, admin_client
 
 
@@ -125,21 +123,9 @@ class Designate(driver.ExternalDNSService):
                                                   in_addr_name, 'PTR',
                                                   [recordset_name])
             except d_exc.NotFound:
-                # Note(jh): If multiple PTRs get created at the same time,
-                # the creation of the zone may fail with a conflict because
-                # it has already been created by a parallel job. So we
-                # ignore that error and try to create the recordset
-                # anyway. That call will still fail in the end if something
-                # is really broken. See bug 1891309.
-                try:
-                    designate_admin.zones.create(
-                        in_addr_zone_name, email=ptr_zone_email,
-                        description=in_addr_zone_description)
-                except d_exc.Conflict:
-                    LOG.debug('Conflict when trying to create PTR zone %s,'
-                              ' assuming it exists.',
-                              in_addr_zone_name)
-                    pass
+                designate_admin.zones.create(
+                    in_addr_zone_name, email=ptr_zone_email,
+                    description=in_addr_zone_description)
                 designate_admin.recordsets.create(in_addr_zone_name,
                                                   in_addr_name, 'PTR',
                                                   [recordset_name])
@@ -191,7 +177,7 @@ class Designate(driver.ExternalDNSService):
         try:
             recordsets = designate_client.recordsets.list(
                 dns_domain, criterion={"name": "%s" % name})
-        except d_exc.NotFound:
+        except (d_exc.NotFound, d_exc.Forbidden):
             raise dns_exc.DNSDomainNotFound(dns_domain=dns_domain)
         ids = [rec['id'] for rec in recordsets]
         ips = [str(ip) for rec in recordsets for ip in rec['records']]
