@@ -30,17 +30,19 @@ from neutron_lib.services import base as service_base
 from oslo_log import log
 from oslo_utils import excutils
 
-from neutron._i18n import _
 from neutron.common.ovn import constants as ovn_const
 from neutron.common.ovn import extensions
 from neutron.common.ovn import utils
+from neutron.common import utils as common_utils
 from neutron.db.availability_zone import router as router_az_db
 from neutron.db import dns_db
 from neutron.db import extraroute_db
 from neutron.db import l3_fip_port_details
+from neutron.db import l3_fip_qos
 from neutron.db import l3_gwmode_db
 from neutron.db.models import l3 as l3_models
 from neutron.db import ovn_revision_numbers_db as db_rev
+from neutron.extensions import qos_fip as qos_fip_api
 from neutron.plugins.ml2.drivers.ovn.mech_driver.ovsdb import ovn_client
 from neutron.quota import resource_registry
 from neutron.scheduler import l3_ovn_scheduler
@@ -58,7 +60,8 @@ class OVNL3RouterPlugin(service_base.ServicePluginBase,
                         l3_gwmode_db.L3_NAT_db_mixin,
                         dns_db.DNSDbMixin,
                         l3_fip_port_details.Fip_port_details_db_mixin,
-                        router_az_db.RouterAvailabilityZoneMixin):
+                        router_az_db.RouterAvailabilityZoneMixin,
+                        l3_fip_qos.FloatingQoSDbMixin):
     """Implementation of the OVN L3 Router Service Plugin.
 
     This class implements a L3 service plugin that provides
@@ -91,6 +94,13 @@ class OVNL3RouterPlugin(service_base.ServicePluginBase,
             self.create_floatingip_precommit, resources.FLOATING_IP,
             events.PRECOMMIT_CREATE)
 
+    @staticmethod
+    def disable_qos_fip_extension_by_extension_drivers(aliases):
+        qos_service_plugin = directory.get_plugin(plugin_constants.QOS)
+        qos_aliases = qos_fip_api.FIP_QOS_ALIAS in aliases
+        if not qos_service_plugin and qos_aliases:
+            aliases.remove(qos_fip_api.FIP_QOS_ALIAS)
+
     @staticmethod
     def disable_dns_extension_by_extension_drivers(aliases):
         core_plugin = directory.get_plugin()
@@ -102,6 +112,7 @@ class OVNL3RouterPlugin(service_base.ServicePluginBase,
     def supported_extension_aliases(self):
         if not hasattr(self, '_aliases'):
             self._aliases = self._supported_extension_aliases[:]
+            self.disable_qos_fip_extension_by_extension_drivers(self._aliases)
             self.disable_dns_extension_by_extension_drivers(self._aliases)
         return self._aliases
 
@@ -153,7 +164,8 @@ class OVNL3RouterPlugin(service_base.ServicePluginBase,
     def create_router_precommit(self, resource, event, trigger, context,
                                 router, router_id, router_db):
         db_rev.create_initial_revision(
-            context, router_id, ovn_const.TYPE_ROUTERS)
+            context, router_id, ovn_const.TYPE_ROUTERS,
+            std_attr_id=router_db.standard_attr.id)
 
     def create_router(self, context, router):
         router = super(OVNL3RouterPlugin, self).create_router(context, router)
@@ -248,7 +260,8 @@ class OVNL3RouterPlugin(service_base.ServicePluginBase,
     def create_floatingip_precommit(self, resource, event, trigger, context,
                                     floatingip, floatingip_id, floatingip_db):
         db_rev.create_initial_revision(
-            context, floatingip_id, ovn_const.TYPE_FLOATINGIPS)
+            context, floatingip_id, ovn_const.TYPE_FLOATINGIPS,
+            std_attr_id=floatingip_db.standard_attr.id)
 
     def create_floatingip(self, context, floatingip,
                           initial_status=n_const.FLOATINGIP_STATUS_DOWN):
@@ -258,25 +271,13 @@ class OVNL3RouterPlugin(service_base.ServicePluginBase,
         return fip
 
     def delete_floatingip(self, context, id):
-        # TODO(lucasagomes): Passing ``original_fip`` object as a
-        # parameter to the OVNClient's delete_floatingip() method is done
-        # for backward-compatible reasons. Remove it in the Rocky release
-        # of OpenStack.
-        original_fip = self.get_floatingip(context, id)
         super(OVNL3RouterPlugin, self).delete_floatingip(context, id)
-        self._ovn_client.delete_floatingip(context, id,
-                                           fip_object=original_fip)
+        self._ovn_client.delete_floatingip(context, id)
 
     def update_floatingip(self, context, id, floatingip):
-        # TODO(lucasagomes): Passing ``original_fip`` object as a
-        # parameter to the OVNClient's update_floatingip() method is done
-        # for backward-compatible reasons. Remove it in the Rocky release
-        # of OpenStack.
-        original_fip = self.get_floatingip(context, id)
         fip = super(OVNL3RouterPlugin, self).update_floatingip(context, id,
                                                                floatingip)
-        self._ovn_client.update_floatingip(context, fip,
-                                           fip_object=original_fip)
+        self._ovn_client.update_floatingip(context, fip)
         return fip
 
     def update_floatingip_status(self, context, floatingip_id, status):
@@ -286,7 +287,7 @@ class OVNL3RouterPlugin(service_base.ServicePluginBase,
         return fip
 
     def disassociate_floatingips(self, context, port_id, do_notify=True):
-        fips = self.get_floatingips(context.elevated(),
+        fips = self.get_floatingips(common_utils.get_elevated_context(context),
                                     filters={'port_id': [port_id]})
         router_ids = super(OVNL3RouterPlugin, self).disassociate_floatingips(
             context, port_id, do_notify)
@@ -294,8 +295,11 @@ class OVNL3RouterPlugin(service_base.ServicePluginBase,
             router_id = fip.get('router_id')
             fixed_ip_address = fip.get('fixed_ip_address')
             if router_id and fixed_ip_address:
-                update_fip = {'logical_ip': fixed_ip_address,
-                              'external_ip': fip['floating_ip_address']}
+                update_fip = {
+                    'id': fip['id'],
+                    'logical_ip': fixed_ip_address,
+                    'external_ip': fip['floating_ip_address'],
+                    'floating_network_id': fip['floating_network_id']}
                 try:
                     self._ovn_client.disassociate_floatingip(update_fip,
                                                              router_id)
@@ -461,33 +465,15 @@ class OVNL3RouterPlugin(service_base.ServicePluginBase,
                     context, router_id, add, remove, txn=txn)
 
     @staticmethod
-    @registry.receives(
-        resources.PORT,
-        [events.BEFORE_UPDATE, events.AFTER_UPDATE]
-    )
+    @registry.receives(resources.PORT, [events.AFTER_UPDATE])
     def _port_update(resource, event, trigger, **kwargs):
         l3plugin = directory.get_plugin(plugin_constants.L3)
         if not l3plugin:
             return
 
         current = kwargs['port']
-        original = kwargs['original_port']
 
-        # The OVN NB DB has a constraint where network has to be
-        # greater than 0. Updating it with an empty network would
-        # cause a constraint violation error. This problem happens
-        # when the last IP of a LRP is deleted, in order to avoid it
-        # an exception needs to be thrown before any write is performed
-        # to the DB, since if not it would leave the Neutron DB and the
-        # OVN DB unsync.
-        # https://bugs.launchpad.net/neutron/+bug/1948457
-        if (event == events.BEFORE_UPDATE and
-                'fixed_ips' in current and not current['fixed_ips'] and
-                utils.is_lsp_router_port(original)):
-            reason = _("Router port must have at least one IP.")
-            raise n_exc.ServicePortInUse(port_id=original['id'], reason=reason)
-
-        if event == events.AFTER_UPDATE and utils.is_lsp_router_port(current):
+        if utils.is_lsp_router_port(current):
             # We call the update_router port with if_exists, because neutron,
             # internally creates the port, and then calls update, which will
             # trigger this callback even before we had the chance to create
