@@ -33,6 +33,7 @@ from neutron_lib.objects import exceptions as obj_exc
 from neutron_lib.plugins import constants
 from neutron_lib.plugins import directory
 from oslo_config import cfg
+from oslo_db import exception as oslo_db_exc
 from oslo_log import log as logging
 
 from neutron._i18n import _
@@ -141,8 +142,9 @@ class PortForwardingPlugin(fip_pf.PortForwardingPluginBase):
         # dvr_no_external host to one dvr host. So we just do not allow
         # all dvr router's floating IP to be binded to a port which
         # already has port forwarding.
-        router = self.l3_plugin.get_router(payload.context.elevated(),
-                                           pf_objs[0].router_id)
+        router = self.l3_plugin.get_router(
+            utils.get_elevated_context(payload.context),
+            pf_objs[0].router_id)
         if l3_dvr_db.is_distributed_router(router):
             raise pf_exc.PortHasPortForwarding(port_id=port_id)
 
@@ -209,7 +211,7 @@ class PortForwardingPlugin(fip_pf.PortForwardingPluginBase):
         # context to check if the floatingip or port forwarding resources
         # are owned by other tenants.
         if not context.is_admin:
-            context = context.elevated()
+            context = utils.get_elevated_context(context)
         # If the logic arrives here, that means we have got update_ip_set and
         # its value is not None. So we need to get all port forwarding
         # resources based on the request port_id for preparing the next
@@ -329,7 +331,7 @@ class PortForwardingPlugin(fip_pf.PortForwardingPluginBase):
     def _check_port_has_binding_floating_ip(self, context, port_forwarding):
         port_id = port_forwarding['internal_port_id']
         floatingip_objs = l3_obj.FloatingIP.get_objects(
-            context.elevated(),
+            utils.get_elevated_context(context),
             fixed_port_id=port_id)
         if floatingip_objs:
             floating_ip_address = floatingip_objs[0].floating_ip_address
@@ -382,13 +384,15 @@ class PortForwardingPlugin(fip_pf.PortForwardingPluginBase):
                 raise lib_exc.BadRequest(resource=apidef.RESOURCE_NAME,
                                          msg=message)
 
-            if self._rpc_notifications_required:
-                self.push_api.push(context, [pf_obj], rpc_events.CREATED)
-            registry.notify(pf_consts.PORT_FORWARDING, events.AFTER_CREATE,
-                            self,
-                            payload=[callbacks.PortForwardingPayload(context,
-                                current_pf=pf_obj)])
-            return pf_obj
+        registry.notify(pf_consts.PORT_FORWARDING, events.AFTER_CREATE,
+                        self,
+                        payload=[callbacks.PortForwardingPayload(context,
+                            current_pf=pf_obj)])
+
+        if self._rpc_notifications_required:
+            self.push_api.push(context, [pf_obj], rpc_events.CREATED)
+
+        return pf_obj
 
     @db_base_plugin_common.convert_result_to_dict
     def update_floatingip_port_forwarding(self, context, id, floatingip_id,
@@ -430,7 +434,7 @@ class PortForwardingPlugin(fip_pf.PortForwardingPluginBase):
                 pf_obj.update_fields(port_forwarding, reset_changes=True)
                 self._check_port_forwarding_update(context, pf_obj)
                 pf_obj.update()
-        except obj_exc.NeutronDbObjectDuplicateEntry:
+        except oslo_db_exc.DBDuplicateEntry:
             (__, conflict_params) = self._find_existing_port_forwarding(
                 context, floatingip_id, pf_obj.to_dict())
             message = _("A duplicate port forwarding entry with same "
