@@ -28,6 +28,7 @@ from neutron.api.rpc.callbacks import events
 from neutron.api.rpc.callbacks.producer import registry
 from neutron.api.rpc.callbacks import resources
 from neutron.api.rpc.handlers import resources_rpc
+from neutron.common import utils as common_utils
 from neutron.objects import trunk as trunk_objects
 from neutron.services.trunk import exceptions as trunk_exc
 from neutron.services.trunk.rpc import constants
@@ -82,7 +83,7 @@ class TrunkSkeleton(object):
     @log_helpers.log_method_call
     def update_subport_bindings(self, context, subports):
         """Update subport bindings to match trunk host binding."""
-        el = context.elevated()
+        el = common_utils.get_elevated_context(context)
         ports_by_trunk_id = collections.defaultdict(list)
         updated_ports = collections.defaultdict(list)
 
@@ -127,6 +128,12 @@ class TrunkSkeleton(object):
         trunk_port_id = trunk.port_id
         trunk_port = self.core_plugin.get_port(context, trunk_port_id)
         trunk_host = trunk_port.get(portbindings.HOST_ID)
+        migrating_to_host = trunk_port.get(
+            portbindings.PROFILE, {}).get('migrating_to')
+        if migrating_to_host and trunk_host != migrating_to_host:
+            # Trunk is migrating now, so lets update host of the subports
+            # to the new host already
+            trunk_host = migrating_to_host
 
         # NOTE(status_police) Set the trunk in BUILD state before
         # processing subport bindings. The trunk will stay in BUILD
