@@ -45,6 +45,7 @@ from neutron.db import db_base_plugin_common as db_base
 from neutron.plugins.ml2.drivers.linuxbridge.agent import \
     linuxbridge_neutron_agent as linuxbridge_agent
 from neutron.tests.common import base as common_base
+from neutron.tests.common import helpers
 from neutron.tests import tools
 
 LOG = logging.getLogger(__name__)
@@ -110,8 +111,9 @@ def assert_ping(src_namespace, dst_ip, timeout=1, count=3):
     ipversion = netaddr.IPAddress(dst_ip).version
     ping_command = 'ping' if ipversion == 4 else 'ping6'
     ns_ip_wrapper = ip_lib.IPWrapper(src_namespace)
-    ns_ip_wrapper.netns.execute([ping_command, '-W', timeout, '-c', count,
-                                 dst_ip])
+    ns_ip_wrapper.netns.execute(
+        [ping_command, '-W', timeout, '-c', count, dst_ip],
+        privsep_exec=True)
 
 
 def assert_async_ping(src_namespace, dst_ip, timeout=1, count=1, interval=1):
@@ -123,8 +125,9 @@ def assert_async_ping(src_namespace, dst_ip, timeout=1, count=1, interval=1):
     # cannot be used and it needs to be done using the following workaround.
     for _index in range(count):
         start_time = time.time()
-        ns_ip_wrapper.netns.execute([ping_command, '-W', timeout, '-c', '1',
-                                     dst_ip])
+        ns_ip_wrapper.netns.execute(
+            [ping_command, '-W', timeout, '-c', '1', dst_ip],
+            privsep_exec=True)
         end_time = time.time()
         diff = end_time - start_time
         if 0 < diff < interval:
@@ -166,7 +169,7 @@ def assert_arping(src_namespace, dst_ip, source=None, timeout=1, count=1):
     if source:
         arping_cmd.extend(['-s', source])
     arping_cmd.append(dst_ip)
-    ns_ip_wrapper.netns.execute(arping_cmd)
+    ns_ip_wrapper.netns.execute(arping_cmd, privsep_exec=True)
 
 
 def assert_no_arping(src_namespace, dst_ip, source=None, timeout=1, count=1):
@@ -192,7 +195,8 @@ def _get_source_ports_from_ss_output(output):
 def get_unused_port(used, start=1024, end=None):
     if end is None:
         port_range = utils.execute(
-            ['sysctl', '-n', 'net.ipv4.ip_local_port_range'], run_as_root=True)
+            ['sysctl', '-n', 'net.ipv4.ip_local_port_range'], run_as_root=True,
+            privsep_exec=True)
         end = int(port_range.split()[0]) - 1
 
     candidates = set(range(start, end + 1))
@@ -222,7 +226,8 @@ def get_free_namespace_port(protocol, namespace=None, start=1024, end=None):
         raise ValueError("Unsupported protocol %s" % protocol)
 
     ip_wrapper = ip_lib.IPWrapper(namespace=namespace)
-    output = ip_wrapper.netns.execute(['ss', param], run_as_root=True)
+    output = ip_wrapper.netns.execute(['ss', param], run_as_root=True,
+                                      privsep_exec=True)
     used_ports = _get_source_ports_from_ss_output(output)
 
     return get_unused_port(used_ports, start, end)
@@ -231,11 +236,12 @@ def get_free_namespace_port(protocol, namespace=None, start=1024, end=None):
 def set_local_port_range(start, end):
     utils.execute(
         ['sysctl', '-w', 'net.ipv4.ip_local_port_range=%d %d' % (start, end)],
-        run_as_root=True)
-    utils.execute(['sysctl', '-p'], run_as_root=True)
+        run_as_root=True, privsep_exec=True)
+    utils.execute(['sysctl', '-p'], run_as_root=True, privsep_exec=True)
     # verify
     port_range = utils.execute(
-        ['sysctl', '-n', 'net.ipv4.ip_local_port_range'], run_as_root=True)
+        ['sysctl', '-n', 'net.ipv4.ip_local_port_range'], run_as_root=True,
+        privsep_exec=True)
     assert int(port_range.split()[0]) == start
     assert int(port_range.split()[1]) == end
 
@@ -612,10 +618,18 @@ class NamespaceFixture(fixtures.Fixture):
         self.addCleanup(self.destroy)
 
     def destroy(self):
-        if self.ip_wrapper.netns.exists(self.name):
-            for pid in ip_lib.list_namespace_pids(self.name):
-                utils.kill_process(pid, signal.SIGKILL, run_as_root=True)
-            self.ip_wrapper.netns.delete(self.name)
+        # TODO(ralonsoh): once the issue in LP#1838793 is properly fixed, we
+        # can remove this workaround (TestTimer context).
+        with helpers.TestTimer(5):
+            try:
+                if self.ip_wrapper.netns.exists(self.name):
+                    for pid in ip_lib.list_namespace_pids(self.name):
+                        utils.kill_process(pid, signal.SIGKILL,
+                                           run_as_root=True)
+                    self.ip_wrapper.netns.delete(self.name)
+            except helpers.TestTimerTimeout:
+                LOG.warning('Namespace %s was not deleted due to a timeout.',
+                            self.name)
 
 
 class VethFixture(fixtures.Fixture):
@@ -637,8 +651,8 @@ class VethFixture(fixtures.Fixture):
     def destroy(self):
         for port in self.ports:
             ip_wrapper = ip_lib.IPWrapper(port.namespace)
-            if (port.namespace is None or
-                    ip_wrapper.netns.exists(port.namespace)):
+            if (ip_wrapper.netns.exists(port.namespace) or
+                    port.namespace is None):
                 try:
                     ip_wrapper.del_veth(port.name)
                     break
@@ -828,9 +842,6 @@ class OVSPortFixture(PortFixture):
         # on ports that we intend to use as fake vm interfaces, they
         # need to be flat. This is related to lp#1767422
         self.bridge.clear_db_attribute("Port", port_name, "tag")
-        # Clear vlan_mode that is added for each new port. lp#1930414
-        self.bridge.clear_db_attribute("Port", port_name, "vlan_mode")
-        self.bridge.clear_db_attribute("Port", port_name, "trunks")
         self.addCleanup(self.bridge.delete_port, port_name)
         self.port = ip_lib.IPDevice(port_name, self.namespace)
 
