@@ -62,7 +62,7 @@ class ConfigFixture(config_fixtures.ConfigFileFixture):
 class NeutronConfigFixture(ConfigFixture):
 
     def __init__(self, env_desc, host_desc, temp_dir,
-                 connection, rabbitmq_environment):
+                 connection, rabbitmq_environment, use_local_apipaste=True):
         super(NeutronConfigFixture, self).__init__(
             env_desc, host_desc, temp_dir, base_filename='neutron.conf')
 
@@ -70,7 +70,6 @@ class NeutronConfigFixture(ConfigFixture):
             'DEFAULT': {
                 'host': self._generate_host(),
                 'state_path': self._generate_state_path(self.temp_dir),
-                'api_paste_config': self._generate_api_paste(),
                 'core_plugin': 'ml2',
                 'service_plugins': env_desc.service_plugins,
                 'auth_strategy': 'noauth',
@@ -96,7 +95,12 @@ class NeutronConfigFixture(ConfigFixture):
                 'log_agent_heartbeats': 'True',
             },
         })
-        policy_file = self._generate_policy_json()
+
+        if use_local_apipaste:
+            self.config['DEFAULT']['api_paste_config'] = (
+                self._generate_api_paste())
+
+        policy_file = self._generate_policy_yaml()
         if policy_file:
             self.config['oslo_policy'] = {'policy_file': policy_file}
 
@@ -122,6 +126,10 @@ class NeutronConfigFixture(ConfigFixture):
                     env_desc.placement_port
                 }
             })
+        if env_desc.dhcp_scheduler_class:
+            self.config['DEFAULT']['dhcp_agents_per_network'] = '1'
+            self.config['DEFAULT']['network_scheduler_driver'] = (
+                env_desc.dhcp_scheduler_class)
 
         net_helpers.set_local_port_range(CLIENT_CONN_PORT_START,
                                          CLIENT_CONN_PORT_END)
@@ -146,8 +154,8 @@ class NeutronConfigFixture(ConfigFixture):
     def _generate_api_paste(self):
         return c_helpers.find_sample_file('api-paste.ini')
 
-    def _generate_policy_json(self):
-        return c_helpers.find_sample_file('policy.json')
+    def _generate_policy_yaml(self):
+        return c_helpers.find_sample_file('policy.yaml')
 
     def get_host(self):
         return self.config['DEFAULT']['host']
@@ -163,19 +171,25 @@ class ML2ConfigFixture(ConfigFixture):
         if self.env_desc.l2_pop:
             mechanism_drivers += ',l2population'
 
+        net_vlan_ranges_extra = ''
+        if 'segments' in env_desc.service_plugins:
+            net_vlan_ranges_extra = (',' + PHYSICAL_NETWORK_NAME +
+                                     '_lb:1050:1059')
+
         self.config.update({
             'ml2': {
                 'tenant_network_types': tenant_network_types,
                 'mechanism_drivers': mechanism_drivers,
             },
             'ml2_type_vlan': {
-                'network_vlan_ranges': PHYSICAL_NETWORK_NAME + ':1000:2999',
+                'network_vlan_ranges': PHYSICAL_NETWORK_NAME + ':1000:1029' +
+                net_vlan_ranges_extra,
             },
             'ml2_type_gre': {
-                'tunnel_id_ranges': '1:1000',
+                'tunnel_id_ranges': '1:30',
             },
             'ml2_type_vxlan': {
-                'vni_ranges': '1001:2000',
+                'vni_ranges': '1001:1030',
             },
         })
 
@@ -202,6 +216,7 @@ class OVSConfigFixture(ConfigFixture):
                 'integration_bridge': self._generate_integration_bridge(),
                 'bridge_mappings': '%s:%s' % (PHYSICAL_NETWORK_NAME, ext_dev),
                 'of_inactivity_probe': '0',
+                'ovsdb_debug': 'True',
             },
             'securitygroup': {
                 'firewall_driver': host_desc.firewall_driver,
@@ -328,6 +343,8 @@ class LinuxBridgeConfigFixture(ConfigFixture):
             env_desc, host_desc, temp_dir,
             base_filename="linuxbridge_agent.ini"
         )
+        self.service_plugins = env_desc.service_plugins
+
         self.config.update({
             'VXLAN': {
                 'enable_vxlan': str(self.env_desc.tunneling_enabled),
@@ -367,7 +384,10 @@ class LinuxBridgeConfigFixture(ConfigFixture):
             })
 
     def _generate_bridge_mappings(self, device_name):
-        return '%s:%s' % (PHYSICAL_NETWORK_NAME, device_name)
+        bridge_mappings_extra = ('_lb' if 'segments' in self.service_plugins
+                                 else '')
+        return '%s%s:%s' % (PHYSICAL_NETWORK_NAME, bridge_mappings_extra,
+                            device_name)
 
 
 class L3ConfigFixture(ConfigFixture):
@@ -403,7 +423,9 @@ class L3ConfigFixture(ConfigFixture):
             'DEFAULT': {
                 'interface_driver': ('neutron.agent.linux.interface.'
                                      'OVSInterfaceDriver'),
-                'ovs_integration_bridge': integration_bridge,
+            },
+            'OVS': {
+                'integration_bridge': integration_bridge,
             }
         })
 
@@ -447,7 +469,9 @@ class DhcpConfigFixture(ConfigFixture):
         self.config.update({
             'DEFAULT': {
                 'interface_driver': 'openvswitch',
-                'ovs_integration_bridge': integration_bridge,
+            },
+            'OVS': {
+                'integration_bridge': integration_bridge,
             }
         })
 
