@@ -226,7 +226,7 @@ class L3AgentTestCase(framework.L3AgentTestFramework):
 
         # make sure all events are processed
         while not self.agent._queue._queue.empty():
-            self.agent._process_update()
+            self.agent._process_router_update()
 
         for r in routers_to_keep:
             self.assertIn(r['id'], self.agent.router_info)
@@ -320,7 +320,7 @@ class L3AgentTestCase(framework.L3AgentTestFramework):
         # Verify that the ping replys with fip
         ns_ip_wrapper = ip_lib.IPWrapper(src_machine.namespace)
         result = ns_ip_wrapper.netns.execute(
-            ['ping', '-W', 5, '-c', 1, dst_fip])
+            ['ping', '-W', 5, '-c', 1, dst_fip], privsep_exec=True)
         self._assert_ping_reply_from_expected_address(result, dst_fip)
 
     def _setup_address_scope(self, internal_address_scope1,
@@ -436,37 +436,3 @@ class L3AgentTestCase(framework.L3AgentTestFramework):
         # networks that are in the same address scope as external network
         net_helpers.assert_ping(machine_diff_scope.namespace,
                                 machine_same_scope.ip)
-
-    def test_router_interface_mtu_update(self):
-        original_mtu = 1450
-        router_info = self.generate_router_info(False)
-        router_info['_interfaces'][0]['mtu'] = original_mtu
-        router_info['gw_port']['mtu'] = original_mtu
-
-        router = self.manage_router(self.agent, router_info)
-
-        interface_name = router.get_internal_device_name(
-            router_info['_interfaces'][0]['id'])
-        gw_interface_name = router.get_external_device_name(
-            router_info['gw_port']['id'])
-
-        self.assertEqual(
-            original_mtu,
-            ip_lib.IPDevice(interface_name, router.ns_name).link.mtu)
-        self.assertEqual(
-            original_mtu,
-            ip_lib.IPDevice(gw_interface_name, router.ns_name).link.mtu)
-
-        updated_mtu = original_mtu + 1
-        router_info_copy = copy.deepcopy(router_info)
-        router_info_copy['_interfaces'][0]['mtu'] = updated_mtu
-        router_info_copy['gw_port']['mtu'] = updated_mtu
-
-        self.agent._process_updated_router(router_info_copy)
-
-        self.assertEqual(
-            updated_mtu,
-            ip_lib.IPDevice(interface_name, router.ns_name).link.mtu)
-        self.assertEqual(
-            updated_mtu,
-            ip_lib.IPDevice(gw_interface_name, router.ns_name).link.mtu)
