@@ -47,7 +47,12 @@ WRONG_IP = '0.0.0.0'
 TEST_IP = '240.0.0.1'
 TEST_IP_NEIGH = '240.0.0.2'
 TEST_IP_SECONDARY = '240.0.0.3'
-TEST_IP6_VXLAN_GROUP = 'ff00::1'
+TEST_IP6_NEIGH = 'fd00::2'
+TEST_IP6_SECONDARY = 'fd00::3'
+TEST_IP_NUD_STATES = ((TEST_IP_NEIGH, 'permanent'),
+                      (TEST_IP_SECONDARY, 'reachable'),
+                      (TEST_IP6_NEIGH, 'permanent'),
+                      (TEST_IP6_SECONDARY, 'reachable'))
 
 
 class IpLibTestFramework(functional_base.BaseSudoTestCase):
@@ -233,26 +238,9 @@ class IpLibTestCase(IpLibTestFramework):
         attr = self.generate_device_details()
         ip = ip_lib.IPWrapper(namespace=attr.namespace)
         ip.netns.add(attr.namespace)
-        ip.add_dummy('dummy_device')
         self.addCleanup(ip.netns.delete, attr.namespace)
         self.assertFalse(ip_lib.vxlan_in_use(9999, namespace=attr.namespace))
-        device = ip.add_vxlan(attr.name, 9999, 'dummy_device')
-        self.addCleanup(self._safe_delete_device, device)
-        self.assertTrue(ip_lib.vxlan_in_use(9999, namespace=attr.namespace))
-        device.link.delete()
-        self.assertFalse(ip_lib.vxlan_in_use(9999, namespace=attr.namespace))
-
-    def test_ipv6_vxlan_exists(self):
-        attr = self.generate_device_details(
-            name='test_device', ip_cidrs=["%s/24" % TEST_IP, 'fd00::1/64']
-        )
-        self.manage_device(attr)
-        ip = ip_lib.IPWrapper(namespace=attr.namespace)
-        ip.netns.add(attr.namespace)
-        self.addCleanup(ip.netns.delete, attr.namespace)
-        self.assertFalse(ip_lib.vxlan_in_use(9999, namespace=attr.namespace))
-        device = ip.add_vxlan('test_vxlan_device', 9999, local='fd00::1',
-            group=TEST_IP6_VXLAN_GROUP, dev='test_device')
+        device = ip.add_vxlan(attr.name, 9999)
         self.addCleanup(self._safe_delete_device, device)
         self.assertTrue(ip_lib.vxlan_in_use(9999, namespace=attr.namespace))
         device.link.delete()
@@ -435,7 +423,8 @@ class IpLibTestCase(IpLibTestFramework):
 
         expected_neighs = [{'dst': TEST_IP_NEIGH,
                             'lladdr': mac_address,
-                            'device': attr.name}]
+                            'device': attr.name,
+                            'state': 'permanent'}]
 
         neighs = device.neigh.dump(4)
         self.assertItemsEqual(expected_neighs, neighs)
@@ -468,6 +457,41 @@ class IpLibTestCase(IpLibTestFramework):
         # trying to delete a non-existent entry shouldn't raise an error
         device.neigh.delete(TEST_IP_NEIGH, mac_address)
 
+    def test_flush_neigh_ipv4(self):
+        # Entry with state "reachable" deleted.
+        self._flush_neigh(constants.IP_VERSION_4, TEST_IP_SECONDARY,
+                          {TEST_IP_NEIGH})
+        # Entries belong to "ip_to_flush" passed CIDR, but "permanent" entry
+        # is not deleted.
+        self._flush_neigh(constants.IP_VERSION_4, '240.0.0.0/28',
+                          {TEST_IP_NEIGH})
+        # "all" passed, but "permanent" entry is not deleted.
+        self._flush_neigh(constants.IP_VERSION_4, 'all', {TEST_IP_NEIGH})
+
+    def test_flush_neigh_ipv6(self):
+        # Entry with state "reachable" deleted.
+        self._flush_neigh(constants.IP_VERSION_6, TEST_IP6_SECONDARY,
+                          {TEST_IP6_NEIGH})
+        # Entries belong to "ip_to_flush" passed CIDR, but "permanent" entry
+        # is not deleted.
+        self._flush_neigh(constants.IP_VERSION_6, 'fd00::0/64',
+                          {TEST_IP6_NEIGH})
+        # "all" passed, but "permanent" entry is not deleted.
+        self._flush_neigh(constants.IP_VERSION_6, 'all', {TEST_IP6_NEIGH})
+
+    def _flush_neigh(self, version, ip_to_flush, ips_expected):
+        attr = self.generate_device_details(
+            ip_cidrs=['%s/24' % TEST_IP, 'fd00::1/64'],
+            namespace=utils.get_rand_name(20, 'ns-'))
+        device = self.manage_device(attr)
+        for test_ip, nud_state in TEST_IP_NUD_STATES:
+            mac_address = net.get_random_mac('fa:16:3e:00:00:00'.split(':'))
+            device.neigh.add(test_ip, mac_address, nud_state)
+
+        device.neigh.flush(version, ip_to_flush)
+        ips = {e['dst'] for e in device.neigh.dump(version)}
+        self.assertEqual(ips_expected, ips)
+
     def _check_for_device_name(self, ip, name, should_exist):
         exist = any(d for d in ip.get_devices() if d.name == name)
         self.assertEqual(should_exist, exist)
@@ -678,16 +702,10 @@ class NamespaceTestCase(functional_base.BaseSudoTestCase):
         ip_lib.delete_network_namespace(self.namespace)
 
     def test_network_namespace_exists_ns_exists(self):
-        for use_helper_for_ns_read in (True, False):
-            cfg.CONF.set_override('use_helper_for_ns_read',
-                                  use_helper_for_ns_read, 'AGENT')
-            self.assertTrue(ip_lib.network_namespace_exists(self.namespace))
+        self.assertTrue(ip_lib.network_namespace_exists(self.namespace))
 
     def test_network_namespace_exists_ns_doesnt_exists(self):
-        for use_helper_for_ns_read in (True, False):
-            cfg.CONF.set_override('use_helper_for_ns_read',
-                                  use_helper_for_ns_read, 'AGENT')
-            self.assertFalse(ip_lib.network_namespace_exists('another_ns'))
+        self.assertFalse(ip_lib.network_namespace_exists('another_ns'))
 
     def test_network_namespace_exists_ns_exists_try_is_ready(self):
         self.assertTrue(ip_lib.network_namespace_exists(self.namespace,
@@ -758,7 +776,7 @@ class IpMonitorTestCase(testscenarios.WithScenarios,
                 if ip_address not in registers:
                     return False
             return True
-        except (OSError, IOError):
+        except (OSError, IOError, ValueError):
             return False
 
     def _check_read_file(self, ip_addresses):
@@ -901,7 +919,8 @@ class IpRouteCommandTestCase(functional_base.BaseSudoTestCase):
                'scope': scope,
                'device': 'test_device',
                'via': via,
-               'metric': metric}
+               'metric': metric,
+               'proto': 'static'}
         try:
             utils.wait_until_true(fn, timeout=5)
         except utils.WaitTimeout:
@@ -1118,14 +1137,3 @@ class GetDevicesWithIpTestCase(functional_base.BaseSudoTestCase):
             ip_addresses = self._remove_loopback_interface(ip_addresses)
             ip_addresses = self._remove_ipv6_scope_link(ip_addresses)
             self.assertEqual(0, len(ip_addresses))
-
-
-class IpLinkCommandTestCase(IpLibTestFramework):
-
-    def test_set_netns(self):
-        device_name = ('int_' + uuidutils.generate_uuid())[
-                      :constants.DEVICE_NAME_MAX_LEN]
-        device = ip_lib.IPDevice(device_name, kind='dummy')
-        device.link.create()
-        namespace = self.useFixture(net_helpers.NamespaceFixture())
-        device.link.set_netns(namespace.name)
