@@ -13,6 +13,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import signal
+
 from oslo_config import cfg
 
 from neutron._i18n import _
@@ -46,7 +48,15 @@ class KeepalivedManagerTestCase(base.BaseSudoTestCase,
             'router1', self.expected_config, self.process_monitor,
             conf_path=cfg.CONF.state_path,
             namespace=self.namespace)
-        self.addCleanup(self.manager.disable)
+        self.addCleanup(self._stop_keepalived_manager)
+
+    def _stop_keepalived_manager(self):
+        self.manager.disable()
+        try:
+            common_utils.wait_until_true(
+                lambda: not self.manager.get_process().active, timeout=5)
+        except common_utils.WaitTimeout:
+            self.manager.get_process().disable(sig=signal.SIGKILL)
 
     def _prepare_devices(self):
         # NOTE(slaweq): those are devices used in keepalived config file,
@@ -82,7 +92,8 @@ class KeepalivedManagerTestCase(base.BaseSudoTestCase,
 
         # Exit the process, and see that when it comes back
         # It's indeed a different process
-        self.ip_wrapper.netns.execute(['kill', exit_code, pid])
+        self.ip_wrapper.netns.execute(['kill', exit_code, pid],
+                                      privsep_exec=True)
         common_utils.wait_until_true(
             lambda: process.active and pid != process.pid,
             timeout=5,
