@@ -85,9 +85,6 @@ class DHCPAgentOVSTestFramework(base.BaseSudoTestCase):
 
         self.conf.set_override('check_child_processes_interval', 1, 'AGENT')
 
-        mock.patch('neutron.agent.common.ovs_lib.'
-                   'OVSBridge._set_port_dead').start()
-
     def network_dict_for_dhcp(self, dhcp_enabled=True,
                               ip_version=lib_const.IP_VERSION_4,
                               prefix_override=None):
@@ -181,7 +178,8 @@ class DHCPAgentOVSTestFramework(base.BaseSudoTestCase):
 
     def assert_accept_ra_disabled(self, namespace):
         actual = ip_lib.IPWrapper(namespace=namespace).netns.execute(
-            ['sysctl', '-b', 'net.ipv6.conf.default.accept_ra'])
+            ['sysctl', '-b', 'net.ipv6.conf.default.accept_ra'],
+            privsep_exec=True)
         self.assertEqual('0', actual)
 
     def assert_dhcp_device(self, namespace, dhcp_iface_name, dhcp_enabled):
@@ -359,12 +357,17 @@ class DHCPAgentOVSTestCase(DHCPAgentOVSTestFramework):
         new_network.subnets.append(dhcp_enabled_ipv4_subnet)
 
         self.mock_plugin_api.get_network_info.return_value = new_network
-        dhcp_port_mock = self.create_port_dict(
-            network.id, dhcp_enabled_ipv4_subnet.id,
+        fixed_ip_mock = mock.Mock(
+            ip_address='192.168.10.2',
+            subnet_id=dhcp_enabled_ipv4_subnet.id)
+        dhcp_port_mock = mock.Mock(
+            dns_assignment={},
+            extra_dhcp_opts=[],
+            fixed_ips=[fixed_ip_mock],
+            id=new_network.ports[0].id,
             mac_address=str(self._DHCP_PORT_MAC_ADDRESS))
-        self.mock_plugin_api.create_dhcp_port.return_value = dhcp_port_mock
-        network.ports = []
-        new_network.ports = []
+        self.mock_plugin_api.get_dhcp_port.return_value = dhcp_port_mock
+        self.mock_plugin_api.update_dhcp_port.return_value = dhcp_port_mock
 
         self.agent.refresh_dhcp_helper(network.id)
         # Metadata proxy should be spawned for the newly added subnet
