@@ -23,7 +23,6 @@ from oslo_config import fixture as config_fixture
 from oslo_db.sqlalchemy import test_migrations
 from oslo_log import log as logging
 from oslotest import base as oslotest_base
-import six
 import sqlalchemy
 from sqlalchemy import event  # noqa
 from sqlalchemy.sql import ddl as sqla_ddl
@@ -273,7 +272,7 @@ class _TestModelsMigrations(test_migrations.ModelsMigrationsSync):
             """Identify excepted operations that are allowed for the branch."""
             # For alembic the clause is AddColumn or DropColumn
             column = clauseelement.column.name
-            table = clauseelement.column.table.name
+            table = clauseelement.table_name
             element_name = '.'.join([table, column])
             for alembic_type, excepted_names in exceptions.items():
                 if alembic_type == sqlalchemy.Column:
@@ -325,9 +324,25 @@ class _TestModelsMigrations(test_migrations.ModelsMigrationsSync):
                     self.alembic_config, 'upgrade',
                     '%s@head' % migration.CONTRACT_BRANCH)
 
+    def _test_has_offline_migrations(self, revision, expected):
+        engine = self.get_engine()
+        cfg.CONF.set_override('connection', engine.url, group='database')
+        migration.do_alembic_command(self.alembic_config, 'upgrade', revision)
+        self.assertEqual(expected,
+                         migration.has_offline_migrations(self.alembic_config,
+                                                          'unused'))
+
+    def test_has_offline_migrations_pending_contract_scripts(self):
+        self._test_has_offline_migrations('kilo', True)
+
+    def test_has_offline_migrations_all_heads_upgraded(self):
+        self._test_has_offline_migrations('heads', False)
+
     # NOTE(ihrachys): if this test fails for you, it probably means that you
     # attempt to add an unsafe contract migration script, that is in
     # contradiction to blueprint online-upgrades
+    # TODO(ihrachys): revisit later in Pike+ where some contract scripts may be
+    # safe again
     def test_forbid_offline_migrations_starting_newton(self):
         engine = self.get_engine()
         cfg.CONF.set_override('connection', engine.url, group='database')
@@ -381,6 +396,16 @@ class TestModelsMigrationsMysql(testlib_api.MySQLTestCaseMixin,
     def test_branches(self):
         super(TestModelsMigrationsMysql, self).test_branches()
 
+    @test_base.skip_if_timeout("bug 1687027")
+    def test_has_offline_migrations_pending_contract_scripts(self):
+        super(TestModelsMigrationsMysql,
+              self).test_has_offline_migrations_pending_contract_scripts()
+
+    @test_base.skip_if_timeout("bug 1687027")
+    def test_has_offline_migrations_all_heads_upgraded(self):
+        super(TestModelsMigrationsMysql,
+              self).test_has_offline_migrations_all_heads_upgraded()
+
     @test_base.skip_if_timeout("bug 1687027")
     def test_models_sync(self):
         super(TestModelsMigrationsMysql, self).test_models_sync()
@@ -550,7 +575,7 @@ class _TestWalkMigrations(object):
         self.script_dir = alembic_script.ScriptDirectory.from_config(db_config)
         db_config.neutron_config = cfg.CONF
         db_config.neutron_config.set_override('connection',
-                                              six.text_type(uri),
+                                              str(uri),
                                               group='database')
         return db_config
 
