@@ -15,9 +15,6 @@
 from unittest import mock
 
 import fixtures as og_fixtures
-from oslo_concurrency import processutils
-from oslo_config import cfg
-from oslo_serialization import jsonutils
 from oslo_utils import uuidutils
 
 from neutron.common.ovn import constants as ovn_const
@@ -32,6 +29,18 @@ from neutron_lib.api.definitions import portbindings
 from neutron_lib.plugins import constants as plugin_constants
 from neutron_lib.plugins import directory
 from ovsdbapp.backend.ovs_idl import event
+from ovsdbapp.backend.ovs_idl import idlutils
+
+
+class WaitForMACBindingDeleteEvent(event.WaitEvent):
+    event_name = 'WaitForMACBindingDeleteEvent'
+
+    def __init__(self, entry):
+        table = 'MAC_Binding'
+        events = (self.ROW_DELETE,)
+        conditions = (('_uuid', '=', entry),)
+        super(WaitForMACBindingDeleteEvent, self).__init__(
+            events, table, conditions, timeout=15)
 
 
 class WaitForDataPathBindingCreateEvent(event.WaitEvent):
@@ -61,6 +70,10 @@ class DistributedLockTestEvent(event.WaitEvent):
         self.event.set()
 
 
+class GlobalTestEvent(DistributedLockTestEvent):
+    GLOBAL = True
+
+
 class TestNBDbMonitor(base.TestOVNFunctionalBase):
 
     def setUp(self):
@@ -118,40 +131,6 @@ class TestNBDbMonitor(base.TestOVNFunctionalBase):
                 'port_id': port['id']}})
         return r1_f2
 
-    def _check_mac_binding_exists(self, macb_id):
-        cmd = ['ovsdb-client', 'transact',
-               self.mech_driver.sb_ovn.connection_string]
-
-        if self._ovsdb_protocol == 'ssl':
-            cmd += ['-p', self.ovsdb_server_mgr.private_key, '-c',
-                    self.ovsdb_server_mgr.certificate, '-C',
-                    self.ovsdb_server_mgr.ca_cert]
-
-        cmd += ['["OVN_Southbound", {"op": "select", "table": "MAC_Binding", '
-                '"where": [["_uuid", "==", ["uuid", "%s"]]]}]' % macb_id]
-
-        out, _ = processutils.execute(*cmd,
-                                      log_errors=False)
-        return str(macb_id) in out
-
-    def _add_mac_binding_row(self, ip, datapath):
-        cmd = ['ovsdb-client', 'transact',
-               self.mech_driver.sb_ovn.connection_string]
-
-        if self._ovsdb_protocol == 'ssl':
-            cmd += ['-p', self.ovsdb_server_mgr.private_key, '-c',
-                    self.ovsdb_server_mgr.certificate, '-C',
-                    self.ovsdb_server_mgr.ca_cert]
-
-        cmd += ['["OVN_Southbound", {"op": "insert", "table": "MAC_Binding", '
-                '"row": {"ip": "%s", "datapath": '
-                '["uuid", "%s"]}}]' % (ip, datapath)]
-
-        out, _ = processutils.execute(*cmd,
-                                      log_errors=False)
-        out_parsed = jsonutils.loads(out)
-        return out_parsed[0]['uuid'][1]
-
     def test_floatingip_mac_bindings(self):
         """Check that MAC_Binding entries are cleared on FIP add/removal
 
@@ -173,31 +152,27 @@ class TestNBDbMonitor(base.TestOVNFunctionalBase):
         dp = self.sb_api.db_find(
             'Datapath_Binding',
             ('external_ids', '=', {'name2': net_name})).execute()
-        macb_id = self._add_mac_binding_row(
-            ip='100.0.0.21', datapath=dp[0]['_uuid'])
+        macb_id = self.sb_api.db_create('MAC_Binding', datapath=dp[0]['_uuid'],
+                                        ip='100.0.0.21').execute()
         port = self.create_port()
 
         # Ensure that the MAC_Binding entry gets deleted after creating a FIP
+        row_event = WaitForMACBindingDeleteEvent(macb_id)
+        self.mech_driver._sb_ovn.idl.notify_handler.watch_event(row_event)
         fip = self._create_fip(port, '100.0.0.21')
-        n_utils.wait_until_true(
-            lambda: not self._check_mac_binding_exists(macb_id),
-            timeout=15, sleep=1)
+        self.assertTrue(row_event.wait())
 
         # Now that the FIP is created, add a new MAC_Binding entry with the
         # same IP address
-        macb_id = self._add_mac_binding_row(
-            ip='100.0.0.21', datapath=dp[0]['_uuid'])
+
+        macb_id = self.sb_api.db_create('MAC_Binding', datapath=dp[0]['_uuid'],
+                                        ip='100.0.0.21').execute()
 
         # Ensure that the MAC_Binding entry gets deleted after deleting the FIP
+        row_event = WaitForMACBindingDeleteEvent(macb_id)
+        self.mech_driver._sb_ovn.idl.notify_handler.watch_event(row_event)
         self.l3_plugin.delete_floatingip(self.context, fip['id'])
-        n_utils.wait_until_true(
-            lambda: not self._check_mac_binding_exists(macb_id),
-            timeout=15, sleep=1)
-
-    def _get_port_uuid(self, port_id):
-        sb_port = self.sb_api.db_find(
-            'Port_Binding', ('logical_port', '=', port_id)).execute()[0]
-        return sb_port['_uuid']
+        self.assertTrue(row_event.wait())
 
     def _test_port_binding_and_status(self, port_id, action, status):
         # This function binds or unbinds port to chassis and
@@ -205,7 +180,6 @@ class TestNBDbMonitor(base.TestOVNFunctionalBase):
         core_plugin = directory.get_plugin()
         self.sb_api.check_for_row_by_value_and_retry(
             'Port_Binding', 'logical_port', port_id)
-        port_uuid = self._get_port_uuid(port_id)
 
         def check_port_status(status):
             port = core_plugin.get_ports(
@@ -214,21 +188,8 @@ class TestNBDbMonitor(base.TestOVNFunctionalBase):
         if action == 'bind':
             self.sb_api.lsp_bind(port_id, self.chassis,
                                  may_exist=True).execute(check_error=True)
-            try:
-                self.sb_api.db_set('Port_Binding', port_uuid,
-                                   ('up', True)).execute(check_error=True)
-            except KeyError:
-                self.sb_api.db_set('Port_Binding', port_uuid,
-                                   ('chassis', 'host-1')).execute(
-                                       check_error=True)
         else:
             self.sb_api.lsp_unbind(port_id).execute(check_error=True)
-            try:
-                self.sb_api.db_set('Port_Binding', port_uuid,
-                                   ('up', False)).execute(check_error=True)
-            except KeyError:
-                self.sb_api.db_set('Port_Binding', port_uuid,
-                                   ('chassis', None)).execute(check_error=True)
         n_utils.wait_until_true(lambda: check_port_status(status))
 
     def test_port_up_down_events(self):
@@ -243,15 +204,12 @@ class TestNBDbMonitor(base.TestOVNFunctionalBase):
         self._test_port_binding_and_status(port['id'], 'bind', 'ACTIVE')
         self._test_port_binding_and_status(port['id'], 'unbind', 'DOWN')
 
-    def test_distributed_lock(self):
-        api_workers = 11
-        cfg.CONF.set_override('api_workers', api_workers)
-        row_event = DistributedLockTestEvent()
+    def _create_workers(self, row_event, worker_num):
         self.mech_driver._nb_ovn.idl.notify_handler.watch_event(row_event)
-        worker_list = [self.mech_driver._nb_ovn, ]
+        worker_list = [self.mech_driver._nb_ovn]
 
         # Create 10 fake workers
-        for _ in range(api_workers - len(worker_list)):
+        for _ in range(worker_num):
             node_uuid = uuidutils.generate_uuid()
             db_hash_ring.add_node(
                 self.context, ovn_const.HASH_RING_ML2_GROUP, node_uuid)
@@ -260,7 +218,7 @@ class TestNBDbMonitor(base.TestOVNFunctionalBase):
                 hash_ring_group=ovn_const.HASH_RING_ML2_GROUP)
             _idl = ovsdb_monitor.OvnNbIdl.from_server(
                 self.ovsdb_server_mgr.get_ovsdb_connection_path(),
-                'OVN_Northbound', fake_driver)
+                self.nb_api.schema_helper, fake_driver)
             worker = self.useFixture(
                 fixtures.OVNIdlConnectionFixture(
                     idl=_idl, timeout=10)).connection
@@ -273,11 +231,17 @@ class TestNBDbMonitor(base.TestOVNFunctionalBase):
 
         # Assert we have 11 active workers in the ring
         self.assertEqual(
-            11, len(db_hash_ring.get_active_nodes(
-                    self.context,
-                    interval=ovn_const.HASH_RING_NODES_TIMEOUT,
-                    group_name=ovn_const.HASH_RING_ML2_GROUP)))
+            worker_num + 1,
+            len(db_hash_ring.get_active_nodes(
+                self.context,
+                interval=ovn_const.HASH_RING_NODES_TIMEOUT,
+                group_name=ovn_const.HASH_RING_ML2_GROUP)))
 
+        return worker_list
+
+    def test_distributed_lock(self):
+        row_event = DistributedLockTestEvent()
+        self._create_workers(row_event, worker_num=10)
         # Trigger the event
         self.create_port()
 
@@ -287,6 +251,30 @@ class TestNBDbMonitor(base.TestOVNFunctionalBase):
         # Assert that only one worker handled the event
         self.assertEqual(1, row_event.COUNTER)
 
+    def test_global_events(self):
+        worker_num = 10
+        distributed_event = DistributedLockTestEvent()
+        global_event = GlobalTestEvent()
+        worker_list = self._create_workers(distributed_event, worker_num)
+        for worker in worker_list:
+            worker.idl.notify_handler.watch_event(global_event)
+
+        # This should generate one distributed even handled by a single worker
+        # and one global event, that should be handled by all workers
+        self.create_port()
+
+        # Wait for the distributed event to complete
+        self.assertTrue(distributed_event.wait())
+
+        # Assert that only one worker handled the distributed event
+        self.assertEqual(1, distributed_event.COUNTER)
+
+        n_utils.wait_until_true(
+            lambda: global_event.COUNTER == worker_num + 1,
+            exception=Exception(
+                "Fanout event didn't get handled expected %d times" %
+                (worker_num + 1)))
+
 
 class TestNBDbMonitorOverTcp(TestNBDbMonitor):
     def get_ovsdb_server_protocol(self):
@@ -300,20 +288,18 @@ class TestNBDbMonitorOverSsl(TestNBDbMonitor):
 
 class TestOvnIdlProbeInterval(base.TestOVNFunctionalBase):
     def setUp(self):
-        # skip parent setUp, we don't need it, but we do need grandparent
-        # pylint: disable=bad-super-call
-        super(base.TestOVNFunctionalBase, self).setUp()
-        mm = directory.get_plugin().mechanism_manager
-        self.mech_driver = mm.mech_drivers['ovn'].obj
-        self.temp_dir = self.useFixture(og_fixtures.TempDir()).path
+        # We need an OvsdbServer that uses TCP because probe_interval is always
+        # zero for unix socket connections, which is what the parent uses
+        temp_dir = self.useFixture(og_fixtures.TempDir()).path
         install_share_path = self._get_install_share_path()
-        self.mgr = self.useFixture(
-            process.OvsdbServer(self.temp_dir, install_share_path,
+        mgr = self.useFixture(
+            process.OvsdbServer(temp_dir, install_share_path,
                                 ovn_nb_db=True, ovn_sb_db=True,
                                 protocol='tcp'))
-        connection = self.mgr.get_ovsdb_connection_path
+        connection = mgr.get_ovsdb_connection_path
         self.connections = {'OVN_Northbound': connection(),
                             'OVN_Southbound': connection(db_type='sb')}
+        super().setUp()
 
     def test_ovsdb_probe_interval(self):
         klasses = {
@@ -322,8 +308,11 @@ class TestOvnIdlProbeInterval(base.TestOVNFunctionalBase):
                                      {'driver': self.mech_driver}),
             ovsdb_monitor.OvnSbIdl: ('OVN_Southbound',
                                      {'driver': self.mech_driver})}
-        idls = [kls.from_server(self.connections[schema], schema, **kwargs)
-                for kls, (schema, kwargs) in klasses.items()]
+        idls = [
+            kls.from_server(
+                self.connections[schema],
+                idlutils.get_schema_helper(self.connections[schema], schema),
+                **kwargs) for kls, (schema, kwargs) in klasses.items()]
         interval = ovn_conf.get_ovn_ovsdb_probe_interval()
         for idl in idls:
             self.assertEqual(interval, idl._session.reconnect.probe_interval)
