@@ -13,9 +13,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import multiprocessing
 import os
-import queue
 import signal
 import socket
 import time
@@ -41,8 +39,8 @@ CONF = cfg.CONF
 
 # Those messages will be written to temporary file each time
 # start/reset methods are called.
-FAKE_START_MSG = 'start'
-FAKE_RESET_MSG = 'reset'
+FAKE_START_MSG = b"start"
+FAKE_RESET_MSG = b"reset"
 
 TARGET_PLUGIN = 'neutron.plugins.ml2.plugin.Ml2Plugin'
 
@@ -52,7 +50,7 @@ class TestNeutronServer(base.BaseLoggingTestCase):
         super(TestNeutronServer, self).setUp()
         self.service_pid = None
         self.workers = None
-        self._mp_queue = multiprocessing.Queue()
+        self.temp_file = self.get_temp_file_path("test_server.tmp")
         self.health_checker = self._check_active
         self.pipein, self.pipeout = os.pipe()
         self.addCleanup(self._destroy_workers)
@@ -62,7 +60,7 @@ class TestNeutronServer(base.BaseLoggingTestCase):
             # Make sure all processes are stopped
             os.kill(self.service_pid, signal.SIGKILL)
 
-    def _start_server(self, callback, workers, processes_queue=None):
+    def _start_server(self, callback, workers):
         """Run a given service.
 
         :param callback: callback that will start the required service
@@ -92,7 +90,15 @@ class TestNeutronServer(base.BaseLoggingTestCase):
         # If number of workers is 1 it is assumed that we run
         # a service in the current process.
         if self.workers > 1:
-            workers = self._get_workers(10, processes_queue=processes_queue)
+            # Wait at most 10 seconds to spawn workers
+            condition = lambda: self.workers == len(self._get_workers())
+
+            utils.wait_until_true(
+                condition, timeout=10, sleep=0.1,
+                exception=RuntimeError(
+                    "Failed to start %d workers." % self.workers))
+
+            workers = self._get_workers()
             self.assertEqual(len(workers), self.workers)
             return workers
 
@@ -103,35 +109,21 @@ class TestNeutronServer(base.BaseLoggingTestCase):
 
         return [self.service_pid]
 
-    def _get_workers(self, timeout, processes_queue=None):
+    def _get_workers(self):
         """Get the list of processes in which WSGI server is running."""
+
         def safe_ppid(proc):
             try:
                 return proc.ppid()
             except psutil.NoSuchProcess:
                 return None
 
-        def get_workers_pid():
-            if self.workers > 1:
-                return [proc.pid for proc in psutil.process_iter()
-                        if safe_ppid(proc) == self.service_pid]
-            else:
-                return [proc.pid for proc in psutil.process_iter()
-                        if proc.pid == self.service_pid]
-
-        exception = RuntimeError('Failed to start %d workers.' % self.workers)
-
-        if processes_queue:
-            try:
-                return processes_queue.get(timeout=timeout)
-            except queue.Empty:
-                raise exception
-
-        # Wait at most 10 seconds to spawn workers
-        condition = lambda: self.workers == len(get_workers_pid())
-        utils.wait_until_true(condition, timeout=timeout, sleep=0.1,
-                              exception=exception)
-        return get_workers_pid()
+        if self.workers > 1:
+            return [proc.pid for proc in psutil.process_iter()
+                    if safe_ppid(proc) == self.service_pid]
+        else:
+            return [proc.pid for proc in psutil.process_iter()
+                    if proc.pid == self.service_pid]
 
     def _check_active(self):
         """Dummy service activity check."""
@@ -139,13 +131,14 @@ class TestNeutronServer(base.BaseLoggingTestCase):
         return True
 
     def _fake_start(self):
-        self._mp_queue.put(FAKE_START_MSG)
+        with open(self.temp_file, 'ab') as f:
+            f.write(FAKE_START_MSG)
 
     def _fake_reset(self):
-        self._mp_queue.put(FAKE_RESET_MSG)
+        with open(self.temp_file, 'ab') as f:
+            f.write(FAKE_RESET_MSG)
 
-    def _test_restart_service_on_sighup(self, service, workers=1,
-                                        processes_queue=None):
+    def _test_restart_service_on_sighup(self, service, workers=1):
         """Test that a service correctly (re)starts on receiving SIGHUP.
 
         1. Start a service with a given number of workers.
@@ -153,8 +146,7 @@ class TestNeutronServer(base.BaseLoggingTestCase):
         3. Wait for workers (if any) to (re)start.
         """
 
-        self._start_server(callback=service, workers=workers,
-                           processes_queue=processes_queue)
+        self._start_server(callback=service, workers=workers)
         os.kill(self.service_pid, signal.SIGHUP)
 
         # After sending SIGHUP it is expected that there will be as many
@@ -166,27 +158,38 @@ class TestNeutronServer(base.BaseLoggingTestCase):
         # Wait for temp file to be created and its size reaching the expected
         # value
         expected_size = len(expected_msg)
-        ret_msg = ''
 
-        def is_ret_buffer_ok():
-            nonlocal ret_msg
-            LOG.debug('Checking returned buffer size')
-            while not self._mp_queue.empty():
-                ret_msg += self._mp_queue.get()
-            LOG.debug('Size of buffer is %s. Expected size: %s',
-                      len(ret_msg), expected_size)
-            return len(ret_msg) == expected_size
+        def is_temp_file_ok():
+            LOG.debug("Checking file %s", self.temp_file)
+            if not os.path.isfile(self.temp_file):
+                LOG.debug("File %s not exists.", self.temp_file)
+                return False
+            temp_file_size = os.stat(self.temp_file).st_size
+            LOG.debug("Size of file %s is %s. Expected size: %s",
+                      self.temp_file, temp_file_size, expected_size)
+            return temp_file_size == expected_size
 
         try:
-            utils.wait_until_true(is_ret_buffer_ok, timeout=5, sleep=1)
-        except utils.WaitTimeout:
-            raise RuntimeError('Expected buffer size: %s, current size: %s' %
-                               (len(ret_msg), expected_size))
+            utils.wait_until_true(is_temp_file_ok, timeout=5, sleep=1)
+        except utils.TimerTimeout:
+            if not os.path.isfile(self.temp_file):
+                raise RuntimeError(
+                    "Timed out waiting for file %(filename)s to be created" %
+                    {'filename': self.temp_file})
+            else:
+                raise RuntimeError(
+                    "Expected size for file %(filename)s: %(size)s, current "
+                    "size: %(current_size)s" %
+                    {'filename': self.temp_file,
+                     'size': expected_size,
+                     'current_size': os.stat(self.temp_file).st_size})
 
         # Verify that start has been called twice for each worker (one for
         # initial start, and the second one on SIGHUP after children were
         # terminated).
-        self.assertEqual(expected_msg, ret_msg)
+        with open(self.temp_file, 'rb') as f:
+            res = f.readline()
+            self.assertEqual(expected_msg, res)
 
 
 class TestWsgiServer(TestNeutronServer):
@@ -253,7 +256,6 @@ class TestRPCServer(TestNeutronServer):
         self._plugin_patcher = mock.patch(TARGET_PLUGIN, autospec=True)
         self.plugin = self._plugin_patcher.start()
         self.plugin.return_value.rpc_workers_supported = True
-        self._processes_queue = multiprocessing.Queue()
 
     def _serve_rpc(self, workers=1):
         """Start RPC server with a given number of workers."""
@@ -274,13 +276,11 @@ class TestRPCServer(TestNeutronServer):
             CONF.set_override("rpc_state_report_workers", 0)
 
             rpc_workers_launcher = service.start_rpc_workers()
-            self._processes_queue.put(list(rpc_workers_launcher.children))
             rpc_workers_launcher.wait()
 
     def test_restart_rpc_on_sighup_multiple_workers(self):
-        self._test_restart_service_on_sighup(
-            service=self._serve_rpc, workers=2,
-            processes_queue=self._processes_queue)
+        self._test_restart_service_on_sighup(service=self._serve_rpc,
+                                             workers=2)
 
 
 class TestPluginWorker(TestNeutronServer):
