@@ -13,10 +13,11 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import mock
+from unittest import mock
 
 from neutron.agent.linux import iptables_firewall
 from neutron.agent.linux.openvswitch_firewall import iptables
+from neutron.agent.linux import utils
 from neutron.tests import base
 
 
@@ -25,6 +26,8 @@ class TestHelper(base.BaseTestCase):
         super(TestHelper, self).setUp()
         self.helper = iptables.Helper(mock.Mock())
         mock.patch.object(iptables_firewall, 'cfg').start()
+        self.mock_execute = mock.patch.object(utils, 'execute',
+                                              return_value=False).start()
         mock.patch('neutron.agent.linux.ip_conntrack.get_conntrack').start()
 
     def test_get_hybrid_ports(self):
@@ -107,8 +110,10 @@ class TestHybridIptablesHelper(base.BaseTestCase):
     def test_overloaded_remove_conntrack(self):
         with mock.patch.object(iptables_firewall.IptablesFirewallDriver,
                 '_remove_conntrack_entries_from_port_deleted') as rcefpd, \
-             mock.patch("neutron.agent.linux.ip_conntrack.IpConntrackManager"
-                        "._populate_initial_zone_map"):
+                mock.patch("neutron.agent.linux.ip_conntrack."
+                           "IpConntrackManager._populate_initial_zone_map"), \
+                mock.patch.object(iptables_firewall.IptablesFirewallDriver,
+                                  '_check_netfilter_for_bridges'):
             firewall = iptables.get_iptables_driver_instance()
             firewall._remove_conntrack_entries_from_port_deleted(None)
             rcefpd.assert_not_called()
