@@ -13,14 +13,15 @@
 #    under the License.
 
 import os.path
+from unittest import mock
 
-import mock
 from neutron_lib import fixture as lib_fixtures
 from oslo_config import cfg
 from oslo_utils import fileutils
 import psutil
 
 from neutron.agent.linux import external_process as ep
+from neutron.common import utils as common_utils
 from neutron.tests import base
 
 
@@ -137,7 +138,8 @@ class TestProcessManager(base.BaseTestCase):
                                                      check_exit_code=True,
                                                      extra_ok_codes=None,
                                                      run_as_root=False,
-                                                     log_fail_as_error=True)
+                                                     log_fail_as_error=True,
+                                                     privsep_exec=False)
 
     def test_enable_with_namespace(self):
         callback = mock.Mock()
@@ -169,6 +171,24 @@ class TestProcessManager(base.BaseTestCase):
                 manager.enable(callback)
                 self.assertFalse(callback.called)
 
+    def test_enable_with_ensure_active(self):
+        def _create_cmd(*args):
+            return ['sleep', 0]
+
+        pm = ep.ProcessManager(self.conf, 'uuid', pid_file='pid_file',
+                               default_cmd_callback=_create_cmd)
+        with mock.patch.object(psutil, 'Process') as mock_psutil_process, \
+                mock.patch.object(ep.ProcessManager, 'pid',
+                                  new_callable=mock.PropertyMock) as mock_pid:
+            mock_pid.return_value = 'pid_value'
+            mock_process = mock.Mock()
+            mock_process.cmdline.side_effect = [[], ['the', 'cmd', 'uuid']]
+            mock_psutil_process.return_value = mock_process
+            try:
+                pm.enable(ensure_active=True)
+            except common_utils.WaitTimeout:
+                self.fail('ProcessManager.enable() raised WaitTimeout')
+
     def test_reload_cfg_without_custom_reload_callback(self):
         with mock.patch.object(ep.ProcessManager, 'disable') as disable:
             manager = ep.ProcessManager(self.conf, 'uuid', namespace='ns')
@@ -209,7 +229,8 @@ class TestProcessManager(base.BaseTestCase):
                     manager.disable()
                     utils.assert_has_calls([
                         mock.call.execute(['kill', '-9', 4],
-                                          run_as_root=False)])
+                                          run_as_root=False,
+                                          privsep_exec=True)])
 
     def test_disable_namespace(self):
         with mock.patch.object(ep.ProcessManager, 'pid') as pid:
@@ -223,7 +244,8 @@ class TestProcessManager(base.BaseTestCase):
                     manager.disable()
                     utils.assert_has_calls([
                         mock.call.execute(['kill', '-9', 4],
-                                          run_as_root=True)])
+                                          run_as_root=True,
+                                          privsep_exec=True)])
 
     def test_disable_not_active(self):
         with mock.patch.object(ep.ProcessManager, 'pid') as pid:
@@ -265,7 +287,8 @@ class TestProcessManager(base.BaseTestCase):
                                           return_value=kill_script_exists):
                     manager.disable()
                     utils.execute.assert_called_with(
-                        expected_cmd, run_as_root=bool(namespace))
+                        expected_cmd, run_as_root=bool(namespace),
+                        privsep_exec=True)
 
     def test_disable_custom_kill_script_no_namespace(self):
         self._test_disable_custom_kill_script(
