@@ -13,7 +13,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import mock
+from unittest import mock
+
 import netaddr
 from neutron_lib.api.definitions import portbindings
 from neutron_lib import constants
@@ -26,6 +27,7 @@ from neutron.db import db_base_plugin_v2
 from neutron.db import ipam_backend_mixin
 from neutron.db import portbindings_db
 from neutron.objects import subnet as subnet_obj
+from neutron.services.segments import db as segments_db
 from neutron.tests import base
 from neutron.tests.unit.db import test_db_base_plugin_v2
 
@@ -96,6 +98,9 @@ class TestIpamBackendMixin(base.BaseTestCase):
         self.assertFalse(self.mixin._is_distributed_service(port))
         port = {'device_owner': constants.DEVICE_OWNER_DHCP,
                 'device_id': 'ovnmeta-%s' % uuidutils.generate_uuid()}
+        self.assertFalse(self.mixin._is_distributed_service(port))
+        port = {'device_owner': constants.DEVICE_OWNER_DISTRIBUTED,
+                'device_id': 'ovnmeta-%s' % uuidutils.generate_uuid()}
         self.assertTrue(self.mixin._is_distributed_service(port))
 
     def _test_get_changed_ips_for_port(self, expected, original_ips,
@@ -342,7 +347,8 @@ class TestIpamBackendMixin(base.BaseTestCase):
 
 
 class TestPlugin(db_base_plugin_v2.NeutronDbPluginV2,
-                 portbindings_db.PortBindingMixin):
+                 portbindings_db.PortBindingMixin,
+                 segments_db.SegmentDbMixin):
     __native_pagination_support = True
     __native_sorting_support = True
 
