@@ -13,13 +13,15 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import mock
+from unittest import mock
+
 from neutron_lib.api.definitions import portbindings
 from neutron_lib.callbacks import events
 from neutron_lib.callbacks import registry
 from neutron_lib.callbacks import resources
 from neutron_lib import constants as const
 from neutron_lib import context
+from neutron_lib.db import api as db_api
 from neutron_lib import exceptions
 from neutron_lib.exceptions import l3 as l3_exc
 from neutron_lib.objects import exceptions as o_exc
@@ -28,6 +30,7 @@ from neutron_lib.plugins import directory
 from neutron_lib.plugins import utils as plugin_utils
 from oslo_utils import uuidutils
 
+from neutron.common import utils as common_utils
 from neutron.db import agents_db
 from neutron.db import l3_dvr_db
 from neutron.db import l3_dvrscheduler_db
@@ -35,7 +38,6 @@ from neutron.db.models import l3 as l3_models
 from neutron.db import models_v2
 from neutron.objects import agent as agent_obj
 from neutron.objects import l3agent as rb_obj
-from neutron.objects import ports as port_obj
 from neutron.objects import router as router_obj
 from neutron.tests.unit.db import test_db_base_plugin_v2
 from neutron.tests.unit.extensions import test_l3
@@ -60,11 +62,11 @@ class L3DvrTestCase(test_db_base_plugin_v2.NeutronDbPluginV2TestCase):
         directory.add_plugin(plugin_constants.L3, self.mixin)
 
     def _create_router(self, router):
-        with self.ctx.session.begin(subtransactions=True):
+        with db_api.CONTEXT_WRITER.using(self.ctx):
             return self.mixin._create_router_db(self.ctx, router, 'foo_tenant')
 
     def create_port(self, net_id, port_info):
-        with self.ctx.session.begin(subtransactions=True):
+        with db_api.CONTEXT_WRITER.using(self.ctx):
             return self._create_port(self.fmt, net_id, **port_info)
 
     def _test__create_router_db(self, expected=False, distributed=None):
@@ -326,9 +328,9 @@ class L3DvrTestCase(test_db_base_plugin_v2.NeutronDbPluginV2TestCase):
              check_for_fip_and_create_agent_gw_port_on_host_if_not_exists(
                     ctxt, port, 'host'))
             if fip:
-                c_fip.assert_called_once_with(ctxt.elevated(),
-                                              fip['floating_network_id'],
-                                              'host')
+                c_fip.assert_called_once_with(
+                    common_utils.get_elevated_context(ctxt),
+                    fip['floating_network_id'], 'host')
             else:
                 c_fip.assert_not_called()
 
@@ -347,11 +349,11 @@ class L3DvrTestCase(test_db_base_plugin_v2.NeutronDbPluginV2TestCase):
         plugin = mock.Mock()
         directory.add_plugin(plugin_constants.CORE, plugin)
         plugin.get_port.return_value = port
-        self.mixin._router_exists = mock.Mock(return_value=True)
-        self.assertRaises(exceptions.ServicePortInUse,
-                          self.mixin.prevent_l3_port_deletion,
-                          self.ctx,
-                          port['id'])
+        with mock.patch.object(router_obj.Router, 'objects_exist',
+                               return_value=True):
+            self.assertRaises(exceptions.ServicePortInUse,
+                              self.mixin.prevent_l3_port_deletion,
+                              self.ctx, port['id'])
 
     def test_prevent_delete_floatingip_agent_gateway_port(self):
         port = {
@@ -463,20 +465,20 @@ class L3DvrTestCase(test_db_base_plugin_v2.NeutronDbPluginV2TestCase):
             'admin_state_up': True,
             'distributed': True
         }
-        router = self._create_router(router_db)
-        if gw_port:
-            with self.subnet(cidr='10.10.10.0/24') as subnet:
-                port_dict = {
-                    'device_id': router.id,
-                    'device_owner': const.DEVICE_OWNER_ROUTER_GW,
-                    'admin_state_up': True,
-                    'fixed_ips': [{'subnet_id': subnet['subnet']['id'],
-                                   'ip_address': '10.10.10.100'}]
-                }
-            net_id = subnet['subnet']['network_id']
-            port_res = self.create_port(net_id, port_dict)
-            port_res_dict = self.deserialize(self.fmt, port_res)
-            with self.ctx.session.begin(subtransactions=True):
+        with db_api.CONTEXT_WRITER.using(self.ctx):
+            router = self._create_router(router_db)
+            if gw_port:
+                with self.subnet(cidr='10.10.10.0/24') as subnet:
+                    port_dict = {
+                        'device_id': router.id,
+                        'device_owner': const.DEVICE_OWNER_ROUTER_GW,
+                        'admin_state_up': True,
+                        'fixed_ips': [{'subnet_id': subnet['subnet']['id'],
+                                       'ip_address': '10.10.10.100'}]
+                    }
+                net_id = subnet['subnet']['network_id']
+                port_res = self.create_port(net_id, port_dict)
+                port_res_dict = self.deserialize(self.fmt, port_res)
                 port_db = self.ctx.session.query(models_v2.Port).filter_by(
                     id=port_res_dict['port']['id']).one()
                 router.gw_port = port_db
@@ -487,9 +489,8 @@ class L3DvrTestCase(test_db_base_plugin_v2.NeutronDbPluginV2TestCase):
                 )
                 self.ctx.session.add(router)
                 self.ctx.session.add(router_port)
-
-        else:
-            net_id = None
+            else:
+                net_id = None
 
         plugin = mock.Mock()
         directory.add_plugin(plugin_constants.CORE, plugin)
@@ -815,115 +816,6 @@ class L3DvrTestCase(test_db_base_plugin_v2.NeutronDbPluginV2TestCase):
                 fip, floatingip, router))
         self.assertFalse(create_fip.called)
 
-    def test_get_ext_nets_by_host(self):
-        ports = [mock.Mock(id=_uuid()) for _ in range(3)]
-        fips = [mock.Mock(fixed_port_id=p.id, floating_network_id=_uuid())
-                for p in ports]
-        expected_ext_nets = set([fip.floating_network_id for fip in fips])
-        with mock.patch.object(
-            port_obj.Port, 'get_ports_by_host',
-            return_value=[p.id for p in ports]
-        ) as get_ports_by_host, mock.patch.object(
-            self.mixin, '_get_floatingips_by_port_id', return_value=fips
-        ) as get_floatingips_by_port_id:
-            self.assertEqual(
-                expected_ext_nets,
-                self.mixin._get_ext_nets_by_host(self.ctx, 'host'))
-            get_ports_by_host.assert_called_once_with(self.ctx, 'host')
-            get_floatingips_by_port_id.assert_has_calls(
-                [mock.call(self.ctx, p.id) for p in ports])
-
-    def _test_create_fip_agent_gw_ports(self, agent_type, agent_mode=None):
-        agent = {
-            'id': _uuid(),
-            'host': 'host',
-            'agent_type': agent_type,
-            'configurations': {'agent_mode': agent_mode}}
-        payload = events.DBEventPayload(
-            self.ctx, states=(agent,), resource_id=agent['id'])
-
-        ext_nets = ['ext-net-1', 'ext-net-2']
-        with mock.patch.object(
-            self.mixin,
-            'create_fip_agent_gw_port_if_not_exists'
-        ) as create_fip_gw, mock.patch.object(
-            self.mixin, "_get_ext_nets_by_host",
-            return_value=ext_nets
-        ) as get_ext_nets_by_host:
-
-            registry.publish(resources.AGENT, events.AFTER_CREATE, mock.Mock(),
-                             payload=payload)
-
-            if agent_type == 'L3 agent' and agent_mode in ['dvr', 'dvr_snat']:
-                get_ext_nets_by_host.assert_called_once_with(
-                    mock.ANY, 'host')
-                create_fip_gw.assert_has_calls(
-                    [mock.call(mock.ANY, ext_net, 'host') for
-                        ext_net in ext_nets])
-            else:
-                get_ext_nets_by_host.assert_not_called()
-                create_fip_gw.assert_not_called()
-
-    def test_create_fip_agent_gw_ports(self):
-        self._test_create_fip_agent_gw_ports(
-            agent_type='L3 agent', agent_mode='dvr')
-        self._test_create_fip_agent_gw_ports(
-            agent_type='L3 agent', agent_mode='dvr_snat')
-
-    def test_create_fip_agent_gw_ports_dvr_no_external_agent(self):
-        self._test_create_fip_agent_gw_ports(
-            agent_type='L3 agent', agent_mode='dvr_no_external')
-
-    def test_create_fip_agent_gw_ports_non_dvr_agent(self):
-        self._test_create_fip_agent_gw_ports(
-            agent_type='L3 agent', agent_mode='legacy')
-
-    def test_create_fip_agent_gw_ports_deleted_non_l3_agent(self):
-        self._test_create_fip_agent_gw_ports('Other agent type')
-
-    def _test_delete_fip_agent_gw_ports(self, agent_type, agent_mode=None):
-        agent = agent_obj.Agent(
-            self.ctx, id=_uuid(), agent_type=agent_type,
-            configurations={"agent_mode": agent_mode})
-        payload = events.DBEventPayload(
-            self.ctx, states=(agent,), resource_id=agent.id)
-
-        gw_port = {'id': _uuid(), 'network_id': _uuid()}
-        with mock.patch.object(
-            self.mixin, '_get_agent_gw_ports',
-            return_value=[gw_port]
-        ) as get_agent_gw_ports, mock.patch.object(
-            self.core_plugin, 'delete_port'
-        ) as delete_port:
-            registry.publish(resources.AGENT, events.AFTER_DELETE, mock.Mock(),
-                             payload=payload)
-
-            if agent_type == 'L3 agent' and agent_mode in ['dvr', 'dvr_snat']:
-                get_agent_gw_ports.assert_called_once_with(payload.context,
-                                                           agent['id'])
-                delete_port.assert_called_once_with(payload.context,
-                                                    gw_port['id'])
-            else:
-                get_agent_gw_ports.assert_not_called()
-                delete_port.assert_not_called()
-
-    def test_delete_fip_agent_gw_ports(self):
-        self._test_delete_fip_agent_gw_ports(
-            agent_type='L3 agent', agent_mode='dvr')
-        self._test_delete_fip_agent_gw_ports(
-            agent_type='L3 agent', agent_mode='dvr_snat')
-
-    def test_delete_fip_agent_gw_ports_dvr_no_external_agent(self):
-        self._test_delete_fip_agent_gw_ports(
-            agent_type='L3 agent', agent_mode='dvr_no_external')
-
-    def test_delete_fip_agent_gw_ports_non_dvr_agent(self):
-        self._test_delete_fip_agent_gw_ports(
-            agent_type='L3 agent', agent_mode='legacy')
-
-    def test_delete_fip_agent_gw_ports_deleted_non_l3_agent(self):
-        self._test_delete_fip_agent_gw_ports('Other agent type')
-
     def _test_update_router_gw_info_external_network_change(self):
         router_dict = {'name': 'test_router', 'admin_state_up': True,
                        'distributed': True}
@@ -1241,6 +1133,10 @@ class L3DvrTestCase(test_db_base_plugin_v2.NeutronDbPluginV2TestCase):
             mock_notify.assert_called_once_with(
                 'router', 'before_update', self.mixin, **kwargs)
 
+    def _assert_mock_called_with_router(self, mock_fn, router_id):
+        router = mock_fn.call_args[1].get('router_db')
+        self.assertEqual(router_id, router.id)
+
     def test__validate_router_migration_notify_advanced_services_mocked(self):
         # call test with admin_state_down_before_update ENABLED
         self._test__validate_router_migration_notify_advanced_services()
@@ -1261,9 +1157,16 @@ class L3DvrTestCase(test_db_base_plugin_v2.NeutronDbPluginV2TestCase):
             interface_info = {'subnet_id': sub['subnet']['id']}
             self.mixin.add_router_interface(self.ctx, router_db.id,
                                             interface_info)
-            mock_notify.assert_called_once_with(self.ctx, router_db=router_db,
+            # NOTE(slaweq): here we are just checking if mock_notify was called
+            # with kwargs which we are expecting, but we can't check exactly if
+            # router_db was object which we are expecting and because of that
+            # below we are checking if router_db used as argument in
+            # mock_notify call is the has same id as the one which we are
+            # expecting
+            mock_notify.assert_called_once_with(self.ctx, router_db=mock.ANY,
                                                 port=mock.ANY,
                                                 interface_info=interface_info)
+            self._assert_mock_called_with_router(mock_notify, router_db.id)
 
     def test_validate_add_router_interface_by_port_notify_advanced_services(
             self):
@@ -1278,9 +1181,16 @@ class L3DvrTestCase(test_db_base_plugin_v2.NeutronDbPluginV2TestCase):
             interface_info = {'port_id': port['port']['id']}
             self.mixin.add_router_interface(self.ctx, router_db.id,
                                             interface_info)
-            mock_notify.assert_called_once_with(self.ctx, router_db=router_db,
+            # NOTE(slaweq): here we are just checking if mock_notify was called
+            # with kwargs which we are expecting, but we can't check exactly if
+            # router_db was object which we are expecting and because of that
+            # below we are checking if router_db used as argument in
+            # mock_notify call is the has same id as the one which we are
+            # expecting.
+            mock_notify.assert_called_once_with(self.ctx, router_db=mock.ANY,
                                                 port=mock.ANY,
                                                 interface_info=interface_info)
+            self._assert_mock_called_with_router(mock_notify, router_db.id)
 
     def test__generate_arp_table_and_notify_agent(self):
         fixed_ip = {
@@ -1501,8 +1411,6 @@ class L3DvrTestCase(test_db_base_plugin_v2.NeutronDbPluginV2TestCase):
         router = self._create_router(router_dict)
         with self.network() as network,\
                 self.subnet(network=network) as subnet:
-            self.mixin._core_plugin.get_allowed_address_pairs_for_ports = (
-                mock.Mock())
             fake_bound_ports_ids = []
 
             def fake_is_port_bound(port):
@@ -1524,8 +1432,6 @@ class L3DvrTestCase(test_db_base_plugin_v2.NeutronDbPluginV2TestCase):
                 self.ctx, subnet['subnet']['id'])
             dvr_subnet_ports_ids = [p['id'] for p in dvr_subnet_ports]
             self.assertItemsEqual(fake_bound_ports_ids, dvr_subnet_ports_ids)
-            (self.mixin._core_plugin.get_allowed_address_pairs_for_ports.
-                assert_called_once_with(self.ctx, dvr_subnet_ports_ids))
 
     @mock.patch.object(plugin_utils, 'can_port_be_bound_to_virtual_bridge',
                        return_value=True)
