@@ -13,7 +13,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import mock
+from unittest import mock
+
 from neutron_lib.api.definitions import provider_net
 from neutron_lib import context
 from neutron_lib import fixture
@@ -70,7 +71,7 @@ class ProvidernetExtensionTestCase(testlib_api.WebTestCase):
         instance = self.plugin.return_value
         instance.get_networks_count.return_value = 1
         # Register mock plugin and enable the 'provider' extension
-        instance.supported_extension_aliases = ["provider"]
+        instance.supported_extension_aliases = [provider_net.ALIAS]
         tools.make_mock_plugin_json_encodable(instance)
         directory.add_plugin(constants.CORE, instance)
         ext_mgr = ProviderExtensionManager()
