@@ -14,9 +14,9 @@ import collections
 import copy
 import itertools
 import random
+from unittest import mock
 
 import fixtures
-import mock
 import netaddr
 from neutron_lib import constants
 from neutron_lib import context
@@ -33,17 +33,19 @@ from oslo_db import exception as obj_exc
 from oslo_db.sqlalchemy import utils as db_utils
 from oslo_utils import uuidutils
 from oslo_versionedobjects import base as obj_base
-from oslo_versionedobjects import exception
 from oslo_versionedobjects import fields as obj_fields
 from sqlalchemy import orm
 import testtools
 
 from neutron import objects
+from neutron.objects import address_group
 from neutron.objects import agent
 from neutron.objects import base
 from neutron.objects.db import api as obj_db_api
 from neutron.objects import flavor
 from neutron.objects import network as net_obj
+from neutron.objects.port.extensions import port_device_profile
+from neutron.objects.port.extensions import port_numa_affinity_policy
 from neutron.objects import ports
 from neutron.objects.qos import policy as qos_policy
 from neutron.objects import rbac_db
@@ -100,8 +102,6 @@ class FakeSmallNeutronObjectNewEngineFacade(base.NeutronDbObject):
 
     db_model = ObjectFieldsModel
 
-    new_facade = True
-
     primary_keys = ['field1']
 
     foreign_keys = {
@@ -541,6 +541,10 @@ FIELD_TYPE_VALUE_GENERATOR_MAP = {
     obj_fields.ListOfStringsField: tools.get_random_string_list,
     obj_fields.ObjectField: lambda: None,
     obj_fields.StringField: lambda: helpers.get_random_string(10),
+    port_numa_affinity_policy.NumaAffinityPoliciesEnumField:
+        tools.get_random_port_numa_affinity_policy,
+    port_device_profile.PortDeviceProfile:
+        lambda: helpers.get_random_string(255)
 }
 
 
@@ -1641,6 +1645,17 @@ class BaseDbObjectTestCase(_BaseObjectTestCase,
         _securitygroup.create()
         return _securitygroup.id
 
+    def _create_test_address_group_id(self, fields=None):
+        ag_fields = self.get_random_object_fields(address_group.AddressGroup)
+        fields = fields or {}
+        for field, value in ((f, v) for (f, v) in fields.items() if
+                             f in ag_fields):
+            ag_fields[field] = value
+        _address_group = address_group.AddressGroup(
+            self.context, **ag_fields)
+        _address_group.create()
+        return _address_group.id
+
     def _create_test_agent_id(self):
         attrs = self.get_random_object_fields(obj_cls=agent.Agent)
         _agent = agent.Agent(self.context, **attrs)
@@ -1699,14 +1714,6 @@ class BaseDbObjectTestCase(_BaseObjectTestCase,
                                 **remove_timestamps_from_fields(
                                     fields, self._test_class.fields))
 
-    def test_downgrade_to_1_0(self):
-        for obj in self.objs:
-            try:
-                obj.obj_to_primitive(target_version='1.0')
-            except exception.IncompatibleObjectVersion:
-                # the only exception we should allow is IncompatibleVersion
-                pass
-
     def test_get_object_create_update_delete(self):
         # Timestamps can't be initialized and multiple objects may use standard
         # attributes so we need to remove timestamps when creating objects
@@ -1790,7 +1797,7 @@ class BaseDbObjectTestCase(_BaseObjectTestCase,
 
     def test_get_objects_single_transaction(self):
         with mock.patch(self._get_ro_txn_exit_func_name()) as mock_exit:
-            with db_api.autonested_transaction(self.context.session):
+            with db_api.CONTEXT_READER.using(self.context):
                 self._test_class.get_objects(self.context)
         self.assertEqual(1, mock_exit.call_count)
 
@@ -1805,7 +1812,7 @@ class BaseDbObjectTestCase(_BaseObjectTestCase,
         obj.create()
 
         with mock.patch(self._get_ro_txn_exit_func_name()) as mock_exit:
-            with db_api.autonested_transaction(self.context.session):
+            with db_api.CONTEXT_READER.using(self.context):
                 obj = self._test_class.get_object(self.context,
                                                   **obj._get_composite_keys())
         self.assertEqual(1, mock_exit.call_count)
