@@ -26,6 +26,9 @@ from neutron.tests import base as test_base
 # corresponding version bump in the affected objects. Please keep the list in
 # alphabetic order.
 object_data = {
+    'AddressAssociation': '1.0-b92160a3dd2fb7b951adcd2e6ae1665a',
+    'AddressGroup': '1.2-1ddbf0a9f61785033ce31818ac62687e',
+    'AddressGroupRBAC': '1.0-192845c5ed0718e1c54fac36936fcd7d',
     'AddressScope': '1.1-dd0dfdb67775892d3adc090e28e43bd8',
     'AddressScopeRBAC': '1.0-192845c5ed0718e1c54fac36936fcd7d',
     'Agent': '1.1-64b670752d57b3c7602cb136e0338507',
@@ -42,7 +45,7 @@ object_data = {
     'FlatAllocation': '1.0-bf666f24f4642b047eeca62311fbcb41',
     'Flavor': '1.0-82194de5c9aafce08e8527bb7977f5c6',
     'FlavorServiceProfileBinding': '1.0-a2c8731e16cefdac4571f80abf1f8930',
-    'FloatingIP': '1.0-0205cc99ec79e8089d641ed1b565ddae',
+    'FloatingIP': '1.1-595514f8e992849cfc67981d1fd6614e',
     'FloatingIPDNS': '1.0-ee3db848500fa1825235f701828c06d5',
     'GeneveAllocation': '1.0-d5f76e8eac60a778914d61dd8e23e90f',
     'GeneveEndpoint': '1.0-040f026996b5952e2ae4ccd40ac61ca6',
@@ -57,7 +60,7 @@ object_data = {
     'L3HARouterNetwork': '1.0-87acea732853f699580179a94d2baf91',
     'L3HARouterVRIdAllocation': '1.0-37502aebdbeadc4f9e3bd5e9da714ab9',
     'MeteringLabel': '1.0-cc4b620a3425222447cbe459f62de533',
-    'MeteringLabelRule': '1.0-b5c5717e7bab8d1af1623156012a5842',
+    'MeteringLabelRule': '2.0-0ad09894c62e1ce6e868f725158959ba',
     'Log': '1.0-6391351c0f34ed34375a19202f361d24',
     'Network': '1.1-c3e9ecc0618ee934181d91b143a48901',
     'NetworkDhcpAgentBinding': '1.1-d9443c88809ffa4c45a0a5a48134b54a',
@@ -67,14 +70,16 @@ object_data = {
     'NetworkSegment': '1.0-57b7f2960971e3b95ded20cbc59244a8',
     'NetworkSegmentRange': '1.0-bdec1fffc9058ea676089b1f2f2b3cf3',
     'NetworkSubnetLock': '1.0-140de39d4b86ae346dc3d70b885bea53',
-    'Port': '1.5-98f35183d876c9beb188f4bf44d4d886',
+    'Port': '1.7-d8c1cfe42cfa3719a5d810eeab79e006',
+    'PortDeviceProfile': '1.0-b98c7083cc3e93d176fd7a91ae13af32',
+    'PortNumaAffinityPolicy': '1.0-38fcea43e7bfb2536461f3d053c43aa3',
     'PortBinding': '1.0-3306deeaa6deb01e33af06777d48d578',
     'PortBindingLevel': '1.1-50d47f63218f87581b6cd9a62db574e5',
     'PortDataPlaneStatus': '1.0-25be74bda46c749653a10357676c0ab2',
     'PortDNS': '1.1-c5ca2dc172bdd5fafee3fc986d1d7023',
     'PortForwarding': '1.2-f772f03b82a616603c7f3d4497bf577f',
     'PortSecurity': '1.0-b30802391a87945ee9c07582b4ff95e3',
-    'PortUplinkStatusPropagation': '1.0-3cfb3f7da716ca9687e4f04ca72b081d',
+    'PortUplinkStatusPropagation': '1.1-f0a4ca451a941910376c33616dea5de2',
     'ProviderResourceAssociation': '1.0-05ab2d5a3017e5ce9dd381328f285f34',
     'ProvisioningBlock': '1.0-c19d6d05bfa8143533471c1296066125',
     'QosBandwidthLimitRule': '1.3-51b662b12a8d1dfa89288d826c6d26d3',
@@ -99,10 +104,10 @@ object_data = {
     'RouterL3AgentBinding': '1.0-c5ba6c95e3a4c1236a55f490cd67da82',
     'RouterPort': '1.0-c8c8f499bcdd59186fcd83f323106908',
     'RouterRoute': '1.0-07fc5337c801fb8c6ccfbcc5afb45907',
-    'SecurityGroup': '1.2-7b63b834e511856f54a09282d6843ecc',
+    'SecurityGroup': '1.4-7b63b834e511856f54a09282d6843ecc',
     'SecurityGroupPortBinding': '1.0-6879d5c0af80396ef5a72934b6a6ef20',
     'SecurityGroupRBAC': '1.0-192845c5ed0718e1c54fac36936fcd7d',
-    'SecurityGroupRule': '1.0-e9b8dace9d48b936c62ad40fe1f339d5',
+    'SecurityGroupRule': '1.2-27793368d4ac35f2ed6e0bb653c6aaad',
     'SegmentHostMapping': '1.0-521597cf82ead26217c3bd10738f00f0',
     'ServiceProfile': '1.0-9beafc9e7d081b8258f3c5cb66ac5eed',
     'StandardAttribute': '1.0-617d4f46524c4ce734a6fc1cc0ac6a0b',
