@@ -55,24 +55,16 @@ class OpenvswitchMechanismBaseTestCase(base.AgentMechanismBaseTestCase):
 
     AGENTS = [{'alive': True,
                'configurations': GOOD_CONFIGS,
-               'host': 'host',
-               'agent_type': AGENT_TYPE,
-               }]
+               'host': 'host'}]
     AGENTS_DEAD = [{'alive': False,
                     'configurations': GOOD_CONFIGS,
-                    'host': 'dead_host',
-                    'agent_type': AGENT_TYPE,
-                    }]
+                    'host': 'dead_host'}]
     AGENTS_BAD = [{'alive': False,
                    'configurations': GOOD_CONFIGS,
-                   'host': 'bad_host_1',
-                   'agent_type': AGENT_TYPE,
-                   },
+                   'host': 'bad_host_1'},
                   {'alive': True,
                    'configurations': BAD_CONFIGS,
-                   'host': 'bad_host_2',
-                   'agent_type': AGENT_TYPE,
-                   }]
+                   'host': 'bad_host_2'}]
 
     def setUp(self):
         super(OpenvswitchMechanismBaseTestCase, self).setUp()
@@ -136,9 +128,7 @@ class OpenvswitchMechanismSGDisabledBaseTestCase(
                     'tunnel_types': GOOD_TUNNEL_TYPES}
     AGENTS = [{'alive': True,
                'configurations': GOOD_CONFIGS,
-               'host': 'host',
-               'agent_type': constants.AGENT_TYPE_OVS,
-               }]
+               'host': 'host'}]
 
     def setUp(self):
         cfg.CONF.set_override('enable_security_group',
@@ -161,9 +151,7 @@ class OpenvswitchMechanismHybridPlugTestCase(OpenvswitchMechanismBaseTestCase):
         self.driver.vif_details[hybrid] = False
         agents = [{'alive': True,
                    'configurations': {hybrid: True},
-                   'host': 'host',
-                   'agent_type': self.AGENT_TYPE,
-                   }]
+                   'host': 'host'}]
         context = self._make_port_ctx(agents)
         self.driver.bind_port(context)
         self.assertTrue(context._bound_vif_details[hybrid])
@@ -175,9 +163,7 @@ class OpenvswitchMechanismHybridPlugTestCase(OpenvswitchMechanismBaseTestCase):
         self.driver.vif_details[hybrid] = True
         agents = [{'alive': True,
                    'configurations': {hybrid: False},
-                   'host': 'host',
-                   'agent_type': self.AGENT_TYPE,
-                   }]
+                   'host': 'host'}]
         context = self._make_port_ctx(agents)
         self.driver.bind_port(context)
         self.assertFalse(context._bound_vif_details[hybrid])
@@ -346,7 +332,9 @@ class OpenvswitchMechVnicTypesTestCase(OpenvswitchMechanismBaseTestCase):
 
     supported_vnics = [portbindings.VNIC_NORMAL,
                        portbindings.VNIC_DIRECT,
-                       portbindings.VNIC_SMARTNIC]
+                       portbindings.VNIC_SMARTNIC,
+                       portbindings.VNIC_VHOST_VDPA,
+                       ]
 
     def setUp(self):
         self.prohibit_list_cfg = {
