@@ -354,6 +354,7 @@ class TestNBImplIdlOvn(TestDBImplIdlOvn):
         self._tables['DHCP_Options'] = self.dhcp_table
         self._tables['Address_Set'] = self.address_set_table
         self._tables['Load_Balancer'] = self.lb_table
+        self._tables['NAT'] = self.nat_table
 
         with mock.patch.object(impl_idl_ovn.OvsdbNbOvnIdl, 'from_worker',
                                return_value=mock.Mock()):
@@ -692,10 +693,10 @@ class TestNBImplIdlOvn(TestDBImplIdlOvn):
             'ports': []}, subnet_options)
         subnet_options = self.nb_ovn_idl.get_subnet_dhcp_options(
             'subnet-id-11-0-2-0')['subnet']
-        self.assertEqual({}, subnet_options)
+        self.assertIsNone(subnet_options)
         subnet_options = self.nb_ovn_idl.get_subnet_dhcp_options(
             'port-id-30-0-1-0')['subnet']
-        self.assertEqual({}, subnet_options)
+        self.assertIsNone(subnet_options)
 
     def test_get_subnet_dhcp_options_with_ports(self):
         # Test empty
