@@ -17,6 +17,7 @@ from unittest import mock
 
 from futurist import periodics
 from neutron_lib import context
+from neutron_lib.db import api as db_api
 from oslo_config import cfg
 
 from neutron.common.ovn import constants
@@ -139,39 +140,40 @@ class TestDBInconsistenciesPeriodics(testlib_api.SqlTestCaseLight,
                                                      never_again=False)
 
     def _test_fix_create_update_network(self, ovn_rev, neutron_rev):
-        self.net['revision_number'] = neutron_rev
+        with db_api.CONTEXT_WRITER.using(self.ctx):
+            self.net['revision_number'] = neutron_rev
 
-        # Create an entry to the revision_numbers table and assert the
-        # initial revision_number for our test object is the expected
-        ovn_revision_numbers_db.create_initial_revision(
-            self.ctx, self.net['id'], constants.TYPE_NETWORKS,
-            revision_number=ovn_rev)
-        row = ovn_revision_numbers_db.get_revision_row(self.ctx,
-                                                       self.net['id'])
-        self.assertEqual(ovn_rev, row.revision_number)
+            # Create an entry to the revision_numbers table and assert the
+            # initial revision_number for our test object is the expected
+            ovn_revision_numbers_db.create_initial_revision(
+                self.ctx, self.net['id'], constants.TYPE_NETWORKS,
+                revision_number=ovn_rev)
+            row = ovn_revision_numbers_db.get_revision_row(self.ctx,
+                                                           self.net['id'])
+            self.assertEqual(ovn_rev, row.revision_number)
 
-        if ovn_rev < 0:
-            self.fake_ovn_client._nb_idl.get_lswitch.return_value = None
-        else:
-            fake_ls = mock.Mock(external_ids={
-                constants.OVN_REV_NUM_EXT_ID_KEY: ovn_rev})
-            self.fake_ovn_client._nb_idl.get_lswitch.return_value = fake_ls
+            if ovn_rev < 0:
+                self.fake_ovn_client._nb_idl.get_lswitch.return_value = None
+            else:
+                fake_ls = mock.Mock(external_ids={
+                    constants.OVN_REV_NUM_EXT_ID_KEY: ovn_rev})
+                self.fake_ovn_client._nb_idl.get_lswitch.return_value = fake_ls
 
-        self.fake_ovn_client._plugin.get_network.return_value = self.net
-        self.periodic._fix_create_update(self.ctx, row)
+            self.fake_ovn_client._plugin.get_network.return_value = self.net
+            self.periodic._fix_create_update(self.ctx, row)
 
-        # Since the revision number was < 0, make sure create_network()
-        # is invoked with the latest version of the object in the neutron
-        # database
-        if ovn_rev < 0:
-            self.fake_ovn_client.create_network.assert_called_once_with(
-                self.ctx, self.net)
-        # If the revision number is > 0 it means that the object already
-        # exist and we just need to update to match the latest in the
-        # neutron database so, update_network() should be called.
-        else:
-            self.fake_ovn_client.update_network.assert_called_once_with(
-                self.ctx, self.net)
+            # Since the revision number was < 0, make sure create_network()
+            # is invoked with the latest version of the object in the neutron
+            # database
+            if ovn_rev < 0:
+                self.fake_ovn_client.create_network.assert_called_once_with(
+                    self.ctx, self.net)
+            # If the revision number is > 0 it means that the object already
+            # exist and we just need to update to match the latest in the
+            # neutron database so, update_network() should be called.
+            else:
+                self.fake_ovn_client.update_network.assert_called_once_with(
+                    self.ctx, self.net)
 
     def test_fix_network_create(self):
         self._test_fix_create_update_network(ovn_rev=-1, neutron_rev=2)
@@ -180,40 +182,41 @@ class TestDBInconsistenciesPeriodics(testlib_api.SqlTestCaseLight,
         self._test_fix_create_update_network(ovn_rev=5, neutron_rev=7)
 
     def _test_fix_create_update_port(self, ovn_rev, neutron_rev):
-        self.port['revision_number'] = neutron_rev
+        _nb_idl = self.fake_ovn_client._nb_idl
+        with db_api.CONTEXT_WRITER.using(self.ctx):
+            self.port['revision_number'] = neutron_rev
 
-        # Create an entry to the revision_numbers table and assert the
-        # initial revision_number for our test object is the expected
-        ovn_revision_numbers_db.create_initial_revision(
-            self.ctx, self.port['id'], constants.TYPE_PORTS,
-            revision_number=ovn_rev)
-        row = ovn_revision_numbers_db.get_revision_row(self.ctx,
-                                                       self.port['id'])
-        self.assertEqual(ovn_rev, row.revision_number)
+            # Create an entry to the revision_numbers table and assert the
+            # initial revision_number for our test object is the expected
+            ovn_revision_numbers_db.create_initial_revision(
+                self.ctx, self.port['id'], constants.TYPE_PORTS,
+                revision_number=ovn_rev)
+            row = ovn_revision_numbers_db.get_revision_row(self.ctx,
+                                                           self.port['id'])
+            self.assertEqual(ovn_rev, row.revision_number)
 
-        if ovn_rev < 0:
-            self.fake_ovn_client._nb_idl.get_lswitch_port.return_value = None
-        else:
-            fake_lsp = mock.Mock(external_ids={
-                constants.OVN_REV_NUM_EXT_ID_KEY: ovn_rev})
-            self.fake_ovn_client._nb_idl.get_lswitch_port.return_value = (
-                fake_lsp)
+            if ovn_rev < 0:
+                _nb_idl.get_lswitch_port.return_value = None
+            else:
+                fake_lsp = mock.Mock(external_ids={
+                    constants.OVN_REV_NUM_EXT_ID_KEY: ovn_rev})
+                _nb_idl.get_lswitch_port.return_value = fake_lsp
 
-        self.fake_ovn_client._plugin.get_port.return_value = self.port
-        self.periodic._fix_create_update(self.ctx, row)
+            self.fake_ovn_client._plugin.get_port.return_value = self.port
+            self.periodic._fix_create_update(self.ctx, row)
 
-        # Since the revision number was < 0, make sure create_port()
-        # is invoked with the latest version of the object in the neutron
-        # database
-        if ovn_rev < 0:
-            self.fake_ovn_client.create_port.assert_called_once_with(
-                self.ctx, self.port)
-        # If the revision number is > 0 it means that the object already
-        # exist and we just need to update to match the latest in the
-        # neutron database so, update_port() should be called.
-        else:
-            self.fake_ovn_client.update_port.assert_called_once_with(
-                self.ctx, self.port)
+            # Since the revision number was < 0, make sure create_port()
+            # is invoked with the latest version of the object in the neutron
+            # database
+            if ovn_rev < 0:
+                self.fake_ovn_client.create_port.assert_called_once_with(
+                    self.ctx, self.port)
+            # If the revision number is > 0 it means that the object already
+            # exist and we just need to update to match the latest in the
+            # neutron database so, update_port() should be called.
+            else:
+                self.fake_ovn_client.update_port.assert_called_once_with(
+                    self.ctx, self.port)
 
     def test_fix_port_create(self):
         self._test_fix_create_update_port(ovn_rev=-1, neutron_rev=2)
@@ -223,14 +226,16 @@ class TestDBInconsistenciesPeriodics(testlib_api.SqlTestCaseLight,
 
     @mock.patch.object(ovn_revision_numbers_db, 'bump_revision')
     def _test_fix_security_group_create(self, mock_bump, revision_number):
-        sg_name = utils.ovn_addrset_name('fake_id', 'ip4')
-        sg = self._make_security_group(self.fmt, sg_name, '')['security_group']
+        with db_api.CONTEXT_WRITER.using(self.ctx):
+            sg_name = utils.ovn_addrset_name('fake_id', 'ip4')
+            sg = self._make_security_group(
+                self.fmt, sg_name, '')['security_group']
 
-        ovn_revision_numbers_db.create_initial_revision(
-            self.ctx, sg['id'], constants.TYPE_SECURITY_GROUPS,
-            revision_number=revision_number)
-        row = ovn_revision_numbers_db.get_revision_row(self.ctx, sg['id'])
-        self.assertEqual(revision_number, row.revision_number)
+            ovn_revision_numbers_db.create_initial_revision(
+                self.ctx, sg['id'], constants.TYPE_SECURITY_GROUPS,
+                revision_number=revision_number)
+            row = ovn_revision_numbers_db.get_revision_row(self.ctx, sg['id'])
+            self.assertEqual(revision_number, row.revision_number)
 
         if revision_number < 0:
             self.fake_ovn_client._nb_idl.get_address_set.return_value = None
@@ -320,14 +325,21 @@ class TestDBInconsistenciesPeriodics(testlib_api.SqlTestCaseLight,
                    'other_config': {
                         constants.MCAST_SNOOP: 'true',
                         constants.MCAST_FLOOD_UNREGISTERED: 'false'}})
+        ls3 = fakes.FakeOvsdbRow.create_one_ovsdb_row(
+            attrs={'name': '',
+                   'other_config': {}})
+        ls4 = fakes.FakeOvsdbRow.create_one_ovsdb_row(
+            attrs={'name': '',
+                   'other_config': {constants.MCAST_SNOOP: 'false'}})
 
-        nb_idl.ls_list.return_value.execute.return_value = [ls0, ls1, ls2]
+        nb_idl.ls_list.return_value.execute.return_value = [ls0, ls1, ls2, ls3,
+                                                            ls4]
 
         self.assertRaises(periodics.NeverAgain,
                           self.periodic.check_for_igmp_snoop_support)
 
         # "ls2" is not part of the transaction because it already
-        # have the right value set
+        # have the right value set; "ls3" and "ls4" do not have a name set.
         expected_calls = [
             mock.call('Logical_Switch', 'ls0',
                       ('other_config', {
@@ -400,8 +412,7 @@ class TestDBInconsistenciesPeriodics(testlib_api.SqlTestCaseLight,
         nb_idl = self.fake_ovn_client._nb_idl
         lsp0 = fakes.FakeOvsdbRow.create_one_ovsdb_row(
             attrs={'name': 'lsp0',
-                   'options': {
-                       constants.LSP_OPTIONS_MCAST_FLOOD_REPORTS: 'true'},
+                   'options': {'mcast_flood_reports': 'true'},
                    'type': ""})
         lsp1 = fakes.FakeOvsdbRow.create_one_ovsdb_row(
             attrs={'name': 'lsp1', 'options': {}, 'type': ""})
@@ -416,106 +427,20 @@ class TestDBInconsistenciesPeriodics(testlib_api.SqlTestCaseLight,
                    'type': "router"})
         lsp5 = fakes.FakeOvsdbRow.create_one_ovsdb_row(
             attrs={'name': 'lsp5', 'options': {}, 'type': 'localnet'})
-        lsp6 = fakes.FakeOvsdbRow.create_one_ovsdb_row(
-            attrs={'name': 'lsp6',
-                   'options': {
-                       constants.LSP_OPTIONS_MCAST_FLOOD_REPORTS: 'true',
-                       constants.LSP_OPTIONS_MCAST_FLOOD: 'true'},
-                   'type': 'localnet'})
-        lsp7 = fakes.FakeOvsdbRow.create_one_ovsdb_row(
-            attrs={'name': 'lsp7',
-                   'options': {
-                       constants.LSP_OPTIONS_MCAST_FLOOD_REPORTS: 'true',
-                       constants.LSP_OPTIONS_MCAST_FLOOD: 'false'},
-                   'type': 'localnet'})
 
         nb_idl.lsp_list.return_value.execute.return_value = [
-            lsp0, lsp1, lsp2, lsp3, lsp4, lsp5, lsp6, lsp7]
+            lsp0, lsp1, lsp2, lsp3, lsp4, lsp5]
 
         # Invoke the periodic method, it meant to run only once at startup
         # so NeverAgain will be raised at the end
         self.assertRaises(periodics.NeverAgain,
                           self.periodic.check_for_mcast_flood_reports)
 
-        # Assert only lsp1, lsp5 and lsp6 were called because they are the
-        # only ones meeting the criteria
+        # Assert only lsp1 and lsp5 were called because they are the only
+        # ones meeting the criteria ("mcast_flood_reports" not yet set,
+        # and type "" or localnet)
         expected_calls = [
             mock.call('lsp1', mcast_flood_reports='true'),
-            mock.call('lsp5', mcast_flood_reports='true', mcast_flood='false'),
-            mock.call('lsp6', mcast_flood_reports='true', mcast_flood='false')]
+            mock.call('lsp5', mcast_flood_reports='true', mcast_flood='true')]
 
         nb_idl.lsp_set_options.assert_has_calls(expected_calls)
-
-    def test_check_router_mac_binding_options(self):
-        nb_idl = self.fake_ovn_client._nb_idl
-        lr0 = fakes.FakeOvsdbRow.create_one_ovsdb_row(
-            attrs={'name': 'lr0',
-                   'options': {'always_learn_from_arp_request': 'false',
-                               'dynamic_neigh_routers': 'true'}})
-        lr1 = fakes.FakeOvsdbRow.create_one_ovsdb_row(
-            attrs={'name': 'lr1', 'options': {}})
-        lr2 = fakes.FakeOvsdbRow.create_one_ovsdb_row(
-            attrs={'name': 'lr2', 'options': {}})
-        nb_idl.lr_list.return_value.execute.return_value = [lr0, lr1, lr2]
-
-        # Invoke the periodic method, it meant to run only once at startup
-        # so NeverAgain will be raised at the end
-        self.assertRaises(periodics.NeverAgain,
-                          self.periodic.check_router_mac_binding_options)
-
-        # Assert lr1 and lr2 had their options updated since the values
-        # were not set
-        expected_calls = [
-            mock.call('lr1',
-                      options={'always_learn_from_arp_request': 'false',
-                               'dynamic_neigh_routers': 'true'}),
-            mock.call('lr2',
-                      options={'always_learn_from_arp_request': 'false',
-                               'dynamic_neigh_routers': 'true'})]
-        nb_idl.update_lrouter.assert_has_calls(expected_calls)
-
-    def _test_check_vlan_distributed_ports(self, opt_value=None):
-        fake_net0 = {'id': 'net0'}
-        fake_net1 = {'id': 'net1'}
-        fake_port0 = {'id': 'port0'}
-        fake_port1 = {'id': 'port1'}
-
-        self.fake_ovn_client._plugin.get_networks.return_value = [
-            fake_net0, fake_net1]
-        self.fake_ovn_client._plugin.get_ports.return_value = [
-            fake_port0, fake_port1]
-
-        fake_lrp = fakes.FakeOvsdbRow.create_one_ovsdb_row(
-            attrs={
-                'name': 'lrp',
-                'options': {constants.LRP_OPTIONS_RESIDE_REDIR_CH: opt_value}})
-        self.fake_ovn_client._nb_idl.get_lrouter_port.return_value = fake_lrp
-
-        # Invoke the periodic method, it meant to run only once at startup
-        # so NeverAgain will be raised at the end
-        self.assertRaises(periodics.NeverAgain,
-                          self.periodic.check_vlan_distributed_ports)
-
-    def test_check_vlan_distributed_ports_expected_value(self):
-        cfg.CONF.set_override('enable_distributed_floating_ip', 'False',
-                              group='ovn')
-        self._test_check_vlan_distributed_ports(opt_value='true')
-
-        # If the "reside-on-redirect-chassis" option value do match
-        # the expected value, assert we do not update the database
-        self.assertFalse(
-            self.fake_ovn_client._nb_idl.db_set.called)
-
-    def test_check_vlan_distributed_ports_non_expected_value(self):
-        cfg.CONF.set_override('enable_distributed_floating_ip', 'False',
-                              group='ovn')
-        self._test_check_vlan_distributed_ports(opt_value='false')
-
-        # If the "reside-on-redirect-chassis" option value does not match
-        # the expected value, assert we update the database
-        opt = {constants.LRP_OPTIONS_RESIDE_REDIR_CH: 'true'}
-        expected_calls = [
-            mock.call('Logical_Router_Port', 'lrp-port0', ('options', opt)),
-            mock.call('Logical_Router_Port', 'lrp-port1', ('options', opt))]
-        self.fake_ovn_client._nb_idl.db_set.assert_has_calls(
-            expected_calls)
