@@ -33,6 +33,7 @@ from neutron.common.ovn import hash_ring_manager
 from neutron.common.ovn import utils
 from neutron.conf.plugins.ml2.drivers.ovn import ovn_conf
 from neutron.db import ovn_hash_ring_db
+from neutron.plugins.ml2.drivers.ovn.mech_driver.ovsdb import impl_idl_ovn
 from neutron.plugins.ml2.drivers.ovn.mech_driver.ovsdb import ovsdb_monitor
 from neutron.services.ovn_l3 import plugin  # noqa
 from neutron.tests import base
@@ -85,15 +86,7 @@ OVN_SB_SCHEMA = {
                              "min": 0, "max": "unlimited"}}},
             "isRoot": True,
             "indexes": [["name"]]
-        },
-        "Port_Binding": {
-            "columns": {
-                "logical_port": {"type": "string"},
-                "type": {"type": "string"},
-                "chassis": {"type": "string"},
-                "up": {"type": {"key": "boolean", "min": 0, "max": 1}}},
-            "indexes": [["logical_port"]],
-            "isRoot": True},
+        }
     }
 }
 
@@ -176,7 +169,9 @@ class TestOvnConnection(base.BaseTestCase):
                                idl_class, schema):
         mock_gsh.return_value = ovs_idl.SchemaHelper(
             location=schema_files[schema])
-        _idl = idl_class.from_server('punix:/tmp/fake', schema, mock.Mock())
+        impl_idl_ovn.Backend.schema = schema
+        helper = impl_idl_ovn.OvsdbNbOvnIdl.schema_helper
+        _idl = idl_class.from_server('punix:/fake', helper, mock.Mock())
         self.ovn_connection = connection.Connection(_idl, mock.Mock())
         with mock.patch.object(poller, 'Poller'), \
                 mock.patch('threading.Thread'):
@@ -224,14 +219,21 @@ class TestOvnIdlDistributedLock(base.BaseTestCase):
         self.mock_get_node = mock.patch.object(
             hash_ring_manager.HashRingManager,
             'get_node', return_value=self.node_uuid).start()
+        self.mock_update_tables = mock.patch.object(
+            self.idl, 'update_tables').start()
+
+    def _assert_has_notify_calls(self):
+        self.idl.notify_handler.notify.assert_has_calls([
+            mock.call(self.fake_event, self.fake_row, None, global_=True),
+            mock.call(self.fake_event, self.fake_row, None)])
+        self.assertEqual(2, len(self.idl.notify_handler.mock_calls))
 
     @mock.patch.object(ovn_hash_ring_db, 'touch_node')
     def test_notify(self, mock_touch_node):
         self.idl.notify(self.fake_event, self.fake_row)
 
         mock_touch_node.assert_called_once_with(mock.ANY, self.node_uuid)
-        self.idl.notify_handler.notify.assert_called_once_with(
-            self.fake_event, self.fake_row, None)
+        self._assert_has_notify_calls()
 
     @mock.patch.object(ovn_hash_ring_db, 'touch_node')
     def test_notify_skip_touch_node(self, mock_touch_node):
@@ -241,8 +243,7 @@ class TestOvnIdlDistributedLock(base.BaseTestCase):
 
         # Assert that touch_node() wasn't called
         self.assertFalse(mock_touch_node.called)
-        self.idl.notify_handler.notify.assert_called_once_with(
-            self.fake_event, self.fake_row, None)
+        self._assert_has_notify_calls()
 
     @mock.patch.object(ovn_hash_ring_db, 'touch_node')
     def test_notify_last_touch_expired(self, mock_touch_node):
@@ -258,8 +259,7 @@ class TestOvnIdlDistributedLock(base.BaseTestCase):
 
         # Assert that touch_node() was invoked
         mock_touch_node.assert_called_once_with(mock.ANY, self.node_uuid)
-        self.idl.notify_handler.notify.assert_called_once_with(
-            self.fake_event, self.fake_row, None)
+        self._assert_has_notify_calls()
 
     @mock.patch.object(ovsdb_monitor.LOG, 'exception')
     @mock.patch.object(ovn_hash_ring_db, 'touch_node')
@@ -272,14 +272,106 @@ class TestOvnIdlDistributedLock(base.BaseTestCase):
         mock_touch_node.assert_called_once_with(mock.ANY, self.node_uuid)
         # Assert we are logging the exception
         self.assertTrue(mock_log.called)
-        self.idl.notify_handler.notify.assert_called_once_with(
-            self.fake_event, self.fake_row, None)
+        self._assert_has_notify_calls()
 
     def test_notify_different_node(self):
         self.mock_get_node.return_value = 'different-node-uuid'
         self.idl.notify('fake-event', self.fake_row)
         # Assert that notify() wasn't called for a different node uuid
-        self.assertFalse(self.idl.notify_handler.notify.called)
+        self.idl.notify_handler.notify.assert_called_once_with(
+            self.fake_event, self.fake_row, None, global_=True)
+
+    @staticmethod
+    def _create_fake_row(table_name):
+        # name is a parameter in Mock() so it can't be passed to constructor
+        table = mock.Mock()
+        table.name = table_name
+        return fakes.FakeOvsdbRow.create_one_ovsdb_row(
+            attrs={'_table': table, 'schema': ['foo']})
+
+    def test_handle_db_schema_changes_no_match_events(self):
+        other_table_row = self._create_fake_row('other')
+        database_table_row = self._create_fake_row('Database')
+
+        self.idl.handle_db_schema_changes(
+            ovsdb_monitor.BaseEvent.ROW_UPDATE, other_table_row)
+        self.idl.handle_db_schema_changes(
+            ovsdb_monitor.BaseEvent.ROW_CREATE, other_table_row)
+        self.idl.handle_db_schema_changes(
+            ovsdb_monitor.BaseEvent.ROW_UPDATE, database_table_row)
+
+        self.assertFalse(self.mock_update_tables.called)
+
+    def _test_handle_db_schema(self, agent_table, chassis_private_present):
+        database_table_row = self._create_fake_row('Database')
+        self.idl._tables_to_register[database_table_row.name] = 'foo'
+
+        self.fake_driver.agent_chassis_table = agent_table
+        if chassis_private_present:
+            self.idl.tables['Chassis_Private'] = 'foo'
+        else:
+            try:
+                del self.idl.tables['Chassis_Private']
+            except KeyError:
+                pass
+
+        self.idl.handle_db_schema_changes(
+            ovsdb_monitor.BaseEvent.ROW_CREATE, database_table_row)
+
+    def test_handle_db_schema_changes_old_schema_to_old_schema(self):
+        """Agents use Chassis and should keep using Chassis table"""
+        self._test_handle_db_schema('Chassis', chassis_private_present=False)
+        self.assertEqual('Chassis', self.fake_driver.agent_chassis_table)
+
+    def test_handle_db_schema_changes_old_schema_to_new_schema(self):
+        """Agents use Chassis and should start using Chassis_Private table"""
+        self._test_handle_db_schema('Chassis', chassis_private_present=True)
+        self.assertEqual('Chassis_Private',
+                         self.fake_driver.agent_chassis_table)
+
+    def test_handle_db_schema_changes_new_schema_to_old_schema(self):
+        """Agents use Chassis_Private and should start using Chassis table"""
+        self._test_handle_db_schema('Chassis_Private',
+                                    chassis_private_present=False)
+        self.assertEqual('Chassis', self.fake_driver.agent_chassis_table)
+
+    def test_handle_db_schema_changes_new_schema_to_new_schema(self):
+        """Agents use Chassis_Private and should keep using Chassis_Private
+           table.
+        """
+        self._test_handle_db_schema('Chassis_Private',
+                                    chassis_private_present=True)
+        self.assertEqual('Chassis_Private',
+                         self.fake_driver.agent_chassis_table)
+
+
+class TestPortBindingChassisUpdateEvent(base.BaseTestCase):
+    def setUp(self):
+        super(TestPortBindingChassisUpdateEvent, self).setUp()
+        self.driver = mock.Mock()
+        self.event = ovsdb_monitor.PortBindingChassisUpdateEvent(self.driver)
+
+    def _test_event(self, event, row, old):
+        if self.event.matches(event, row, old):
+            self.event.run(event, row, old)
+            self.driver.set_port_status_up.assert_called()
+        else:
+            self.driver.set_port_status_up.assert_not_called()
+
+    def test_event_matches(self):
+        # NOTE(twilson) This primarily tests implementation details. If a
+        # scenario test is written that handles shutting down a compute
+        # node uncleanly and performing a 'host-evacuate', this can be removed
+        pbtable = fakes.FakeOvsdbTable.create_one_ovsdb_table(
+            attrs={'name': 'Port_Binding'})
+        ovsdb_row = fakes.FakeOvsdbRow.create_one_ovsdb_row
+        self.driver._nb_ovn.lookup.return_value = ovsdb_row(attrs={'up': True})
+        self._test_event(
+            self.event.ROW_UPDATE,
+            ovsdb_row(attrs={'_table': pbtable, 'chassis': 'one',
+                             'type': '_fake_', 'logical_port': 'foo'}),
+            ovsdb_row(attrs={'_table': pbtable, 'chassis': 'two',
+                             'type': '_fake_'}))
 
 
 class TestOvnNbIdlNotifyHandler(test_mech_driver.OVNMechanismDriverTestCase):
@@ -292,6 +384,106 @@ class TestOvnNbIdlNotifyHandler(test_mech_driver.OVNMechanismDriverTestCase):
         self.lp_table = self.idl.tables.get('Logical_Switch_Port')
         self.driver.set_port_status_up = mock.Mock()
         self.driver.set_port_status_down = mock.Mock()
+        mock.patch.object(self.idl, 'handle_db_schema_changes').start()
+
+    def _test_lsp_helper(self, event, new_row_json, old_row_json=None,
+                         table=None):
+        row_uuid = uuidutils.generate_uuid()
+        if not table:
+            table = self.lp_table
+        lp_row = ovs_idl.Row.from_json(self.idl, table,
+                                       row_uuid, new_row_json)
+        if old_row_json:
+            old_row = ovs_idl.Row.from_json(self.idl, table,
+                                            row_uuid, old_row_json)
+        else:
+            old_row = None
+        self.idl.notify(event, lp_row, updates=old_row)
+        # Add a STOP EVENT to the queue
+        self.idl.notify_handler.shutdown()
+        # Execute the notifications queued
+        self.idl.notify_handler.notify_loop()
+
+    def test_lsp_up_create_event(self):
+        row_data = {"up": True, "name": "foo-name"}
+        self._test_lsp_helper('create', row_data)
+        self.driver.set_port_status_up.assert_called_once_with("foo-name")
+        self.assertFalse(self.driver.set_port_status_down.called)
+
+    def test_lsp_down_create_event(self):
+        row_data = {"up": False, "name": "foo-name"}
+        self._test_lsp_helper('create', row_data)
+        self.driver.set_port_status_down.assert_called_once_with("foo-name")
+        self.assertFalse(self.driver.set_port_status_up.called)
+
+    def test_lsp_up_not_set_event(self):
+        row_data = {"up": ['set', []], "name": "foo-name"}
+        self._test_lsp_helper('create', row_data)
+        self.assertFalse(self.driver.set_port_status_up.called)
+        self.assertFalse(self.driver.set_port_status_down.called)
+
+    def test_unwatch_logical_switch_port_create_events(self):
+        self.idl.unwatch_logical_switch_port_create_events()
+        row_data = {"up": True, "name": "foo-name"}
+        self._test_lsp_helper('create', row_data)
+        self.assertFalse(self.driver.set_port_status_up.called)
+        self.assertFalse(self.driver.set_port_status_down.called)
+
+        row_data["up"] = False
+        self._test_lsp_helper('create', row_data)
+        self.assertFalse(self.driver.set_port_status_up.called)
+        self.assertFalse(self.driver.set_port_status_down.called)
+
+    def test_post_connect(self):
+        self.idl.post_connect()
+        self.assertIsNone(self.idl._lsp_create_up_event)
+        self.assertIsNone(self.idl._lsp_create_down_event)
+
+    def test_lsp_up_update_event(self):
+        new_row_json = {"up": True, "name": "foo-name"}
+        old_row_json = {"up": False}
+        self._test_lsp_helper('update', new_row_json,
+                              old_row_json=old_row_json)
+        self.driver.set_port_status_up.assert_called_once_with("foo-name")
+        self.assertFalse(self.driver.set_port_status_down.called)
+
+    def test_lsp_down_update_event(self):
+        new_row_json = {"up": False, "name": "foo-name"}
+        old_row_json = {"up": True}
+        self._test_lsp_helper('update', new_row_json,
+                              old_row_json=old_row_json)
+        self.driver.set_port_status_down.assert_called_once_with("foo-name")
+        self.assertFalse(self.driver.set_port_status_up.called)
+
+    def test_lsp_up_update_event_no_old_data(self):
+        new_row_json = {"up": True, "name": "foo-name"}
+        self._test_lsp_helper('update', new_row_json,
+                              old_row_json=None)
+        self.assertFalse(self.driver.set_port_status_up.called)
+        self.assertFalse(self.driver.set_port_status_down.called)
+
+    def test_lsp_down_update_event_no_old_data(self):
+        new_row_json = {"up": False, "name": "foo-name"}
+        self._test_lsp_helper('update', new_row_json,
+                              old_row_json=None)
+        self.assertFalse(self.driver.set_port_status_up.called)
+        self.assertFalse(self.driver.set_port_status_down.called)
+
+    def test_lsp_other_column_update_event(self):
+        new_row_json = {"up": False, "name": "foo-name",
+                        "addresses": ["10.0.0.2"]}
+        old_row_json = {"addresses": ["10.0.0.3"]}
+        self._test_lsp_helper('update', new_row_json,
+                              old_row_json=old_row_json)
+        self.assertFalse(self.driver.set_port_status_up.called)
+        self.assertFalse(self.driver.set_port_status_down.called)
+
+    def test_notify_other_table(self):
+        new_row_json = {"name": "foo-name"}
+        self._test_lsp_helper('create', new_row_json,
+                              table=self.idl.tables.get("Logical_Switch"))
+        self.assertFalse(self.driver.set_port_status_up.called)
+        self.assertFalse(self.driver.set_port_status_down.called)
 
     @mock.patch.object(hash_ring_manager.HashRingManager, 'get_node')
     def test_notify_different_target_node(self, mock_get_node):
@@ -300,8 +492,9 @@ class TestOvnNbIdlNotifyHandler(test_mech_driver.OVNMechanismDriverTestCase):
         self.idl.notify_handler.notify = mock.Mock()
         self.idl.notify("create", row)
         # Assert that if the target_node returned by the ring is different
-        # than this driver's node_uuid, notify() won't be called
-        self.assertFalse(self.idl.notify_handler.notify.called)
+        # than this driver's node_uuid, only global notify() won't be called
+        self.idl.notify_handler.notify.assert_called_once_with(
+            "create", row, None, global_=True)
 
 
 class TestOvnSbIdlNotifyHandler(test_mech_driver.OVNMechanismDriverTestCase):
@@ -312,24 +505,24 @@ class TestOvnSbIdlNotifyHandler(test_mech_driver.OVNMechanismDriverTestCase):
         super(TestOvnSbIdlNotifyHandler, self).setUp()
         sb_helper = ovs_idl.SchemaHelper(schema_json=OVN_SB_SCHEMA)
         sb_helper.register_table('Chassis')
-        sb_helper.register_table('Port_Binding')
         self.driver.agent_chassis_table = 'Chassis'
         self.sb_idl = ovsdb_monitor.OvnSbIdl(self.driver, "remote", sb_helper)
+        self.sb_idl.post_connect()
+        self.chassis_table = self.sb_idl.tables.get('Chassis')
         self.driver.update_segment_host_mapping = mock.Mock()
-        self.driver.set_port_status_up = mock.Mock()
-        self.driver.set_port_status_down = mock.Mock()
         self.l3_plugin = directory.get_plugin(n_const.L3)
         self.l3_plugin.schedule_unhosted_gateways = mock.Mock()
 
-        self.chassis_row_json = {
+        self.row_json = {
             "name": "fake-name",
             "hostname": "fake-hostname",
             "external_ids": ['map', [["ovn-bridge-mappings",
                                       "fake-phynet1:fake-br1"]]]
         }
 
-    def _test_helper(self, event, table, new_row_json, old_row_json=None):
+    def _test_chassis_helper(self, event, new_row_json, old_row_json=None):
         row_uuid = uuidutils.generate_uuid()
+        table = self.chassis_table
         row = ovs_idl.Row.from_json(self.sb_idl, table, row_uuid, new_row_json)
         if old_row_json:
             old_row = ovs_idl.Row.from_json(self.sb_idl, table,
@@ -342,223 +535,76 @@ class TestOvnSbIdlNotifyHandler(test_mech_driver.OVNMechanismDriverTestCase):
         # Execute the notifications queued
         self.sb_idl.notify_handler.notify_loop()
 
-    def _test_chassis_helper(self, event, new_row_json, old_row_json=None,
-                             table=None):
-        self.sb_idl.post_connect()
-        self.chassis_table = self.sb_idl.tables.get('Chassis')
-        if not table:
-            table = self.chassis_table
-        self._test_helper(event, table, new_row_json, old_row_json)
-
     def test_chassis_create_event(self):
-        self._test_chassis_helper('create', self.chassis_row_json)
+        self._test_chassis_helper('create', self.row_json)
         self.driver.update_segment_host_mapping.assert_called_once_with(
             'fake-hostname', ['fake-phynet1'])
         self.l3_plugin.schedule_unhosted_gateways.assert_called_once_with(
             event_from_chassis=None)
 
     def test_chassis_delete_event(self):
-        self._test_chassis_helper('delete', self.chassis_row_json)
+        self._test_chassis_helper('delete', self.row_json)
         self.driver.update_segment_host_mapping.assert_called_once_with(
             'fake-hostname', [])
         self.l3_plugin.schedule_unhosted_gateways.assert_called_once_with(
             event_from_chassis='fake-name')
 
     def test_chassis_update_event(self):
-        old_row_json = copy.deepcopy(self.chassis_row_json)
+        old_row_json = copy.deepcopy(self.row_json)
         old_row_json['external_ids'][1][0][1] = (
             "fake-phynet2:fake-br2")
-        self._test_chassis_helper('update', self.chassis_row_json,
-                                  old_row_json)
+        self._test_chassis_helper('update', self.row_json, old_row_json)
         self.driver.update_segment_host_mapping.assert_called_once_with(
             'fake-hostname', ['fake-phynet1'])
         self.l3_plugin.schedule_unhosted_gateways.assert_called_once_with(
             event_from_chassis=None)
 
     def test_chassis_update_event_reschedule_not_needed(self):
-        self.chassis_row_json['external_ids'][1].append(['foo_field',
-                                                         'foo_value_new'])
-        old_row_json = copy.deepcopy(self.chassis_row_json)
+        self.row_json['external_ids'][1].append(['foo_field', 'foo_value_new'])
+        old_row_json = copy.deepcopy(self.row_json)
         old_row_json['external_ids'][1][1][1] = (
             "foo_value")
-        self._test_chassis_helper('update', self.chassis_row_json,
-                                  old_row_json)
+        self._test_chassis_helper('update', self.row_json, old_row_json)
         self.driver.update_segment_host_mapping.assert_not_called()
         self.l3_plugin.schedule_unhosted_gateways.assert_not_called()
 
     def test_chassis_update_event_reschedule_lost_physnet(self):
-        old_row_json = copy.deepcopy(self.chassis_row_json)
-        self.chassis_row_json['external_ids'][1][0][1] = ''
-        self._test_chassis_helper('update', self.chassis_row_json,
-                                  old_row_json)
+        old_row_json = copy.deepcopy(self.row_json)
+        self.row_json['external_ids'][1][0][1] = ''
+        self._test_chassis_helper('update', self.row_json, old_row_json)
         self.l3_plugin.schedule_unhosted_gateways.assert_called_once_with(
             event_from_chassis='fake-name')
 
     def test_chassis_update_event_reschedule_add_physnet(self):
-        old_row_json = copy.deepcopy(self.chassis_row_json)
-        self.chassis_row_json['external_ids'][1][0][1] += ',foo_physnet:foo_br'
-        self._test_chassis_helper('update', self.chassis_row_json,
-                                  old_row_json)
+        old_row_json = copy.deepcopy(self.row_json)
+        self.row_json['external_ids'][1][0][1] += ',foo_physnet:foo_br'
+        self._test_chassis_helper('update', self.row_json, old_row_json)
         self.driver.update_segment_host_mapping.assert_called_once_with(
             'fake-hostname', ['fake-phynet1', 'foo_physnet'])
         self.l3_plugin.schedule_unhosted_gateways.assert_called_once_with(
             event_from_chassis=None)
 
     def test_chassis_update_event_reschedule_add_and_remove_physnet(self):
-        old_row_json = copy.deepcopy(self.chassis_row_json)
-        self.chassis_row_json['external_ids'][1][0][1] = 'foo_physnet:foo_br'
-        self._test_chassis_helper('update', self.chassis_row_json,
-                                  old_row_json)
+        old_row_json = copy.deepcopy(self.row_json)
+        self.row_json['external_ids'][1][0][1] = 'foo_physnet:foo_br'
+        self._test_chassis_helper('update', self.row_json, old_row_json)
         self.driver.update_segment_host_mapping.assert_called_once_with(
             'fake-hostname', ['foo_physnet'])
         self.l3_plugin.schedule_unhosted_gateways.assert_called_once_with(
             event_from_chassis=None)
 
     def test_chassis_update_empty_no_external_ids(self):
-        old_row_json = copy.deepcopy(self.chassis_row_json)
+        old_row_json = copy.deepcopy(self.row_json)
         old_row_json.pop('external_ids')
         with mock.patch(
             'neutron.plugins.ml2.drivers.ovn.mech_driver.ovsdb.'
             'ovsdb_monitor.ChassisEvent.'
                 'handle_ha_chassis_group_changes') as mock_ha:
-            self._test_chassis_helper('update', self.chassis_row_json,
-                                      old_row_json)
+            self._test_chassis_helper('update', self.row_json, old_row_json)
             self.driver.update_segment_host_mapping.assert_not_called()
             self.l3_plugin.schedule_unhosted_gateways.assert_not_called()
             mock_ha.assert_not_called()
 
-    def _test_port_binding_helper(self, event, new_row_json,
-                                  old_row_json=None, table=None):
-        self.port_binding_table = self.sb_idl.tables.get('Port_Binding')
-        if not table:
-            table = self.port_binding_table
-        self._test_helper(event, table, new_row_json, old_row_json)
-
-    def test_port_binding_up_create_event(self):
-        row_data = {"type": "", "up": True, "logical_port": "foo-name",
-                    "chassis": "foo-host"}
-        self._test_port_binding_helper('create', row_data)
-        self.driver.set_port_status_up.assert_called_once_with("foo-name")
-        self.assertFalse(self.driver.set_port_status_down.called)
-
-    def test_port_binding_up_create_patch_type_event(self):
-        # Note(ltomasbo): This will behave the same for up = True|False
-        # as the port needs to transition to up anyway
-        row_data = {"type": "patch", "logical_port": "foo-name",
-                    "chassis": "foo-host"}
-        self._test_port_binding_helper('create', row_data)
-        self.driver.set_port_status_up.assert_called_once_with("foo-name")
-        self.assertFalse(self.driver.set_port_status_down.called)
-
-        row_data = {"type": "patch", "logical_port": "lrp-foo-name"}
-        self._test_port_binding_helper('create', row_data)
-        self.assertEqual(1, self.driver.set_port_status_up.call_count)
-        self.assertFalse(self.driver.set_port_status_down.called)
-
-    def test_port_binding_up_create_virtual_type_event(self):
-        row_data = {"type": "virtual", "up": True,
-                    "logical_port": "foo-name", "chassis": "foo-host"}
-        self._test_port_binding_helper('create', row_data)
-        self.assertFalse(self.driver.set_port_status_up.called)
-        self.assertFalse(self.driver.set_port_status_down.called)
-
-    def test_port_binding_down_create_event(self):
-        row_data = {"type": "", "up": False, "logical_port": "foo-name",
-                    "chassis": None}
-        self._test_port_binding_helper('create', row_data)
-        self.driver.set_port_status_down.assert_called_once_with("foo-name")
-        self.assertFalse(self.driver.set_port_status_up.called)
-
-    def test_port_binding_down_create_virtual_type_event(self):
-        row_data = {"type": "virtual", "up": False,
-                    "logical_port": "foo-name", "chassis": None}
-        self._test_port_binding_helper('create', row_data)
-        self.assertFalse(self.driver.set_port_status_up.called)
-        self.assertFalse(self.driver.set_port_status_down.called)
-
-    def test_port_binding_up_update_event(self):
-        new_row_json = {"type": "", "up": True, "logical_port": "foo-name",
-                        "chassis": "foo-host"}
-        old_row_json = {"type": "", "up": False, "logical_port": "foo-name",
-                        "chassis": None}
-        self._test_port_binding_helper('update', new_row_json,
-                                       old_row_json=old_row_json)
-        self.driver.set_port_status_up.assert_called_once_with("foo-name")
-        self.assertFalse(self.driver.set_port_status_down.called)
-
-    def test_port_binding_down_update_event(self):
-        new_row_json = {"type": "", "up": False, "logical_port": "foo-name",
-                        "chassis": None}
-        old_row_json = {"type": "", "up": True, "logical_port": "foo-name",
-                        "chassis": "foo-host"}
-        self._test_port_binding_helper('update', new_row_json,
-                                       old_row_json=old_row_json)
-        self.driver.set_port_status_down.assert_called_once_with("foo-name")
-        self.assertFalse(self.driver.set_port_status_up.called)
-
-    def test_port_binding_down_update_event_up_not_updated(self):
-        new_row_json = {"type": "", "up": True, "logical_port": "foo-name",
-                        "chassis": None}
-        old_row_json = {"type": "", "up": True, "logical_port": "foo-name",
-                        "chassis": "foo-host"}
-        self._test_port_binding_helper('update', new_row_json,
-                                       old_row_json=old_row_json)
-        self.driver.set_port_status_down.assert_called_once_with("foo-name")
-        self.assertFalse(self.driver.set_port_status_up.called)
-
-    def test_port_binding_up_update_event_no_old_data(self):
-        new_row_json = {"type": "", "up": True, "logical_port": "foo-name",
-                        "chassis": "foo-host"}
-        self._test_port_binding_helper('update', new_row_json,
-                                       old_row_json=None)
-        self.assertFalse(self.driver.set_port_status_up.called)
-        self.assertFalse(self.driver.set_port_status_down.called)
-
-    def test_port_binding_down_update_event_no_old_data(self):
-        new_row_json = {"type": "", "up": False, "logical_port": "foo-name",
-                        "chassis": None}
-        self._test_port_binding_helper('update', new_row_json,
-                                       old_row_json=None)
-        self.assertFalse(self.driver.set_port_status_up.called)
-        self.assertFalse(self.driver.set_port_status_down.called)
-
-    def test_port_binding_other_column_update_event(self):
-        new_row_json = {"type": "", "up": True, "name": "foo-name",
-                        "addresses": ["10.0.0.2"]}
-        old_row_json = {"type": "", "up": True, "name": "foo-name",
-                        "addresses": ["10.0.0.3"]}
-        self._test_port_binding_helper('update', new_row_json,
-                                       old_row_json=old_row_json)
-        self.assertFalse(self.driver.set_port_status_up.called)
-        self.assertFalse(self.driver.set_port_status_down.called)
-
-    def test_notify_other_table(self):
-        new_row_json = {"type": "", "up": True, "logical_port": "foo-name"}
-        self._test_port_binding_helper(
-            'create', new_row_json,
-            table=self.sb_idl.tables.get("Chassis"))
-        self.assertFalse(self.driver.set_port_status_up.called)
-        self.assertFalse(self.driver.set_port_status_down.called)
-
-    def test_unwatch_port_binding_create_events(self):
-        self.sb_idl.unwatch_port_binding_create_events()
-        row_data = {"type": "", "up": True, "logical_port": "foo-name"}
-        self._test_port_binding_helper('create', row_data)
-        self.assertFalse(self.driver.set_port_status_up.called)
-        self.assertFalse(self.driver.set_port_status_down.called)
-
-        row_data["up"] = False
-        self._test_port_binding_helper('create', row_data)
-        self.assertFalse(self.driver.set_port_status_up.called)
-        self.assertFalse(self.driver.set_port_status_down.called)
-
-    def test_post_connect(self):
-        self.assertIsNotNone(self.sb_idl._pb_create_up_event)
-        self.assertIsNotNone(self.sb_idl._pb_create_down_event)
-        self.sb_idl.post_connect()
-        self.assertIsNone(self.sb_idl._pb_create_up_event)
-        self.assertIsNone(self.sb_idl._pb_create_down_event)
-
 
 class TestChassisEvent(base.BaseTestCase):
 
