@@ -18,8 +18,8 @@ Unit Tests for ml2 rpc
 """
 
 import collections
+from unittest import mock
 
-import mock
 from neutron_lib.agent import topics
 from neutron_lib.api.definitions import portbindings
 from neutron_lib.callbacks import resources
@@ -479,8 +479,8 @@ class RpcApiTestCase(base.BaseTestCase):
                            devices_down=['fake_device3', 'fake_device4'],
                            agent_id='fake_agent_id',
                            host='fake_host',
-                           agent_restarted=False,
-                           version='1.5')
+                           refresh_tunnels=False,
+                           version='1.8')
 
     def test_get_devices_details_list_and_failed_devices(self):
         rpcapi = agent_rpc.PluginApi(topics.PLUGIN)
