@@ -56,6 +56,8 @@ class TestOVNL3RouterPlugin(test_mech_driver.Ml2PluginV2TestCase):
         return patch
 
     def setUp(self):
+        mock.patch('neutron.plugins.ml2.drivers.ovn.mech_driver.ovsdb.'
+                   'impl_idl_ovn.Backend.schema_helper').start()
         super(TestOVNL3RouterPlugin, self).setUp()
         revision_plugin.RevisionPlugin()
         network_attrs = {external_net.EXTERNAL: True, 'mtu': 1500}
@@ -538,8 +540,7 @@ class TestOVNL3RouterPlugin(test_mech_driver.Ml2PluginV2TestCase):
                         ovn_const.OVN_ROUTER_AZ_HINTS_EXT_ID_KEY: ''}
         self.l3_inst._ovn.create_lrouter.assert_called_once_with(
             'neutron-router-id', external_ids=external_ids,
-            enabled=True, options={'always_learn_from_arp_request': 'false',
-                                   'dynamic_neigh_routers': 'true'})
+            enabled=True, options={})
         self.l3_inst._ovn.add_lrouter_port.assert_called_once_with(
             **self.fake_ext_gw_port_assert)
         expected_calls = [
@@ -890,6 +891,8 @@ class TestOVNL3RouterPlugin(test_mech_driver.Ml2PluginV2TestCase):
                 'ovn_client.OVNClient._check_external_ips_changed')
     @mock.patch.object(utils, 'get_lrouter_snats')
     @mock.patch.object(utils, 'get_lrouter_ext_gw_static_route')
+    @mock.patch('neutron.common.ovn.utils.is_snat_enabled',
+                mock.Mock(return_value=True))
     @mock.patch('neutron.plugins.ml2.drivers.ovn.mech_driver.ovsdb.'
                 'ovn_client.OVNClient._get_router_ports')
     @mock.patch('neutron.db.extraroute_db.ExtraRoute_dbonly_mixin.'
@@ -928,7 +931,9 @@ class TestOVNL3RouterPlugin(test_mech_driver.Ml2PluginV2TestCase):
                 self.fake_floating_ip['port_id'],
             ovn_const.OVN_ROUTER_NAME_EXT_ID_KEY: utils.ovn_name(
                 self.fake_floating_ip['router_id']),
-            ovn_const.OVN_FIP_EXT_MAC_KEY: 'aa:aa:aa:aa:aa:aa'}
+            ovn_const.OVN_FIP_EXT_MAC_KEY: 'aa:aa:aa:aa:aa:aa',
+            ovn_const.OVN_FIP_NET_ID:
+                self.fake_floating_ip['floating_network_id']}
         self.l3_inst._ovn.add_nat_rule_in_lrouter.assert_called_once_with(
             'neutron-router-id',
             type='dnat_and_snat',
@@ -952,7 +957,9 @@ class TestOVNL3RouterPlugin(test_mech_driver.Ml2PluginV2TestCase):
                 self.fake_floating_ip['port_id'],
             ovn_const.OVN_ROUTER_NAME_EXT_ID_KEY: utils.ovn_name(
                 self.fake_floating_ip['router_id']),
-            ovn_const.OVN_FIP_EXT_MAC_KEY: '00:01:02:03:04:05'}
+            ovn_const.OVN_FIP_EXT_MAC_KEY: '00:01:02:03:04:05',
+            ovn_const.OVN_FIP_NET_ID:
+                self.fake_floating_ip['floating_network_id']}
         self.l3_inst._ovn.add_nat_rule_in_lrouter.assert_called_once_with(
             'neutron-router-id', type='dnat_and_snat', logical_ip='10.0.0.10',
             external_ip='192.168.0.10', external_mac='00:01:02:03:04:05',
@@ -978,7 +985,9 @@ class TestOVNL3RouterPlugin(test_mech_driver.Ml2PluginV2TestCase):
                 self.fake_floating_ip['port_id'],
             ovn_const.OVN_ROUTER_NAME_EXT_ID_KEY: utils.ovn_name(
                 self.fake_floating_ip['router_id']),
-            ovn_const.OVN_FIP_EXT_MAC_KEY: '00:01:02:03:04:05'}
+            ovn_const.OVN_FIP_EXT_MAC_KEY: '00:01:02:03:04:05',
+            ovn_const.OVN_FIP_NET_ID:
+                self.fake_floating_ip['floating_network_id']}
         self.l3_inst._ovn.add_nat_rule_in_lrouter.assert_called_once_with(
             'neutron-router-id', type='dnat_and_snat', logical_ip='10.0.0.10',
             external_ip='192.168.0.10',
@@ -999,7 +1008,9 @@ class TestOVNL3RouterPlugin(test_mech_driver.Ml2PluginV2TestCase):
                 self.fake_floating_ip['port_id'],
             ovn_const.OVN_ROUTER_NAME_EXT_ID_KEY: utils.ovn_name(
                 self.fake_floating_ip['router_id']),
-            ovn_const.OVN_FIP_EXT_MAC_KEY: 'aa:aa:aa:aa:aa:aa'}
+            ovn_const.OVN_FIP_EXT_MAC_KEY: 'aa:aa:aa:aa:aa:aa',
+            ovn_const.OVN_FIP_NET_ID:
+                self.fake_floating_ip['floating_network_id']}
         self.l3_inst._ovn.add_nat_rule_in_lrouter.assert_called_once_with(
             'neutron-router-id',
             type='dnat_and_snat',
@@ -1023,7 +1034,9 @@ class TestOVNL3RouterPlugin(test_mech_driver.Ml2PluginV2TestCase):
                 self.fake_floating_ip['port_id'],
             ovn_const.OVN_ROUTER_NAME_EXT_ID_KEY: utils.ovn_name(
                 self.fake_floating_ip['router_id']),
-            ovn_const.OVN_FIP_EXT_MAC_KEY: 'aa:aa:aa:aa:aa:aa'}
+            ovn_const.OVN_FIP_EXT_MAC_KEY: 'aa:aa:aa:aa:aa:aa',
+            ovn_const.OVN_FIP_NET_ID:
+                self.fake_floating_ip['floating_network_id']}
         self.l3_inst._ovn.add_nat_rule_in_lrouter.assert_called_once_with(
             'neutron-router-id',
             type='dnat_and_snat',
@@ -1052,14 +1065,22 @@ class TestOVNL3RouterPlugin(test_mech_driver.Ml2PluginV2TestCase):
         self.get_port.return_value = self.member_port
         self.l3_inst._ovn.lookup.return_value = self.lb_network
         self.l3_inst._ovn.get_lswitch_port.return_value = self.member_lsp
-        self.l3_inst.create_floatingip(self.context, 'floatingip')
+        fip = self.l3_inst.create_floatingip(self.context, 'floatingip')
         # Validate that there is no external_mac and logical_port while
         # setting the NAT entry.
+        expected_ext_ids = {
+              ovn_const.OVN_FIP_EXT_ID_KEY: fip['id'],
+              ovn_const.OVN_REV_NUM_EXT_ID_KEY: mock.ANY,
+              ovn_const.OVN_FIP_PORT_EXT_ID_KEY: fip['port_id'],
+              ovn_const.OVN_ROUTER_NAME_EXT_ID_KEY: 'neutron-router-id',
+              ovn_const.OVN_FIP_EXT_MAC_KEY: self.member_port['mac_address'],
+              ovn_const.OVN_FIP_NET_ID: fip['floating_network_id']}
         self.l3_inst._ovn.add_nat_rule_in_lrouter.assert_called_once_with(
             'neutron-router-id',
             external_ip='192.168.0.10',
             logical_ip='10.0.0.10',
-            type='dnat_and_snat')
+            type='dnat_and_snat',
+            external_ids=expected_ext_ids)
 
     def test_create_floatingip_lb_vip_fip(self):
         config.cfg.CONF.set_override(
@@ -1072,15 +1093,23 @@ class TestOVNL3RouterPlugin(test_mech_driver.Ml2PluginV2TestCase):
             [self.fake_ovn_nat_rule],
         ]
         self.l3_inst._ovn.lookup.return_value = self.lb_network
+        fip = self.l3_inst.create_floatingip(self.context, 'floatingip')
+        expected_ext_ids = {
+              ovn_const.OVN_FIP_EXT_ID_KEY: fip['id'],
+              ovn_const.OVN_REV_NUM_EXT_ID_KEY: mock.ANY,
+              ovn_const.OVN_FIP_PORT_EXT_ID_KEY: fip['port_id'],
+              ovn_const.OVN_ROUTER_NAME_EXT_ID_KEY: 'neutron-router-id',
+              ovn_const.OVN_FIP_EXT_MAC_KEY: self.member_port['mac_address'],
+              ovn_const.OVN_FIP_NET_ID: fip['floating_network_id']}
 
-        self.l3_inst.create_floatingip(self.context, 'floatingip')
         self.l3_inst._ovn.add_nat_rule_in_lrouter.assert_called_once_with(
             'neutron-router-id',
             external_ip='192.168.0.10',
             external_mac='aa:aa:aa:aa:aa:aa',
             logical_ip='10.0.0.10',
             logical_port='port_id',
-            type='dnat_and_snat')
+            type='dnat_and_snat',
+            external_ids=expected_ext_ids)
         self.l3_inst._ovn.db_find_rows.assert_called_with(
             'NAT', ('external_ids', '=', {ovn_const.OVN_FIP_PORT_EXT_ID_KEY:
                                           self.member_lsp.name}))
@@ -1178,7 +1207,9 @@ class TestOVNL3RouterPlugin(test_mech_driver.Ml2PluginV2TestCase):
                 self.fake_floating_ip_new['port_id'],
             ovn_const.OVN_ROUTER_NAME_EXT_ID_KEY: utils.ovn_name(
                 self.fake_floating_ip_new['router_id']),
-            ovn_const.OVN_FIP_EXT_MAC_KEY: 'aa:aa:aa:aa:aa:aa'}
+            ovn_const.OVN_FIP_EXT_MAC_KEY: 'aa:aa:aa:aa:aa:aa',
+            ovn_const.OVN_FIP_NET_ID:
+                self.fake_floating_ip['floating_network_id']}
         self.l3_inst._ovn.add_nat_rule_in_lrouter.assert_called_once_with(
             'neutron-new-router-id',
             type='dnat_and_snat',
@@ -1202,7 +1233,9 @@ class TestOVNL3RouterPlugin(test_mech_driver.Ml2PluginV2TestCase):
                 self.fake_floating_ip_new['port_id'],
             ovn_const.OVN_ROUTER_NAME_EXT_ID_KEY: utils.ovn_name(
                 self.fake_floating_ip_new['router_id']),
-            ovn_const.OVN_FIP_EXT_MAC_KEY: 'aa:aa:aa:aa:aa:aa'}
+            ovn_const.OVN_FIP_EXT_MAC_KEY: 'aa:aa:aa:aa:aa:aa',
+            ovn_const.OVN_FIP_NET_ID:
+                self.fake_floating_ip['floating_network_id']}
         self.l3_inst._ovn.add_nat_rule_in_lrouter.assert_called_once_with(
             'neutron-new-router-id',
             type='dnat_and_snat',
@@ -1236,7 +1269,9 @@ class TestOVNL3RouterPlugin(test_mech_driver.Ml2PluginV2TestCase):
                 self.fake_floating_ip_new['port_id'],
             ovn_const.OVN_ROUTER_NAME_EXT_ID_KEY: utils.ovn_name(
                 self.fake_floating_ip_new['router_id']),
-            ovn_const.OVN_FIP_EXT_MAC_KEY: '00:01:02:03:04:05'}
+            ovn_const.OVN_FIP_EXT_MAC_KEY: '00:01:02:03:04:05',
+            ovn_const.OVN_FIP_NET_ID:
+                self.fake_floating_ip['floating_network_id']}
         self.l3_inst._ovn.add_nat_rule_in_lrouter.assert_called_once_with(
             'neutron-new-router-id', type='dnat_and_snat',
             logical_ip='10.10.10.10', external_ip='192.168.0.10',
@@ -1265,7 +1300,9 @@ class TestOVNL3RouterPlugin(test_mech_driver.Ml2PluginV2TestCase):
                 self.fake_floating_ip_new['port_id'],
             ovn_const.OVN_ROUTER_NAME_EXT_ID_KEY: utils.ovn_name(
                 self.fake_floating_ip_new['router_id']),
-            ovn_const.OVN_FIP_EXT_MAC_KEY: 'aa:aa:aa:aa:aa:aa'}
+            ovn_const.OVN_FIP_EXT_MAC_KEY: 'aa:aa:aa:aa:aa:aa',
+            ovn_const.OVN_FIP_NET_ID:
+                self.fake_floating_ip['floating_network_id']}
         self.l3_inst._ovn.add_nat_rule_in_lrouter.assert_called_once_with(
             'neutron-new-router-id',
             type='dnat_and_snat',
@@ -1298,7 +1335,9 @@ class TestOVNL3RouterPlugin(test_mech_driver.Ml2PluginV2TestCase):
                 self.fake_floating_ip_new['port_id'],
             ovn_const.OVN_ROUTER_NAME_EXT_ID_KEY: utils.ovn_name(
                 self.fake_floating_ip_new['router_id']),
-            ovn_const.OVN_FIP_EXT_MAC_KEY: 'aa:aa:aa:aa:aa:aa'}
+            ovn_const.OVN_FIP_EXT_MAC_KEY: 'aa:aa:aa:aa:aa:aa',
+            ovn_const.OVN_FIP_NET_ID:
+                self.fake_floating_ip['floating_network_id']}
         self.l3_inst._ovn.add_nat_rule_in_lrouter.assert_called_once_with(
             'neutron-new-router-id',
             type='dnat_and_snat',
@@ -1314,13 +1353,15 @@ class TestOVNL3RouterPlugin(test_mech_driver.Ml2PluginV2TestCase):
                              'router_id': 'router-id',
                              'port_id': 'port_id',
                              'floating_port_id': 'fip-port-id1',
-                             'fixed_ip_address': '10.0.0.10'},
+                             'fixed_ip_address': '10.0.0.10',
+                             'floating_network_id': 'net1'},
                             {'id': 'fip-id2',
                              'floating_ip_address': '192.167.0.10',
                              'router_id': 'router-id',
                              'port_id': 'port_id',
                              'floating_port_id': 'fip-port-id2',
-                             'fixed_ip_address': '10.0.0.11'}]
+                             'fixed_ip_address': '10.0.0.11',
+                             'floating_network_id': 'net2'}]
         self.l3_inst.disassociate_floatingips(self.context, 'port_id',
                                               do_notify=False)
 
@@ -1339,14 +1380,12 @@ class TestOVNL3RouterPlugin(test_mech_driver.Ml2PluginV2TestCase):
                 'ovn_client.OVNClient.update_router_port')
     def test_port_update_postcommit(self, update_rp_mock):
         kwargs = {'port': {'device_owner': 'foo'},
-                  'original_port': '',
                   'context': 'fake_context'}
         self.l3_inst._port_update(resources.PORT, events.AFTER_UPDATE, None,
                                   **kwargs)
         update_rp_mock.assert_not_called()
 
         kwargs = {'port': {'device_owner': constants.DEVICE_OWNER_ROUTER_INTF},
-                  'original_port': '',
                   'context': 'fake_context'}
         self.l3_inst._port_update(resources.PORT, events.AFTER_UPDATE, None,
                                   **kwargs)
@@ -1355,26 +1394,6 @@ class TestOVNL3RouterPlugin(test_mech_driver.Ml2PluginV2TestCase):
                                                kwargs['port'],
                                                if_exists=True)
 
-    def test_port_update_before_update_router_port_without_ip(self):
-        context = 'fake_context'
-        og_port = {'device_owner': constants.DEVICE_OWNER_ROUTER_INTF,
-                'fixed_ips': ['10.0.0.10'],
-                'id': 'port-id'}
-        port = {'device_owner': constants.DEVICE_OWNER_ROUTER_INTF,
-                'fixed_ips': [],
-                'id': 'port-id'}
-        kwargs = {'original_port': og_port,
-                  'port': port,
-                  'context': context}
-        self.assertRaises(
-            n_exc.ServicePortInUse,
-            self.l3_inst._port_update,
-            resources.PORT,
-            events.BEFORE_UPDATE,
-            None,
-            **kwargs,
-        )
-
     @mock.patch('neutron.plugins.ml2.plugin.Ml2Plugin.update_port_status')
     @mock.patch('neutron.plugins.ml2.plugin.Ml2Plugin.update_port')
     @mock.patch('neutron.db.db_base_plugin_v2.NeutronDbPluginV2.get_ports')
@@ -1505,28 +1524,20 @@ class TestOVNL3RouterPlugin(test_mech_driver.Ml2PluginV2TestCase):
         self.nb_idl().get_unhosted_gateways.assert_called_once_with(
             {'foo-1': 'physnet1'}, mock.ANY, mock.ANY)
 
-    @mock.patch('neutron.plugins.ml2.plugin.Ml2Plugin.get_network')
-    @mock.patch('neutron.plugins.ml2.plugin.Ml2Plugin.get_networks')
+    @mock.patch('neutron.db.db_base_plugin_v2.NeutronDbPluginV2.get_network')
     @mock.patch('neutron.plugins.ml2.drivers.ovn.mech_driver.ovsdb.'
                 'ovn_client.OVNClient._get_router_ports')
     @mock.patch('neutron.db.l3_db.L3_NAT_dbonly_mixin.add_router_interface')
-    def test_add_router_interface_need_to_frag_enabled(
-            self, ari, grps, gns, gn):
+    def test_add_router_interface_need_to_frag_enabled(self, ari, grps, gn):
         config.cfg.CONF.set_override(
             'ovn_emit_need_to_frag', True, group='ovn')
         router_id = 'router-id'
-        interface_info = {'port_id': 'router-port-id',
-                'network_id': 'priv-net'}
+        interface_info = {'port_id': 'router-port-id'}
         ari.return_value = self.fake_router_interface_info
-        grps.return_value = [interface_info]
         self.get_router.return_value = self.fake_router_with_ext_gw
-        network_attrs = {'id': 'prov-net', 'mtu': 1200}
-        prov_net = fake_resources.FakeNetwork.create_one_network(
-                attrs=network_attrs).info()
+        gn.return_value = self.fake_network
         self.fake_router_port['device_owner'] = (
             constants.DEVICE_OWNER_ROUTER_GW)
-        gn.return_value = prov_net
-        gns.return_value = [self.fake_network]
 
         self.l3_inst.add_router_interface(self.context, router_id,
                                           interface_info)
@@ -1536,7 +1547,7 @@ class TestOVNL3RouterPlugin(test_mech_driver.Ml2PluginV2TestCase):
         fake_router_port_assert['gateway_chassis'] = mock.ANY
         fake_router_port_assert['options'] = {
             ovn_const.OVN_ROUTER_PORT_GW_MTU_OPTION:
-            str(prov_net['mtu'])}
+            str(self.fake_network['mtu'])}
 
         self.l3_inst._ovn.add_lrouter_port.assert_called_once_with(
             **fake_router_port_assert)
@@ -1552,52 +1563,6 @@ class TestOVNL3RouterPlugin(test_mech_driver.Ml2PluginV2TestCase):
             mock.ANY, self.fake_router_port,
             ovn_const.TYPE_ROUTER_PORTS)
 
-    @mock.patch('neutron.plugins.ml2.plugin.Ml2Plugin.get_network')
-    @mock.patch('neutron.plugins.ml2.plugin.Ml2Plugin.get_networks')
-    @mock.patch('neutron.plugins.ml2.drivers.ovn.mech_driver.ovsdb.'
-                'ovn_client.OVNClient._get_router_ports')
-    @mock.patch('neutron.db.l3_db.L3_NAT_dbonly_mixin.add_router_interface')
-    def test_add_router_interface_need_to_frag_enabled_then_remove(
-            self, ari, grps, gns, gn):
-        config.cfg.CONF.set_override(
-            'ovn_emit_need_to_frag', True, group='ovn')
-        router_id = 'router-id'
-        interface_info = {'port_id': 'router-port-id',
-                'network_id': 'priv-net'}
-        ari.return_value = self.fake_router_interface_info
-        # If we remove the router halfway the return value of
-        # _get_routers_ports will be RouterNotFound
-        grps.side_effect = l3_exc.RouterNotFound(router_id=router_id)
-        self.get_router.return_value = self.fake_router_with_ext_gw
-        network_attrs = {'id': 'prov-net', 'mtu': 1200}
-        prov_net = fake_resources.FakeNetwork.create_one_network(
-                attrs=network_attrs).info()
-        self.fake_router_port['device_owner'] = (
-            constants.DEVICE_OWNER_ROUTER_GW)
-        gn.return_value = prov_net
-        gns.return_value = [self.fake_network]
-
-        self.l3_inst.add_router_interface(self.context, router_id,
-                                          interface_info)
-
-        # Make sure that the "gateway_mtu" option was set to the router port
-        fake_router_port_assert = self.fake_router_port_assert
-        fake_router_port_assert['gateway_chassis'] = mock.ANY
-        fake_router_port_assert['options'] = {}
-
-        self.l3_inst._ovn.add_lrouter_port.assert_called_once_with(
-            **fake_router_port_assert)
-        # Since if_exists = True it will safely return
-        self.l3_inst._ovn.update_lrouter_port(
-            name='lrp-router-port-id', if_exists=True,
-            options=fake_router_port_assert)
-        # If no if_exists is provided, it is defaulted to true, so this
-        # operation is safe.
-        self.l3_inst._ovn.set_lrouter_port_in_lswitch_port.\
-            assert_called_once_with(
-                'router-port-id', 'lrp-router-port-id', is_gw_port=True,
-                lsp_address=ovn_const.DEFAULT_ADDR_FOR_LSP_WITH_PEER)
-
     def _test_get_router_availability_zones(self, azs, expected):
         lr = fake_resources.FakeOvsdbRow.create_one_ovsdb_row(
             attrs={'id': 'fake-router', 'external_ids': {
@@ -1737,8 +1702,13 @@ class OVNL3ExtrarouteTests(test_l3_gw.ExtGwModeIntTestCase,
     def test_router_update_gateway_upon_subnet_create_max_ips_ipv6(self):
         super(OVNL3ExtrarouteTests, self). \
             test_router_update_gateway_upon_subnet_create_max_ips_ipv6()
+        expected_ext_ids = {
+                ovn_const.OVN_ROUTER_IS_EXT_GW: 'true',
+                ovn_const.OVN_SUBNET_EXT_ID_KEY: mock.ANY}
         add_static_route_calls = [
-            mock.call(mock.ANY, ip_prefix='0.0.0.0/0', nexthop='10.0.0.1'),
-            mock.call(mock.ANY, ip_prefix='::/0', nexthop='2001:db8::')]
+            mock.call(mock.ANY, ip_prefix='0.0.0.0/0', nexthop='10.0.0.1',
+                external_ids=expected_ext_ids),
+            mock.call(mock.ANY, ip_prefix='::/0', nexthop='2001:db8::',
+                external_ids=expected_ext_ids)]
         self.l3_inst._ovn.add_static_route.assert_has_calls(
             add_static_route_calls, any_order=True)
